@@ -9,11 +9,10 @@
  *    6. Additional string functions, esl_str*()
  *    7. File path/name manipulation, including tmpfiles.
  *    8. Typed comparison functions.
- *    9. Commonly used background composition (iid) frequencies.
- *   10. Unit tests.
- *   11. Test driver.
- *   12. Examples. 
- *   13. Copyright and license. 
+ *    9. Unit tests.
+ *   10. Test driver.
+ *   11. Examples. 
+ *   12. Copyright and license. 
  */
 #include "esl_config.h"
 
@@ -33,6 +32,10 @@
 #include <sys/types.h>
 #endif
 
+#ifdef HAVE_MPI
+#include <mpi.h>		/* MPI_Abort() may be used in esl_fatal() or other program killers */
+#endif
+
 #include "easel.h"
 
 
@@ -41,6 +44,36 @@
  *****************************************************************/
 static esl_exception_handler_f esl_exception_handler = NULL;
 
+/* Function:  esl_fail()
+ * Synopsis:  Handle a normal failure code/message before returning to caller.
+ *
+ * Purpose:   A "failure" is a normal error that we want to handle
+ *            without terminating the program; we're going to return
+ *            control to the caller with a nonzero error code and
+ *            (optionally) an informative error message formatted
+ *            in <errbuf>.
+ *            
+ *            <esl_fail()> is called internally by the <ESL_FAIL()>
+ *            and <ESL_XFAIL()> macros (see easel.h). The reason to
+ *            have the failure macros call such a simple little
+ *            function is to give us a convenient debugging
+ *            breakpoint. For example, in a <_Validate()> routine that
+ *            needs to do a normal return to a caller, you can set a
+ *            breakpoint in <esl_fail()> to see exactly where the
+ *            validation failed.
+ */
+void
+esl_fail(char *errbuf, const char *format, ...)
+{
+  if (format) {
+    va_list ap;
+    va_start(ap, format);
+    if (errbuf) vsnprintf(errbuf, eslERRBUFSIZE, format, ap);
+    va_end(ap);
+  }
+}
+
+
 /* Function:  esl_exception()
  * Synopsis:  Throw an exception.
  *
@@ -53,8 +86,9 @@ static esl_exception_handler_f esl_exception_handler = NULL;
  *            a non-fatal exception handler.
  *            
  *            Easel programs normally call one of the exception-handling
- *            wrappers <ESL_EXCEPTION()> or <ESL_XEXCEPTION()>, rather
- *            than calling  <esl_exception> directly.
+ *            wrappers <ESL_EXCEPTION()> or <ESL_XEXCEPTION()>, which
+ *            handle the overhead of passing in <use_errno>, <sourcefile>,
+ *            and <sourceline>. <esl_exception> is rarely called directly.
  *            
  *            If no custom exception handler has been registered, the
  *            default behavior is to print a brief message to <stderr>
@@ -65,6 +99,10 @@ static esl_exception_handler_f esl_exception_handler = NULL;
  *            
  *            Fatal exception (source file foo.c, line 42):
  *            Something wicked this way came.
+ *
+ *            Additionally, in an MPI parallel program, the default fatal 
+ *            handler aborts all processes (with <MPI_Abort()>), not just
+ *            the one that called <esl_exception()>. 
  *            
  * Args:      errcode     - Easel error code, such as eslEINVAL. See easel.h.
  *            use_errno   - if TRUE, also use perror() to report POSIX errno message.
@@ -82,6 +120,9 @@ void
 esl_exception(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, ...)
 {
   va_list argp;
+#ifdef HAVE_MPI
+  int     mpiflag;
+#endif
 
   if (esl_exception_handler != NULL) 
     {
@@ -99,6 +140,10 @@ esl_exception(int errcode, int use_errno, char *sourcefile, int sourceline, char
       fprintf(stderr, "\n");
       if (use_errno && errno) perror("system error");
       fflush(stderr);
+#ifdef HAVE_MPI
+      MPI_Initialized(&mpiflag);                 /* we're assuming we can do this, even in a corrupted, dying process...? */
+      if (mpiflag) MPI_Abort(MPI_COMM_WORLD, 1);
+#endif
       abort();
     }
 }
@@ -212,10 +257,17 @@ esl_nonfatal_handler(int errcode, int use_errno, char *sourcefile, int sourcelin
  *            AND the error is guaranteed to be a coding error. For an example,
  *            see <esl_opt_IsOn()>, which triggers a violation if the code
  *            checks for an option that isn't in the code.
+ *            
+ *            In an MPI-parallel program, the entire job is
+ *            terminated; all processes are aborted (<MPI_Abort()>,
+ *            not just the one that called <esl_fatal()>.
+ *            
+ *            If caller is feeling lazy and just wants to terminate
+ *            without any informative message, use <abort()>.
  * 
  * Args:      format  - <sprintf()> formatted exception message, followed
  *                      by any additional necessary arguments for that 
- *                      message.
+ *                      message. 
  *
  * Returns:   (void)
  *
@@ -225,12 +277,20 @@ void
 esl_fatal(const char *format, ...)
 {
   va_list argp;
+#ifdef HAVE_MPI
+  int mpiflag;
+#endif
 
   va_start(argp, format);
   vfprintf(stderr, format, argp);
   va_end(argp);
   fprintf(stderr, "\n");
   fflush(stderr);
+
+#ifdef HAVE_MPI
+  MPI_Initialized(&mpiflag);
+  if (mpiflag) MPI_Abort(MPI_COMM_WORLD, 1);
+#endif
   exit(1);
 }
 /*---------------- end, error handling conventions --------------*/
@@ -412,6 +472,118 @@ esl_usage(FILE *fp, char *progname, char *usage)
   if (appname) free(appname);
   return status;
 }
+
+
+/* Function:  esl_dataheader()
+ * Synopsis:  Standard #-prefixed header lines for output data table
+ *
+ * Purpose:   Print column headers for a space-delimited, fixed-column-width
+ *            data table to <fp>.
+ * 
+ *            Takes a variable number of argument pairs. Each pair is
+ *            <width, label>. The absolute value of <width> is the max
+ *            width of the column. <label> is the column label.
+ *            
+ *            If <width> is negative, left justify the label. (This is
+ *            supposed to mirror the %-8s vs %8s of a printf format.)
+ *             
+ *            Caller marks the end of the argument list
+ *            with a 0 sentinel.
+ *            
+ *            Example: <esl_dataheader(stdout, 8, "name", 3, "A", -4, "B", 0)>
+ *            gives three columns:
+ *            
+ *            \begin{cchunk}
+ *            # name     A B
+ *            #------- --- ----
+ *            \end{cchunk}
+ *            
+ *            The <width> arguments match the widths given in
+ *            <fprintf()>'s or whatever generates the data rows.
+ *            Because the first header line is prefixed by \verb+#+, the
+ *            first column's width argument is inclusive of these two
+ *            extra chars, and therefore the first column label must
+ *            have no more than its <width>-2 chars.  For all other
+ *            column labels, a label's length cannot exceed its
+ *            <width>.
+ *            
+ *            Up to 1024 columns are allowed. (The only reason there's
+ *            a limit is because you're going to forget to add the 0
+ *            sentinel, and we don't want to risk a <while(1)> infinite
+ *            loop.)
+ *
+ * Args:      <fp>                 : output stream
+ *            [<width>, <label]... : width, label pairs
+ *            0                    : sentinel for end of argument list
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEINVAL> if a label is too wide for its width, or if
+ *            the number of columns exceeds the max limit.
+ *            <eslEWRITE> if a write to <fp> fails, which can happen
+ *            if a disk fills up, for example.
+ */
+int 
+esl_dataheader(FILE *fp, ...)
+{
+  va_list ap, ap2;
+  int     width, len;
+  char   *s;
+  int     col     = 0;
+  int     maxcols = 1024;  // limit, to avoid scary while(1) alternative
+  int     leftjustify;
+  int     status;
+
+  va_start(ap, fp);
+  va_copy(ap2, ap);
+  if ( fputc('#', fp) == EOF) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed");
+  for (col = 0; col < maxcols; col++)
+    {
+      width = va_arg(ap, int);
+      if (width == 0) break;
+      if (width < 0)  { leftjustify = TRUE;  width = -width; }
+      else            { leftjustify = FALSE;                 }
+      if (col == 0)  width -= 2;   // First column header -2 char for the "# " prefix
+
+      s   = va_arg(ap, char *);
+      len = strlen(s);
+      if (len > width) {
+	if (col == 0) ESL_XEXCEPTION(eslEINVAL, "esl_dataheader(): first arg (%s) too wide for %d-char column ('# ' leader took 2 chars)", col, s, width+2);
+	else          ESL_XEXCEPTION(eslEINVAL, "esl_dataheader(): arg %d (%s) too wide for %d-char column", col, s, width);
+      }
+
+      if (leftjustify) { if ( fprintf(fp, " %-*s", width, s) < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed"); }
+      else             { if ( fprintf(fp, " %*s",  width, s) < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed"); }
+    }
+  if (col == maxcols)          ESL_XEXCEPTION(    eslEINVAL, "esl_dataheader(): too many args");
+  if ( fputc('\n', fp) == EOF) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed");
+
+  maxcols = col;
+  for (col = 0; col < maxcols; col++)
+    {
+      width = va_arg(ap2, int);
+      if (width < 0) width = -width;
+      if (col == 0) width -= 1;   
+      (void) va_arg(ap2, char *);
+
+      if (col == 0) { if ( fputc('#', fp)   == EOF) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");  }
+      else          { if ( fputc(' ', fp)   == EOF) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");  }
+      while (width--) 
+	if ( fputc('-', fp) == EOF) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); 
+    }
+  if (fputc('\n', fp)       == EOF) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); 
+
+  va_end(ap);
+  va_end(ap2);
+  return eslOK;
+  
+ ERROR:
+  va_end(ap);
+  va_end(ap2);
+  return status;
+}
+
+
 /*-------------------- end, standard miniapp banner --------------------------*/
 
 
@@ -545,19 +717,18 @@ esl_strdup(const char *s, int64_t n, char **ret_dup)
   int   status;
   char *new = NULL;
 
-  if (ret_dup != NULL) *ret_dup = NULL;
-  if (s == NULL) return eslOK;
+  if (s == NULL) {*ret_dup = NULL; return eslOK; }
   if (n < 0) n = strlen(s);
 
   ESL_ALLOC(new, sizeof(char) * (n+1));
   strcpy(new, s);
 
-  if (ret_dup != NULL) *ret_dup = new; else free(new);
+  *ret_dup = new; 
   return eslOK;
 
  ERROR:
-  if (new     != NULL) free(new);
-  if (ret_dup != NULL) *ret_dup = NULL;
+  if (new) free(new);
+  *ret_dup = NULL;
   return status;
 }
 
@@ -743,6 +914,8 @@ esl_strmapcat_noalloc(const ESL_DSQ *inmap, char *dest, int64_t *ldest, const ch
 
   for (xpos = *ldest, cpos = 0; cpos < lsrc; cpos++)
     {
+      if (! isascii(src[cpos])) { dest[xpos++] = inmap[0]; status = eslEINVAL;  continue; }
+
       x = inmap[(int) src[cpos]];
       if       (x <= 127)      dest[xpos++] = x;
       else switch (x) {
@@ -857,6 +1030,10 @@ esl_strtok_adv(char **s, char *delim, char **ret_tok, int *opt_toklen, char *opt
   int   n      = 0;
   int   status = eslEOL;  /* unless proven otherwise */
 
+  /* contract checks */
+  ESL_DASSERT1(( s     != NULL ));
+  ESL_DASSERT1(( delim != NULL ));
+
   tok += strspn(tok, delim);
   if (! *tok) tok = NULL;         /* if *tok = 0, EOL, no token left */
   else
@@ -946,16 +1123,16 @@ esl_vsprintf(char **ret_s, const char *format, va_list *ap)
   if ((n2 = vsnprintf(s, n1+1, format, *ap)) >= n1) 
     {
       ESL_REALLOC(s, sizeof(char) * (n2+1));
-      if (vsnprintf(s, n2+1, format, ap2) == -1) ESL_EXCEPTION(eslESYS, "vsnprintf() failed");
+      if (vsnprintf(s, n2+1, format, ap2) == -1) ESL_XEXCEPTION(eslESYS, "vsnprintf() failed");
     }
-  else if (n2 == -1) ESL_EXCEPTION(eslESYS, "vsnprintf() failed");
+  else if (n2 == -1) ESL_XEXCEPTION(eslESYS, "vsnprintf() failed");
 
   va_end(ap2);
   *ret_s = s;
   return eslOK;
 
  ERROR:
-  if (s != NULL) free(s);
+  if (s) free(s);
   va_end(ap2);
   *ret_s = NULL;
   return status;
@@ -1129,91 +1306,62 @@ esl_str_IsBlank(char *s)
 }
 
 /* Function:  esl_str_IsInteger()
- * Synopsis:  Return TRUE if <s> is an integer; else FALSE.
+ * Synopsis:  Return TRUE if <s> represents an integer; else FALSE.
  *
  * Purpose:   Given a NUL-terminated string <s>, return TRUE
- *            if the complete string is convertible to an integer 
- *            by the rules of <atoi()>.
+ *            if the complete string is convertible to a base-10 integer 
+ *            by the rules of <strtol()> or <atoi()>. 
  *            
- *            Leading whitespace is skipped. A leading sign character
- *            + or - is allowed. A prefix of 0x or 0X indicates
- *            a hexadecimal number follows; a prefix of 0 indicates
- *            that an octal number follows.
+ *            Leading and trailing whitespace is allowed, but otherwise
+ *            the entire string <s> must be convertable. (Unlike <strtol()>
+ *            itself, which will convert a prefix. ' 99 foo' converts
+ *            to 99, but <esl_str_IsInteger()> will return FALSE.
  *            
  *            If <s> is <NULL>, FALSE is returned.
  */
 int
 esl_str_IsInteger(char *s)
 {
-  int hex = FALSE;
+  char *endp;
 
-  if (s == NULL) return FALSE;
-  while (isspace((int) (*s))) s++;      /* skip whitespace */
-  if (*s == '-' || *s == '+') s++;      /* skip leading sign */
-				        /* skip leading conversion signals */
-  if ((strncmp(s, "0x", 2) == 0 && (int) strlen(s) > 2) ||
-      (strncmp(s, "0X", 2) == 0 && (int) strlen(s) > 2))
-    {
-      s += 2;
-      hex = 1;
-    }
-  else if (*s == '0' && (int) strlen(s) > 1)
-    s++;
-				/* examine remainder for garbage chars */
-  if (!hex)  while (*s != '\0') { if (!isdigit ((int) (*s))) return FALSE; s++; }
-  else       while (*s != '\0') { if (!isxdigit((int) (*s))) return FALSE; s++; }
+  if (s == NULL) return FALSE;	        /* it's NULL */
+  (void) strtol(s, &endp, 10);		/* don't need result itself, discard to void */
+  if (endp == s) return FALSE;          /* strtol() can't convert it */
+  for (s = endp; *s != '\0'; s++)
+    if (! isspace(*s)) return FALSE;    /* it has trailing nonconverted nonwhitespace */
   return TRUE;
 }
 
 /* Function:  esl_str_IsReal()
- * Synopsis:  Return TRUE if <s> is a real number; else FALSE.
+ * Synopsis:  Return TRUE if string <s> represents a real number; else FALSE.
  *
  * Purpose:   Given a NUL-terminated string <s>, return <TRUE>
- *            if the string is convertible to a floating-point
- *            real number by the rules of <atof()>. 
+ *            if the string is completely convertible to a floating-point
+ *            real number by the rules of <strtod()> and <atof()>. 
+ *            (Which allow for exponential forms, hexadecimal forms,
+ *            and case-insensitive INF, INFINITY, NAN, all w/ optional
+ *            leading +/- sign.)
+ * 
+ *            No trailing garbage is allowed, unlike <strtod()>. The
+ *            entire string must be convertible, allowing leading and
+ *            trailing whitespace is allowed. '99.0 foo' converts
+ *            to 99.0 with <strtod()> but is <FALSE> for 
+ *            <esl_str_IsReal()>. '  99.0  ' is <TRUE>.
  *            
- *            Leading space is skipped. A leading sign of either
- *            + or - is allowed. Scientific notation is expressed
- *            with either e or E, as in 1.0e12 or 2.1E42.
+ *            If <s> is <NULL>, return <FALSE>.
  */
 int
 esl_str_IsReal(char *s)
 {
-  int gotdecimal = 0;
-  int gotexp     = 0;
-  int gotreal    = 0;
+  char   *endp;
+  double  val;
 
-  if (s == NULL) return FALSE;
-
-  while (isspace((int) (*s))) s++; /* skip leading whitespace */
-  if (*s == '-' || *s == '+') s++; /* skip leading sign */
-
-  /* Examine remainder for garbage. Allowed one '.' and
-   * one 'e' or 'E'; if both '.' and e/E occur, '.'
-   * must be first.
-   */
-  while (*s != '\0')
-    {
-      if (isdigit((int) (*s))) 	gotreal++;
-      else if (*s == '.')
-	{
-	  if (gotdecimal) return FALSE; /* can't have two */
-	  if (gotexp)     return FALSE; /* e/E preceded . */
-	  else gotdecimal++;
-	}
-      else if (*s == 'e' || *s == 'E')
-	{
-	  if (gotexp) return FALSE;	/* can't have two */
-	  else gotexp++;
-	}
-      else if (isspace((int) (*s)))
-	break;
-      s++;
-    }
-
-  while (isspace((int) (*s))) s++;         /* skip trailing whitespace */
-  if (*s == '\0' && gotreal) return TRUE;
-  else return FALSE;
+  if (! s) return FALSE;		      /* <s> is NULL */
+  val = strtod(s, &endp);
+  if (val == 0.0f && endp == s) return FALSE; /* strtod() can't convert it */
+  for (s = endp; *s != '\0'; s++)
+    if (! isspace(*s)) return FALSE;          /* it has trailing nonconverted nonwhitespace */
+  return TRUE;
 }
 
 
@@ -1439,11 +1587,19 @@ esl_FileConcat(const char *dir, const char *file, char **ret_path)
  *            
  *            For example, if <filename> is "foo" and <sfx> is "ssi",
  *            returns "foo.ssi". If <filename> is "foo.db" and <sfx>
- *            is "idx", returns "foo.idx". 
+ *            is "idx", returns "foo.idx". You can remove a suffix
+ *            too; if <filename> is "foo.db", and <sfx> is "", the
+ *            result is "foo".
+ *            
+ *            Caller can either ask for <*ret_newpath> to be a new
+ *            allocation by passing <*ret_newpath = NULL>, or can
+ *            provide a ptr to a preallocated space.
  *
  * Returns:   <eslOK> on success, and <ret_newpath> is set
- *            string "<base_filename>.<sfx>". Caller must <free()>
- *            this string.
+ *            string "<base_filename>.<sfx>". Caller is 
+ *            responsible for free'ing this string, whether it
+ *            provided it as preallocated space or asked for a new
+ *            allocation.
  *
  * Throws:    <eslEMEM> on allocation failure.
  *
@@ -1452,30 +1608,27 @@ esl_FileConcat(const char *dir, const char *file, char **ret_path)
 int 
 esl_FileNewSuffix(const char *filename, const char *sfx, char **ret_newpath)
 {
-  char *new = NULL;
+  char *new = *ret_newpath;  // caller either provides memory, or asks for allocation w/ <NULL>
   char *lastdot;
   int   nf;
   int   status;
 
-  if (ret_newpath != NULL) *ret_newpath = NULL;
-
   lastdot   = strrchr(filename, '.'); /* check for suffix to replace */
   if (lastdot != NULL && 
       strchr(lastdot, eslDIRSLASH) != NULL) 
     lastdot = NULL; /*foo.1/filename case - don't be fooled.*/
   nf = (lastdot == NULL)? strlen(filename) : lastdot-filename;
   
-  ESL_ALLOC(new, sizeof(char) * (nf+strlen(sfx)+2)); /* '.' too */
+  if (! new) ESL_ALLOC(new, sizeof(char) * (nf+strlen(sfx)+2)); /* '.' too */
   strncpy(new, filename, nf);
   *(new+nf) = '.';
   strcpy(new+nf+1, sfx);
 
-  if (ret_newpath != NULL) *ret_newpath = new; else free(new);
+  *ret_newpath = new;  
   return eslOK;
 
  ERROR:
-  if (new         != NULL) free(new);
-  if (ret_newpath != NULL) *ret_newpath = NULL;
+  if (!(*ret_newpath) && new) free(new);
   return status;
 }
 
@@ -1846,7 +1999,7 @@ esl_FCompare(float a, float b, float tol)
  *            approximate equality, by absolute difference.  Return
  *            <eslOK> if equal, <eslFAIL> if not.
  *            
- *            Equality is defined as <fabs(a-b) \leq tol> for finite
+ *            Equality is defined as <fabs(a-b) $\leq$ tol> for finite
  *            <a,b>; or <inf=inf>, <NaN=NaN> when either value is not
  *            finite.
  *            
@@ -1912,162 +2065,47 @@ esl_CCompare(char *s1, char *s2)
 
 
 
-/*****************************************************************
- * 9. Commonly used background composition (iid) frequencies. 
- *****************************************************************/
-
-/* Function:  esl_composition_BL62()
- *
- * Purpose:   Sets <f> to the background frequencies used in
- *            \citep{Henikoff92} to calculate the BLOSUM62
- *            substitution matrix. Caller provides space in <f>
- *            allocated for at least 20 doubles.  The entries are in
- *            alphabetic order A..Y, same as the standard Easel amino
- *            acid alphabet order.
- *
- * Returns:   <eslOK> on success.
- */
-int
-esl_composition_BL62(double *f)
-{
-  f[0]  = 0.074;
-  f[1]  = 0.025;
-  f[2]  = 0.054;
-  f[3]  = 0.054;
-  f[4]  = 0.047;
-  f[5]  = 0.074;
-  f[6]  = 0.026;
-  f[7]  = 0.068;
-  f[8]  = 0.058;
-  f[9]  = 0.099;
-  f[10] = 0.025;
-  f[11] = 0.045;
-  f[12] = 0.039;
-  f[13] = 0.034;
-  f[14] = 0.052;
-  f[15] = 0.057;
-  f[16] = 0.051;
-  f[17] = 0.073;
-  f[18] = 0.013;
-  f[19] = 0.032;
-  return eslOK;
-}
-
-/* Function:  esl_composition_WAG()
- *
- * Purpose:   Sets <f> to the background frequencies used in
- *            \citep{WhelanGoldman01} to calculate the WAG rate
- *            matrix. Caller provides space in <f> allocated for at
- *            least 20 doubles.  The entries are in alphabetic order
- *            A..Y, same as the standard Easel amino acid alphabet
- *            order.
- *
- * Returns:   <eslOK> on success.
- */
-int
-esl_composition_WAG(double *f)
-{
-  f[0]  = 0.086628;                     /* A */
-  f[1]  = 0.019308;	                /* C */
-  f[2]  = 0.057045;	                /* D */
-  f[3]  = 0.058059;	                /* E */
-  f[4]  = 0.038432;	                /* F */
-  f[5]  = 0.083252;	                /* G */
-  f[6]  = 0.024431;	                /* H */
-  f[7]  = 0.048466;	                /* I */
-  f[8]  = 0.062029;	                /* K */
-  f[9]  = 0.086209;	                /* L */
-  f[10] = 0.019503;	                /* M */
-  f[11] = 0.039089;	                /* N */
-  f[12] = 0.045763;	                /* P */
-  f[13] = 0.036728;	                /* Q */
-  f[14] = 0.043972;	                /* R */
-  f[15] = 0.069518;	                /* S */
-  f[16] = 0.061013;	                /* T */
-  f[17] = 0.070896;	                /* V */
-  f[18] = 0.014386;	                /* W */
-  f[19] = 0.035274;	                /* Y */
-  return eslOK;
-}
-
-/* Function:  esl_composition_SW34()
- *
- * Purpose:   Sets <f> to the background frequencies observed in
- *            Swiss-Prot release 34 (21.2M residues).  Caller provides
- *            space in <f> allocated for at least 20 doubles.  The
- *            entries are in alphabetic order A..Y, same as the
- *            standard Easel amino acid alphabet order.
- *
- * Returns:   <eslOK> on success.
- */
-int
-esl_composition_SW34(double *f)
-{
-  f[0]  = 0.075520;                     /* A */
-  f[1]  = 0.016973;                     /* C */
-  f[2]  = 0.053029;                     /* D */
-  f[3]  = 0.063204;                     /* E */
-  f[4]  = 0.040762;                     /* F */
-  f[5]  = 0.068448;                     /* G */
-  f[6]  = 0.022406;                     /* H */
-  f[7]  = 0.057284;                     /* I */
-  f[8]  = 0.059398;                     /* K */
-  f[9]  = 0.093399;                     /* L */
-  f[10] = 0.023569;                     /* M */
-  f[11] = 0.045293;                     /* N */
-  f[12] = 0.049262;                     /* P */
-  f[13] = 0.040231;                     /* Q */
-  f[14] = 0.051573;                     /* R */
-  f[15] = 0.072214;                     /* S */
-  f[16] = 0.057454;                     /* T */
-  f[17] = 0.065252;                     /* V */
-  f[18] = 0.012513;                     /* W */
-  f[19] = 0.031985;                     /* Y */
-  return eslOK;
-}
-
-
-/* Function:  esl_composition_SW50()
- *
- * Purpose:   Sets <f> to the background frequencies observed in
- *            Swiss-Prot release 50.8 (86.0M residues; Oct 2006).
- *
- * Returns:   <eslOK> on success.
- */
-int
-esl_composition_SW50(double *f)
-{
-  f[0] = 0.0787945;		/* A */
-  f[1] = 0.0151600;		/* C */
-  f[2] = 0.0535222;		/* D */
-  f[3] = 0.0668298;		/* E */
-  f[4] = 0.0397062;		/* F */
-  f[5] = 0.0695071;		/* G */
-  f[6] = 0.0229198;		/* H */
-  f[7] = 0.0590092;		/* I */
-  f[8] = 0.0594422;		/* K */
-  f[9] = 0.0963728;		/* L */
-  f[10]= 0.0237718;		/* M */
-  f[11]= 0.0414386;		/* N */
-  f[12]= 0.0482904;		/* P */
-  f[13]= 0.0395639;		/* Q */
-  f[14]= 0.0540978;		/* R */
-  f[15]= 0.0683364;		/* S */
-  f[16]= 0.0540687;		/* T */
-  f[17]= 0.0673417;		/* V */
-  f[18]= 0.0114135;		/* W */
-  f[19]= 0.0304133;		/* Y */
-  return eslOK;
-}
-/*-------------- end, background compositions -------------------*/
 
 
 
 /*****************************************************************
- * 10. Unit tests.
+ * 9. Unit tests.
  *****************************************************************/
 #ifdef eslEASEL_TESTDRIVE
 
+static void
+utest_IsInteger(void)
+{
+  char *goodones[] = { " 99 " };
+  char *badones[]  = {  "",  " 99 foo " };
+  int ngood = sizeof(goodones) / sizeof(char *);
+  int nbad  = sizeof(badones)  / sizeof(char *);
+  int i;
+
+  for (i = 0; i < ngood; i++)
+    if (! esl_str_IsInteger(goodones[i])) esl_fatal("esl_str_IsInteger() should have recognized %s", goodones[i]);
+  for (i = 0; i < nbad;  i++)
+    if (  esl_str_IsInteger(badones[i]))  esl_fatal("esl_str_IsInteger() should not have recognized %s", badones[i]);
+}
+
+static void
+utest_IsReal(void)
+{
+  char *goodones[] = { "99", " \t 99", "-99.00", "+99.00e-12", "+0xabc.defp-12",  "  +INFINITY", "-nan" };
+  char *badones[] = {  "", 
+		       "FIBB_BOVIN/67-212",	/* testing for a fixed bug, 17 Dec 2012, reported by ER */
+  };
+  int ngood = sizeof(goodones) / sizeof(char *);
+  int nbad  = sizeof(badones)  / sizeof(char *);
+  int i;
+
+  for (i = 0; i < ngood; i++)
+    if (! esl_str_IsReal(goodones[i])) esl_fatal("esl_str_IsReal() should have recognized %s", goodones[i]);
+  for (i = 0; i < nbad;  i++)
+    if (  esl_str_IsReal(badones[i]))  esl_fatal("esl_str_IsReal() should not have recognized %s", badones[i]);
+}
+
+
 static void
 utest_strmapcat(void)
 {
@@ -2161,8 +2199,8 @@ utest_sprintf(void)
   if (strcmp(s, "99 bottles of beer")                != 0)     esl_fatal(msg);
   free(s); 
 
-  if (esl_sprintf(&s, NULL)                          != eslOK) esl_fatal(msg);
-  if (s                                              != NULL)  esl_fatal(msg);
+  if (esl_sprintf(&s, NULL) != eslOK) esl_fatal(msg);
+  if (s                     != NULL)  esl_fatal(msg);
 }
 
 
@@ -2221,7 +2259,7 @@ utest_tmpfile_named(void)
 
 
 /*****************************************************************
- * 11. Test driver.
+ * 10. Test driver.
  *****************************************************************/
 
 #ifdef eslEASEL_TESTDRIVE
@@ -2236,6 +2274,8 @@ int main(void)
   esl_exception_SetHandler(&esl_nonfatal_handler);
 #endif
 
+  utest_IsInteger();
+  utest_IsReal();
   utest_strmapcat();
   utest_strtok();
   utest_sprintf();
@@ -2246,7 +2286,7 @@ int main(void)
 #endif /*eslEASEL_TESTDRIVE*/
 
 /*****************************************************************
- * 12. Examples.
+ * 11. Examples.
  *****************************************************************/
 
 #ifdef eslEASEL_EXAMPLE
@@ -2292,13 +2332,13 @@ int main(void)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: easel.c 748 2012-02-14 21:23:06Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/easel.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/  
