@@ -45,22 +45,28 @@
  * ESL_XEXCEPTION() - throwing an exception, with cleanup.
  * 
  * The X versions (with cleanup) require the caller to have an
- * <int status> variable and a <ERROR:> goto target in scope.
+ * <int status> variable and a <ERROR:> goto target in scope, 
+ * which, yes, is a little hacky.
  *
  * Wrapping these macros in <while(0)> loops allows a statement:
  *       if (something) ESL_XEXCEPTION(code,mesg);
  * without the trailing semicolon becoming a null statement after 
  * macro expansion.
+ *
+ * All esl_fail() does is vsnprintf() to the <errbuf>; the reason to
+ * have ESL_FAIL and ESL_XFAIL call esl_fail() is to enable us to set
+ * a debugging breakpoint in esl_fail(), so we can break execution at
+ * a normal failure.
  */
 /*::cexcerpt::error_macros::begin::*/
 #define ESL_FAIL(code, errbuf, ...) do {				\
-    if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);	\
+    esl_fail(errbuf, __VA_ARGS__);                                      \
     return code; }							\
   while (0)
 
 #define ESL_XFAIL(code, errbuf, ...) do {				\
     status = code;							\
-    if (errbuf != NULL) snprintf(errbuf, eslERRBUFSIZE, __VA_ARGS__);	\
+    esl_fail(errbuf, __VA_ARGS__);                                      \
     goto ERROR; }							\
   while (0)
 
@@ -85,10 +91,11 @@
     esl_exception(code, TRUE, __FILE__, __LINE__, __VA_ARGS__);	\
     goto ERROR; }							\
   while (0)
-
 /*::cexcerpt::error_macros::end::*/
 
 
+
+
 /* Return codes for error handler
  */
 /*::cexcerpt::statuscodes::begin::*/
@@ -120,6 +127,7 @@
 #define eslENOFORMAT      25	/* couldn't guess file format   */
 #define eslENOALPHABET    26	/* couldn't guess seq alphabet  */
 #define eslEWRITE         27   	/* write failed (fprintf, etc)  */
+#define eslEINACCURATE    28    /* return val may be inaccurate */
 /*::cexcerpt::statuscodes::end::*/
 
 
@@ -139,12 +147,25 @@
  * ESL_REALLOC() is a newer version of ESL_RALLOC() which doesn't
  * need a tmp ptr. All ESL_RALLOC() calls can be safely converted
  * to ESL_REALLOC() calls.
+ * 
+ * The result of malloc(0) is implementation-defined (either NULL or
+ * a ptr that may not be dereferenced), a bit of a hole in the C
+ * standard. In Easel, we want to avoid having NULL as a valid
+ * non-error result of malloc(), because it confuses static analysis
+ * tools when they see dereferences of possibly NULL pointers. We
+ * therefore treat malloc(0) as an eslEMEM error.
  */
 /*::cexcerpt::alloc_macros::begin::*/
 #define ESL_ALLOC(p, size) do {\
-    if (((p) = malloc(size)) == NULL && (size)) {	\
+    if ( size <= 0 ) { \
+       p = NULL; \
+       status = eslEMEM; \
+       esl_exception(status, FALSE, __FILE__, __LINE__, "zero malloc disallowed"); \
+       goto ERROR;\
+    }\
+    if ( ((p) = malloc(size)) == NULL)  { \
        status = eslEMEM;\
-       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "malloc of size %d failed", size); \
+       esl_exception(status, FALSE, __FILE__, __LINE__, "malloc of size %d failed", size); \
        goto ERROR;\
      }} while (0)
 
@@ -154,7 +175,7 @@
      if ((tmp) != NULL) (p) = (tmp);\
      else {\
        status = eslEMEM;\
-       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize);	\
+       esl_exception(status, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize);	\
        goto ERROR;\
      }} while (0)
 
@@ -165,13 +186,14 @@
      if ((esltmpp) != NULL) (p) = (esltmpp);\
      else {\
        status = eslEMEM;\
-       esl_exception(eslEMEM, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize); \
+       esl_exception(status, FALSE, __FILE__, __LINE__, "realloc for size %d failed", newsize); \
        goto ERROR;\
      }} while (0)
 /*::cexcerpt::alloc_macros::end::*/
 
-/* Convert MB,GB,TB to bytes, using binary definitions (2^20, 2^30, 2^40)
- * More pedantically: mebibytes (MiB), gibibytes (GiB), tebibytes (TiB).
+/* Convert MiB,GiB,TiB to bytes, using binary definitions (2^20, 2^30, 2^40):
+ * Pedantically speaking, that's: mebibytes (MiB), gibibytes (GiB), tebibytes (TiB).
+ * 1 TB = 10^12 bytes; 1 TiB = 2^40 bytes.
  */
 #define ESL_MBYTES(x) ((x) * 1048576) 
 #define ESL_GBYTES(x) ((x) * 1024 * 1048576) 
@@ -191,6 +213,12 @@
 #define esl_byp_IsReturned(p) ((p) != NULL && (*p) == NULL)
 #define esl_byp_IsProvided(p) ((p) != NULL && (*p) != NULL)
 
+/* Sometimes a shared function API dictates arguments that a function
+ * doesn't use, and we want to silence compiler warnings about this.
+ * Putting ESL_UNUSED(x) in the function, for an unused argument <x>,
+ * should silence the compiler, and should generate a no-op.
+ */
+#define ESL_UNUSED(x) (void)(sizeof((x)))
 
 
 /*****************************************************************
@@ -225,8 +253,6 @@
 #endif
 
 
-
-
 /*****************************************************************
  * 5. Defined constants
  *****************************************************************/
@@ -334,11 +360,13 @@ typedef uint8_t ESL_DSQ;
 
 /* Some generic macros for swapping, min, and max.
  */
-#define ESL_SWAP(x, y, type)  do { type tmpxyz = (x); (x) = (y); (y) = tmpxyz; } while (0)
+#define ESL_SWAP(x, y, type)  do { type esltmpxyz = (x); (x) = (y); (y) = esltmpxyz; } while (0)
 #define ESL_MIN(a,b)          (((a)<(b))?(a):(b))
 #define ESL_MAX(a,b)          (((a)>(b))?(a):(b))
 
-static inline float esl_logf(float x) { return (x == 0.0 ? -eslINFINITY : logf(x)); }
+static inline float esl_log (double x) { return (x == 0.0 ? -eslINFINITY : log(x));  } /* avoid fp exceptions; log(0) = -inf is fine */
+static inline float esl_logf(float x)  { return (x == 0.0 ? -eslINFINITY : logf(x)); }
+static inline float esl_log2f(float x) { return (x == 0.0 ? -eslINFINITY : eslCONST_LOG2R * logf(x)); }
 
 /* Typedef: <esl_pos_t> 
  * 
@@ -356,6 +384,28 @@ static inline float esl_logf(float x) { return (x == 0.0 ? -eslINFINITY : logf(x
  */
 typedef int64_t esl_pos_t;
 
+
+/* ESL_ANALYZER_NORETURN
+ *    adds some optional support for clang static analysis. 
+ *    The static analyzer sometimes needs to be clued in when a
+ *    function cannot return: fatal error handlers, for example.
+ *    clang, gcc, and other gcc-like compilers support the __attribute__
+ *    extension on function declarations. We detect this support
+ *    at compile-time in the configure script. Functions that
+ *    don't return are declared like:
+ *       extern void fatal(char *msg, ...) ESL_ANALYZER_NORETURN;    
+ */
+#ifndef ESL_ANALYZER_NORETURN
+#ifdef  HAVE_FUNC_ATTRIBUTE_NORETURN
+#define ESL_ANALYZER_NORETURN __attribute__((__noreturn__))
+#else
+#define ESL_ANALYZER_NORETURN
+#endif
+#endif
+
+
+
+
 /*****************************************************************
  * 8. Void declarations of missing augmentations
  *****************************************************************/
@@ -372,19 +422,21 @@ typedef void ESL_KEYHASH;
 
 /* 1. Error handling. */
 typedef void (*esl_exception_handler_f)(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, va_list argp);
+extern void esl_fail(char *errbuf, const char *format, ...);
 extern void esl_exception(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, ...);
 extern void esl_exception_SetHandler(esl_exception_handler_f);
 extern void esl_exception_ResetDefaultHandler(void);
 extern void esl_nonfatal_handler(int errcode, int use_errno, char *sourcefile, int sourceline, char *format, va_list argp);
-extern void esl_fatal(const char *format, ...);
+extern void esl_fatal(const char *format, ...) ESL_ANALYZER_NORETURN;
 
 /* 2. Memory allocation/deallocation conventions. */
 extern void esl_Free2D(void  **p, int dim1);
 extern void esl_Free3D(void ***p, int dim1, int dim2);
 
 /* 3. Standard banner for Easel miniapplications. */
-extern int  esl_banner(FILE *fp, char *progname, char *banner);
-extern int  esl_usage (FILE *fp, char *progname, char *usage);
+extern int  esl_banner    (FILE *fp, char *progname, char *banner);
+extern int  esl_usage     (FILE *fp, char *progname, char *usage);
+extern int  esl_dataheader(FILE *fp, ...);
 
 /* 4. Improved replacements for some C library functions */
 extern int  esl_fgets(char **buf, int *n, FILE *fp);
@@ -435,25 +487,18 @@ extern int  esl_DCompareAbs(double a, double b, double tol);
 extern int  esl_FCompareAbs(float  a, float  b, float  tol);
 extern int  esl_CCompare(char *s1, char *s2);
 
-/* 9. Commonly used background composition (iid) frequencies. */
-extern int  esl_composition_BL62(double *f);
-extern int  esl_composition_WAG (double *f);
-extern int  esl_composition_SW34(double *f);
-extern int  esl_composition_SW50(double *f);
-
-
 #endif /*eslEASEL_INCLUDED*/
 
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: easel.h 748 2012-02-14 21:23:06Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/easel.h $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
