@@ -10,6 +10,11 @@
  *   6. Test driver
  *   7. Example
  *   8. Copyright and license information
+ *   
+ * To-do:
+ *   -  Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *      on failure due to small n. Compare esl_gumbel. xref J12/93.
+ *      SRE, Wed Nov 27 11:18:12 2013
  */
 #include <esl_config.h>
 
@@ -53,6 +58,9 @@ esl_mixdchlet_Create(int N, int K)
   ESL_MIXDCHLET *pri = NULL;
   int q;
 
+  ESL_DASSERT1( (N > 0) );
+  ESL_DASSERT1( (K > 0) );
+
   ESL_ALLOC(pri, sizeof(ESL_MIXDCHLET));
   pri->pq = NULL; 
   pri->alpha = NULL;
@@ -62,6 +70,7 @@ esl_mixdchlet_Create(int N, int K)
   pri->alpha[0] = NULL;
 
   ESL_ALLOC(pri->alpha[0], sizeof(double) * N * K);
+  if (pri->alpha[0] == NULL) goto ERROR;               // to silence clang static analysis, which gets overzealous about N=0/K=0 -> NULL result
   for (q = 1; q < N; q++)
     pri->alpha[q] = pri->alpha[0] + q*K;
 
@@ -170,17 +179,17 @@ esl_mixdchlet_PerfectBipartiteMatchExists(int **A, int N )
 int
 esl_mixdchlet_Compare(ESL_MIXDCHLET *d1, ESL_MIXDCHLET *d2, double tol)
 {
-  int i,j;
-  int status;
+  int   i,j;
+  int **A = NULL;
+  int   status;
 
   if (d1->N != d2->N) return eslFAIL;
   if (d1->K != d2->K) return eslFAIL;
 
   //set up a 2-D matrix, to store the pairs of components that meet tolerance requirements
-  int **A;
   ESL_ALLOC(A, d1->N * sizeof(int*));
-  for (i=0; i<d1->N; i++)
-    ESL_ALLOC(A[i], d1->N * sizeof(int) );
+  for (i = 0; i < d1->N; i++) A[i] = NULL;
+  for (i = 0; i < d1->N; i++) ESL_ALLOC(A[i], d1->N * sizeof(int) );
 
   // Fill in matrix - OK if component i from d1 is a viable match with component q from d2
   for (i=0; i<d1->N; i++)
@@ -200,12 +209,14 @@ esl_mixdchlet_Compare(ESL_MIXDCHLET *d1, ESL_MIXDCHLET *d2, double tol)
    * matching (aka the marriage problem)
    */
   status = esl_mixdchlet_PerfectBipartiteMatchExists( A, d1->N);
-
-  /* fallthrough*/
+  
+  /* fallthrough */
  ERROR:
-  for (i=0; i<d1->N; i++)
-    free (A[i]);
-  free (A);
+  if (A) {
+    for (i = 0; i < d1->N; i++)
+      if (A[i]) free (A[i]);
+    free (A);
+  }
   return status;
 }
 
@@ -274,6 +285,38 @@ esl_mixdchlet_Dump(FILE *fp, ESL_MIXDCHLET *d)
   return eslOK;
 }
 
+/* esl_dirichlet_MixturePosterior()
+ *
+ * Purpose:   For a count vector <c> of cardinality <K>, and a
+ *            mixture Dirichlet prior <pri>. Calculate mix[],
+ *            the posterior probability P(q | c) of mixture
+ *            component q given the count vector c. Caller must
+ *            provide allocated space for <mix>, of length <K>.
+ *
+ * Returns:   <eslOK> on success, <mix> contains posterior probabilities of
+ *            the Dirichlet components.
+ */
+static int
+esl_dirichlet_MixturePosterior(double *c, int K, ESL_MIXDCHLET *pri, double *mix)
+{
+  int q;      /* counter over mixture components */
+  double val;
+
+  for (q = 0; q < pri->N; q++) {
+    if (pri->pq[q] > 0.0) {
+      esl_dirichlet_LogProbData(c, pri->alpha[q], K, &val);
+      mix[q] =  val + log(pri->pq[q]);
+    }
+    else
+    {
+      mix[q] = -HUGE_VAL;
+    }
+  }
+
+  esl_vec_DLogNorm(mix, pri->N); /* mix[q] is now P(q|c) */
+
+  return eslOK;
+}
 
 /* Function:  esl_mixdchlet_MPParameters()
  *
@@ -286,7 +329,7 @@ esl_mixdchlet_Dump(FILE *fp, ESL_MIXDCHLET *d)
  *            of length <K>.
  *
  * Returns:   <eslOK> on success; <mix> contains posterior probabilities of
- *            the Dirichlet components, and <p> contains mean posterior 
+ *            the Dirichlet components, and <p> contains mean posterior
  *            probability parameter estimates.
  *
  * Throws:    <esl_EINCOMPAT> if <pri> has different cardinality than <c>.
@@ -296,7 +339,6 @@ esl_mixdchlet_MPParameters(double *c, int K, ESL_MIXDCHLET *pri, double *mix, do
 {
   int q;			/* counter over mixture components */
   int x;
-  double val;
   double totc;
   double tota;
   
@@ -305,16 +347,11 @@ esl_mixdchlet_MPParameters(double *c, int K, ESL_MIXDCHLET *pri, double *mix, do
   /* Calculate mix[], the posterior probability
    * P(q | c) of mixture component q given the count vector c.
    */
-  for (q = 0; q < pri->N; q++)
-    if (pri->pq[q] > 0.0)  
-      {
-	esl_dirichlet_LogProbData(c, pri->alpha[q], K, &val);
-	mix[q] =  val + log(pri->pq[q]);
-      }
-    else
-      mix[q] = -HUGE_VAL;
-  esl_vec_DLogNorm(mix, pri->N); /* mix[q] is now P(q|c) */
+  esl_dirichlet_MixturePosterior(c, K, pri, mix);
 
+
+  /* Compute mean posterior estimates for probability parameters
+   */
   totc = esl_vec_DSum(c, K);
   esl_vec_DSet(p, K, 0.);
   for (x = 0; x < K; x++)
@@ -327,6 +364,85 @@ esl_mixdchlet_MPParameters(double *c, int K, ESL_MIXDCHLET *pri, double *mix, do
   esl_vec_DNorm(p, K);
   return eslOK;
 }
+
+
+/* Function:  esl_mixdchlet_BILD_score()
+ *
+ * Purpose:   Compute the BILD score (sensu Altschul et al PLos Compbio 2010)
+ *            for a given count vector <c> of cardinality (alphabet size) <K>,
+ *            under a mixture Dirichlet prior <pri>, and a background
+ *            character distribution <bg>, also cardinality K. The score is
+ *            in bits. Also computes posterior values for (1) Dirichlet mixture
+ *            coefficients ($P(q \mid c)$, performed and returned in a previously
+ *            allocated array, <mix>).
+ *
+ *            Caller must provide allocated space for <mix> (length K), and
+ *            <q> (length 1).
+ *
+ * Returns:   <eslOK> on success; <mix> contains posterior probabilities of
+ *            the Dirichlet components, and <sc> contains the BILD score of
+ *            observation under the prior and bg.
+ *
+ * Throws:    <esl_EINCOMPAT> if <pri> has different cardinality than <c>.
+ */
+int
+esl_mixdchlet_BILD_score(double *c, int K, int N, ESL_MIXDCHLET *pri,
+                   double *mix, double *bg, double *sc)
+{
+  int i;      /* counter over mixture components */
+  int j;
+  double tmp;
+  double val;
+  double totc;
+  double tota;
+
+  if (K != pri->K) ESL_EXCEPTION(eslEINCOMPAT, "cvec's K != mixture Dirichlet's K");
+  if (N != pri->N) ESL_EXCEPTION(eslEINCOMPAT, "cvec's N != mixture Dirichlet's N");
+
+  /* Calculate mix[], the posterior probability
+   * P(q | c) of mixture component q given the count vector c.
+   */
+  esl_dirichlet_MixturePosterior(c, K, pri, mix);
+
+
+  /* Compute probability of observing the given count vector
+   * under the mixture Dirichlet prior, which depends on the
+   * posterior.
+   */
+  *sc = 0.0;
+  totc = esl_vec_DSum(c, K);
+  for (i = 0; i < N; i++) {
+    if (mix[i] > 0) {
+      tota = esl_vec_DSum(pri->alpha[i], K);
+      esl_stats_LogGamma(tota, &tmp);
+      val = tmp;
+
+      esl_stats_LogGamma(tota + totc, &tmp);
+      val -= tmp;
+
+      for (j = 0; j < K; j++) {
+        esl_stats_LogGamma(pri->alpha[i][j] + c[j], &tmp);
+        val += tmp;
+        esl_stats_LogGamma(pri->alpha[i][j], &tmp);
+        val -= tmp;
+      }
+
+      *sc += mix[i] * exp(val);
+    }
+  }
+
+  /* At this point, sc holds the Q value from the Altschul paper.
+   * Get the odds ratio by dividing by the product of background
+   * probabilities for observed counts, (accounting for sequence
+   * weighting).
+   */
+  for (j = 0; j < K; j++) {
+    *sc /= pow(bg[j], c[j]);
+  }
+  *sc = log(*sc)*eslCONST_LOG2R;
+
+  return eslOK;
+}
 /*---------------- end, ESL_MIXDCHLET ---------------------------*/
 
 
@@ -394,6 +510,9 @@ esl_dirichlet_LogProbData(double *c, double *alpha, int K, double *ret_answer)
  *
  * Returns:   <eslOK> on success, and puts result $\log P(c \mid \alpha)$
  *            in <ret_answer>.
+ *            
+ * Throws:    <eslEMEM> on allocation error. Now <*ret_answer> is 
+ *            <-eslINFINITY>.           
  */
 int
 esl_dirichlet_LogProbData_Mixture(double *c, ESL_MIXDCHLET *d, double *ret_answer)
@@ -419,6 +538,7 @@ esl_dirichlet_LogProbData_Mixture(double *c, ESL_MIXDCHLET *d, double *ret_answe
 
  ERROR:
   free(mixq);
+  *ret_answer = -eslINFINITY;
   return status;
 }
 
@@ -439,22 +559,29 @@ esl_dirichlet_LogProbData_Mixture(double *c, ESL_MIXDCHLET *d, double *ret_answe
  *
  * Returns:   <eslOK> on success, and puts result $\log P(c \mid \alpha)$
  *            in <ret_answer>.
+ *            
+ * Throws:    <eslEMEM> on allocation error. Now <*ret_answer> is            
+ *            <-eslINFINITY>.
  */
 static int 
 esl_dirichlet_LogProbDataSet_Mixture(int ntrials, double** counts, ESL_MIXDCHLET* md, double *ret_answer) 
 {
   double val;
-  int i;
+  int    i;
+  int    status;
 
   *ret_answer = 0;
   for (i = 0; i < ntrials; i++) 
     {
-      esl_dirichlet_LogProbData_Mixture(counts[i], md, &val);
+      if (( status = esl_dirichlet_LogProbData_Mixture(counts[i], md, &val)) != eslOK) goto ERROR;
       *ret_answer += val;
     }
   return eslOK;
-}
 
+ ERROR:
+  *ret_answer = -eslINFINITY;
+  return status;
+}
 
 /* Function:  esl_dirichlet_LogProbProbs()
  *
@@ -775,7 +902,7 @@ esl_mixdchlet_Fit(double **c, int nc, ESL_MIXDCHLET *d, int be_verbose)
 					    &mixdchlet_complete_gradient,
 					    (void *) (&data), tol, wrk, &fx);
   if (status != eslOK && status != eslENOHALT) // eslENOHALT? Then take what we've got - it's probably pretty good
-	  goto ERROR;
+    goto ERROR;
 
   /* Convert the final parameter vector back to a mixdchlet
    */
@@ -794,6 +921,7 @@ esl_mixdchlet_Fit(double **c, int nc, ESL_MIXDCHLET *d, int be_verbose)
 }
 
 
+#ifdef eslAUGMENT_RANDOM
 /* Function:  esl_mixdchlet_Fit_Multipass()
  *
  * Purpose:   Given a set of count vectors <c>, find maximum
@@ -818,29 +946,26 @@ esl_mixdchlet_Fit(double **c, int nc, ESL_MIXDCHLET *d, int be_verbose)
  * Returns:   <eslOK> on success, and <best_md> contains the fitted
  *            mixdchlet parameters with best likelihood.
  *
- * Throws:    <eslEMEM> on allocation error, and <d> is left in
- *            in its initial state.
+ * Throws:    <eslEMEM> on allocation error, and the state of <best_md> 
+ *            is undefined.
  */
 int
-esl_mixdchlet_Fit_Multipass(ESL_RANDOMNESS *r, double **c, int nc, int reps, ESL_MIXDCHLET *best_md, int verbose)
+esl_mixdchlet_Fit_Multipass(ESL_RANDOMNESS *rng, double **c, int nc, int reps, ESL_MIXDCHLET *best_md, int verbose)
 {
-  int i, q, k, status;
-  double best_lk = -eslINFINITY;
-  double lk;
-  ESL_MIXDCHLET *md = esl_mixdchlet_Create(best_md->N, best_md->K);
-  int err_cnt = 0;
+  ESL_MIXDCHLET *md      = esl_mixdchlet_Create(best_md->N, best_md->K);
+  double         best_lk = -eslINFINITY;
+  int            err_cnt = 0;
+  int            i, q, k;
+  double         lk;
+  int            status;
   
-  for (i=0; i<reps; i++) 
+  for (i = 0; i < reps; i++) 
     {
       /* for each pass, establish a new random starting point */
+      if (( status = esl_dirichlet_DSampleUniform(rng, md->N, md->pq)) != eslOK) goto ERROR;
       for (q = 0; q < md->N; q++) 
-	{
-	  md->pq[q] = esl_rnd_UniformPositive(r);
-
-	  for (k = 0; k < md->K; k++)
-	    md->alpha[q][k] = 10.0 * esl_rnd_UniformPositive(r);
-	}
-      esl_vec_DNorm(md->pq, md->N);
+	for (k = 0; k < md->K; k++)
+	  md->alpha[q][k] = 10.0 * esl_rnd_UniformPositive(rng);
 
       /* then use Fit to do local search */
       status = esl_mixdchlet_Fit(c, nc, md, 0);
@@ -877,10 +1002,14 @@ esl_mixdchlet_Fit_Multipass(ESL_RANDOMNESS *r, double **c, int nc, int reps, ESL
       fprintf(stdout, "llk = %.3f", best_lk);
     }
 
+  esl_mixdchlet_Destroy(md);
+  return eslOK;
+
  ERROR:
   esl_mixdchlet_Destroy(md);
   return status;
 }
+#endif /*eslAUGMENT_RANDOM*/
 
 #endif /*eslAUGMENT_MINIMIZER*/
 /*----------- end, Dirichlet Maximum likelihood fit from counts ---------------*/
@@ -1140,6 +1269,62 @@ utest_io(ESL_MIXDCHLET *d, double tol)
   return;
 }
 
+static void
+utest_bild()
+{
+  char           *msg         = "esl_dirichlet: BILD unit test failed";
+  ESL_MIXDCHLET  *d           = NULL;
+  int             K           = 4;
+  int             N           = 2;
+  double         *counts;
+  double         *mix;
+  double         *bg;
+  double          sc;
+
+
+  /* Create a mixture Dirichlet */
+  if ((d = esl_mixdchlet_Create(N, K)) == NULL) esl_fatal(msg);
+  //esl_vec_DSet(d->pq,       N, 1.0/N);
+  d->pq[0]       = 0.4;
+  d->pq[1]       = 0.6;
+
+  d->alpha[0][0] = 0.1;
+  d->alpha[0][1] = 0.2;
+  d->alpha[0][2] = 0.3;
+  d->alpha[0][3] = 0.4;
+  esl_vec_DSet(d->alpha[1], K, 1.0);
+
+
+  //simulate count vector
+  counts = malloc(K*sizeof(double));
+  counts[0] = 3.0; //2.2;
+  counts[1] = 1.0; //0.9;
+  counts[2] = 0.0; //4.5;
+  counts[3] = 0.0; //3.0;
+
+  //simulate background probabilities
+  bg = malloc(K*sizeof(double));
+  esl_vec_DSet(bg, K, 1.0/K);
+
+  //allocate working space
+  mix = malloc(K*sizeof(double));
+
+  esl_mixdchlet_BILD_score(counts, K, N, d, mix, bg, &sc);
+
+  if (esl_DCompare(sc, 0.701, 0.001) != eslOK)
+    esl_fatal(msg);
+
+//  fprintf(stderr, "Score is %.3f\n", sc);
+
+  esl_mixdchlet_Destroy(d);
+  free(bg);
+  free(counts);
+  free(mix);
+
+  return;
+}
+
+
 /*
  * For any given sampling effort, there is always a possibility that the resulting
  * count vector will have a higher likelihood under the wrong component than under the
@@ -1393,7 +1578,9 @@ main(int argc, char **argv)
   utest_io(d, tol);
   utest_fit(r, d, ntrials, ncounts, tol, nfit_reps, be_verbose);
   utest_inference(r, d, ntrials, ncounts, be_verbose);
- 
+  utest_bild();
+
+
   esl_randomness_Destroy(r);
   esl_mixdchlet_Destroy(d);
   esl_getopts_Destroy(go);
@@ -1490,14 +1677,14 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_dirichlet.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_dirichlet.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
