@@ -13,6 +13,11 @@
  *  10. Copyright and license information
  *
  * xref STL9/138  
+ * 
+ * To do:
+ *    - Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *      on failure due to small n. Compare esl_gumbel. xref J12/93.
+ *      SRE, Wed Nov 27 11:03:07 2013
  */
 #include "esl_config.h"
 
@@ -314,8 +319,9 @@ esl_exp_FitComplete(double *x, int n, double *ret_mu, double *ret_lambda)
 {
   double mu, mean;
   int    i;
+  int    status;
 
-  if (!n) ESL_EXCEPTION(eslEINVAL, "empty data vector provided for exponential fit");
+  if (!n) ESL_XEXCEPTION(eslEINVAL, "empty data vector provided for exponential fit");
 
   /* ML mu is the lowest score. mu=x is ok in the exponential. */
   mu = x[0];
@@ -328,6 +334,11 @@ esl_exp_FitComplete(double *x, int n, double *ret_mu, double *ret_lambda)
   *ret_mu     = mu;
   *ret_lambda = 1./mean;	/* ML estimate trivial & analytic */
   return eslOK;
+
+ ERROR:
+  *ret_mu     = 0.0;
+  *ret_lambda = 0.0;
+  return status;
 }
 
 /* Function:  esl_exp_FitCompleteScale()
@@ -572,7 +583,7 @@ main(int argc, char **argv)
       else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
       else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
       else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
-      else ESL_EXCEPTION(eslEINVAL, "bad option");
+      else esl_fatal("bad option");
     }
 
   if (be_verbose)
@@ -581,8 +592,7 @@ main(int argc, char **argv)
   r = esl_randomness_Create(0);
   h = esl_histogram_CreateFull(mu, 100., binwidth);
   if (plotfile != NULL) {
-    if ((pfp = fopen(plotfile, "w")) == NULL) 
-      ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
+    if ((pfp = fopen(plotfile, "w")) == NULL) esl_fatal("Failed to open plotfile");
   }
   if (! xmin_set)  xmin  = mu;
   if (! xmax_set)  xmax  = mu+20* (1./lambda);
@@ -596,20 +606,14 @@ main(int argc, char **argv)
   esl_histogram_GetData(h, &data, &ndata);
 
   esl_exp_FitComplete(data, ndata, &emu, &elambda);
-  if (be_verbose)
-    printf("Complete data fit:  mu = %f   lambda = %f\n", emu, elambda);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 10%\n");
+  if (be_verbose) printf("Complete data fit:  mu = %f   lambda = %f\n", emu, elambda);
+  if (fabs( (emu-mu)/mu )             > 0.01) esl_fatal("Error in (complete) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (complete) fitted lambda > 10%\n");
 
   esl_exp_FitCompleteBinned(h, &emu, &elambda);
-  if (be_verbose)
-    printf("Binned data fit:  mu = %f   lambda = %f\n", emu, elambda);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
+  if (be_verbose) printf("Binned data fit:  mu = %f   lambda = %f\n", emu, elambda);
+  if (fabs( (emu-mu)/mu )             > 0.01) esl_fatal("Error in (binned) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (binned) fitted lambda > 10%\n");
 
   if (plot_pdf)     esl_exp_Plot(pfp, mu, lambda, &esl_exp_pdf,     xmin, xmax, xstep);
   if (plot_logpdf)  esl_exp_Plot(pfp, mu, lambda, &esl_exp_logpdf,  xmin, xmax, xstep);
@@ -689,13 +693,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_exponential.c 770 2012-06-06 19:33:59Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_exponential.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
