@@ -13,10 +13,6 @@ typedef struct {
   char *s;			/* used by esl_strtok(); current position in buf.  */
   char  commentchar;		/* often '#'                                       */
 
-  char *tok;			/* _NextLine() may remember a token...             */
-  int   toklen;			/* ... and its length                              */
-  char  tokchar;		/* ... and char that got overwritten by \0, if any */
-
   char *filename;		/* name of opened file; or NULL (if just a stream) */
   int   linenumber;		/* what line is loaded into buf; 1..nlines         */
   char  errbuf[eslERRBUFSIZE];  /* for holding error diagnostics                   */
@@ -42,10 +38,10 @@ extern void esl_fileparser_Close           (ESL_FILEPARSER *efp);
 #endif /*eslFILEPARSER_INCLUDED */
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
