@@ -1,6 +1,21 @@
 /* Statistical routines for gamma distributions.
  * 
- * xref STL10/65
+ * Contents:
+ *   1. Routines for evaluating densities and distributions
+ *   2. Generic API routines: for general interface w/ histogram module
+ *   3. Dumping plots for files
+ *   4. Sampling (requires augmentation w/ random module)
+ *   5. ML fitting to complete data
+ *   6. Test driver
+ *   7. Example
+ *   8. Copyright and license information
+ *   
+ * Xref:  STL10/65
+ * 
+ * To do:
+ *    - Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *      on failure due to small n. Compare esl_gumbel. xref J12/93.
+ *      SRE, Wed Nov 27 11:18:19 2013
  */
 #include "esl_config.h"
 
@@ -14,14 +29,19 @@
 #ifdef eslAUGMENT_RANDOM
 #include "esl_random.h"
 #endif
+#ifdef eslAUGMENT_HISTOGRAM
+#include "esl_histogram.h"
+#endif
 
 static int    tau_by_moments(double *x, int n, double mu, double *ret_tau, 
 			     double *ret_mean, double *ret_logsum);
+static int    tau_by_moments_binned(ESL_HISTOGRAM *g, double mu, double *ret_tau, 
+				    double *ret_mean, double *ret_logsum);
 static double tau_function(double tau, double mean, double logsum);
 
 
 /****************************************************************************
- * Routines for evaluating densities and distributions
+ * 1. Routines for evaluating densities and distributions
  ****************************************************************************/ 
 
 /* Function:  esl_gam_pdf()
@@ -190,7 +210,7 @@ esl_gam_invcdf(double p, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Generic API routines: for general interface w/ histogram module
+ * 2. Generic API routines: for general interface w/ histogram module
  ****************************************************************************/ 
 
 /* Function:  esl_gam_generic_pdf()
@@ -252,7 +272,7 @@ esl_gam_generic_invcdf(double x, void *params)
 
 
 /****************************************************************************
- * Routines for dumping plots for files
+ * 3. Dumping plots for files
  ****************************************************************************/ 
 
 /* Function:  esl_gam_Plot()
@@ -281,7 +301,7 @@ esl_gam_Plot(FILE *fp, double mu, double lambda, double tau,
 
 
 /****************************************************************************
- * Routines for sampling (requires augmentation w/ random module)
+ * 4. Sampling (requires augmentation w/ random module)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 /* Function:  esl_gam_Sample()
@@ -302,7 +322,7 @@ esl_gam_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Maximum likelihood fitting
+ * 5. ML fitting to complete data
  ****************************************************************************/ 
 
 /* Function:  esl_gam_FitComplete()
@@ -335,9 +355,9 @@ esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret
   double b, fb;
   int    status;
 
-  if ((status = tau_by_moments(x, n, mu, &c, &mean, &logsum) != eslOK)) return status;
+  if ((status = tau_by_moments(x, n, mu, &c, &mean, &logsum) != eslOK)) goto ERROR;
   a = b = c;
-  fa=fb=fc = tau_function(c, mean, logsum);
+  fc = tau_function(c, mean, logsum);
 
   /* Rootfinding, 1.: bracketing the root with points a,b.
    */
@@ -348,9 +368,9 @@ esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret
 	  b = a * 2.;
 	  fb = tau_function(b, mean, logsum);
 	  if (fb < 0.) break;	/* a,b now bracket */
-	  a = b; fa = fb;	/* else fb>0, so b is a better left bracket than a */
+	  a = b;                /* else fb>0, so b is a better left bracket than a */
 	}
-      if (i == 100) ESL_EXCEPTION(eslENOHALT, "failed to bracket");
+      if (i == 100) ESL_XEXCEPTION(eslENOHALT, "failed to bracket");
     }
   else if (fc < 0.)		/* fx<0 means tau is too large, search left */
     {
@@ -359,9 +379,9 @@ esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret
 	  a = b/2.;
 	  fa = tau_function(a, mean, logsum);
 	  if (fa > 0.) break;   /* a,b now bracket */
-	  b = a; fb = fa;	/* else fa<0, so a is a better right bracket than b */
+	  b = a;                /* else fa<0, so a is a better right bracket than b */
 	}
-      if (i == 100) ESL_EXCEPTION(eslENOHALT, "failed to bracket");
+      if (i == 100) ESL_XEXCEPTION(eslENOHALT, "failed to bracket");
     }  
   
   /* Rootfinding, 2.: Bisection search.
@@ -371,8 +391,8 @@ esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret
     {
       c  = (a+b)/2.;		/* bisection */
       fc = tau_function(c, mean, logsum);
-      if      (fc > 0.) { a = c; fa = fc; }
-      else if (fc < 0.) { b = c; fb = fc; }
+      if      (fc > 0.)  a = c;
+      else if (fc < 0.)  b = c;
       else    break;		/* unlikely event that we nail it */
 
       if ((b-a) <= 2.* DBL_EPSILON) { 
@@ -380,11 +400,17 @@ esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret
 	break;
       }
     }
-  if (i == 100) ESL_EXCEPTION(eslENOHALT, "bisection search failed");
+  if (i == 100) ESL_XEXCEPTION(eslENOHALT, "bisection search failed");
 
   *ret_lambda = c / mean;
   *ret_tau    = c;
   return eslOK;
+
+
+ ERROR:
+  *ret_lambda = 0.0;
+  *ret_tau    = 0.0;
+  return status;
 }
 
 /* tau_by_moments()
@@ -423,7 +449,6 @@ tau_by_moments(double *x, int n, double mu, double *ret_tau, double *ret_mean, d
 }
 
 
-
 /* tau_function()
  *
  * This is the rootfinding equation for tau...
@@ -447,6 +472,306 @@ tau_function(double tau, double mean, double logsum)
   return ( ((log(tau) - psitau) - log(mean)) + logsum );  
 }
 
+#ifdef eslAUGMENT_HISTOGRAM
+/* Function:  esl_gam_FitCompleteBinned()
+ *
+ * Purpose:   Fit a complete exponential distribution to the observed
+ *            binned data in a histogram <g>, where each
+ *            bin i holds some number of observed samples x with values from 
+ *            lower bound l to upper bound u (that is, $l < x \leq u$);
+ *            determine and return maximum likelihood estimates for the
+ *            parameters $\mu, \lambda, \tau$ and 
+ *            return them in <*ret_mu>, <*ret_lambda>, <*ret_tau>.
+ *            
+ *            Unlike the <esl_exp_FitCompleteBinned()> case where the
+ *            ML fit optimizes $\sum_i n_i \log P(a_i \leq x < b_i)$
+ *            where $a_i \leq b_i$ are the bounds of bin i with
+ *            occupancy $n_i$, here we take the approximation that
+ *            $c_i = a_i + 0.5*(b_i-a_i)$ and optimize $\log P(a_i
+ *            \leq x < b_i) \simeq \log(w) + \log P(x=c_i)$.
+ *
+ *            Since $b_i-a_i = w$ is fixed, optimizing the above
+ *            becomes equivalent to optimizing $\sum_i n_i * log P(x=c_i)$.
+ *
+ *            The optimization is then equivalent to the non-binned case,
+ *            but subsituting in averages such as $\sum_i x(i)$ by
+ *            $\sum_i n_i*c_i i$, and so forth.
+ *
+ *            If the binned data in <g> were set to focus on 
+ *            a tail by virtual censoring, the "complete" exponential is 
+ *            fitted to this tail. The caller then also needs to
+ *            remember what fraction of the probability mass was in this
+ *            tail.
+ *
+ * Args:      g          - histogram
+ *            ret_mu     - RETURN: given by the histogram           
+ *            ret_lambda - RETURN: ML estimate of lambda            
+ *            ret_tau    - RETURN: ML estimate of tau
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslENOHALT> if bracketing or bisection fails;
+ *            <eslEINVAL> if data cannot be gamma distributed (some <x[i] < mu>,
+ *            or zero variance in x).
+ *
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEINVAL> if dataset is true-censored.
+ */
+int
+esl_gam_FitCompleteBinned(ESL_HISTOGRAM *g, double *ret_mu, double *ret_lambda, double *ret_tau)
+{
+  double mu = 0.;
+  double mean, logsum;
+  int    i;
+  double c, fc;
+  double a, fa;
+  double b, fb;
+  double tol = 1e-6;
+  int    maxit = 100;
+  int    status;
+
+  if (g->dataset_is == COMPLETE)
+    {
+      if   (g->is_rounded) mu = esl_histogram_Bin2LBound(g, g->imin);
+      else                 mu = g->xmin;
+    }
+  else if (g->dataset_is == VIRTUAL_CENSORED) /* i.e., we'll fit to tail */
+    mu = g->phi;
+  else if (g->dataset_is == TRUE_CENSORED)
+    ESL_EXCEPTION(eslEINVAL, "can't fit true censored dataset");
+
+  if ((status = tau_by_moments_binned(g, mu, &c, &mean, &logsum) != eslOK)) goto ERROR;
+  a = b = c;
+  if (c == 1.0) {
+    *ret_mu     = mu;
+    *ret_lambda = c / mean;
+    *ret_tau    = c;
+    return eslOK;
+  }
+  fc = tau_function(c, mean, logsum);
+
+  /* Rootfinding, 1.: bracketing the root with points a,b.
+   */
+  if (fc > 0.)			/* fx>0 means tau is too small, search right */
+    {
+      for (i = 0; i < maxit; i++)	/* max iterations */
+	{
+	  b = a * 2.;
+	  fb = tau_function(b, mean, logsum);
+
+	  if (fb < 0.) break;	/* a,b now bracket */
+	  a = b;                /* else fb>0, so b is a better left bracket than a */
+	}
+      if (i == maxit) ESL_XEXCEPTION(eslENOHALT, "failed to bracket");
+    }
+  else if (fc < 0.)		/* fx<0 means tau is too large, search left */
+    {
+      for (i = 0; i < maxit; i++)
+	{
+	  a = b/2.;
+	  fa = tau_function(a, mean, logsum);
+	  if (fa > 0.) break;   /* a,b now bracket */
+	  b = a;                /* else fa<0, so a is a better right bracket than b */
+	}
+      if (i == maxit) ESL_XEXCEPTION(eslENOHALT, "failed to bracket");
+    }  
+
+  /* Rootfinding, 2.: Bisection search.
+   * We have the root in interval (a,b).
+   */
+  for (i = 0; i < maxit; i++)
+    {
+      c  = (a+b)/2.;		/* bisection */
+      fc = tau_function(c, mean, logsum);
+
+      if      (fc > 0.)  a = c;
+      else if (fc < 0.)  b = c;
+      else    break;		/* unlikely event that we nail it */
+
+      if ((b-a) <= tol) { 
+	c  = (a+b)/2.;
+	break;
+      }
+    }
+  if (i == maxit) ESL_XEXCEPTION(eslENOHALT, "bisection search failed");
+
+  *ret_mu     = mu;
+  *ret_lambda = (mean > 0.)? c / mean : 0.0;
+  *ret_tau    = c;
+  return eslOK;
+
+ ERROR:
+  *ret_mu     = 0.;
+  *ret_lambda = 0.;
+  *ret_tau    = 0.;
+  return status;
+}
+
+/* tau_by_moments_binned()
+ * 
+ * similar to tau_by_moments()
+ * where mean=\sum_i x_i now becomes mean=\sum_i n(i)*ci, ...
+ *
+ * note: the whole method relies on the property log(sum) >= logsum;
+ * which works if all points are valide, that is positive; 
+ * log(0) = -inf is not a valid point,
+ * and the inequality (Jensen's inequality) does not hold. 
+ */
+static int
+tau_by_moments_binned(ESL_HISTOGRAM *g, double mu, double *ret_tau, double *ret_mean, double *ret_logsum)
+{
+  int    i;
+  double ai, bi, ci;
+  double sum, mean, var, logsum;
+  double tol = 1e-6;
+
+  sum = mean = var = logsum = 0.;
+  for (i = g->cmin+1; i <= g->imax; i++) /* for each occupied bin */
+    {
+      if (g->obs[i] == 0) continue;
+      ai = esl_histogram_Bin2LBound(g,i);
+      bi = esl_histogram_Bin2UBound(g,i);
+      ci = ai + 0.5 * (bi-ai);
+      
+      if (ci < mu) ESL_EXCEPTION(eslEINVAL, "No point can be < mu in gamma data");
+      sum    += (double)g->obs[i];
+      mean   += (double)g->obs[i] * (ci-mu);	                   /* mean is temporarily just the sum */
+      logsum += (ci>mu)? (double)g->obs[i] * log(ci-mu):0.0;       
+      var    += (double)g->obs[i] * (ci-mu) * (ci-mu);             /* var is temporarily the sum of squares */
+    }
+
+  var     = (sum > 1.)? (var - mean*mean/sum) / (sum-1.) : 0.0; /* now var is the variance */
+  mean   /= (sum > 0.)? sum : 1.;	                                        /* and now mean is the mean */
+  logsum /= (sum > 0.)? sum : 1.;
+
+  if (ret_tau    != NULL) *ret_tau    = (var < tol || mean == 0.)? 1. :  mean * mean / var;
+  if (ret_mean   != NULL) *ret_mean   = mean;
+  if (ret_logsum != NULL) *ret_logsum = logsum;
+  return eslOK;
+}
+
+
+#endif /*eslAUGMENT_HISTOGRAM*/
+
+
+/****************************************************************************
+ * 6. Test driver
+ ****************************************************************************/ 
+#ifdef eslGAMMA_TESTDRIVE
+/* Compile:
+   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test -DeslGAMMA_TESTDRIVE\
+    esl_gamma.c -leasel -lm
+*/
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "easel.h"
+#include "esl_random.h"
+#include "esl_histogram.h"
+#include "esl_gamma.h"
+
+int
+main(int argc, char **argv)
+{
+  ESL_HISTOGRAM  *h;
+  ESL_RANDOMNESS *r;
+  double  mu        = -5.0;
+  double  lambda    =  2.0;  
+  double  tau       =  0.7;
+  int     n         = 10000;
+  double  binwidth  = 0.0001;
+  double  emu, elambda, etau;
+  int     i;
+  double  x;
+  double *data;
+  int     ndata;
+
+  int     opti;
+  int     be_verbose   = FALSE;
+  char   *plotfile     = NULL;
+  FILE   *pfp          = stdout;
+  int     plot_pdf     = FALSE;
+  int     plot_logpdf  = FALSE;
+  int     plot_cdf     = FALSE;
+  int     plot_logcdf  = FALSE;
+  int     plot_surv    = FALSE;
+  int     plot_logsurv = FALSE;
+  int     xmin_set     = FALSE;
+  double  xmin;
+  int     xmax_set     = FALSE;
+  double  xmax;
+  int     xstep_set    = FALSE;
+  double  xstep;
+
+  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
+    {
+      if      (strcmp(argv[opti], "-m")  == 0) mu           = atof(argv[++opti]);
+      else if (strcmp(argv[opti], "-l")  == 0) lambda       = atof(argv[++opti]);
+      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
+      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
+      else if (strcmp(argv[opti], "-t")  == 0) tau          = atof(argv[++opti]);
+      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
+      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
+      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
+      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
+      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
+      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
+      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
+      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
+      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
+      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
+      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
+      else esl_fatal("bad option");
+    }
+
+  if (be_verbose)
+    printf("Parametric:  mu = %f   lambda = %f    tau = %f\n", mu, lambda, tau);
+
+  r = esl_randomness_Create(0);
+  h = esl_histogram_CreateFull(mu, 100., binwidth);
+  if (plotfile != NULL) {
+    if ((pfp = fopen(plotfile, "w")) == NULL) esl_fatal("Failed to open plotfile");
+  }
+  if (! xmin_set)  xmin  = mu;
+  if (! xmax_set)  xmax  = mu+40*(1./lambda);
+  if (! xstep_set) xstep = 0.1;
+
+  for (i = 0; i < n; i++)
+    {
+      x = esl_gam_Sample(r, mu, lambda, tau);
+      esl_histogram_Add(h, x);
+    }
+  esl_histogram_GetData(h, &data, &ndata);
+
+  esl_gam_FitComplete(data, ndata, mu, &elambda, &etau);
+  if (be_verbose)
+    printf("Complete data fit:  mu = %f   lambda = %f   tau = %f\n", mu, elambda, etau);
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (complete) fitted lambda > 10%%\n");
+  if (fabs( (etau-tau)/tau )          > 0.10) esl_fatal("Error in (complete) fitted tau > 10%%\n");
+
+  esl_gam_FitCompleteBinned(h, &emu, &elambda, &etau);
+  if (be_verbose) printf("Binned data fit:  mu = %f   lambda = %f  tau = %f\n", emu, elambda, etau);
+  if (fabs( (emu-mu)/mu )             > 0.01) esl_fatal("Error in (binned) fitted mu > 1%%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (binned) fitted lambda > 10%%\n");
+  if (fabs( (etau-tau)/tau )          > 0.10) esl_fatal("Error in (binned) fitted tau > 10%%\n");
+
+  if (plot_pdf)     esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_pdf,     xmin, xmax, xstep);
+  if (plot_logpdf)  esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logpdf,  xmin, xmax, xstep);
+  if (plot_cdf)     esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_cdf,     xmin, xmax, xstep);
+  if (plot_logcdf)  esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logcdf,  xmin, xmax, xstep);
+  if (plot_surv)    esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_surv,    xmin, xmax, xstep);
+  if (plot_logsurv) esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logsurv, xmin, xmax, xstep);
+
+  if (plotfile != NULL) fclose(pfp);
+
+  esl_randomness_Destroy(r);
+  esl_histogram_Destroy(h);
+  return 0;
+}
+#endif /*eslGAMMA_TESTDRIVE*/
 
 /****************************************************************************
  * Example main()
@@ -506,130 +831,15 @@ main(int argc, char **argv)
 
 
 
-/****************************************************************************
- * Test driver
- ****************************************************************************/ 
-#ifdef eslGAMMA_TESTDRIVE
-/* Compile:
-   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test -DeslGAMMA_TESTDRIVE\
-    esl_gamma.c -leasel -lm
-*/
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "easel.h"
-#include "esl_random.h"
-#include "esl_histogram.h"
-#include "esl_gamma.h"
-
-int
-main(int argc, char **argv)
-{
-  ESL_HISTOGRAM  *h;
-  ESL_RANDOMNESS *r;
-  double  mu        = -5.0;
-  double  lambda    =  2.0;  
-  double  tau       =  0.7;
-  int     n         = 10000;
-  double  binwidth  = 0.1;
-  double  elambda, etau;
-  int     i;
-  double  x;
-  double *data;
-  int     ndata;
-
-  int     opti;
-  int     be_verbose   = FALSE;
-  char   *plotfile     = NULL;
-  FILE   *pfp          = stdout;
-  int     plot_pdf     = FALSE;
-  int     plot_logpdf  = FALSE;
-  int     plot_cdf     = FALSE;
-  int     plot_logcdf  = FALSE;
-  int     plot_surv    = FALSE;
-  int     plot_logsurv = FALSE;
-  int     xmin_set     = FALSE;
-  double  xmin;
-  int     xmax_set     = FALSE;
-  double  xmax;
-  int     xstep_set    = FALSE;
-  double  xstep;
-
-  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
-    {
-      if      (strcmp(argv[opti], "-m")  == 0) mu           = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-l")  == 0) lambda       = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
-      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
-      else if (strcmp(argv[opti], "-t")  == 0) tau          = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
-      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
-      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
-      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
-      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
-      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
-      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
-      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
-      else ESL_EXCEPTION(eslEINVAL, "bad option");
-    }
-
-  if (be_verbose)
-    printf("Parametric:  mu = %f   lambda = %f    tau = %f\n", mu, lambda, tau);
-
-  r = esl_randomness_Create(0);
-  h = esl_histogram_CreateFull(mu, 100., binwidth);
-  if (plotfile != NULL) {
-    if ((pfp = fopen(plotfile, "w")) == NULL) 
-      ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
-  }
-  if (! xmin_set)  xmin  = mu;
-  if (! xmax_set)  xmax  = mu+40*(1./lambda);
-  if (! xstep_set) xstep = 0.1;
-
-  for (i = 0; i < n; i++)
-    {
-      x = esl_gam_Sample(r, mu, lambda, tau);
-      esl_histogram_Add(h, x);
-    }
-  esl_histogram_GetData(h, &data, &ndata);
-
-  esl_gam_FitComplete(data, ndata, mu, &elambda, &etau);
-  if (be_verbose)
-    printf("Complete data fit:  mu = %f   lambda = %f   tau = %f\n", 
-	   mu, elambda, etau);
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 10%\n");
-  if (fabs( (etau-tau)/tau ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted tau > 10%\n");
-
-  if (plot_pdf)     esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_pdf,     xmin, xmax, xstep);
-  if (plot_logpdf)  esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logpdf,  xmin, xmax, xstep);
-  if (plot_cdf)     esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_cdf,     xmin, xmax, xstep);
-  if (plot_logcdf)  esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logcdf,  xmin, xmax, xstep);
-  if (plot_surv)    esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_surv,    xmin, xmax, xstep);
-  if (plot_logsurv) esl_gam_Plot(pfp, mu, lambda, tau, &esl_gam_logsurv, xmin, xmax, xstep);
-
-  if (plotfile != NULL) fclose(pfp);
-
-  esl_randomness_Destroy(r);
-  esl_histogram_Destroy(h);
-  return 0;
-}
-#endif /*eslGAMMA_TESTDRIVE*/
-
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_gamma.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gamma.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
