@@ -1,8 +1,8 @@
 /* Gamma distributions.
  * 
  * SRE, Wed Nov 16 19:15:33 2005 [St. Louis]
- * SVN $Id: esl_gamma.h 664 2011-02-27 17:08:36Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_gamma.h $
+ * SVN $Id$
+ * SVN $URL$
  */
 #ifndef eslGAMMA_INCLUDED
 #define eslGAMMA_INCLUDED
@@ -10,6 +10,10 @@
 #ifdef eslAUGMENT_RANDOM
 #include <esl_random.h>
 #endif
+#ifdef eslAUGMENT_HISTOGRAM
+#include "esl_histogram.h"
+#endif
+
 
 double esl_gam_pdf    (double x, double mu, double lambda, double tau);
 double esl_gam_logpdf (double x, double mu, double lambda, double tau);
@@ -34,13 +38,17 @@ extern double esl_gam_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double
 
 extern int esl_gam_FitComplete(double *x, int n, double mu, double *ret_lambda, double *ret_tau);
 
+#ifdef eslAUGMENT_HISTOGRAM
+extern int esl_gam_FitCompleteBinned(ESL_HISTOGRAM *h, double *ret_mu, double *ret_lambda, double *ret_tau);
+#endif
+
 #endif /*eslGAMMA_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
