@@ -141,8 +141,17 @@ esl_getopts_Create(ESL_OPTIONS *opt)
  *            
  *            <options> is an array of <ESL_OPTIONS> structures describing
  *            the options, terminated by an all-<NULL> structure.
- *            <nargs> is the number of command-line arguments
- *            expected. <argc> and <argv> are the command line
+ *
+ *            <nargs> is the number of commandline arguments
+ *            expected. If the number of commandline arguments isn't
+ *            equal to this, an error message is printed, with the
+ *            <usage> string, and <exit()> is called. If <nargs> is
+ *            -1, this check isn't done; if your program deliberately
+ *            has a variable number of commandline arguments (i.e.
+ *            if the number is unknown at compile time), pass -1
+ *            for <nargs>.
+ *
+ *            <argc> and <argv> are the command line
  *            arguments (number and pointer array) from <main()>.
  *            
  *            <banner> is an optional one-line description of the
@@ -208,7 +217,7 @@ esl_getopts_CreateDefaultApp(ESL_OPTIONS *options, int nargs, int argc, char **a
       esl_opt_DisplayHelp(stdout, go, 0, 2, 80);
       exit(0);
     }
-  if (esl_opt_ArgNumber(go) != nargs) 
+  if (nargs != -1 && esl_opt_ArgNumber(go) != nargs) 
     {
       puts("Incorrect number of command line arguments.");
       esl_usage(stdout, argv[0], usage);
@@ -887,7 +896,7 @@ esl_opt_GetBoolean(const ESL_GETOPTS *g, char *optname)
   if (get_optidx_exactly(g, optname, &opti) == eslENOTFOUND)
     esl_fatal("no such option %s\n", optname);
   if (g->opt[opti].type != eslARG_NONE)
-    esl_fatal("option %s is not a boolean", optname);
+    esl_fatal("option %s is not a boolean; code called _GetBoolean", optname);
 
   if (g->val[opti] == NULL) return FALSE;
   else                      return TRUE;
@@ -907,7 +916,7 @@ esl_opt_GetInteger(const ESL_GETOPTS *g, char *optname)
   if (get_optidx_exactly(g, optname, &opti) == eslENOTFOUND)
     esl_fatal("no such option %s\n", optname);
   if (g->opt[opti].type != eslARG_INT)
-    esl_fatal("option %s does not take an integer arg", optname);
+    esl_fatal("option %s does not take an integer arg; code called _GetInteger", optname);
   return atoi(g->val[opti]);
 }
 		
@@ -925,7 +934,7 @@ esl_opt_GetReal(const ESL_GETOPTS *g, char *optname)
   if (get_optidx_exactly(g, optname, &opti) == eslENOTFOUND)
     esl_fatal("no such option %s\n", optname);
   if (g->opt[opti].type != eslARG_REAL)
-    esl_fatal("option %s does not take a real-valued arg", optname);
+    esl_fatal("option %s does not take a real-valued arg; code called _GetReal", optname);
 
   return (atof(g->val[opti]));
 }
@@ -944,7 +953,7 @@ esl_opt_GetChar(const ESL_GETOPTS *g, char *optname)
   if (get_optidx_exactly(g, optname, &opti) == eslENOTFOUND)
     esl_fatal("no such option %s\n", optname);
   if (g->opt[opti].type != eslARG_CHAR)
-    esl_fatal("option %s does not take a char arg", optname);
+    esl_fatal("option %s does not take a char arg; code called _GetChar", optname);
 
   return (*g->val[opti]);
 }
@@ -969,7 +978,7 @@ esl_opt_GetString(const ESL_GETOPTS *g, char *optname)
   if (g->opt[opti].type != eslARG_STRING &&
       g->opt[opti].type != eslARG_INFILE &&
       g->opt[opti].type != eslARG_OUTFILE)
-    esl_fatal("option %s does not take a string arg", optname);
+    esl_fatal("option %s does not take a string arg; code called _GetString", optname);
 
   return g->val[opti];
 }
@@ -1165,7 +1174,7 @@ int
 set_option(ESL_GETOPTS *g, int opti, char *optarg, int setby, int do_alloc)
 {
   int   arglen;
-  char *where;
+  char *where = NULL;
   char *s;
   int   togi;
   int   status;
@@ -1254,7 +1263,8 @@ set_option(ESL_GETOPTS *g, int opti, char *optarg, int setby, int do_alloc)
  * the index of the option, and return eslOK. <optname>
  * must exactly match one of the options in <g>.
  * 
- * If the option is not found, return eslENOTFOUND.
+ * If the option is not found, return <eslENOTFOUND>; in this
+ * case <*ret_opti> is -1 (and caller should not use it!)
  */
 static int
 get_optidx_exactly(const ESL_GETOPTS *g, char *optname, int *ret_opti)
@@ -1263,6 +1273,7 @@ get_optidx_exactly(const ESL_GETOPTS *g, char *optname, int *ret_opti)
 
   for (i = 0; i < g->nopts; i++)
     if (strcmp(optname, g->opt[i].name) == 0) { *ret_opti = i; return eslOK; }
+  *ret_opti = -1;
   return eslENOTFOUND;
 }
 
@@ -1560,7 +1571,7 @@ process_stdopt(ESL_GETOPTS *g, int *ret_opti, char **ret_optarg)
 static int
 verify_type_and_range(ESL_GETOPTS *g, int i, char *val, int setby)
 {
-  char *where;
+  char *where = NULL;
 
   if       (setby == eslARG_SETBY_DEFAULT) where = "as default";
   else if  (setby == eslARG_SETBY_CMDLINE) where = "on cmdline";
@@ -2150,15 +2161,15 @@ main(int argc, char **argv)
 
 /*****************************************************************  
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_getopts.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_getopts.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 
