@@ -32,34 +32,39 @@ static int esl_histogram_sort(ESL_HISTOGRAM *h);
  * Synopsis:  Create a new <ESL_HISTOGRAM>.
  *
  * Purpose:   Creates and returns a new histogram object, initially
- *            allocated to count scores $>$ <xmin> and $<=$ <xmax> into
- *            bins of width <w>. Thus, a total of <xmax>-<xmin>/<w> bins
+ *            allocated to count values $>$ <bmin> and $<=$ <bmax> into
+ *            bins of width <w>. Thus, a total of <bmax>-<bmin>/<w> bins
  *            are initially created. 
  *            
- *            The lower bound <xmin> and the width <w> permanently
+ *            The lower bound <bmin> and the width <w> permanently
  *            determine the offset and width of the binning, but not
  *            the range.  For example, <esl_histogram_Create(-100,
- *            100, 0.5)> would init the object to collect scores into
+ *            100, 0.5)> would initialize the object to collect scores into
  *            400 bins $[-100< x \leq -99.5],[-99.5 < x \leq
  *            -99.0]...[99.5 <x \leq 100.0]$.  Aside from this, the
- *            range specified by the bounds <xmin> and <xmax> only
+ *            range specified by the bounds <bmin> and <bmax> only
  *            needs to be an initial guess. The histogram object will
  *            reallocate itself dynamically as needed to accommodate
  *            scores that exceed current bounds.
  *
- *            You can be sloppy about <xmax>; it does not have to
+ *            You can be sloppy about <bmax>; it does not have to
  *            exactly match a bin upper bound. The initial allocation
  *            is for all full-width bins with upper bounds $\leq
- *            xmax$.
+ *            bmax$.
  *
  *            <esl_histogram_Create()> creates a simplified histogram
  *            object that collates only the "display" histogram. For
  *            a more complex object that also keeps the raw data samples,
  *            better suited for fitting distributions and goodness-of-fit
  *            testing, use <esl_histogram_CreateFull()>.
+ *            
+ *            There is currently no way to alter where the equals sign
+ *            is, in setting the bin bounds: that is, you can't make bins
+ *            that have <bmin> $\leq x$ and $x <$ <bmax>, alas.
+
  *  
- * Args:      xmin - caller guesses that minimum score will be > xmin
- *            xmax - caller guesses that max score will be <= xmax
+ * Args:      bmin - caller guesses that minimum score will be > bmin
+ *            bmax - caller guesses that max score will be <= bmax
  *            w    - size of bins (1.0, for example)
  *            
  * Returns:   ptr to new <ESL_HISTOGRAM> object, which caller is responsible
@@ -68,7 +73,7 @@ static int esl_histogram_sort(ESL_HISTOGRAM *h);
  * Throws:    <NULL> on allocation failure.
  */
 ESL_HISTOGRAM *
-esl_histogram_Create(double xmin, double xmax, double w)
+esl_histogram_Create(double bmin, double bmax, double w)
 {
   ESL_HISTOGRAM *h = NULL;
   int status;
@@ -80,9 +85,9 @@ esl_histogram_Create(double xmin, double xmax, double w)
   h->xmax      = -DBL_MAX;
   h->n         = 0;
   h->obs       = NULL;		/* will get allocated below... */
-  h->bmin      = xmin;		/* bmin/bmax are the allocated bounds */
-  h->bmax      = xmax;
-  h->nb        = (int)((xmax-xmin)/w);
+  h->bmin      = bmin;		/* bmin/bmax are the allocated bounds */
+  h->bmax      = bmax;
+  h->nb        = (int)((bmax-bmin)/w);
   h->imin      = h->nb;
   h->imax      = -1;
   h->w         = w;
@@ -128,10 +133,10 @@ esl_histogram_Create(double xmin, double xmax, double w)
  *            tests, but will consume more memory. 
  */
 ESL_HISTOGRAM *
-esl_histogram_CreateFull(double xmin, double xmax, double w)
+esl_histogram_CreateFull(double bmin, double bmax, double w)
 {
   int status;
-  ESL_HISTOGRAM *h = esl_histogram_Create(xmin, xmax, w);
+  ESL_HISTOGRAM *h = esl_histogram_Create(bmin, bmax, w);
   if (h == NULL) return NULL;
 
   h->n      = 0;		/* make sure */
@@ -750,9 +755,9 @@ int
 esl_histogram_SetExpect(ESL_HISTOGRAM *h, 
 			double (*cdf)(double x, void *params), void *params)
 {
-  int    status;
   int    i;
   double ai,bi;			/* ai < x <= bi : lower,upper bounds in bin */
+  int    status;
 
   if (h->expect == NULL) 
     ESL_ALLOC(h->expect, sizeof(double) * h->nb);
@@ -761,6 +766,7 @@ esl_histogram_SetExpect(ESL_HISTOGRAM *h,
     {
       ai = esl_histogram_Bin2LBound(h, i);
       bi = esl_histogram_Bin2UBound(h, i);
+
       h->expect[i] = h->Nc * ( (*cdf)(bi, params) - (*cdf)(ai, params) );
 
       if (h->emin == -1 && h->expect[i] > 0.) h->emin = i;
@@ -877,8 +883,10 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
   int      pos;			  /* position in output line buffer */
   uint64_t lowcount, highcount;	  
   int      ilowbound, ihighbound; 
+  int      nlines;
   int      emptybins = 3;
 
+
   /* Find out how we'll scale the histogram.  We have 58 characters to
    * play with on a standard 80-column terminal display: leading "%6.1f
    * %6d %6d|" occupies 21 chars.  Save the peak position, we'll use
@@ -917,16 +925,22 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 		/* maxbar might need to be raised now; then set our units  */
   if (lowcount  > maxbar) maxbar = lowcount;
   if (highcount > maxbar) maxbar = highcount;
-  units = ((maxbar-1)/ 58) + 1;
 
-  /* Print the histogram
+  if (maxbar > 0) units = ((maxbar-1)/ 58) + 1;
+  else            units = 1;	                 /* watch out for an empty histogram w/ no data points. */
+
+  /* Print the histogram header
    */
   if (fprintf(fp, "%6s %6s %6s  (one = represents %llu sequences)\n", 
 	      "score", "obs", "exp", (unsigned long long) units) < 0) 
     ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
-  if (fprintf(fp, "%6s %6s %6s\n", "-----", "---", "---") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+  if (fprintf(fp, "%6s %6s %6s\n", "-----", "---", "---") < 0)
+    ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+
+  /* Print the histogram itself */
   buffer[80] = '\0';
   buffer[79] = '\n';
+  nlines     = 0;		/* Count the # of lines we print, so we know if it ends up being zero */
   for (i = h->imin; i <= h->imax; i++)
     {
       memset(buffer, ' ', 79 * sizeof(char));
@@ -944,6 +958,7 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 	    for (pos = 21; num > 0; num--)  buffer[pos++] = '=';
 	  }
 	  if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+	  nlines++;
 	  continue;
 	}
       else if (i == ihighbound && i != h->imax)
@@ -954,6 +969,7 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 	    for (pos = 21; num > 0; num--)  buffer[pos++] = '=';
 	  }
 	  if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+	  nlines++;
 	  continue;
 	}
 
@@ -972,11 +988,9 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 	    sprintf(buffer, "%6.1f %6.2e %6.2e|", x, (double) h->obs[i], h->expect[i]);
 	  else
 	    sprintf(buffer, "%6.1f %6.2e %6s|",   x, (double) h->obs[i], "-");
-
 	}
       buffer[21] = ' ';		/* sprintf writes a null char; replace it */
 
-
       /* Mark the histogram bar for observed hits
        */ 
       if (h->obs[i] > 0) {
@@ -997,8 +1011,12 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
       /* Print the line
        */
       if (fputs(buffer, fp) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");
+      nlines++;
     }
 
+  if (nlines == 0 && fprintf(fp, "[histogram contained no data points]\n") < 0)
+    ESL_EXCEPTION_SYS(eslEWRITE, "histogram write failed");    
+
   return eslOK;
 }
   
@@ -1016,29 +1034,33 @@ esl_histogram_Write(FILE *fp, ESL_HISTOGRAM *h)
 int
 esl_histogram_Plot(FILE *fp, ESL_HISTOGRAM *h)
 {
+  int    imin, imax;
   int    i;
   double x;
 
   /* First data set is the observed histogram
    */
   for (i = h->imin; i <= h->imax; i++)
-    if (h->obs[i] > 0)
-      {
-	x = esl_histogram_Bin2LBound(h,i);
-	if (fprintf(fp, "%f %llu\n", x, (unsigned long long) h->obs[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
-      }
+    {
+      x = esl_histogram_Bin2LBound(h,i);
+      if (fprintf(fp, "%f %llu\n", x, (unsigned long long) h->obs[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
+    }
   if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
 
   /* Second data set is the theoretical (expected) histogram
    */
   if (h->expect != NULL)
     {
-      for (i = 0; i < h->nb; i++)
-	if (h->expect[i] > 0.)	/* >0 suffices to remove censored region */
-	  {
-	    x = esl_histogram_Bin2LBound(h,i);
-	    if (fprintf(fp, "%.2f %g\n", x, h->expect[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
-	  }
+      for (imin = 0; imin < h->nb; imin++) 
+	if (h->expect[imin] > 0.) break;
+      for (imax = h->nb-1; imax >= 0; imax--)
+	if (h->expect[imax] > 0.) break;
+
+      for (i = imin; i <= imax; i++)
+	{
+	  x = esl_histogram_Bin2LBound(h,i);
+	  if (fprintf(fp, "%f %g\n", x, h->expect[i]) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
+	}
       if (fprintf(fp, "&\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "histogram plot write failed");
     }
   return eslOK;
@@ -1193,10 +1215,22 @@ esl_histogram_PlotQQ(FILE *fp, ESL_HISTOGRAM *h,
  *            The two tests should give similar
  *            probabilities. However, both tests are sensitive to
  *            arbitrary choices in how the data are binned, and
- *            neither seems to be on an entirely sound theoretical footing.
+ *            neither seems to be on an entirely sound theoretical
+ *            footing.
+ *
+ *            On some datasets, pathological and/or very small, it may
+ *            be impossible to calculate goodness of fit
+ *            statistics. In this case, <eslENORESULT> is returned.
  *            
  * Returns:   <eslOK> on success.
  *
+ *            <eslENORESULT> if the data are such that goodness-of-fit
+ *            statistics can't be calculated, probably because there
+ *            just aren't many data points. On this error, <*ret_G>
+ *            and <*ret_X2> are 0.0, and <*ret_Gp> and <*ret_X2p> are
+ *            1.0. (Because suppose n=1: then any fit to a single data
+ *            point is "perfect".)
+ *
  * Throws:    <eslEINVAL> if expected counts have not been set in
  *            the histogram; <eslERANGE> or <eslENOHALT> on different internal
  *            errors that can arise in calculating the probabilities;
@@ -1241,6 +1275,7 @@ esl_histogram_Goodness(ESL_HISTOGRAM *h,
       nobs += h->obs[i];
       if (h->obs[i] > hmax) hmax = h->obs[i];
     }
+  if (nobs == 0) { status = eslENORESULT; goto ERROR; }
 
   /* Figure out how many eval bins we'd like to have, then allocate
    * for re-binning.
@@ -1257,8 +1292,8 @@ esl_histogram_Goodness(ESL_HISTOGRAM *h,
    * The most important thing seems to be to get the # of counts
    * in each bin to be roughly equal.
    */
-  nb   = 2* (int) pow((double) nobs, 0.4); /* "desired" nb. */
-  minc = 1 + nobs / (2*nb);	/* arbitrarily set min = 1/2 of the target # */
+  nb   = 2* (int) pow((double) nobs, 0.4);      /* "desired" nb. */
+  minc = 1 + nobs / (2*nb);	                /* arbitrarily set min = 1/2 of the target # */
   ESL_ALLOC(obs,  sizeof(uint64_t) * (nb*2+1)); /* final nb must be <= 2*nb+1 */
   ESL_ALLOC(exp,  sizeof(double)   * (nb*2+1));
   ESL_ALLOC(topx, sizeof(double)   * (nb*2+1));
@@ -1287,11 +1322,18 @@ esl_histogram_Goodness(ESL_HISTOGRAM *h,
 	i++;
       }
     }
+  if (i == 0) { status = eslENORESULT; goto ERROR; }
   obs[i-1]  += nobs;		/* add the right tail to final bin */
   exp[i-1]  += nexp;
   topx[i-1]  = esl_histogram_Bin2UBound(h, h->imax);
   nb         = i;		/* nb is now actual # of bins, not target */
 
+  /* We have to have at least one degree of freedom, else
+   * goodness-of-fit testing isn't defined (and moreover, will
+   * fail numerically if we proceed)
+   */
+  if (nb-nfitted-1 <= 0) { status = eslENORESULT; goto ERROR; }
+
   /* Calculate the X^2 statistic: \sum (obs_i - exp_i)^2 / exp_i */
   X2 = 0.;
   for (i = 0; i < nb; i++)
@@ -1300,15 +1342,17 @@ esl_histogram_Goodness(ESL_HISTOGRAM *h,
       X2 += tmp*tmp / exp[i];
     }
   /* X^2 is distributed approximately chi^2. */
-  if (nb-nfitted >= 0 && X2 != eslINFINITY)
-    {
-      status = esl_stats_ChiSquaredTest(nb-nfitted, X2, &X2p);
-      if (status != eslOK) return status;
-    }
-  else X2p = 0.;
+  if (X2 == 0.) 
+    X2p = 1.0;
+  else if (X2 != eslINFINITY) {
+    if ((status = esl_stats_ChiSquaredTest(nb-nfitted, X2, &X2p)) != eslOK) goto ERROR;
+  }
+  else 
+    X2p = 0.;
 
   /* The G test assumes that #exp=#obs (the X^2 test didn't).
    * If that's not true, renormalize to make it so. 
+   * This normalization subtracts a degree of freedom.
    */
   nobs = 0;
   nexp = 0.;
@@ -1327,35 +1371,35 @@ esl_histogram_Goodness(ESL_HISTOGRAM *h,
   G *= 2;
   
   /* G is distributed approximately as \chi^2.
-   * -1 is because total #obs=#exp (which is must be)
+   * -1 is because total #obs=#exp 
    */
-  ESL_DASSERT1( (G >= 0.));
-  if (nb-nfitted-1 >= 0 && G != eslINFINITY)
+  if (G == 0.)
+    Gp = 1.0;
+  else if (G != eslINFINITY)
     {
-      status = esl_stats_ChiSquaredTest(nb-nfitted-1, G, &Gp);
-      if (status != eslOK) return status;
+      if ((status = esl_stats_ChiSquaredTest(nb-nfitted-1, G, &Gp)) != eslOK) goto ERROR;
     }
   else Gp = 0.;
 
-  if (ret_nbins != NULL) *ret_nbins = nb;
-  if (ret_G     != NULL) *ret_G     = G;
-  if (ret_Gp    != NULL) *ret_Gp    = Gp;
-  if (ret_X2    != NULL) *ret_X2    = X2;
-  if (ret_X2p   != NULL) *ret_X2p   = X2p;
+  if (ret_nbins) *ret_nbins = nb;
+  if (ret_G)     *ret_G     = G;
+  if (ret_Gp)    *ret_Gp    = Gp;
+  if (ret_X2)    *ret_X2    = X2;
+  if (ret_X2p)   *ret_X2p   = X2p;
   free(obs);
   free(exp);
   free(topx);
   return eslOK;
 
  ERROR:
-  if (ret_nbins != NULL) *ret_nbins = 0;
-  if (ret_G     != NULL) *ret_G     = 0.;
-  if (ret_Gp    != NULL) *ret_Gp    = 0.;
-  if (ret_X2    != NULL) *ret_X2    = 0.;
-  if (ret_X2p   != NULL) *ret_X2p   = 0.;
-  if (obs  != NULL) free(obs);
-  if (exp  != NULL) free(exp);
-  if (topx != NULL) free(topx);
+  if (ret_nbins) *ret_nbins = 0;
+  if (ret_G)     *ret_G     = 0.;
+  if (ret_Gp)    *ret_Gp    = 1.;
+  if (ret_X2)    *ret_X2    = 0.;
+  if (ret_X2p)   *ret_X2p   = 1.;
+  if (obs)       free(obs);
+  if (exp)       free(exp);
+  if (topx)      free(topx);
   return status;
 }
 
@@ -1549,6 +1593,8 @@ main(int argc, char **argv)
       bstrategy = FIT_BINNED;
       fstrategy = FIT_TAIL;
     }
+  else
+    esl_fatal("no such test type");
 
 
   for (trial = 0; trial < ntrials; trial++)
@@ -1583,28 +1629,28 @@ main(int argc, char **argv)
 	  fstrategy == FIT_COMPLETE)
 	{
 	  esl_histogram_GetData(h, &xv, &n);
-	  esl_gumbel_FitComplete(xv, n, &(ep[0]), &ep[1]);
+	  if (esl_gumbel_FitComplete(xv, n, &(ep[0]), &ep[1]) != eslOK) esl_fatal("gumbel complete fit failed");
 	}
       else if (cstrategy == COLLECT_COMPLETE &&
 	       bstrategy == FIT_SAMPLES &&
 	       fstrategy == FIT_CENSORED)
 	{
 	  esl_histogram_GetTailByMass(h, cmass, &xv, &n, &z);
-	  esl_gumbel_FitCensored(xv, n, z, xv[0], &(ep[0]), &ep[1]);
+	  if (esl_gumbel_FitCensored(xv, n, z, xv[0], &(ep[0]), &ep[1]) != eslOK) esl_fatal("gumbel censored fit failed");
 	}
       else if (cstrategy == COLLECT_COMPLETE &&
 	       bstrategy == FIT_SAMPLES &&
 	       fstrategy == FIT_TAIL)
 	{
 	  esl_histogram_GetTailByMass(h, tailmass, &xv, &n, &z);
-	  esl_exp_FitComplete(xv, n, &(ep[0]), &ep[1]);
+	  if (esl_exp_FitComplete(xv, n, &(ep[0]), &ep[1]) != eslOK) esl_fatal("exponential complete fit failed");
 	}
       else if (cstrategy == COLLECT_CENSORED &&
 	       bstrategy == FIT_SAMPLES &&
 	       fstrategy == FIT_CENSORED)
 	{
 	  esl_histogram_GetData(h, &xv, &n);
-	  esl_gumbel_FitCensored(xv, n, h->z, h->phi, &(ep[0]), &ep[1]);
+	  if (esl_gumbel_FitCensored(xv, n, h->z, h->phi, &(ep[0]), &ep[1]) != eslOK) esl_fatal("gumbel censored fit failed");
 	}
       else if (cstrategy == COLLECT_COMPLETE &&
 	       bstrategy == FIT_BINNED &&
@@ -1612,7 +1658,7 @@ main(int argc, char **argv)
 	{
 	  tailmass = save_tailmass; /* reset to original for each trial. */
 	  esl_histogram_SetTailByMass(h, tailmass, &tailmass);
-	  esl_exp_FitCompleteBinned(h, &(ep[0]), &ep[1]);
+	  if (esl_exp_FitCompleteBinned(h, &(ep[0]), &ep[1]) != eslOK) esl_fatal("exponential binned complete fit failed");
 	}
       else
 	ESL_EXCEPTION(eslEINVAL, "not a scenario we currently test");
@@ -1655,7 +1701,8 @@ main(int argc, char **argv)
       /* Evaluate goodness-of-fit
        */
       nfitted =  (ntest == 0)? 2 : 0;
-      esl_histogram_Goodness(h1, nfitted, &nbins, &G, &Gp, &X2, &X2p);
+      if (esl_histogram_Goodness(h1, nfitted, &nbins, &G, &Gp, &X2, &X2p) != eslOK)
+	esl_fatal("esl_histogram unit testing: goodness-of-fit failed");
 
       /* Track minimum goodness of fit probs, for automated testing
        */
@@ -1770,14 +1817,17 @@ main(int argc, char **argv)
   }
 
   esl_histogram_GetData(h, &xv, &n);
-  esl_gumbel_FitComplete(xv, n, &mu, &lambda);
+  if (esl_gumbel_FitComplete(xv, n, &mu, &lambda) != eslOK)
+    esl_fatal("gumbel complete data fit failed");
 
   params[0] = mu;
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
   esl_histogram_Write(stdout, h);
-  esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
+  if (esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p) != eslOK)
+    esl_fatal("goodness of fit testing failed");
+
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
 
@@ -1822,14 +1872,17 @@ main(int argc, char **argv)
   }
 
   esl_histogram_GetTailByMass(h, 0.5, &xv, &n, &z); /* fit to right 50% */
-  esl_gumbel_FitCensored(xv, n, z, xv[0], &mu, &lambda);
+  if (esl_gumbel_FitCensored(xv, n, z, xv[0], &mu, &lambda) != eslOK)
+    esl_fatal("gumbel censored fit failed");
 
   params[0] = mu;
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
   esl_histogram_Write(stdout, h);
-  esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
+  if (esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p) != eslOK)
+    esl_fatal("goodness of fit testing failed");
+
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
 
@@ -1881,7 +1934,9 @@ main(int argc, char **argv)
   esl_histogram_SetExpectedTail(h, mu, 0.1, &esl_exp_generic_cdf, &params);
 
   esl_histogram_Write(stdout, h);
-  esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
+  if (esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p) != eslOK)
+    esl_fatal("goodness of fit testing failed");
+
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
 
@@ -1928,14 +1983,17 @@ main(int argc, char **argv)
   }
 
   esl_histogram_GetData(h, &xv, &n);
-  esl_gumbel_FitCensored(xv, n, z, phi, &mu, &lambda);
+  if (esl_gumbel_FitCensored(xv, n, z, phi, &mu, &lambda) != eslOK)
+    esl_fatal("gumbel censored fit failed");
 
   params[0] = mu;
   params[1] = lambda;
   esl_histogram_SetExpect(h, &esl_gumbel_generic_cdf, &params);
 
   esl_histogram_Write(stdout, h);
-  esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
+  if (esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p) != eslOK)
+    esl_fatal("goodness of fit testing failed");
+
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
 
@@ -1981,14 +2039,17 @@ main(int argc, char **argv)
 
   esl_histogram_SetTailByMass(h, 0.1, &actual_mass);
   esl_histogram_DeclareRounding(h);
-  esl_exp_FitCompleteBinned(h, &mu, &lambda);
+  if (esl_exp_FitCompleteBinned(h, &mu, &lambda) != eslOK)
+    esl_fatal("exponential ML fitting failed");
 
   params[0] = mu;
   params[1] = lambda;
   esl_histogram_SetExpectedTail(h, mu, actual_mass, &esl_exp_generic_cdf, &params);
 
   esl_histogram_Write(stdout, h);
-  esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p);
+  if (esl_histogram_Goodness(h, 0, NULL, &G, &Gp, &X2, &X2p) != eslOK)
+    esl_fatal("goodness of fit testing failed");
+
   printf("G   = %f  p = %f\n", G, Gp);
   printf("X^2 = %f  p = %f\n", X2, X2p);
 
@@ -2001,15 +2062,15 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_histogram.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_histogram.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 
