@@ -31,20 +31,20 @@ esl_hmm_Create(const ESL_ALPHABET *abc, int M)
   int      status;
 
   ESL_ALLOC(hmm, sizeof(ESL_HMM));
-  hmm->t = NULL;
-  hmm->e = NULL;
+  hmm->t  = NULL;
+  hmm->e  = NULL;
+  hmm->eo = NULL;
+  hmm->pi = NULL;
 
-  ESL_ALLOC(hmm->t,  sizeof(float *) * M);
-  ESL_ALLOC(hmm->e,  sizeof(float *) * M);
-  ESL_ALLOC(hmm->eo, sizeof(float *) * abc->Kp);
-  hmm->t[0]  = NULL;
-  hmm->e[0]  = NULL;
-  hmm->eo[0] = NULL;
+  ESL_ALLOC(hmm->t,  sizeof(float *) * M);           hmm->t[0]  = NULL;
+  ESL_ALLOC(hmm->e,  sizeof(float *) * M);           hmm->e[0]  = NULL;
+  ESL_ALLOC(hmm->eo, sizeof(float *) * abc->Kp);     hmm->eo[0] = NULL;
+  ESL_ALLOC(hmm->pi, sizeof(float) * (M+1));         // initial transition to state M means a L=0 sequence 
 
-  ESL_ALLOC(hmm->t[0],  sizeof(float) * M * (M+1));  /* state M is the implicit end state */
+  ESL_ALLOC(hmm->t[0],  sizeof(float) * M * (M+1));  // state M is the implicit end state 
   ESL_ALLOC(hmm->e[0],  sizeof(float) * M * abc->K);
   ESL_ALLOC(hmm->eo[0], sizeof(float) * abc->Kp * M);
-  ESL_ALLOC(hmm->pi,    sizeof(float) * (M+1));      /* initial transition to state M means a L=0 sequence */
+
   
   for (k = 1; k < M; k++)
     {
@@ -784,10 +784,11 @@ main(int argc, char **argv)
 
       esl_sq_Reuse(sq);
     }
-  if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s line %" PRId64 "):\n%s\n",
-					    sqfp->filename, sqfp->linenumber, sqfp->errbuf);     
+  if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s)\n%s\n",
+					   sqfp->filename, sqfp->get_error(sqfp));     
   else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
-					    status, sqfp->filename);
+					   status, sqfp->filename);
+ 
   
   esl_sqfile_Close(sqfp);
   esl_sq_Destroy(sq);
@@ -805,13 +806,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_hmm.c 694 2011-06-14 21:57:16Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_hmm.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
