@@ -1,4 +1,26 @@
 /* Statistical routines for hyperexponential distributions.
+ * 
+ * Contents:
+ *   1. The ESL_HYPEREXP object
+ *   2. Evaluating densities and distributions
+ *   3. Generic API routines: for general interface w/ histogram module
+ *   4. Dumping plots for files
+ *   5. Sampling                    (augmentation: random)
+ *   6. File input                  (augmentation: fileparser)
+ *   7. ML fitting to complete data (augmentation: minimizer)
+ *   8. ML fitting to binned data   (augmentation: histogram, minimizer)
+ *   9. Test driver
+ *  10. Example
+ *  11. Copyright and license information
+ *   
+ * Xrefs:
+ *   STL9/140     :  original implementation
+ *   STL9/143-144 :  ML fitting to binned data  
+ *   
+ * To-do:
+ *   - Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *     on failure due to small n. Compare esl_gumbel. xref J12/93.  
+ *     SRE, Wed Nov 27 11:17:59 2013
  */
 #include "esl_config.h"
 
@@ -26,7 +48,7 @@
 #endif
 
 /****************************************************************************
- *# Routines for the ESL_HYPEREXP object
+ *# 1. The ESL_HYPEREXP object
  ****************************************************************************/ 
 
 /* Function:  esl_hyperexp_Create()
@@ -230,7 +252,7 @@ esl_hyperexp_Dump(FILE *fp, ESL_HYPEREXP *hxp)
 
 
 /****************************************************************************
- * Routines for evaluating densities and distributions
+ * 2. Evaluating densities and distributions
  ****************************************************************************/ 
 /* all lambda_k > 0
  * all q_k are probabilities, \sum_k q_k = 1 [watch out for q_k=0 in log(q_k)].
@@ -404,7 +426,7 @@ esl_hxp_invcdf(double p, ESL_HYPEREXP *h)
 
 
 /****************************************************************************
- * Generic API routines: for general interface w/ histogram module
+ * 3. Generic API routines: for general interface w/ histogram module
  ****************************************************************************/ 
 
 /* Function:  esl_hxp_generic_pdf()
@@ -458,7 +480,7 @@ esl_hxp_generic_invcdf(double p, void *params)
 
 
 /****************************************************************************
- * Routines for dumping plots for files
+ * 4. Dumping plots for files
  ****************************************************************************/ 
 
 /* Function:  esl_hxp_Plot()
@@ -490,7 +512,7 @@ esl_hxp_Plot(FILE *fp, ESL_HYPEREXP *h,
 
 
 /****************************************************************************
- * Routines for sampling (requires augmentation w/ random module)
+ * 5. Sampling (requires augmentation w/ random module)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 /* Function:  esl_hxp_Sample()
@@ -511,7 +533,7 @@ esl_hxp_Sample(ESL_RANDOMNESS *r, ESL_HYPEREXP *h)
 
 
 /****************************************************************************
- * File input (mixture models are a little too complex to set on commandline)
+ * 6. File input (mixture models are a little too complex to set on commandline)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_FILEPARSER
 /* Function:  esl_hyperexp_Read()
@@ -649,7 +671,7 @@ esl_hyperexp_ReadFile(char *filename, ESL_HYPEREXP **ret_hxp)
 
 
 /****************************************************************************
- * Maximum likelihood fitting, complete unbinned data
+ * 7. ML fitting to complete data
  ****************************************************************************/ 
 #ifdef eslAUGMENT_MINIMIZER
 /* This structure is used to sneak the data into minimizer's generic
@@ -909,7 +931,7 @@ esl_hxp_FitComplete(double *x, int n, ESL_HYPEREXP *h)
 
 
 /****************************************************************************
- * Maximum likelihood fitting, complete binned data         xref STL9/143-144
+ * 8. Maximum likelihood fitting, complete binned data         xref STL9/143-144
  ****************************************************************************/ 
 #ifdef eslAUGMENT_HISTOGRAM
 /* minimizer API only allows us one generic void ptr to pass
@@ -1137,6 +1159,185 @@ esl_hxp_FitCompleteBinned(ESL_HISTOGRAM *g, ESL_HYPEREXP *h)
 
 
 
+
+
+
+/****************************************************************************
+ * 9. Test driver
+ ****************************************************************************/ 
+#ifdef eslHYPEREXP_TESTDRIVE
+/* Compile:
+   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test\
+    -DeslHYPEREXP_TESTDRIVE esl_hyperexp.c -leasel -lm
+*/
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "easel.h"
+#include "esl_random.h"
+#include "esl_histogram.h"
+#include "esl_hyperexp.h"
+
+int
+main(int argc, char **argv)
+{
+  ESL_HISTOGRAM  *h;
+  ESL_RANDOMNESS *r;
+  ESL_HYPEREXP   *hxp;
+  ESL_HYPEREXP   *ehxp;
+  int     n         = 20000;
+  double  binwidth  = 0.1;
+  int     i;
+  double  x;
+  double *data;
+  int     ndata;
+  int     k, ek, mink;
+  double  mindiff, diff;
+
+  int     opti;
+  int     be_verbose   = FALSE;
+  char   *paramfile    = NULL;
+  char   *plotfile     = NULL;
+  FILE   *pfp          = stdout;
+  int     plot_pdf     = FALSE;
+  int     plot_logpdf  = FALSE;
+  int     plot_cdf     = FALSE;
+  int     plot_logcdf  = FALSE;
+  int     plot_surv    = FALSE;
+  int     plot_logsurv = FALSE;
+  int     xmin_set     = FALSE;
+  double  xmin;
+  int     xmax_set     = FALSE;
+  double  xmax;
+  int     xstep_set    = FALSE;
+  double  xstep;
+  int     do_fixmix    = FALSE;
+  int     status;
+
+  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
+    {
+      if      (strcmp(argv[opti], "-f")  == 0) do_fixmix    = TRUE;
+      else if (strcmp(argv[opti], "-i")  == 0) paramfile    = argv[++opti];
+      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
+      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
+      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
+      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
+      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
+      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
+      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
+      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
+      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
+      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
+      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
+      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
+      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
+      else esl_fatal("bad option");
+    }
+
+  if (paramfile != NULL)
+    {
+      status = esl_hyperexp_ReadFile(paramfile, &hxp);
+      if      (status == eslENOTFOUND) esl_fatal("Param file %s not found", paramfile);
+      else if (status == eslEFORMAT)   esl_fatal("Parse failed: param file %s invalid format", paramfile);
+      else if (status != eslOK)        esl_fatal("Unusual failure opening param file %s", paramfile);
+    }
+  else 
+    {
+      hxp = esl_hyperexp_Create(3);
+      hxp->mu = -2.0;
+      hxp->q[0]      = 0.5;    hxp->q[1]      = 0.3;   hxp->q[2]      = 0.2; 
+      hxp->lambda[0] = 1.0;    hxp->lambda[1] = 0.3;   hxp->lambda[2] = 0.1;
+    }
+  if (do_fixmix) esl_hyperexp_FixedUniformMixture(hxp);	/* overrides q's above */
+
+  if (be_verbose) esl_hyperexp_Dump(stdout, hxp);
+
+  r = esl_randomness_Create(42);
+  h = esl_histogram_CreateFull(hxp->mu, 100., binwidth);
+  if (plotfile != NULL) {
+    if ((pfp = fopen(plotfile, "w")) == NULL) 
+      esl_fatal("Failed to open plotfile");
+  }
+  if (! xmin_set)  xmin  = hxp->mu;
+  if (! xmax_set)  xmax  = hxp->mu+ 20*(1. / esl_vec_DMin(hxp->lambda, hxp->K));
+  if (! xstep_set) xstep = 0.1;
+
+  for (i = 0; i < n; i++)
+    {
+      x = esl_hxp_Sample(r, hxp);
+      esl_histogram_Add(h, x);
+    }
+
+  esl_histogram_GetData(h, &data, &ndata); /* get sorted data vector */
+
+  ehxp = esl_hyperexp_Create(hxp->K);
+  if (do_fixmix) esl_hyperexp_FixedUniformMixture(ehxp);
+  esl_hxp_FitGuess(data, ndata, ehxp);  
+  if ( esl_hxp_FitComplete(data, ndata, ehxp) != eslOK) esl_fatal("Failed to fit hyperexponential");
+
+  if (be_verbose) esl_hyperexp_Dump(stdout, ehxp);
+
+  if (fabs( (ehxp->mu-hxp->mu)/hxp->mu ) > 0.01)
+    esl_fatal("Error in (complete) fitted mu > 1%\n");
+  for (ek = 0; ek < ehxp->K; ek++)
+    {  /* try to match each estimated lambda up to a parametric lambda */
+      mindiff = 1.0;
+      mink    = -1;
+      for (k = 0; k < hxp->K; k++)
+	{
+	  diff =  fabs( (ehxp->lambda[ek] - hxp->lambda[k]) / hxp->lambda[k]);
+	  if (diff < mindiff) {
+	    mindiff = diff;
+	    mink    = k;
+	  }
+	}
+      if (mindiff > 0.50)
+	esl_fatal("Error in (complete) fitted lambda > 50%\n");
+      if (fabs( (ehxp->q[ek] - hxp->q[mink]) / hxp->q[mink]) > 1.0)
+	esl_fatal("Error in (complete) fitted q > 2-fold%\n");
+    }
+
+  esl_hxp_FitGuessBinned(h, ehxp);  
+  if ( esl_hxp_FitCompleteBinned(h, ehxp) != eslOK) esl_fatal("Failed to fit binned hyperexponential");
+  if (be_verbose)  esl_hyperexp_Dump(stdout, ehxp);
+
+  if (fabs( (ehxp->mu-hxp->mu)/hxp->mu ) > 0.01)
+    esl_fatal("Error in (binned) fitted mu > 1%\n");
+  for (ek = 0; ek < ehxp->K; ek++)
+    {  /* try to match each estimated lambda up to a parametric lambda */
+      mindiff = 1.0;
+      mink    = -1;
+      for (k = 0; k < hxp->K; k++)
+	{
+	  diff =  fabs( (ehxp->lambda[ek] - hxp->lambda[k]) / hxp->lambda[k]);
+	  if (diff < mindiff) {
+	    mindiff = diff;
+	    mink    = k;
+	  }
+	}
+      if (mindiff > 0.50)
+	esl_fatal("Error in (binned) fitted lambda > 50%\n");
+      if (fabs( (ehxp->q[ek] - hxp->q[mink]) / hxp->q[mink]) > 1.0)
+	esl_fatal("Error in (binned) fitted q > 2-fold\n");
+    }
+
+  if (plot_pdf)     esl_hxp_Plot(pfp, hxp, &esl_hxp_pdf,     xmin, xmax, xstep);
+  if (plot_logpdf)  esl_hxp_Plot(pfp, hxp, &esl_hxp_logpdf,  xmin, xmax, xstep);
+  if (plot_cdf)     esl_hxp_Plot(pfp, hxp, &esl_hxp_cdf,     xmin, xmax, xstep);
+  if (plot_logcdf)  esl_hxp_Plot(pfp, hxp, &esl_hxp_logcdf,  xmin, xmax, xstep);
+  if (plot_surv)    esl_hxp_Plot(pfp, hxp, &esl_hxp_surv,    xmin, xmax, xstep);
+  if (plot_logsurv) esl_hxp_Plot(pfp, hxp, &esl_hxp_logsurv, xmin, xmax, xstep);
+
+  if (plotfile != NULL) fclose(pfp);
+  esl_histogram_Destroy(h);
+  esl_hyperexp_Destroy(hxp);
+  esl_hyperexp_Destroy(ehxp);
+  esl_randomness_Destroy(r);
+  return 0;
+}
+#endif /*eslHYPEREXP_TESTDRIVE*/
+
 /****************************************************************************
  * Example main()
  ****************************************************************************/ 
@@ -1207,190 +1408,17 @@ main(int argc, char **argv)
 /*::cexcerpt::hyperexp_example::end::*/
 #endif /*eslHYPEREXP_EXAMPLE*/
 
-
-
-
-/****************************************************************************
- * Test driver
- ****************************************************************************/ 
-#ifdef eslHYPEREXP_TESTDRIVE
-/* Compile:
-   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test\
-    -DeslHYPEREXP_TESTDRIVE esl_hyperexp.c -leasel -lm
-*/
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "easel.h"
-#include "esl_random.h"
-#include "esl_histogram.h"
-#include "esl_hyperexp.h"
-
-int
-main(int argc, char **argv)
-{
-  ESL_HISTOGRAM  *h;
-  ESL_RANDOMNESS *r;
-  ESL_HYPEREXP   *hxp;
-  ESL_HYPEREXP   *ehxp;
-  int     n         = 20000;
-  double  binwidth  = 0.1;
-  int     i;
-  double  x;
-  double *data;
-  int     ndata;
-  int     k, ek, mink;
-  double  mindiff, diff;
-
-  int     opti;
-  int     be_verbose   = FALSE;
-  char   *paramfile    = NULL;
-  char   *plotfile     = NULL;
-  FILE   *pfp          = stdout;
-  int     plot_pdf     = FALSE;
-  int     plot_logpdf  = FALSE;
-  int     plot_cdf     = FALSE;
-  int     plot_logcdf  = FALSE;
-  int     plot_surv    = FALSE;
-  int     plot_logsurv = FALSE;
-  int     xmin_set     = FALSE;
-  double  xmin;
-  int     xmax_set     = FALSE;
-  double  xmax;
-  int     xstep_set    = FALSE;
-  double  xstep;
-  int     do_fixmix    = FALSE;
-
-  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
-    {
-      if      (strcmp(argv[opti], "-f")  == 0) do_fixmix    = TRUE;
-      else if (strcmp(argv[opti], "-i")  == 0) paramfile    = argv[++opti];
-      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
-      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
-      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
-      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
-      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
-      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
-      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
-      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
-      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
-      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
-      else ESL_EXCEPTION(eslEINVAL, "bad option");
-    }
-
-  if (paramfile != NULL)
-    esl_hyperexp_ReadFile(paramfile, &hxp);
-  else 
-    {
-      hxp = esl_hyperexp_Create(3);
-      hxp->mu = -2.0;
-      hxp->q[0]      = 0.5;    hxp->q[1]      = 0.3;   hxp->q[2]      = 0.2; 
-      hxp->lambda[0] = 1.0;    hxp->lambda[1] = 0.3;   hxp->lambda[2] = 0.1;
-    }
-  if (do_fixmix) esl_hyperexp_FixedUniformMixture(hxp);	/* overrides q's above */
-
-  if (be_verbose) esl_hyperexp_Dump(stdout, hxp);
-
-  r = esl_randomness_Create(42);
-  h = esl_histogram_CreateFull(hxp->mu, 100., binwidth);
-  if (plotfile != NULL) {
-    if ((pfp = fopen(plotfile, "w")) == NULL) 
-      ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
-  }
-  if (! xmin_set)  xmin  = hxp->mu;
-  if (! xmax_set)  xmax  = hxp->mu+ 20*(1. / esl_vec_DMin(hxp->lambda, hxp->K));
-  if (! xstep_set) xstep = 0.1;
-
-  for (i = 0; i < n; i++)
-    {
-      x = esl_hxp_Sample(r, hxp);
-      esl_histogram_Add(h, x);
-    }
-
-  esl_histogram_GetData(h, &data, &ndata); /* get sorted data vector */
-
-  ehxp = esl_hyperexp_Create(hxp->K);
-  if (do_fixmix) esl_hyperexp_FixedUniformMixture(ehxp);
-  esl_hxp_FitGuess(data, ndata, ehxp);  
-  esl_hxp_FitComplete(data, ndata, ehxp);
-
-  if (be_verbose) esl_hyperexp_Dump(stdout, ehxp);
-
-  if (fabs( (ehxp->mu-hxp->mu)/hxp->mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted mu > 1%\n");
-  for (ek = 0; ek < ehxp->K; ek++)
-    {  /* try to match each estimated lambda up to a parametric lambda */
-      mindiff = 1.0;
-      mink    = -1;
-      for (k = 0; k < hxp->K; k++)
-	{
-	  diff =  fabs( (ehxp->lambda[ek] - hxp->lambda[k]) / hxp->lambda[k]);
-	  if (diff < mindiff) {
-	    mindiff = diff;
-	    mink    = k;
-	  }
-	}
-      if (mindiff > 0.50)
-	ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 50%\n");
-      if (fabs( (ehxp->q[ek] - hxp->q[mink]) / hxp->q[mink]) > 1.0)
-	ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted q > 2-fold%\n");
-    }
-
-  esl_hxp_FitGuessBinned(h, ehxp);  
-  esl_hxp_FitCompleteBinned(h, ehxp);
-  if (be_verbose)  esl_hyperexp_Dump(stdout, ehxp);
-
-  if (fabs( (ehxp->mu-hxp->mu)/hxp->mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted mu > 1%\n");
-  for (ek = 0; ek < ehxp->K; ek++)
-    {  /* try to match each estimated lambda up to a parametric lambda */
-      mindiff = 1.0;
-      mink    = -1;
-      for (k = 0; k < hxp->K; k++)
-	{
-	  diff =  fabs( (ehxp->lambda[ek] - hxp->lambda[k]) / hxp->lambda[k]);
-	  if (diff < mindiff) {
-	    mindiff = diff;
-	    mink    = k;
-	  }
-	}
-      if (mindiff > 0.50)
-	ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 50%\n");
-      if (fabs( (ehxp->q[ek] - hxp->q[mink]) / hxp->q[mink]) > 1.0)
-	ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted q > 2-fold\n");
-    }
-
-  if (plot_pdf)     esl_hxp_Plot(pfp, hxp, &esl_hxp_pdf,     xmin, xmax, xstep);
-  if (plot_logpdf)  esl_hxp_Plot(pfp, hxp, &esl_hxp_logpdf,  xmin, xmax, xstep);
-  if (plot_cdf)     esl_hxp_Plot(pfp, hxp, &esl_hxp_cdf,     xmin, xmax, xstep);
-  if (plot_logcdf)  esl_hxp_Plot(pfp, hxp, &esl_hxp_logcdf,  xmin, xmax, xstep);
-  if (plot_surv)    esl_hxp_Plot(pfp, hxp, &esl_hxp_surv,    xmin, xmax, xstep);
-  if (plot_logsurv) esl_hxp_Plot(pfp, hxp, &esl_hxp_logsurv, xmin, xmax, xstep);
-
-  if (plotfile != NULL) fclose(pfp);
-  esl_histogram_Destroy(h);
-  esl_hyperexp_Destroy(hxp);
-  esl_hyperexp_Destroy(ehxp);
-  esl_randomness_Destroy(r);
-  return 0;
-}
-#endif /*eslHYPEREXP_TESTDRIVE*/
-
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_hyperexp.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_hyperexp.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
-/* xref STL9/140 */
+
