@@ -405,7 +405,7 @@ esl_memstrdup(const char *p, esl_pos_t n, char **ret_s)
  *
  * Purpose:   Given memory line <p> of length <n>, copy
  *            it to <dest> and NUL-terminate it. Caller must
- *            be sure that <s> is already allocated for
+ *            be sure that <dest> is already allocated for
  *            at least <n+1> bytes.
  *              
  * Returns:   <eslOK> on success.
@@ -451,7 +451,7 @@ esl_memtod(const char *p, esl_pos_t n, double *ret_val)
       ESL_ALLOC(bigbuf, sizeof(char) * (n+1));
       memcpy(bigbuf, p, sizeof(char) * n);
       bigbuf[n] = '\0';
-      *ret_val = strtod(fixedbuf, NULL);
+      *ret_val = strtod(bigbuf, NULL);
       free(bigbuf);
       return eslOK;
     }
@@ -492,7 +492,7 @@ esl_memtof(const char *p, esl_pos_t n, float *ret_val)
       ESL_ALLOC(bigbuf, sizeof(char) * (n+1));
       memcpy(bigbuf, p, sizeof(char) * n);
       bigbuf[n] = '\0';
-      *ret_val = (float) strtod(fixedbuf, NULL);
+      *ret_val = (float) strtod(bigbuf, NULL);
       free(bigbuf);
       return eslOK;
     }
@@ -817,15 +817,15 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_mem.c 766 2012-06-04 13:11:51Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_mem.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 
