@@ -292,7 +292,7 @@ brent(double *ori, double *dir, int n,
   esl_vec_DAddScaled(xvec, dir, x, n);
   fx=fv=fw = (*func)(xvec, n, prm);   /* initial function evaluation */
 
-  e     = 0.;
+  d = e = 0.;
   niter = 0;
   while (1) /* algorithm is guaranteed to converge. */
     {
@@ -434,12 +434,10 @@ esl_min_ConjugateGradientDescent(double *x, double *u, int n,
 
   oldfx = (*func)(x, n, prm);	/* init the objective function */
   
-  /* Bail out if the function is +/-inf: this can happen if the caller
+  /* Bail out if the function is +/-inf or nan: this can happen if the caller
    * has screwed something up, or has chosen a bad start point.
    */
-  if (oldfx == eslINFINITY || oldfx == -eslINFINITY)
-	  ESL_EXCEPTION(eslERANGE, "minimum not finite");
-
+  if (! isfinite(oldfx)) ESL_EXCEPTION(eslERANGE, "minimum not finite");
 
   if (dfunc != NULL) 
     {
@@ -650,13 +648,13 @@ main(int argc, char **argv)
 
 /*****************************************************************  
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_minimizer.c 685 2011-05-23 14:27:52Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_minimizer.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
