@@ -288,23 +288,34 @@ esl_sq_MPIPackSize(ESL_SQ *sq, MPI_Comm comm, int *ret_n)
 {
   int   status;
   int   n = 0;
+  int   x;       /* index for optional extra residue markups */
   int   sz;
 
-  status = MPI_Pack_size        (                     1, MPI_INT,           comm, &sz); n += 4*sz; if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-  status = MPI_Pack_size        (                     1, MPI_UNSIGNED_LONG, comm, &sz); n += 7*sz; if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-  status = MPI_Pack_size        (            sq->nalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-  status = MPI_Pack_size        (            sq->aalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-  status = MPI_Pack_size        (            sq->dalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-  status = MPI_Pack_size        (          sq->srcalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (                          1, MPI_INT,           comm, &sz); n += 5*sz; if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (                          1, MPI_UNSIGNED_LONG, comm, &sz); n += 7*sz; if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (                 sq->nalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (                 sq->aalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (                 sq->dalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+  status = MPI_Pack_size        (               sq->srcalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
 
-  /* sequence, digital or text; the ss is optional */
+  /* sequence, digital or text; the ss and extra residue markups are optional */
   if (sq->dsq != NULL) {
-    status = MPI_Pack_size      (               sq->n+2, MPI_UNSIGNED_CHAR, comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-    status = esl_mpi_PackOptSize(sq->ss,        sq->n+2, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+    status = MPI_Pack_size      (                    sq->n+2, MPI_UNSIGNED_CHAR, comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+    status = esl_mpi_PackOptSize(sq->ss,             sq->n+2, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+    for (x = 0; x < sq->nxr; x ++) {
+      status = esl_mpi_PackOptSize(sq->xr_tag[x], sq->nalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+      status = esl_mpi_PackOptSize(sq->xr[x],        sq->n+2, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+    }
+    
   }
   else { 
-    status = MPI_Pack_size      (               sq->n+1, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
-    status = esl_mpi_PackOptSize(sq->ss,        sq->n+1, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+    status = MPI_Pack_size      (                    sq->n+1, MPI_CHAR,          comm, &sz); n += sz;   if (status != 0)     ESL_XEXCEPTION(eslESYS, "pack size failed");
+    status = esl_mpi_PackOptSize(sq->ss,             sq->n+1, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+    for (x = 0; x < sq->nxr; x ++) {
+      status = esl_mpi_PackOptSize(sq->xr_tag[x], sq->nalloc, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+      status = esl_mpi_PackOptSize(sq->xr[x],        sq->n+1, MPI_CHAR,          comm, &sz); n += sz;   if (status != eslOK) goto ERROR;
+      }
+    
   }
   
   *ret_n = n;
@@ -350,6 +361,7 @@ esl_sq_MPIPack(ESL_SQ *sq, char *buf, int n, int *pos, MPI_Comm comm)
   unsigned long int  W;
   unsigned long int  L;
   unsigned long int  salloc;
+  int                x;       /* index for optional extra residue markups */
   int                status;
  
   sq_n   = (unsigned long int)sq->n;
@@ -359,12 +371,13 @@ esl_sq_MPIPack(ESL_SQ *sq, char *buf, int n, int *pos, MPI_Comm comm)
   W      = (unsigned long int)sq->W;
   L      = (unsigned long int)sq->L;
   salloc = (unsigned long int)sq->salloc;
-
+ 
   /* pack allocation values */
   status = MPI_Pack         ((              int *) &(sq->nalloc),              1, MPI_INT,           buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
   status = MPI_Pack         ((              int *) &(sq->aalloc),              1, MPI_INT,           buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
   status = MPI_Pack         ((              int *) &(sq->dalloc),              1, MPI_INT,           buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
   status = MPI_Pack         ((              int *) &(sq->srcalloc),            1, MPI_INT,           buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
+  status = MPI_Pack         ((              int *) &(sq->nxr),                 1, MPI_INT,           buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
   status = MPI_Pack         ((unsigned long int *) &(    salloc),              1, MPI_UNSIGNED_LONG, buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
 
   /* pack coordenate info */
@@ -381,14 +394,22 @@ esl_sq_MPIPack(ESL_SQ *sq, char *buf, int n, int *pos, MPI_Comm comm)
   status = MPI_Pack         (sq->desc,                                sq->dalloc, MPI_CHAR,          buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
   status = MPI_Pack         (sq->source,                            sq->srcalloc, MPI_CHAR,          buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
 
-  /* sequences, digital or text; the ss is optional*/
+  /* sequences, digital or text; the ss and extra residue markups are optional */
   if (sq->dsq != NULL) {   
     status = MPI_Pack       (sq->dsq,                                    sq->n+2, MPI_UNSIGNED_CHAR, buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
     status = esl_mpi_PackOpt(sq->ss,                                     sq->n+2, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+    for (x = 0; x < sq->nxr; x ++) {
+      status = esl_mpi_PackOpt(sq->xr_tag[x],                         sq->nalloc, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+      status = esl_mpi_PackOpt(sq->xr[x],                                sq->n+2, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+    }
   }
   else {
     status = MPI_Pack       (sq->seq,                                    sq->n+1, MPI_CHAR,          buf, n, pos,  comm); if (status != 0)     ESL_EXCEPTION(eslESYS, "pack failed");
     status = esl_mpi_PackOpt(sq->ss,                                     sq->n+1, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+    for (x = 0; x < sq->nxr; x ++) {
+      status = esl_mpi_PackOpt(sq->xr_tag[x],                         sq->nalloc, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+      status = esl_mpi_PackOpt(sq->xr[x],                                sq->n+1, MPI_CHAR,          buf, n, pos,  comm); if (status != eslOK) return status;
+    }
   }
   
   if (*pos > n) ESL_EXCEPTION(eslEMEM, "buffer overflow");
@@ -427,6 +448,7 @@ esl_sq_MPIUnpack(const ESL_ALPHABET *abc, char *buf, int n, int *pos, MPI_Comm c
   unsigned long int  W;
   unsigned long int  L;
   unsigned long int  salloc;
+  int                x;       /* index for optional extra residue markups */
   int                do_digital = FALSE;
   int                status;
 
@@ -440,6 +462,7 @@ esl_sq_MPIUnpack(const ESL_ALPHABET *abc, char *buf, int n, int *pos, MPI_Comm c
   status = MPI_Unpack(buf, n, pos, &(sq->aalloc),   1, MPI_INT,            comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
   status = MPI_Unpack(buf, n, pos, &(sq->dalloc),   1, MPI_INT,            comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
   status = MPI_Unpack(buf, n, pos, &(sq->srcalloc), 1, MPI_INT,            comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
+  status = MPI_Unpack(buf, n, pos, &(sq->nxr),      1, MPI_INT,            comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
   status = MPI_Unpack(buf, n, pos, &(salloc),       1, MPI_UNSIGNED_LONG,  comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
 
   /* unpack coordenate info */
@@ -465,7 +488,9 @@ esl_sq_MPIUnpack(const ESL_ALPHABET *abc, char *buf, int n, int *pos, MPI_Comm c
   sq->source = NULL; ESL_ALLOC(sq->source, sizeof(char) * sq->srcalloc);
   sq->seq    = NULL; if (!do_digital) ESL_ALLOC(sq->seq, sizeof(char)    * sq->salloc);
   sq->dsq    = NULL; if ( do_digital) ESL_ALLOC(sq->dsq, sizeof(ESL_DSQ) * sq->salloc); 	
-  sq->ss     = NULL;		/* ss is optional - it will only be allocated if needed */
+  sq->ss     = NULL; /* ss and extra residue markups are optional - they will only be allocated if needed */
+  sq->xr_tag = NULL;
+  sq->xr     = NULL;
 
   /* unpack strings */
   status = MPI_Unpack       (buf, n, pos, sq->name,                   sq->nalloc, MPI_CHAR,           comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
@@ -479,10 +504,22 @@ esl_sq_MPIUnpack(const ESL_ALPHABET *abc, char *buf, int n, int *pos, MPI_Comm c
   else {
     status = MPI_Unpack     (buf, n, pos, sq->seq,                       sq->n+1, MPI_CHAR,           comm); if (status != 0)     ESL_XEXCEPTION(eslESYS, "mpi unpack failed");
   }
-   
+
   /* unpack the optional ss */
   status = esl_mpi_UnpackOpt(buf, n, pos, (void **) &(sq->ss),              NULL, MPI_CHAR,           comm); if (status != eslOK) goto ERROR;
- 
+  
+  /* unpack the optional extra residue markups */
+  if (sq->nxr > 0) { 
+    ESL_ALLOC(sq->xr,     sizeof(char *) * sq->nxr);
+    ESL_ALLOC(sq->xr_tag, sizeof(char *) * sq->nxr);
+    for (x = 0; x < sq->nxr; x ++) {
+      sq->xr[x]     = NULL;
+      sq->xr_tag[x] = NULL;
+       status = esl_mpi_UnpackOpt(buf, n, pos, (void **) &(sq->xr_tag[x]),  NULL, MPI_CHAR,           comm); if (status != eslOK) goto ERROR;
+       status = esl_mpi_UnpackOpt(buf, n, pos, (void **) &(sq->xr[x]),      NULL, MPI_CHAR,           comm); if (status != eslOK) goto ERROR;
+    }
+  }
+  
   /* set disk offset bookkeeping */
   sq->doff  = -1;
   sq->roff  = -1;
@@ -1117,7 +1154,7 @@ main(int argc, char **argv)
 {
   ESL_GETOPTS  *go  = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
   ESL_ALPHABET *abc = NULL;
-  ESLX_MSAFILE *afp = NULL;
+  ESL_MSAFILE  *afp = NULL;
   ESL_MSA      *msa = NULL;
   int           do_stall = FALSE;
   int           my_rank;
@@ -1130,19 +1167,19 @@ main(int argc, char **argv)
   /* Get a test MSA and alphabet. */
   if (esl_opt_GetString(go, "-m") != NULL) 
     {
-      if (eslx_msafile_Open(&abc, esl_opt_GetString(go, "-m"), eslMSAFILE_UNKNOWN, NULL, &afp) != eslOK) esl_fatal("msa file open failed");
-      if (eslx_msafile_Read(afp, &msa)                                                         != eslOK) esl_fatal("msa read failed");
-      eslx_msafile_Close(afp);
+      if (esl_msafile_Open(&abc, esl_opt_GetString(go, "-m"), eslMSAFILE_UNKNOWN, NULL, &afp) != eslOK) esl_fatal("msa file open failed");
+      if (esl_msafile_Read(afp, &msa)                                                         != eslOK) esl_fatal("msa read failed");
+      esl_msafile_Close(afp);
     }
   else
     {
       abc = esl_alphabet_Create(eslAMINO);
-      if ( (eslx_msafile_OpenMem(&abc, 
+      if ( (esl_msafile_OpenMem(&abc, 
 				 "# STOCKHOLM 1.0\n\nNIFE_CLOPA GYVGS\nNIFD_AZOVI GFDGF\nNIFD_BRAJA GYDGF\nNIFK_ANASP GYQGG\n//\n",
 				 -1, eslMSAFILE_STOCKHOLM, NULL, &afp)) != eslOK)
 	esl_fatal("msa creation failed");
-      if (eslx_msafile_Read(afp, &msa) != eslOK) esl_fatal("msa read failed");
-      eslx_msafile_Close(afp);
+      if (esl_msafile_Read(afp, &msa) != eslOK) esl_fatal("msa read failed");
+      esl_msafile_Close(afp);
     }
 
 
@@ -1198,13 +1235,10 @@ int main(void) { return 0; }
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- * 
- * SVN $Id: esl_mpi.c 756 2012-03-23 13:24:47Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_mpi.c $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
