@@ -1,22 +1,24 @@
 /* Clustering sequences in an MSA by % identity.
  * 
- * SRE, Sun Nov  5 10:08:14 2006 [Janelia]
- * SVN $Id: esl_msacluster.h 664 2011-02-27 17:08:36Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msacluster.h $
  */
 #ifndef eslMSACLUSTER_INCLUDED
 #define eslMSACLUSTER_INCLUDED
 
+#include "esl_msa.h"
+
 extern int esl_msacluster_SingleLinkage(const ESL_MSA *msa, double maxid, 
 					int **opt_c, int **opt_nin, int *opt_nc);
 
 #endif /*eslMSACLUSTER_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
+ *
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
