@@ -1,6 +1,6 @@
 /* Memory-efficient multiple sequence alignment i/o from Pfam format
  * 
- * Legacy interface, now that ESLX_MSAFILE is rewritten. Just enough
+ * Legacy interface, now that ESL_MSAFILE is rewritten. Just enough
  * of the old interface is retained to support --small option in
  * various tools, for reading Pfam format in memory-efficient ways.
  *
@@ -13,7 +13,7 @@
  *    6. Copyright and license information.
  * 
  * to-do:
- *   :: add memory-efficient interface in ESLX_MSAFILE
+ *   :: add memory-efficient interface in ESL_MSAFILE
  *   :: add memory-efficient ESL_MSA w/ API
  *   :: add space-efficient MSA file format
  */
@@ -147,7 +147,7 @@ esl_msafile2_Close(ESL_MSAFILE2 *afp)
  *           
  * Throws:   <eslEMEM> on allocation failure.
  *           <eslEINVAL> if format autodetection is attempted on 
- *           stdin or a gzip pipe.
+ *           stdin or a gzip -dc pipe.
  */
 int
 esl_msafile2_OpenDigital(const ESL_ALPHABET *abc, const char *filename, 
@@ -233,14 +233,14 @@ msafile2_open(const char *filename, const char *env, ESL_MSAFILE2 **ret_afp)
        */
       if ((afp->f = fopen(filename, "r")) != NULL)
 	{
-	  if (esl_strdup(filename, n, &ssifile)                      != eslOK) goto ERROR;
-	  if (esl_strcat(&ssifile, n, ".ssi", 4)                     != eslOK) goto ERROR;
+	  if ((status = esl_strdup(filename, n, &ssifile))           != eslOK) goto ERROR;
+	  if ((status = esl_strcat(&ssifile, n, ".ssi", 4))          != eslOK) goto ERROR;
 	  if ((status = esl_strdup(filename, n, &(afp->fname)))      != eslOK) goto ERROR;
 	}
       else if (esl_FileEnvOpen(filename, env, &(afp->f), &envfile) == eslOK)
 	{
-	  if (esl_strdup(envfile, n, &ssifile)                      != eslOK) goto ERROR;
-	  if (esl_strcat(&ssifile, n, ".ssi", 4)                    != eslOK) goto ERROR;
+	  if ((status = esl_strdup(envfile, n, &ssifile))           != eslOK) goto ERROR;
+	  if ((status = esl_strcat(&ssifile, n, ".ssi", 4))         != eslOK) goto ERROR;
 	  if ((status = esl_strdup(envfile, n, &(afp->fname)))      != eslOK) goto ERROR;
 	}
       else 
@@ -254,7 +254,7 @@ msafile2_open(const char *filename, const char *env, ESL_MSAFILE2 **ret_afp)
    * We should warn, probably, or provide some way for caller to 
    * to know that we've opened the index successfully or not.
    */
-  status = esl_ssi_Open(ssifile, &(afp->ssi)); 
+  esl_ssi_Open(ssifile, &(afp->ssi));
 #endif
 
   if (envfile != NULL) free(envfile);
@@ -907,8 +907,8 @@ esl_msafile2_RegurgitatePfam(ESL_MSAFILE2 *afp, FILE *ofp, int maxname, int maxg
 
 	      /* output, after optionally removing some characters (if useme != NULL) or adding gaps (if add2me != NULL) (contract enforces only one can be non-null) */
 	      if(useme  != NULL) { 
-		/* if this is a GC SS_cons line, remove broken basepairs first */
-		if(strncmp(tag, "SS_cons", 7) == 0) {
+		/* if this is a GC SS_cons line, remove broken basepairs first - only if it's in WUSS RNA format (NOT for a protein SS!) */
+		if (strncmp(tag, "SS_cons", 7) == 0 && afp->abc && (afp->abc->type == eslRNA || afp->abc->type == eslDNA)) {
 		  if((status = esl_msa_RemoveBrokenBasepairsFromSS(text, afp->errbuf, textlen, useme)) != eslOK) ESL_XFAIL(eslEFORMAT, afp->errbuf, "small mem parse failed (line %d): bad #=GC SS_cons line", afp->linenumber);
 		}
 		shrink_string(text, useme, exp_alen); /* this is done in place on text */
@@ -973,7 +973,7 @@ esl_msafile2_RegurgitatePfam(ESL_MSAFILE2 *afp, FILE *ofp, int maxname, int maxg
 		  /* output GR, after optionally removing some characters (if useme != NULL) or adding gaps (if add2me != NULL) (contract enforces only one can be non-null) */
 		  if(useme  != NULL) { 
 		    /* if this is a GR SS line, remove broken basepairs first */
-		    if(strncmp(tag, "SS", 2) == 0) {
+		    if( strncmp(tag, "SS", 2) == 0 && afp->abc && (afp->abc->type == eslRNA || afp->abc->type == eslDNA)) {
 		      if((status = esl_msa_RemoveBrokenBasepairsFromSS(text, afp->errbuf, textlen, useme)) != eslOK) ESL_XFAIL(eslEFORMAT, afp->errbuf, "small mem parse failed (line %d): bad #=GR SS line", afp->linenumber);
 		    }
 		    shrink_string(text, useme, exp_alen); /* this is done in place on text */
@@ -1384,7 +1384,7 @@ utest_RegurgitatePfam(char *filename)
 {
   char         *msg         = "RegurgitatePfam() unit test failure";
   ESL_MSAFILE2 *mfp         = NULL;
-  ESLX_MSAFILE *afp         = NULL;
+  ESL_MSAFILE  *afp         = NULL;
   char          tmpfile[16] = "esltmpXXXXXX";
   FILE         *fp          = NULL;
   ESL_MSA      *msa1        = NULL;
@@ -1409,14 +1409,14 @@ utest_RegurgitatePfam(char *filename)
   esl_msafile2_Close(mfp);
 
   /* Using normal interface, read in msa from filename as msa1 */
-  if (eslx_msafile_Open(NULL, filename, NULL, eslMSAFILE_PFAM, NULL, &afp) != eslOK) esl_fatal(msg); 
-  if (eslx_msafile_Read(afp, &msa1)                                        != eslOK) esl_fatal(msg);
-  eslx_msafile_Close(afp);
+  if (esl_msafile_Open(NULL, filename, NULL, eslMSAFILE_PFAM, NULL, &afp) != eslOK) esl_fatal(msg); 
+  if (esl_msafile_Read(afp, &msa1)                                        != eslOK) esl_fatal(msg);
+  esl_msafile_Close(afp);
 
   /* Using normal interface, read in msa from tmpfile as msa2 */
-  if (eslx_msafile_Open(NULL, tmpfile, NULL, eslMSAFILE_PFAM, NULL, &afp) != eslOK) esl_fatal(msg);
-  if (eslx_msafile_Read(afp, &msa2)                                       != eslOK) esl_fatal(msg);
-  eslx_msafile_Close(afp);
+  if (esl_msafile_Open(NULL, tmpfile, NULL, eslMSAFILE_PFAM, NULL, &afp) != eslOK) esl_fatal(msg);
+  if (esl_msafile_Read(afp, &msa2)                                       != eslOK) esl_fatal(msg);
+  esl_msafile_Close(afp);
 
   if (esl_msa_Compare(msa1, msa2) != eslOK) esl_fatal(msg);
 
@@ -1474,15 +1474,14 @@ main(int argc, char **argv)
 #endif /*eslMSAFILE2_TESTDRIVE*/
 /*----------------- end, test driver ----------------------------*/
 
+
+
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- * 
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msafile2.c $
- * SVN $Id: esl_msafile2.c 833 2012-12-07 18:57:36Z nawrockie $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
