@@ -39,7 +39,7 @@
  *            map ' ' to <eslDSQ_IGNORED>.
  */
 int
-esl_msafile_afa_SetInmap(ESLX_MSAFILE *afp)
+esl_msafile_afa_SetInmap(ESL_MSAFILE *afp)
 {
   int sym;
 
@@ -89,7 +89,7 @@ esl_msafile_afa_SetInmap(ESLX_MSAFILE *afp)
  *            modularity/separability of parsers.
  */
 int
-esl_msafile_afa_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
+esl_msafile_afa_GuessAlphabet(ESL_MSAFILE *afp, int *ret_type)
 {
   int       alphatype     = eslUNKNOWN;
   esl_pos_t anchor        = -1;
@@ -148,7 +148,7 @@ esl_msafile_afa_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
 /* Function:  esl_msafile_afa_Read()
  * Synopsis:  Read in an aligned FASTA format alignment.
  *
- * Purpose:   Read an MSA from an open <ESLX_MSAFILE> <afp>,
+ * Purpose:   Read an MSA from an open <ESL_MSAFILE> <afp>,
  *            parsing for aligned FASTA format. Create
  *            a new MSA, and return a ptr to that alignment
  *            in <*ret_msa>. Caller is responsible for free'ing 
@@ -182,7 +182,7 @@ esl_msafile_afa_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
  *            <afp> is undefined.
  */
 int
-esl_msafile_afa_Read(ESLX_MSAFILE *afp, ESL_MSA **ret_msa)
+esl_msafile_afa_Read(ESL_MSAFILE *afp, ESL_MSA **ret_msa)
 {
   ESL_MSA  *msa  = NULL;
   int       idx  = 0;
@@ -202,7 +202,7 @@ esl_msafile_afa_Read(ESLX_MSAFILE *afp, ESL_MSA **ret_msa)
   if (! afp->abc &&  (msa = esl_msa_Create(                 16, -1)) == NULL) { status = eslEMEM; goto ERROR; }
   
   /* skip leading blank lines in file */
-  while ( (status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK && esl_memspn(afp->line, afp->n, " \t") == afp->n) ;
+  while ( (status = esl_msafile_GetLine(afp, &p, &n)) == eslOK && esl_memspn(afp->line, afp->n, " \t") == afp->n) ;
   if      (status != eslOK)  goto ERROR; /* includes normal EOF */
 
   /* tolerate sloppy space at start of line */
@@ -226,7 +226,7 @@ esl_msafile_afa_Read(ESLX_MSAFILE *afp, ESL_MSA **ret_msa)
      * test on PF00005 Full)
      */
     this_alen = 0;
-    while ((status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK)
+    while ((status = esl_msafile_GetLine(afp, &p, &n)) == eslOK)
     {
       while (n && isspace(*p)) { p++; n--; } /* tolerate and skip leading whitespace on line */
       if (n  == 0)   continue;	       /* tolerate and skip blank lines */
@@ -269,7 +269,8 @@ esl_msafile_afa_Read(ESLX_MSAFILE *afp, ESL_MSA **ret_msa)
  *            If <msa> is in text mode, residues and gaps are written
  *            exactly as they appear in the data structure. If <msa>
  *            is digital, residues are in uppercase and all gaps are
- *            -.
+ *            dots (.). Dots are preferred to dashes because it 
+ *            minimizes confusion with A2M format.
  *
  * Args:      fp  - open stream to write to
  *            msa - MSA to write
@@ -314,7 +315,7 @@ esl_msafile_afa_Write(FILE *fp, const ESL_MSA *msa)
  * 2. Unit tests.
  *****************************************************************/
 #ifdef eslMSAFILE_AFA_TESTDRIVE
-/* a standard globin example, but dusted with evil:
+/* a standard globin example, dusted with evil:
  *  1. \r\n DOS line terminators;
  *  2. extra blank lines and whitespace
  *  3. unusual but legal residues
@@ -347,36 +348,12 @@ utest_write_good1(FILE *ofp, int *ret_alphatype, int *ret_nseq, int *ret_alen)
   *ret_alen      = 165;
 }
 
-/* note: alphabet detection for DNA/RNA doesn't work if unusual codes are used */
-static void
-utest_write_good2(FILE *ofp, int *ret_alphatype, int *ret_nseq, int *ret_alen)
-{
-  fputs(">tRNA2\n", ofp);
-  fputs("UCCGAUAUAGUGUAACGGCUAUCACAUCACGCUUUCACCGUGG-AGACCGGGGUUCGACU\n", ofp);
-  fputs("CCCCGUAUCGGAG\n", ofp);
-  fputs(">tRNA3\n", ofp);
-  fputs("UCCGUGAUAGUUUAAUGGUCAGAAUGG-GCGCUUGUCGCGUGCCAGAUCGGGGUUCAAUU\n", ofp);
-  fputs("CCCCGUCGCGGAG\n", ofp);
-  fputs(">tRNA5\n", ofp);
-  fputs("GGGCACAUGGCGCAGUUGGUAGCGCGCUUCCCUUGCAAGGAAGAGGUCAUCGGUUCGAUU\n", ofp);
-  fputs("CCGGUUGCGUCCA\n", ofp);
-  fputs(">tRNA1\n", ofp);
-  fputs("GCGGAUUUAGCUCAGUUGGGAGAGCGCCAGACUGAAGAUCUGGAGGUCCUGUGUUCGAUC\n", ofp);
-  fputs("CACAGAAUUCGCA\n", ofp);
-  fputs(">tRNA4\n", ofp);
-  fputs("GCUCGUAUGGCGCAGUGG-UAGCGCAGCAGAUUGCAAAUCUGUUGGUCCUUAGUUCGAUC\n", ofp);
-  fputs("CUGAGUGCGAGCU\n", ofp);
-
-  *ret_alphatype = eslRNA;
-  *ret_nseq      = 5;
-  *ret_alen      = 73;
-}
 
 static void
 utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expected_nseq, int expected_alen)
 {
   ESL_ALPHABET        *abc          = NULL;
-  ESLX_MSAFILE        *afp          = NULL;
+  ESL_MSAFILE         *afp          = NULL;
   ESL_MSA             *msa1         = NULL;
   ESL_MSA             *msa2         = NULL;
   char                 tmpfile1[32] = "esltmpXXXXXX";
@@ -385,16 +362,16 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   int                  status;
 
   /* guessing both the format and the alphabet should work: this is a digital open */
-  if ( (status = eslx_msafile_Open(&abc, filename, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: digital open",           testnumber);  
-  if (afp->format != eslMSAFILE_AFA)                                                                esl_fatal("afa good file test %d failed: format autodetection",   testnumber);
-  if (abc->type   != expected_alphatype)                                                            esl_fatal("afa good file test %d failed: alphabet autodetection", testnumber);
+  if ( (status = esl_msafile_Open(&abc, filename, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: digital open",           testnumber);  
+  if (afp->format != eslMSAFILE_AFA)                                                               esl_fatal("afa good file test %d failed: format autodetection",   testnumber);
+  if (abc->type   != expected_alphatype)                                                           esl_fatal("afa good file test %d failed: alphabet autodetection", testnumber);
 
   /* This is a digital read, using <abc>. */
   if ( (status = esl_msafile_afa_Read(afp, &msa1))   != eslOK)     esl_fatal("afa good file test %d failed: msa read, digital", testnumber);  
   if (msa1->nseq != expected_nseq || msa1->alen != expected_alen)  esl_fatal("afa good file test %d failed: nseq/alen",         testnumber);
   if (esl_msa_Validate(msa1, NULL) != eslOK)                       esl_fatal("afa good file test %d failed: msa invalid",       testnumber);
 
-  eslx_msafile_Close(afp);  
+  esl_msafile_Close(afp);  
 
   /* write it back out to a new tmpfile (digital write) */
   if ( (status = esl_tmpfile_named(tmpfile1, &ofp)) != eslOK) esl_fatal("afa good file test %d failed: tmpfile creation",   testnumber);
@@ -402,11 +379,11 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   fclose(ofp);
 
   /* now open and read it as text mode, in known format. (We have to pass fmtd now, to deal with the possibility of a nonstandard name width) */
-  if ( (status = eslx_msafile_Open(NULL, tmpfile1, NULL, eslMSAFILE_AFA, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: text mode open", testnumber);  
-  if ( (status = esl_msafile_afa_Read(afp, &msa2))                                    != eslOK) esl_fatal("afa good file test %d failed: msa read, text", testnumber);  
-  if (msa2->nseq != expected_nseq || msa2->alen != expected_alen)                               esl_fatal("afa good file test %d failed: nseq/alen",      testnumber);
-  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                    esl_fatal("afa good file test %d failed: msa invalid",    testnumber);
-  eslx_msafile_Close(afp);
+  if ( (status = esl_msafile_Open(NULL, tmpfile1, NULL, eslMSAFILE_AFA, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: text mode open", testnumber);  
+  if ( (status = esl_msafile_afa_Read(afp, &msa2))                                   != eslOK) esl_fatal("afa good file test %d failed: msa read, text", testnumber);  
+  if (msa2->nseq != expected_nseq || msa2->alen != expected_alen)                              esl_fatal("afa good file test %d failed: nseq/alen",      testnumber);
+  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                   esl_fatal("afa good file test %d failed: msa invalid",    testnumber);
+  esl_msafile_Close(afp);
   
   /* write it back out to a new tmpfile (text write) */
   if ( (status = esl_tmpfile_named(tmpfile2, &ofp)) != eslOK) esl_fatal("afa good file test %d failed: tmpfile creation", testnumber);
@@ -415,10 +392,10 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   esl_msa_Destroy(msa2);
 
   /* open and read it in digital mode */
-  if ( (status = eslx_msafile_Open(&abc, tmpfile1, NULL, eslMSAFILE_AFA, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: 2nd digital mode open", testnumber);  
-  if ( (status = esl_msafile_afa_Read(afp, &msa2))                                    != eslOK) esl_fatal("afa good file test %d failed: 2nd digital msa read",  testnumber);  
-  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                    esl_fatal("afa good file test %d failed: msa invalid",           testnumber);
-  eslx_msafile_Close(afp);
+  if ( (status = esl_msafile_Open(&abc, tmpfile1, NULL, eslMSAFILE_AFA, NULL, &afp)) != eslOK) esl_fatal("afa good file test %d failed: 2nd digital mode open", testnumber);  
+  if ( (status = esl_msafile_afa_Read(afp, &msa2))                                   != eslOK) esl_fatal("afa good file test %d failed: 2nd digital msa read",  testnumber);  
+  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                   esl_fatal("afa good file test %d failed: msa invalid",           testnumber);
+  esl_msafile_Close(afp);
 
   /* this msa <msa2> should be identical to <msa1> */
   if (esl_msa_Compare(msa1, msa2) != eslOK) esl_fatal("afa good file test %d failed: msa compare", testnumber);  
@@ -430,6 +407,7 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   esl_alphabet_Destroy(abc);
 }
 
+
 static void
 write_test_msas(FILE *ofp1, FILE *ofp2)
 {
@@ -465,43 +443,43 @@ read_test_msas_digital(char *afafile, char *stkfile)
 {
   char msg[]         = "aligned FASTA msa digital read unit test failed";
   ESL_ALPHABET *abc  = NULL;
-  ESLX_MSAFILE *afp1 = NULL;
-  ESLX_MSAFILE *afp2 = NULL;
+  ESL_MSAFILE  *afp1 = NULL;
+  ESL_MSAFILE  *afp2 = NULL;
   ESL_MSA      *msa1, *msa2, *msa3, *msa4;
   FILE         *afafp, *stkfp;
   char          afafile2[32] = "esltmpafa2XXXXXX";
   char          stkfile2[32] = "esltmpstk2XXXXXX";
 
-  if ( eslx_msafile_Open(&abc, afafile, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK)  esl_fatal(msg);
-  if ( !abc || abc->type != eslAMINO)                                                       esl_fatal(msg);
-  if ( eslx_msafile_Open(&abc, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_afa_Read      (afp1, &msa1)                                   != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                   != eslOK)  esl_fatal(msg);
-  if ( esl_msa_Compare(msa1, msa2)                                               != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, afafile, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK)  esl_fatal(msg);
+  if ( !abc || abc->type != eslAMINO)                                                      esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa1)                                  != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                  != eslOK)  esl_fatal(msg);
+  if ( esl_msa_Compare(msa1, msa2)                                              != eslOK)  esl_fatal(msg);
 
-  if ( esl_msafile_afa_Read      (afp1, &msa3)                             != eslEOF) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa3)                             != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa3) != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa3) != eslEOF) esl_fatal(msg);
 
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   /* Now write stk to afa file, and vice versa; then retest */
-  if ( esl_tmpfile_named(afafile2, &afafp)                                  != eslOK) esl_fatal(msg);
-  if ( esl_tmpfile_named(stkfile2, &stkfp)                                  != eslOK) esl_fatal(msg);
-  if ( esl_msafile_afa_Write      (afafp, msa2)                             != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)       != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(afafile2, &afafp)                             != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(stkfile2, &stkfp)                             != eslOK) esl_fatal(msg);
+  if ( esl_msafile_afa_Write      (afafp, msa2)                        != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)  != eslOK) esl_fatal(msg);
   fclose(afafp);
   fclose(stkfp);
-  if ( eslx_msafile_Open(&abc, afafile2, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK) esl_fatal(msg);
-  if ( eslx_msafile_Open(&abc, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
-  if ( esl_msafile_afa_Read      (afp1, &msa3)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msa_Compare(msa3, msa4)                                                != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, afafile2, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa3)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msa_Compare(msa3, msa4)                                               != eslOK) esl_fatal(msg);
 
   remove(afafile2);
   remove(stkfile2);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   esl_msa_Destroy(msa1);
   esl_msa_Destroy(msa2);
@@ -514,40 +492,40 @@ static void
 read_test_msas_text(char *afafile, char *stkfile)
 {
   char msg[]         = "aligned FASTA msa text-mode read unit test failed";
-  ESLX_MSAFILE *afp1 = NULL;
-  ESLX_MSAFILE *afp2 = NULL;
+  ESL_MSAFILE  *afp1 = NULL;
+  ESL_MSAFILE  *afp2 = NULL;
   ESL_MSA      *msa1, *msa2, *msa3, *msa4;
   FILE         *afafp, *stkfp;
   char          afafile2[32] = "esltmpafa2XXXXXX";
   char          stkfile2[32] = "esltmpstk2XXXXXX";
 
-  /*                     vvvv-- everything's the same as the digital utest except these NULLs  */
-  if ( eslx_msafile_Open(NULL, afafile, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK)  esl_fatal(msg);
-  if ( eslx_msafile_Open(NULL, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_afa_Read      (afp1, &msa1)                                   != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                   != eslOK)  esl_fatal(msg);
-  if ( esl_msa_Compare(msa1, msa2)                                               != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_afa_Read      (afp1, &msa3)                                   != eslEOF) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa3)                                   != eslEOF) esl_fatal(msg);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  /*                    vvvv-- everything's the same as the digital utest except these NULLs  */
+  if ( esl_msafile_Open(NULL, afafile, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa1)                                  != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                  != eslOK)  esl_fatal(msg);
+  if ( esl_msa_Compare(msa1, msa2)                                              != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa3)                                  != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa3)                                  != eslEOF) esl_fatal(msg);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
-  if ( esl_tmpfile_named(afafile2, &afafp)                                  != eslOK) esl_fatal(msg);
-  if ( esl_tmpfile_named(stkfile2, &stkfp)                                  != eslOK) esl_fatal(msg);
-  if ( esl_msafile_afa_Write      (afafp, msa2)                             != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)       != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(afafile2, &afafp)                             != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(stkfile2, &stkfp)                             != eslOK) esl_fatal(msg);
+  if ( esl_msafile_afa_Write      (afafp, msa2)                        != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)  != eslOK) esl_fatal(msg);
   fclose(afafp);
   fclose(stkfp);
-  if ( eslx_msafile_Open(NULL, afafile2, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK) esl_fatal(msg);
-  if ( eslx_msafile_Open(NULL, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
-  if ( esl_msafile_afa_Read      (afp1, &msa3)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msa_Compare(msa3, msa4)                                                != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, afafile2, NULL, eslMSAFILE_AFA,       NULL, &afp1) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_afa_Read      (afp1, &msa3)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msa_Compare(msa3, msa4)                                               != eslOK) esl_fatal(msg);
 
   remove(afafile2);
   remove(stkfile2);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   esl_msa_Destroy(msa1);
   esl_msa_Destroy(msa2);
@@ -593,7 +571,7 @@ main(int argc, char **argv)
   char            stkfile[32] = "esltmpstkXXXXXX";
   FILE           *afafp, *stkfp;
   int             testnumber;
-  int             ngoodtests = 2;
+  int             ngoodtests  = 1;
   char            tmpfile[32];
   FILE           *ofp;
   int             expected_alphatype;
@@ -617,7 +595,6 @@ main(int argc, char **argv)
       if (esl_tmpfile_named(tmpfile, &ofp) != eslOK) esl_fatal(msg);
       switch (testnumber) {
       case  1:  utest_write_good1 (ofp, &expected_alphatype, &expected_nseq, &expected_alen); break;
-      case  2:  utest_write_good2 (ofp, &expected_alphatype, &expected_nseq, &expected_alen); break;
       }
       fclose(ofp);
       utest_goodfile(tmpfile, testnumber, expected_alphatype, expected_nseq, expected_alen);
@@ -674,7 +651,7 @@ main(int argc, char **argv)
   char               *filename    = esl_opt_GetArg(go, 1);
   int                 infmt       = eslMSAFILE_UNKNOWN;
   ESL_ALPHABET       *abc         = NULL;
-  ESLX_MSAFILE       *afp         = NULL;
+  ESL_MSAFILE        *afp         = NULL;
   ESL_MSA            *msa         = NULL;
   int                 status;
 
@@ -687,12 +664,12 @@ main(int argc, char **argv)
   /* Text mode: pass NULL for alphabet.
    * Digital mode: pass ptr to expected ESL_ALPHABET; and if abc=NULL, alphabet is guessed 
    */
-  if   (esl_opt_GetBoolean(go, "-t"))  status = eslx_msafile_Open(NULL, filename, NULL, infmt, NULL, &afp);
-  else                                 status = eslx_msafile_Open(&abc, filename, NULL, infmt, NULL, &afp);
-  if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+  if   (esl_opt_GetBoolean(go, "-t"))  status = esl_msafile_Open(NULL, filename, NULL, infmt, NULL, &afp);
+  else                                 status = esl_msafile_Open(&abc, filename, NULL, infmt, NULL, &afp);
+  if (status != eslOK) esl_msafile_OpenFailure(afp, status);
 
   if ((status = esl_msafile_afa_Read(afp, &msa)) != eslOK)
-    eslx_msafile_ReadFailure(afp, status);
+    esl_msafile_ReadFailure(afp, status);
 
   printf("alphabet:       %s\n", (abc ? esl_abc_DecodeType(abc->type) : "none (text mode)"));
   printf("# of seqs:      %d\n", msa->nseq);
@@ -703,7 +680,7 @@ main(int argc, char **argv)
     esl_msafile_afa_Write(stdout, msa);
 
   esl_msa_Destroy(msa);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   if (abc) esl_alphabet_Destroy(abc);
   esl_getopts_Destroy(go);
   exit(0);
@@ -729,18 +706,18 @@ main(int argc, char **argv)
 {
   char         *filename = argv[1];
   int           fmt      = eslMSAFILE_AFA;
-  ESLX_MSAFILE *afp      = NULL;
+  ESL_MSAFILE  *afp      = NULL;
   ESL_MSA      *msa      = NULL;
   int           status;
 
-  if ( (status = eslx_msafile_Open(NULL, filename, NULL, fmt, NULL, &afp)) != eslOK) eslx_msafile_OpenFailure(afp, status);
-  if ( (status = esl_msafile_afa_Read(afp, &msa))                          != eslOK) eslx_msafile_ReadFailure(afp, status);
+  if ( (status = esl_msafile_Open(NULL, filename, NULL, fmt, NULL, &afp)) != eslOK) esl_msafile_OpenFailure(afp, status);
+  if ( (status = esl_msafile_afa_Read(afp, &msa))                         != eslOK) esl_msafile_ReadFailure(afp, status);
 
   printf("%6d seqs, %5d columns\n", msa->nseq, (int) msa->alen);
 
   esl_msafile_afa_Write(stdout, msa);
   esl_msa_Destroy(msa);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   exit(0);
 }
 /*::cexcerpt::msafile_afa_example2::end::*/
@@ -752,13 +729,10 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- * 
- * SVN $Id: esl_msafile_afa.c 754 2012-03-14 20:30:47Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msafile_afa.c $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
