@@ -65,11 +65,11 @@ static ESL_SELEX_BLOCK *selex_block_Create(int nalloc);
 static int              selex_block_Grow(ESL_SELEX_BLOCK *b);
 static void             selex_block_Destroy(ESL_SELEX_BLOCK *b);
 
-static int selex_ErrorInBlock(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, int idx);
-static int selex_read_block  (ESLX_MSAFILE *afp, ESL_SELEX_BLOCK **block_p);
-static int selex_first_block (ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa);
-static int selex_other_block (ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa);
-static int selex_append_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa);
+static int selex_ErrorInBlock(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, int idx);
+static int selex_read_block  (ESL_MSAFILE *afp, ESL_SELEX_BLOCK **block_p);
+static int selex_first_block (ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa);
+static int selex_other_block (ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa);
+static int selex_append_block(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa);
 
 
 /*****************************************************************
@@ -97,7 +97,7 @@ static int selex_append_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *ms
  *            alignment format would use <eslDSQ_IGNORED> mappings.
  */
 int
-esl_msafile_selex_SetInmap(ESLX_MSAFILE *afp)
+esl_msafile_selex_SetInmap(ESL_MSAFILE *afp)
 {
   int sym;
 
@@ -140,7 +140,7 @@ esl_msafile_selex_SetInmap(ESLX_MSAFILE *afp)
  *            started at.
  */
 int
-esl_msafile_selex_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
+esl_msafile_selex_GuessAlphabet(ESL_MSAFILE *afp, int *ret_type)
 {
   int       alphatype     = eslUNKNOWN;
   esl_pos_t anchor        = -1;
@@ -201,7 +201,7 @@ esl_msafile_selex_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
 /* Function:  esl_msafile_selex_Read()
  * Synopsis:  Read in a SELEX format alignment.
  *
- * Purpose:   Read an MSA from an open <ESLX_MSAFILE> <afp>, 
+ * Purpose:   Read an MSA from an open <ESL_MSAFILE> <afp>, 
  *            parsing for SELEX format, starting from the
  *            current point. (<afp->format> is expected to
  *            be <eslMSAFILE_SELEX>.) Create a new multiple
@@ -209,7 +209,7 @@ esl_msafile_selex_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
  *            Caller is responsible for free'ing this
  *            <ESL_MSA>.
  *
- * Args:      afp     - open <ESLX_MSAFILE>
+ * Args:      afp     - open <ESL_MSAFILE>
  *            ret_msa - RETURN: newly parsed <ESL_MSA>
  *
  * Returns:   <eslOK> on success.
@@ -234,7 +234,7 @@ esl_msafile_selex_GuessAlphabet(ESLX_MSAFILE *afp, int *ret_type)
  *            <eslEINCONCEIVABLE> - "impossible" corruption 
  */
 int
-esl_msafile_selex_Read(ESLX_MSAFILE *afp, ESL_MSA **ret_msa)
+esl_msafile_selex_Read(ESL_MSAFILE *afp, ESL_MSA **ret_msa)
 {
   ESL_MSA         *msa     = NULL;
   ESL_SELEX_BLOCK *b       = NULL;
@@ -443,7 +443,7 @@ selex_block_Destroy(ESL_SELEX_BLOCK *b)
  * reset the <afp> so its current line is the one at fault.
  */
 static int
-selex_ErrorInBlock(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, int which)
+selex_ErrorInBlock(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, int which)
 {
   afp->line       = b->line[which];
   afp->n      = b->llen[which];
@@ -475,7 +475,7 @@ selex_ErrorInBlock(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, int which)
  * Throws:  <eslEMEM> on allocation failure.      
  */
 static int
-selex_read_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK **block_p) 
+selex_read_block(ESL_MSAFILE *afp, ESL_SELEX_BLOCK **block_p) 
 {
   ESL_SELEX_BLOCK *b      = *block_p; /* now b==NULL if first block; or on subsequent blocks, reuse prev block storage. */
   int              idx    = 0;
@@ -486,12 +486,12 @@ selex_read_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK **block_p)
    * EOF, we're done.
    */
   do { 
-    if ( ( status = eslx_msafile_GetLine(afp, NULL, NULL)) != eslOK) goto ERROR;                   /* EOF here is a normal EOF   */
+    if ( ( status = esl_msafile_GetLine(afp, NULL, NULL)) != eslOK) goto ERROR;                   /* EOF here is a normal EOF   */
   } while (esl_memspn(afp->line, afp->n, " \t") == afp->n ||                                       /* idiomatic for "blank line" */
 	   (esl_memstrpfx(afp->line, afp->n, "#") && ! esl_memstrpfx(afp->line, afp->n, "#=")));   /* a SELEX comment line       */
 
   /* if this is first block, allocate block; subsequent blocks reuse it */
-  if (!b && (b = selex_block_Create(16)) == NULL) goto ERROR; 
+  if (!b && (b = selex_block_Create(16)) == NULL) { status = eslEMEM; goto ERROR; }
   
   /* Anchor stably at this point. */
   b->anchor = afp->lineoffset;
@@ -509,7 +509,7 @@ selex_read_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK **block_p)
     
     /* Get next non-comment line; this can be next line of block, blank (end of block), or EOF. */
     do { 
-      status = eslx_msafile_GetLine(afp, NULL, NULL); 
+      status = esl_msafile_GetLine(afp, NULL, NULL); 
     } while ( status == eslOK && (esl_memstrpfx(afp->line, afp->n, "#") && ! esl_memstrpfx(afp->line, afp->n, "#=")));
 
   } while (status == eslOK && esl_memspn(afp->line, afp->n, " \t") < afp->n); /* end of block on EOF or blank line */
@@ -538,7 +538,7 @@ selex_read_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK **block_p)
  * 5. Determine lpos[] for each line.
  */
 static int
-selex_first_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa)
+selex_first_block(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa)
 {
   ESL_MSA  *msa = NULL;
   int       nrf, nmm, ncs, nss, nsa, nseq;
@@ -550,7 +550,7 @@ selex_first_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa)
 
   afp->errmsg[0] = '\0';
 
-  nrf = ncs = nss = nsa = nseq = 0;
+  nrf = nmm = ncs = nss = nsa = nseq = 0;
   has_ss = has_sa = FALSE;
   for (idx = 0; idx < b->nlines; idx++)
     {
@@ -591,7 +591,7 @@ selex_first_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa)
       if ( esl_memtok(&p, &n, " \t", &tok, &ntok) != eslOK) ESL_XEXCEPTION(eslEINCONCEIVABLE, "can't happen"); /* because a block by definition consists of non-blank lines */
       if (b->ltype[idx] == eslSELEX_LINE_SQ) /* otherwise, first token is #=XX marking annotation of some sort */
 	{
-	  if ( esl_msa_SetSeqName(msa, seqi, tok, ntok)   != eslOK) goto ERROR;
+	  if ((status = esl_msa_SetSeqName(msa, seqi, tok, ntok)) != eslOK) goto ERROR;
 	  seqi++;
 	}
       b->lpos[idx] = (n ? p-b->line[idx] : -1);  /* set lpos[] to position of first seq or annotation residue */
@@ -613,7 +613,7 @@ selex_first_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA **ret_msa)
  * Validate that a subsequent block has the same.
  */
 static int
-selex_other_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
+selex_other_block(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
 {
   char     *p, *tok;
   esl_pos_t n, ntok;
@@ -647,14 +647,14 @@ selex_other_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
 }
 
 static int
-selex_append_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
+selex_append_block(ESL_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
 {
   char     *p;
   esl_pos_t pos;
   int       idx, seqi;
   esl_pos_t leftmost, rightmost;
   int64_t   nadd;		/* width of this sequence block, in aligned columns added to msa */
-  esl_pos_t nleft, ntext, nright;
+  esl_pos_t nleft, ntext;
   int64_t   alen;
   int       status;
   
@@ -682,7 +682,7 @@ selex_append_block(ESLX_MSAFILE *afp, ESL_SELEX_BLOCK *b, ESL_MSA *msa)
     {
       nleft  = ((b->lpos[idx] != -1) ? b->lpos[idx] - leftmost         : nadd); /* watch special case of all whitespace on data line, lpos>rpos */
       ntext  = ((b->lpos[idx] != -1) ? b->rpos[idx] - b->lpos[idx] + 1 : 0);
-      nright = ((b->lpos[idx] != -1) ? rightmost    - b->rpos[idx]     : 0);
+      //nright = ((b->lpos[idx] != -1) ? rightmost    - b->rpos[idx]     : 0);  // someday you might want to know nright, but for now the code doesn't use it
 
       if      (b->ltype[idx] == eslSELEX_LINE_SQ)
 	{
@@ -871,7 +871,7 @@ static void
 utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expected_nseq, int expected_alen)
 {
   ESL_ALPHABET        *abc          = NULL;
-  ESLX_MSAFILE        *afp          = NULL;
+  ESL_MSAFILE        *afp          = NULL;
   ESL_MSA             *msa1         = NULL;
   ESL_MSA             *msa2         = NULL;
   char                 tmpfile1[32] = "esltmpXXXXXX";
@@ -880,15 +880,15 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   int                  status;
 
   /* guessing both the format and the alphabet should work: this is a digital open */
-  if ( (status = eslx_msafile_Open(&abc, filename, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: digital open",           testnumber);  
-  if (afp->format != eslMSAFILE_SELEX)                                                              esl_fatal("selex good file test %d failed: format autodetection",   testnumber);
-  if (abc->type   != expected_alphatype)                                                            esl_fatal("selex good file test %d failed: alphabet autodetection", testnumber);
+  if ( (status = esl_msafile_Open(&abc, filename, NULL, eslMSAFILE_UNKNOWN, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: digital open",           testnumber);  
+  if (afp->format != eslMSAFILE_SELEX)                                                             esl_fatal("selex good file test %d failed: format autodetection",   testnumber);
+  if (abc->type   != expected_alphatype)                                                           esl_fatal("selex good file test %d failed: alphabet autodetection", testnumber);
 
   /* This is a digital read, using <abc>. */
   if ( (status = esl_msafile_selex_Read(afp, &msa1))   != eslOK)  esl_fatal("selex good file test %d failed: msa read, digital", testnumber);  
   if (msa1->nseq != expected_nseq || msa1->alen != expected_alen) esl_fatal("selex good file test %d failed: nseq/alen",         testnumber);
   if (esl_msa_Validate(msa1, NULL) != eslOK)                      esl_fatal("selex good file test %d failed: msa1 invalid",      testnumber);
-  eslx_msafile_Close(afp);  
+  esl_msafile_Close(afp);  
 
   /* write it back out to a new tmpfile (digital write) */
   if ( (status = esl_tmpfile_named(tmpfile1, &ofp))  != eslOK) esl_fatal("selex good file test %d failed: tmpfile creation",   testnumber);
@@ -896,11 +896,11 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   fclose(ofp);
 
   /* now open and read it as text mode, in known format. (We have to pass fmtd now, to deal with the possibility of a nonstandard name width) */
-  if ( (status = eslx_msafile_Open(NULL, tmpfile1, NULL, eslMSAFILE_SELEX, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: text mode open", testnumber);  
-  if ( (status = esl_msafile_selex_Read(afp, &msa2))                                    != eslOK) esl_fatal("selex good file test %d failed: msa read, text", testnumber);  
-  if (msa2->nseq != expected_nseq || msa2->alen != expected_alen)                                 esl_fatal("selex good file test %d failed: nseq/alen",      testnumber);
-  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                      esl_fatal("selex good file test %d failed: msa2 invalid",   testnumber);
-  eslx_msafile_Close(afp);
+  if ( (status = esl_msafile_Open(NULL, tmpfile1, NULL, eslMSAFILE_SELEX, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: text mode open", testnumber);  
+  if ( (status = esl_msafile_selex_Read(afp, &msa2))                                   != eslOK) esl_fatal("selex good file test %d failed: msa read, text", testnumber);  
+  if (msa2->nseq != expected_nseq || msa2->alen != expected_alen)                                esl_fatal("selex good file test %d failed: nseq/alen",      testnumber);
+  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                     esl_fatal("selex good file test %d failed: msa2 invalid",   testnumber);
+  esl_msafile_Close(afp);
   
   /* write it back out to a new tmpfile (text write) */
   if ( (status = esl_tmpfile_named(tmpfile2, &ofp))   != eslOK) esl_fatal("selex good file test %d failed: tmpfile creation", testnumber);
@@ -909,10 +909,10 @@ utest_goodfile(char *filename, int testnumber, int expected_alphatype, int expec
   esl_msa_Destroy(msa2);
 
   /* open and read it in digital mode */
-  if ( (status = eslx_msafile_Open(&abc, tmpfile1, NULL, eslMSAFILE_SELEX, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: 2nd digital mode open", testnumber);  
-  if ( (status = esl_msafile_selex_Read(afp, &msa2))                                    != eslOK) esl_fatal("selex good file test %d failed: 2nd digital msa read",  testnumber);  
-  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                      esl_fatal("selex good file test %d failed: msa2 invalid",          testnumber);
-  eslx_msafile_Close(afp);
+  if ( (status = esl_msafile_Open(&abc, tmpfile1, NULL, eslMSAFILE_SELEX, NULL, &afp)) != eslOK) esl_fatal("selex good file test %d failed: 2nd digital mode open", testnumber);  
+  if ( (status = esl_msafile_selex_Read(afp, &msa2))                                   != eslOK) esl_fatal("selex good file test %d failed: 2nd digital msa read",  testnumber);  
+  if (esl_msa_Validate(msa2, NULL) != eslOK)                                                     esl_fatal("selex good file test %d failed: msa2 invalid",          testnumber);
+  esl_msafile_Close(afp);
 
   /* this msa <msa2> should be identical to <msa1> */
   if (esl_msa_Compare(msa1, msa2) != eslOK) esl_fatal("selex good file test %d failed: msa compare", testnumber);  
@@ -958,25 +958,25 @@ read_test_msas_digital(char *slxfile, char *stkfile)
 {
   char msg[]         = "SELEX msa digital read unit test failed";
   ESL_ALPHABET *abc  = NULL;
-  ESLX_MSAFILE *afp1 = NULL;
-  ESLX_MSAFILE *afp2 = NULL;
+  ESL_MSAFILE *afp1 = NULL;
+  ESL_MSAFILE *afp2 = NULL;
   ESL_MSA      *msa1, *msa2, *msa3, *msa4;
   FILE         *slxfp, *stkfp;
   char          slxfile2[32]  = "esltmpslx2XXXXXX";
   char          stkfile2[32]  = "esltmpstk2XXXXXX";
 
-  if ( eslx_msafile_Open(&abc, slxfile, NULL, eslMSAFILE_SELEX,     NULL, &afp1)   != eslOK)  esl_fatal(msg);
-  if ( !abc || abc->type != eslAMINO)                                                         esl_fatal(msg);
-  if ( eslx_msafile_Open(&abc, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)   != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_selex_Read    (afp1, &msa1)                                     != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                     != eslOK)  esl_fatal(msg);
-  if ( esl_msa_Compare(msa1, msa2)                                                 != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, slxfile, NULL, eslMSAFILE_SELEX,     NULL, &afp1)   != eslOK)  esl_fatal(msg);
+  if ( !abc || abc->type != eslAMINO)                                                        esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)   != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa1)                                    != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                    != eslOK)  esl_fatal(msg);
+  if ( esl_msa_Compare(msa1, msa2)                                                != eslOK)  esl_fatal(msg);
   
-  if ( esl_msafile_selex_Read    (afp1, &msa3)                               != eslEOF) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa3)                               != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa3)  != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa3)  != eslEOF) esl_fatal(msg);
 
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   /* Now write stk to selex file, and vice versa; then retest */
   if ( esl_tmpfile_named(slxfile2, &slxfp)                                  != eslOK) esl_fatal(msg);
@@ -985,16 +985,16 @@ read_test_msas_digital(char *slxfile, char *stkfile)
   if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)       != eslOK) esl_fatal(msg);
   fclose(slxfp);
   fclose(stkfp);
-  if ( eslx_msafile_Open(&abc, slxfile2, NULL, eslMSAFILE_SELEX,     NULL, &afp1) != eslOK) esl_fatal(msg);
-  if ( eslx_msafile_Open(&abc, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
-  if ( esl_msafile_selex_Read    (afp1, &msa3)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                    != eslOK) esl_fatal(msg);
-  if ( esl_msa_Compare(msa3, msa4)                                                != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, slxfile2, NULL, eslMSAFILE_SELEX,     NULL, &afp1) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(&abc, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2) != eslOK) esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa3)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                   != eslOK) esl_fatal(msg);
+  if ( esl_msa_Compare(msa3, msa4)                                               != eslOK) esl_fatal(msg);
 
   remove(slxfile2);
   remove(stkfile2);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   esl_msa_Destroy(msa1);
   esl_msa_Destroy(msa2);
@@ -1007,40 +1007,40 @@ static void
 read_test_msas_text(char *slxfile, char *stkfile)
 {
   char msg[]         = "SELEX msa text-mode read unit test failed";
-  ESLX_MSAFILE *afp1 = NULL;
-  ESLX_MSAFILE *afp2 = NULL;
+  ESL_MSAFILE *afp1 = NULL;
+  ESL_MSAFILE *afp2 = NULL;
   ESL_MSA      *msa1, *msa2, *msa3, *msa4;
   FILE         *slxfp, *stkfp;
   char          slxfile2[32]  = "esltmpslx2XXXXXX";
   char          stkfile2[32]  = "esltmpstk2XXXXXX";
 
   /*                     vvvv-- everything's the same as the digital utest except these NULLs  */
-  if ( eslx_msafile_Open(NULL, slxfile, NULL, eslMSAFILE_SELEX,     NULL, &afp1)   != eslOK)  esl_fatal(msg);
-  if ( eslx_msafile_Open(NULL, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)   != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_selex_Read    (afp1, &msa1)                                     != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                     != eslOK)  esl_fatal(msg);
-  if ( esl_msa_Compare(msa1, msa2)                                                 != eslOK)  esl_fatal(msg);
-  if ( esl_msafile_selex_Read    (afp1, &msa3)                                     != eslEOF) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa3)                                     != eslEOF) esl_fatal(msg);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  if ( esl_msafile_Open(NULL, slxfile, NULL, eslMSAFILE_SELEX,     NULL, &afp1)   != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, stkfile, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)   != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa1)                                    != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa2)                                    != eslOK)  esl_fatal(msg);
+  if ( esl_msa_Compare(msa1, msa2)                                                != eslOK)  esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa3)                                    != eslEOF) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa3)                                    != eslEOF) esl_fatal(msg);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
-  if ( esl_tmpfile_named(slxfile2, &slxfp)                                   != eslOK) esl_fatal(msg);
-  if ( esl_tmpfile_named(stkfile2, &stkfp)                                   != eslOK) esl_fatal(msg);
-  if ( esl_msafile_selex_Write    (slxfp, msa2)                              != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)        != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(slxfile2, &slxfp)                               != eslOK) esl_fatal(msg);
+  if ( esl_tmpfile_named(stkfile2, &stkfp)                               != eslOK) esl_fatal(msg);
+  if ( esl_msafile_selex_Write    (slxfp, msa2)                          != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Write(stkfp, msa1, eslMSAFILE_STOCKHOLM)    != eslOK) esl_fatal(msg);
   fclose(slxfp);
   fclose(stkfp);
-  if ( eslx_msafile_Open(NULL, slxfile2, NULL, eslMSAFILE_SELEX,     NULL, &afp1)  != eslOK) esl_fatal(msg);
-  if ( eslx_msafile_Open(NULL, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)  != eslOK) esl_fatal(msg);
-  if ( esl_msafile_selex_Read    (afp1, &msa3)                                     != eslOK) esl_fatal(msg);
-  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                     != eslOK) esl_fatal(msg);
-  if ( esl_msa_Compare(msa3, msa4)                                                 != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, slxfile2, NULL, eslMSAFILE_SELEX,     NULL, &afp1)  != eslOK) esl_fatal(msg);
+  if ( esl_msafile_Open(NULL, stkfile2, NULL, eslMSAFILE_STOCKHOLM, NULL, &afp2)  != eslOK) esl_fatal(msg);
+  if ( esl_msafile_selex_Read    (afp1, &msa3)                                    != eslOK) esl_fatal(msg);
+  if ( esl_msafile_stockholm_Read(afp2, &msa4)                                    != eslOK) esl_fatal(msg);
+  if ( esl_msa_Compare(msa3, msa4)                                                != eslOK) esl_fatal(msg);
 
   remove(slxfile2);
   remove(stkfile2);
-  eslx_msafile_Close(afp2);
-  eslx_msafile_Close(afp1);
+  esl_msafile_Close(afp2);
+  esl_msafile_Close(afp1);
 
   esl_msa_Destroy(msa1);
   esl_msa_Destroy(msa2);
@@ -1170,7 +1170,7 @@ main(int argc, char **argv)
   char               *filename    = esl_opt_GetArg(go, 1);
   int                 infmt       = eslMSAFILE_UNKNOWN;
   ESL_ALPHABET       *abc         = NULL;
-  ESLX_MSAFILE       *afp         = NULL;
+  ESL_MSAFILE        *afp         = NULL;
   ESL_MSA            *msa         = NULL;
   int                 status;
 
@@ -1183,12 +1183,12 @@ main(int argc, char **argv)
   /* Text mode: pass NULL for alphabet.
    * Digital mode: pass ptr to expected ESL_ALPHABET; and if abc=NULL, alphabet is guessed 
    */
-  if   (esl_opt_GetBoolean(go, "-t"))  status = eslx_msafile_Open(NULL, filename, NULL, infmt, NULL, &afp);
-  else                                 status = eslx_msafile_Open(&abc, filename, NULL, infmt, NULL, &afp);
-  if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+  if   (esl_opt_GetBoolean(go, "-t"))  status = esl_msafile_Open(NULL, filename, NULL, infmt, NULL, &afp);
+  else                                 status = esl_msafile_Open(&abc, filename, NULL, infmt, NULL, &afp);
+  if (status != eslOK) esl_msafile_OpenFailure(afp, status);
 
   if ( (status = esl_msafile_selex_Read(afp, &msa)) != eslOK)
-    eslx_msafile_ReadFailure(afp, status);
+    esl_msafile_ReadFailure(afp, status);
 
   printf("alphabet:       %s\n", (abc ? esl_abc_DecodeType(abc->type) : "none (text mode)"));
   printf("# of seqs:      %d\n", msa->nseq);
@@ -1199,7 +1199,7 @@ main(int argc, char **argv)
     esl_msafile_selex_Write(stdout, msa);
 
   esl_msa_Destroy(msa);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   if (abc) esl_alphabet_Destroy(abc);
   esl_getopts_Destroy(go);
   exit(0);
@@ -1226,19 +1226,19 @@ main(int argc, char **argv)
 {
   char         *filename = argv[1];
   int           fmt      = eslMSAFILE_SELEX;
-  ESLX_MSAFILE *afp      = NULL;
+  ESL_MSAFILE  *afp      = NULL;
   ESL_MSA      *msa      = NULL;
   int           status;
 
-  if ( (status = eslx_msafile_Open(NULL, filename, NULL, fmt, NULL, &afp)) != eslOK)  eslx_msafile_OpenFailure(afp, status);
-  if ( (status = esl_msafile_selex_Read(afp, &msa))                        != eslOK)  eslx_msafile_ReadFailure(afp, status);
+  if ( (status = esl_msafile_Open(NULL, filename, NULL, fmt, NULL, &afp)) != eslOK)  esl_msafile_OpenFailure(afp, status);
+  if ( (status = esl_msafile_selex_Read(afp, &msa))                       != eslOK)  esl_msafile_ReadFailure(afp, status);
 
   printf("%6d seqs, %5d columns\n",  msa->nseq, (int) msa->alen);
 
   esl_msafile_selex_Write(stdout, msa);
 
   esl_msa_Destroy(msa);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   exit(0);
 }
 /*::cexcerpt::msafile_selex_example2::end::*/
@@ -1251,13 +1251,10 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- *
- * SVN $Id: esl_msafile_selex.c 756 2012-03-23 13:24:47Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msafile_selex.c $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
