@@ -306,7 +306,7 @@ esl_msashuffle_XQRNA(ESL_RANDOMNESS *r, ESL_ALPHABET *abc, ESL_DSQ *x, ESL_DSQ *
     pos              = esl_rnd_Roll(r, nxy);
     xsym             = xs[xycol[pos]];   ysym             = ys[xycol[pos]];    c            = xycol[pos];   
     xs[xycol[pos]]   = xs[xycol[nxy-1]]; ys[xycol[pos]]   = ys[xycol[nxy-1]];  xycol[pos]   = xycol[nxy-1];
-    xs[xycol[nxy-1]] = xsym;             ys[xycol[nxy-1]] = ysym;              xycol[pos]   = xycol[nxy-1];
+    xs[xycol[nxy-1]] = xsym;             ys[xycol[nxy-1]] = ysym;              xycol[pos]   = c;
   }
   for (; nx > 1; nx--) {
     pos            = esl_rnd_Roll(r, nx); 
@@ -404,7 +404,7 @@ esl_msashuffle_CQRNA(ESL_RANDOMNESS *r, ESL_ALPHABET *abc, char *x, char *y, cha
     pos              = esl_rnd_Roll(r, nxy);
     xsym             = xs[xycol[pos]];   ysym             = ys[xycol[pos]];    c            = xycol[pos];   
     xs[xycol[pos]]   = xs[xycol[nxy-1]]; ys[xycol[pos]]   = ys[xycol[nxy-1]];  xycol[pos]   = xycol[nxy-1];
-    xs[xycol[nxy-1]] = xsym;             ys[xycol[nxy-1]] = ysym;              xycol[pos]   = xycol[nxy-1];
+    xs[xycol[nxy-1]] = xsym;             ys[xycol[nxy-1]] = ysym;              xycol[pos]   = c;
   }
   for (; nx > 1; nx--) {
     pos            = esl_rnd_Roll(r, nx); 
@@ -463,13 +463,13 @@ main(int argc, char **argv)
   char           *msafile   = esl_opt_GetArg(go, 1);
   int             fmt       = eslMSAFILE_UNKNOWN;
   ESL_ALPHABET   *abc       = NULL;
-  ESLX_MSAFILE   *afp       = NULL;
+  ESL_MSAFILE    *afp       = NULL;
   ESL_MSA        *msa       = NULL;
   int             textmode  = esl_opt_GetBoolean(go, "--text");
   int             nali      = 0;
   int             status;
 
-  /* If you know the alphabet you want, create it - you'll pass it to eslx_msafile_Open() */
+  /* If you know the alphabet you want, create it - you'll pass it to esl_msafile_Open() */
   if      (esl_opt_GetBoolean(go, "--rna"))   abc = esl_alphabet_Create(eslRNA);
   else if (esl_opt_GetBoolean(go, "--dna"))   abc = esl_alphabet_Create(eslDNA);
   else if (esl_opt_GetBoolean(go, "--amino")) abc = esl_alphabet_Create(eslAMINO); 
@@ -478,16 +478,16 @@ main(int argc, char **argv)
    *   To let the Open() function autoguess the format, you pass <infmt=eslMSAFILE_UNKNOWN>. 
    *   To let it autoguess the alphabet, you set <abc=NULL> and pass <&abc>.
    *   To open in text mode instead of digital, you pass <NULL> for the alphabet argument.
-   * eslx_msafile_OpenFailure() is a convenience, printing various diagnostics of any
+   * esl_msafile_OpenFailure() is a convenience, printing various diagnostics of any
    * open failure to <stderr>. You can of course handle your own diagnostics instead.
    */
-  if (textmode) status = eslx_msafile_Open(NULL, msafile, NULL, fmt, NULL, &afp);
-  else          status = eslx_msafile_Open(&abc, msafile, NULL, fmt, NULL, &afp);
-  if (status != eslOK)   eslx_msafile_OpenFailure(afp, status);
+  if (textmode) status = esl_msafile_Open(NULL, msafile, NULL, fmt, NULL, &afp);
+  else          status = esl_msafile_Open(&abc, msafile, NULL, fmt, NULL, &afp);
+  if (status != eslOK)   esl_msafile_OpenFailure(afp, status);
   
   fmt = afp->format;
 
-  while ((status = eslx_msafile_Read(afp, &msa)) == eslOK)
+  while ((status = esl_msafile_Read(afp, &msa)) == eslOK)
     {	
       /* if digital MSA: msa->ax[idx=0..nseq-1][acol=1..alen] is the alignment data; 
        * if text MSA:  msa->aseq[idx=0..nseq-1][acol=0..alen-1] */
@@ -496,13 +496,13 @@ main(int argc, char **argv)
       /* permute it */
       esl_msashuffle_PermuteSequenceOrder(rng, msa);
 
-      eslx_msafile_Write(stdout, msa, fmt);
+      esl_msafile_Write(stdout, msa, fmt);
       esl_msa_Destroy(msa);
     }
-  if (nali == 0 || status != eslEOF) eslx_msafile_ReadFailure(afp, status); /* a convenience, like eslx_msafile_OpenFailure() */
+  if (nali == 0 || status != eslEOF) esl_msafile_ReadFailure(afp, status); /* a convenience, like esl_msafile_OpenFailure() */
 
   esl_alphabet_Destroy(abc);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   esl_randomness_Destroy(rng);
   esl_getopts_Destroy(go);
   exit(0);
@@ -511,13 +511,10 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- * 
- * SVN $Id: esl_msashuffle.c 761 2012-05-10 20:09:44Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_msashuffle.c $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
