@@ -1,8 +1,8 @@
 /* Statistical routines for normal distributions
  * 
  * SRE, Tue Nov 21 14:29:02 2006 [Janelia]
- * SVN $Id: esl_normal.h 664 2011-02-27 17:08:36Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_normal.h $
+ * SVN $Id$
+ * SVN $URL$
  */
 #ifndef eslNORMAL_INCLUDED
 #define eslNORMAL_INCLUDED
@@ -12,13 +12,17 @@ extern double esl_normal_logpdf(double x, double mu, double sigma);
 extern double esl_normal_cdf   (double x, double mu, double sigma);
 extern double esl_normal_surv  (double x, double mu, double sigma);
 
+extern double esl_normal_generic_pdf (double x, void *params);
+extern double esl_normal_generic_cdf (double x, void *params);
+extern double esl_normal_generic_surv(double x, void *params);
+
 #endif /*eslNORMAL_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
