@@ -14,10 +14,18 @@
  *  
  * See http://csrc.nist.gov/rng/ for the NIST random number
  * generation test suite.
+ *
+ * It'd be nice if we had a debugging/unit testing mode in which
+ * esl_random() deliberately generated extreme values, such as 0 for
+ * example. Routines that use esl_random() can be sensitive to whether
+ * the interval 0,1 is open or closed. We should be able to test for
+ * problems with interval endpoints without taking enormous numbers of
+ * samples.
  */
 #include "esl_config.h"
 
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -102,19 +110,37 @@ esl_randomness_Create(uint32_t seed)
  * Synopsis:  Create the alternative fast generator.
  *
  * Purpose:   Same as <esl_randomness_Create()>, except that a simple
- *            linear congruential generator will be used.
+ *            linear congruential generator (LCG) will be used.
+ *            
+ *            This is a low quality generator. Successive samples from
+ *            an LCG are correlated, and it has a relatively short
+ *            period. IT SHOULD NOT BE USED FOR SERIOUS
+ *            SIMULATIONS. Rather, it's a quick and dirty RNG where
+ *            you're sure that speed is more important than the
+ *            quality of your random numbers. For a high quality RNG,
+ *            use <esl_randomness_Create()> instead.
  *            
  *            This is a $(a=69069, c=1)$ LCG, with a period of
- *            $2^{32}$. Because of the relatively short period, this
- *            generator should not be used for serious simulations
- *            involving large samples.
+ *            $2^{32}$. 
+ *            
+ *            It is about 20x faster to initialize the generator, and
+ *            about 25\% faster to sample each number, compared to the
+ *            default Mersenne Twister. (In most cases, this speed
+ *            differential is not worth the degradation in
+ *            quality. Since we made MT our default generator, the
+ *            speed advantage of the LCG essentially disappeared, so
+ *            in some sense this is legacy code.)
  *
- *            The properties of this generator are not as good as the
- *            default Mersenne Twister, but it is faster, especially
- *            if you only need a small number of samples from the
- *            generator; it is about 20x faster to initialize the
- *            generator, and about 25\% faster to sample a number, 
- *            compared to the default.
+ *            Here's an example of how serial correlation arises in an
+ *            LCG, and how it can lead to serious (and difficult to
+ *            diagnose) failure in a Monte Carlo simulation. Recall
+ *            that an LCG calculates $x_{i+1} = ax_i + c$. Suppose
+ *            $x_i$ is small: in the range 0..6000, say, as a specific
+ *            example. Now $x_{i+1}$ cannot be larger than 4.1e8, for
+ *            an LCG with $a=69069$,$c=1$. So if you take a sample and
+ *            test whether it is $< 1e-6$ (say), the next sample will
+ *            be in a range of about 0..0.1, rather than being uniform
+ *            on 0..1.
  *
  * Args:      seed $>= 0$.
  *
@@ -246,24 +272,13 @@ esl_randomness_Destroy(ESL_RANDOMNESS *r)
 /* Function: esl_random()  
  * Synopsis: Generate a uniform random deviate on [0,1)
  *
- * Purpose:  Returns a uniform deviate x, $0.0 <= x < 1.0$, given
+ * Purpose:  Returns a uniform deviate x, $0.0 \leq x < 1.0$, given
  *           RNG <r>.
  *           
- *           Uses the original Mersenne Twister algorithm, MT19937
- *           [Matsumoto98]. This generator has a period of $2^{19937} -
- *           1$. It generates uniformly distributed variates on the
- *           interval $0..2^{32}-1$. 
- *           
  *           If you cast the return value to float, the [0,1) interval
  *           guarantee is lost because values close to 1 will round to
  *           1.0.
  *           
- * Notes:    Easel previously used a reimplementation of ran2() from
- *           Numerical Recipes in C, which uses L'Ecuyer's algorithm
- *           for combining output of two linear congruential
- *           generators, plus a Bays-Durham shuffle \citep{Press93}.
- *           MT is about 10x faster.
- *
  * Returns:  a uniformly distribute random deviate on interval
  *           $0.0 \leq x < 1.0$.
  */
@@ -275,6 +290,20 @@ esl_random(ESL_RANDOMNESS *r)
 }
 
 
+/* Function:  esl_random_uint32()
+ * Synopsis:  Generate a uniform random deviate on 0..2^32-1
+ * Incept:    SRE, Wed Jan 13 10:59:26 2016
+ *
+ * Purpose:   Returns a uniform deviate x, a 32-bit unsigned
+ *            integer $0 \leq x < 2^{32}$, given RNG <r>.
+ */
+uint32_t 
+esl_random_uint32(ESL_RANDOMNESS *r)
+{
+  return (r->type == eslRND_MERSENNE) ? mersenne_twister(r) : knuth(r);
+}
+
+
 static uint32_t 
 knuth(ESL_RANDOMNESS *r)
 {
@@ -356,21 +385,20 @@ mersenne_fill_table(ESL_RANDOMNESS *r)
 
 /* choose_arbitrary_seed()
  * Return a 'quasirandom' seed > 0.
- * This could be a *lot* better than it is now; see RFC1750
- * for a discussion of securely seeding RNGs.
+ * This should be ok, but could be better.
+ * See RFC1750 for a discussion of securely seeding RNGs.
  */
 static uint32_t
 choose_arbitrary_seed(void)
 {
-  uint32_t a = (uint32_t) time ((time_t *) NULL);
-  uint32_t b = 87654321;	/* anything nonzero */
-  uint32_t c = 12345678;	/* anything nonzero. jenkins' mix3 needs 3 numbers; add an arbitrary one */
+  uint32_t a = (uint32_t) time ((time_t *) NULL);  
+  uint32_t b = 87654321;	                    // we'll use getpid() below, if we can
+  uint32_t c = (uint32_t) clock();                  // clock() gives time since process invocation, in msec at least, if not usec
   uint32_t seed;
 #ifdef HAVE_GETPID
-  b  = (uint32_t) getpid();	  /* preferable b choice, if we have POSIX getpid(); else both b,c are arbitrary */
+  b  = (uint32_t) getpid();	                    // preferable b choice, if we have POSIX getpid()
 #endif
-  seed = jenkins_mix3(a,b,c);	  /* try to decorrelate closely spaced choices of pid/time */
-
+  seed = jenkins_mix3(a,b,c);	                    // try to decorrelate closely spaced choices of pid/times
   return (seed == 0) ? 42 : seed; /* 42 is entirely arbitrary, just to avoid seed==0. */
 }
 
@@ -466,6 +494,13 @@ esl_rnd_UniformPositive(ESL_RANDOMNESS *r)
  *            and James Lovato (M.D. Anderson Cancer Center, Texas
  *            USA) using the method described in
  *            \citep{AhrensDieter73}.
+ *            
+ *            Original algorithm said to use uniform deviates on [0,1)
+ *            interval (i.e. <esl_random()>), but this appears to be
+ *            wrong.  Use uniform deviates on (0,1) interval instead
+ *            (i.e., <esl_rnd_UniformPositive()>). RANLIB, GNU Octave
+ *            have made this alteration, possibly inadvertently.
+ *            [xref cryptogenomicon post, 13 Oct 2014].
  * 
  * Method:    Impenetrability of the code is to be blamed on 
  *            FORTRAN/f2c lineage.
@@ -511,7 +546,7 @@ esl_rnd_Gaussian(ESL_RANDOMNESS *r, double mean, double stddev)
     8.781922E-2,9.930398E-2,0.11556,0.1404344,0.1836142,0.2790016,0.7010474
   };
 
-  u = esl_random(r);
+  u = esl_rnd_UniformPositive(r);
   s = 0.0;
   if(u > 0.5) s = 1.0;
   u += (u-s);
@@ -539,18 +574,18 @@ S60:
   /*
    * CENTER CONTINUED
    */
-  u = esl_random(r);
+  u = esl_rnd_UniformPositive(r);
   w = u*(a[i]-aa);
   tt = (0.5*w+aa)*w;
   goto S80;
 S70:
   tt = u;
-  ustar = esl_random(r);
+  ustar = esl_rnd_UniformPositive(r);
 S80:
   if(ustar > tt) goto S50;
-  u = esl_random(r);
+  u = esl_rnd_UniformPositive(r);
   if(ustar >= u) goto S70;
-  ustar = esl_random(r);
+  ustar = esl_rnd_UniformPositive(r);
   goto S40;
 S100:
   /*
@@ -562,6 +597,7 @@ S100:
 S110:
   aa += d[i-1];
   i += 1;
+  ESL_DASSERT1(( i <= 31 ));
 S120:
   u += u;
   if(u < 1.0) goto S110;
@@ -573,11 +609,11 @@ S140:
 S150:
   tt = u;
 S160:
-  ustar = esl_random(r);
+  ustar = esl_rnd_UniformPositive(r);
   if(ustar > tt) goto S50;
-  u = esl_random(r);
+  u = esl_rnd_UniformPositive(r);
   if(ustar >= u) goto S150;
-  u = esl_random(r);
+  u = esl_rnd_UniformPositive(r);
   goto S140;
 }
 
@@ -650,14 +686,14 @@ gamma_fraction(ESL_RANDOMNESS *r, double a)	/* for fractional a, 0 < a < 1 */
  *
  * Args:     r      - random number generation seed
  *           a      - order of the gamma function; a > 0
- *
- * Throws:   <eslEINVAL> for $a <= 0$.
  */
 double
 esl_rnd_Gamma(ESL_RANDOMNESS *r, double a)
 {
   double aint;
 
+  ESL_DASSERT1(( a > 0. ));
+
   aint = floor(a);
   if (a == aint && a < 12.) 
     return gamma_integer(r, (unsigned int) a);
@@ -667,7 +703,73 @@ esl_rnd_Gamma(ESL_RANDOMNESS *r, double a)
     return gamma_fraction(r, a);
   else 
     return gamma_integer(r, aint) + gamma_fraction(r, a-aint);
-  return eslOK;
+}
+
+
+/* Function:  esl_rnd_Dirichlet()
+ * Synopsis:  Sample a Dirichlet-distributed random probability vector 
+ * Incept:    SRE, Wed Feb 17 12:20:53 2016 [H1/76]
+ *
+ * Purpose:   Using generator <rng>, sample a Dirichlet-distributed
+ *            probabilty vector <p> of <K> elements, using Dirichlet
+ *            parameters <alpha> (also of <K> elements). 
+ *
+ *            Caller provides the allocated space for <p>.
+ * 
+ *            <alpha> is optional. If it isn't provided (i.e. is
+ *            <NULL>), sample <p> uniformly. (That is, use <alpha[i] =
+ *            1.> for all i=0..K-1.)
+ *
+ *            This routine is redundant with <esl_dirichlet_DSample()>
+ *            and <esl_dirichlet_DSampleUniform()> in the dirichlet
+ *            module. Provided here because there's cases where we
+ *            just want to sample a probability vector without
+ *            introducing a dependency on all the stats/dirichlet code
+ *            in Easel.
+ *
+ * Args:      rng   : random number generator
+ *            alpha : OPTIONAL: Dirichlet parameters 0..K-1, or NULL to use alpha[i]=1 for all i
+ *            K     : number of elements in alpha, p
+ *            p     : RESULT: sampled probability vector
+ *
+ * Returns:   (void)
+ */
+void
+esl_rnd_Dirichlet(ESL_RANDOMNESS *rng, const double *alpha, int K, double *p)
+{
+  int    x;
+  double norm = 0.;
+
+  for (x = 0; x < K; x++) 
+    {
+      p[x] = esl_rnd_Gamma(rng, (alpha ? alpha[x] : 1.0));
+      norm += p[x];
+    }
+  for (x = 0; x < K; x++)
+    p[x] /= norm;
+}
+
+
+/* Function:  esl_rnd_mem()
+ * Synopsis:  Overwrite a buffer with random garbage.
+ * Incept:    SRE, Fri Feb 19 08:53:07 2016
+ *
+ * Purpose:   Write <n> bytes of random garbage into buffer
+ *            <buf>, by uniform sampling of values 0..255,
+ *            using generator <rng>.
+ *
+ *            Used in unit tests that are reusing memory, and that
+ *            want to make sure that there's no favorable side effects
+ *            from that reuse.
+ */
+void
+esl_rnd_mem(ESL_RANDOMNESS *rng, void *buf, int n)
+{
+  unsigned char *p = (unsigned char *) buf;
+  int            i;
+
+  for (i = 0; i < n; i++)
+    p[i] = (unsigned char) esl_rnd_Roll(rng, 256);
 }
 
 
@@ -730,7 +832,7 @@ esl_rnd_FChoose(ESL_RANDOMNESS *r, const float *p, int N)
 
   for (i = 0; i < N; i++)
     {
-      sum += p[i];
+      sum += (double) p[i];
       if (roll < (sum / norm) ) return i; 
     }
   esl_fatal("unreached code was reached. universe collapses.");
@@ -800,7 +902,7 @@ esl_rnd_FChooseCDF(ESL_RANDOMNESS *r, const float *cdf, int N)
   ESL_DASSERT1((cdf[N-1] > 0.99 && cdf[N-1] < 1.01));
 
   for (i = 0; i < N; i++) 
-    if (roll < cdf[i]/cdf[N-1]) return i; 
+    if (roll < (double) cdf[i] / (double) cdf[N-1]) return i;  // yes, the casts are NECESSARY. Without them, you get a heisenbug on icc/linux.
   esl_fatal("unreached code is reached. universe goes foom");
   return 0; /*notreached*/
 }
@@ -825,6 +927,7 @@ esl_rnd_FChooseCDF(ESL_RANDOMNESS *r, const float *cdf, int N)
 
  */
 #include "easel.h"
+#include "esl_composition.h"
 #include "esl_getopts.h"
 #include "esl_random.h"
 #include "esl_stopwatch.h"
@@ -1009,8 +1112,6 @@ utest_choose(ESL_RANDOMNESS *r, int n, int nbins, int be_verbose)
   free(ct);
   return;
 }
- 
-
 #endif /*eslRANDOM_TESTDRIVE*/
 /*-------------------- end, unit tests --------------------------*/
 
@@ -1060,6 +1161,10 @@ main(int argc, char **argv)
   int             n          = esl_opt_GetInteger(go, "-n");
   int             be_verbose = esl_opt_GetBoolean(go, "-v");
 
+  fprintf(stderr, "## %s\n", argv[0]);
+  fprintf(stderr, "#  rng seed 1 (slow) = %" PRIu32 "\n", esl_randomness_GetSeed(r1));
+  fprintf(stderr, "#  rng seed 2 (fast) = %" PRIu32 "\n", esl_randomness_GetSeed(r2));
+
   utest_random(r1, n, nbins, be_verbose);
   utest_choose(r1, n, nbins, be_verbose);
   utest_random(r2, n, nbins, be_verbose);
@@ -1068,6 +1173,8 @@ main(int argc, char **argv)
   if (mtbitfile) save_bitfile(mtbitfile, r1, n);
   if (kbitfile)  save_bitfile(kbitfile,  r2, n);
 
+  fprintf(stderr, "#  status = ok\n");
+
   esl_randomness_Destroy(r1);
   esl_randomness_Destroy(r2);
   esl_getopts_Destroy(go);
@@ -1111,31 +1218,51 @@ save_bitfile(char *bitfile, ESL_RANDOMNESS *r, int n)
  *****************************************************************/
 #ifdef eslRANDOM_EXAMPLE
 /*::cexcerpt::random_example::begin::*/
-/* compile: gcc -g -Wall -I. -o esl_random_example -DeslRANDOM_EXAMPLE esl_random.c easel.c -lm
+/* compile: cc -I. -o esl_random_example -DeslRANDOM_EXAMPLE esl_random.c esl_getopts.c easel.c -lm
  * run:     ./random_example 42
  */
 #include <stdio.h>
 #include "easel.h"
+#include "esl_getopts.h"
 #include "esl_random.h"
 
+/* In Easel and HMMER, the option for setting the seed is typically -s, sometimes --seed.
+ * Default is usually 0 because we want a "random" seed. Less commonly, "42" for a fixed seed;
+ * rarely, a different fixed seed.  
+ * 
+ * Generally you want to use esl_randomness_Create(), to get a Mersenne Twister. The "fast"
+ * RNG you get from esl_randomness_CreateFast() isn't all that much faster (~25% per sample)
+ * but has much worse quality in its randomness.
+ */
+static ESL_OPTIONS options[] = {
+  /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
+  { "-h",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",  0 },
+  { "-i",        eslARG_NONE,   FALSE,  NULL, NULL,  NULL,  NULL, NULL, "sample uint32's instead of doubles",    0 },
+  { "-n",        eslARG_INT,     "20",  NULL, NULL,  NULL,  NULL, NULL, "number of random samples to show",      0 },
+  { "-s",        eslARG_INT,      "0",  NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",         0 },
+  {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+};
+static char usage[]  = "[-options]";
+static char banner[] = "example of using random module";
+
 int 
 main(int argc, char **argv)
 {
-  long            seed = atoi(argv[1]);
-  ESL_RANDOMNESS *r    = esl_randomness_Create(seed); 
-  int             n    = 1076;
+  ESL_GETOPTS    *go        = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *rng       = esl_randomness_Create( esl_opt_GetInteger(go, "-s") );   
+  int             do_uint32 = esl_opt_GetBoolean(go, "-i");
+  int             n         = esl_opt_GetInteger(go, "-n");
 
-  printf("RNG seed: %" PRIu32 "\n", esl_randomness_GetSeed(r));
-  printf("A sequence of %d pseudorandom numbers:\n", n);
-  while (n--)  printf("%f\n", esl_random(r));
+  printf("RNG seed: %" PRIu32 "\n", esl_randomness_GetSeed(rng));
+  printf("\nA sequence of %d pseudorandom numbers:\n", n);
+  if (do_uint32)  while (n--)  printf("%" PRIu32 "\n", esl_random_uint32(rng));
+  else            while (n--)  printf("%f\n",          esl_random(rng));
   
-  esl_randomness_Dump(stdout, r);
-  printf("%f\n", esl_random(r));
+  printf("\nInternal dump of RNG state:\n");
+  esl_randomness_Dump(stdout, rng);
 
-  esl_randomness_Dump(stdout, r);
-  printf("%f\n", esl_random(r));
-
-  esl_randomness_Destroy(r);
+  esl_randomness_Destroy(rng);
+  esl_getopts_Destroy(go);
   return 0;
 }
 /*::cexcerpt::random_example::end::*/
@@ -1144,15 +1271,15 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_random.c 738 2011-12-15 17:03:02Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_random.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 
