@@ -1,17 +1,18 @@
 /* Generating, shuffling, and randomizing sequences.
  * 
  * Contents:
- *   1. Generating iid sequences.
- *   2. Shuffling sequences. 
- *   3. Randomizing sequences.
- *   4. Generating iid sequences (digital mode).
- *   5. Shuffling sequences (digital mode).
- *   6. Randomizing sequences (digital mode).
- *   7. Statistics drivers.
- *   8. Unit tests.
- *   9. Test driver.
- *  10. Example.
- *  11. Copyright and license information 
+ *   1. Generating simple random character strings.
+ *   2. Generating iid sequences.
+ *   3. Shuffling sequences. 
+ *   4. Randomizing sequences.
+ *   5. Generating iid sequences (digital mode).
+ *   6. Shuffling sequences (digital mode).
+ *   7. Randomizing sequences (digital mode).
+ *   8. Statistics drivers.
+ *   9. Unit tests.
+ *  10. Test driver.
+ *  11. Example.
+ *  12. Copyright and license information 
  */
 #include "esl_config.h"
 
@@ -22,11 +23,127 @@
 #include <time.h>
 
 #include "easel.h"
+#include "esl_alphabet.h"
 #include "esl_random.h"
 #include "esl_randomseq.h"
-#ifdef eslAUGMENT_ALPHABET 
-#include "esl_alphabet.h"
-#endif
+
+
+/*****************************************************************
+ * 1. Generating simple random character strings.
+ *****************************************************************/
+
+/* Function:  esl_rsq_Sample()
+ * Synopsis:  Generate a random character string.
+ *
+ * Purpose:   Sample a random character string of length <L>, 
+ *            consisting of characters in the set defined by
+ *            an integer flag <allowed_chars>, using 
+ *            random number generator <rng>. 
+ * 
+ *            Return the new NUL-terminated string in <*ret_s>.  This
+ *            may either be a new allocation, or in pre-allocated
+ *            storage provided by the caller. If caller passes
+ *            <*ret_s> as <NULL>, new space is allocated, and the
+ *            caller is responsible for freeing it. That is: 
+ *              \begin{cchunk}
+ *                 char *s  = NULL; 
+ *                 esl_rsq_Sample(..., &s); 
+ *                 free(s);
+ *               \end{cchunk}
+ *
+ *            If caller passes a non-<NULL> <*ret_s>, it is assumed to
+ *            be a preallocated space of at least <L+1> characters,
+ *            and caller is (of course) responsible for freeing
+ *            it. That is: 
+ *                \begin{cchunk}
+ *                   char *s = malloc(L+1);
+ *                   esl_rsq_Sample(...,L, &s);
+ *                   free(s);
+ *                \end{cchunk}
+ *            
+ *            Allowed values of the flag <allowed_char_flag> mirror
+ *            the standard C99 character set functions in <ctype.h>:
+ *            
+ *            | <eslRSQ_SAMPLE_ALNUM>  |  isalnum()  | isalpha() or isdigit() |
+ *            | <eslRSQ_SAMPLE_ALPHA>  |  isalpha()  | islower() or isupper() |
+ *            | <eslRSQ_SAMPLE_LOWER>  |  islower()  | [a-z] |
+ *            | <eslRSQ_SAMPLE_UPPER>  |  isupper()  | [A-Z] |
+ *            | <eslRSQ_SAMPLE_DIGIT>  |  isdigit()  | [0-9] |
+ *            | <eslRSQ_SAMPLE_XDIGIT> |  isxdigit() | [0-9] or [a-f] or [A-F] |
+ *            | <eslRSQ_SAMPLE_CNTRL>  |  iscntrl()  | ASCII control characters |
+ *            | <eslRSQ_SAMPLE_GRAPH>  |  isgraph()  | any printing char except space |
+ *            | <eslRSQ_SAMPLE_SPACE>  |  isspace()  | space, and other whitespace such as tab, newline |
+ *            | <eslRSQ_SAMPLE_BLANK>  |  isblank()  | space or tab |
+ *            | <eslRSQ_SAMPLE_PRINT>  |  isprint()  | any printing char including space |
+ *            | <eslRSQ_SAMPLE_PUNCT>  |  ispunct()  | punctuation |
+ *
+ *            Note that with <eslRSQ_SAMPLE_CNTRL>, your string
+ *            may sample NUL control characters (<0>), in addition to
+ *            the string-terminating one at <(*ret_s)[L]>, so <strlen(*ret_s)>
+ *            may not equal <L> in this case. 
+ *             
+ *            These values are exclusive: you use one and only one of
+ *            them as <allowed_chars>, you can't logically OR or NOT
+ *            them together.
+ *
+ * Args:      rng           - ESL_RANDOMNESS object, the random number generator
+ *            allowed_chars - allowed character set flag: eslRSQ_SAMPLE_*
+ *            L             - length of string to sample
+ *            ret_s         - RETURN: the NUL-terminated string
+ *
+ * Returns:   <eslOK> on success; <*ret_s> is the sampled string, which was
+ *            newly allocated here, and caller becomes responsible for free'ing.
+ *
+ * Throws:    <eslEMEM> on allocation error; <eslEINVAL> if caller 
+ *            passes an invalid value of <allowed_chars>. Now <*ret_s> 
+ *            is <NULL>.
+ */
+int
+esl_rsq_Sample(ESL_RANDOMNESS *rng, int allowed_chars, int L, char **ret_s)
+{
+  char *s = *ret_s;  // if s == NULL, we will allocate here. Else, we're using caller-provided allocation
+  int   n = 0;
+  char  c[127];
+  int   x,i;
+  int   status;
+
+  /* We can't portably make assumptions about char codes (EBCDIC,
+   * ASCII...); and we don't want to write a bunch of fiddly overhead
+   * initializing static tables. So, quickly and portably build an
+   * array c[0..n-1] of characters we will sample uniformly from.
+   * RNG sampling is fairly compute-intensive anyway, so this time
+   * should disappear in the noise of that.
+   */
+  switch (allowed_chars) {
+  case eslRSQ_SAMPLE_ALNUM:  for (x = 0; x < 128; x++) if (isalnum(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_ALPHA:  for (x = 0; x < 128; x++) if (isalpha(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_LOWER:  for (x = 0; x < 128; x++) if (islower(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_UPPER:  for (x = 0; x < 128; x++) if (isupper(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_DIGIT:  for (x = 0; x < 128; x++) if (isdigit(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_XDIGIT: for (x = 0; x < 128; x++) if (isxdigit(x)) c[n++] = x; break;
+  case eslRSQ_SAMPLE_CNTRL:  for (x = 0; x < 128; x++) if (iscntrl(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_GRAPH:  for (x = 0; x < 128; x++) if (isgraph(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_SPACE:  for (x = 0; x < 128; x++) if (isspace(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_BLANK:  for (x = 0; x < 128; x++) if (isblank(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_PRINT:  for (x = 0; x < 128; x++) if (isprint(x))  c[n++] = x; break;
+  case eslRSQ_SAMPLE_PUNCT:  for (x = 0; x < 128; x++) if (ispunct(x))  c[n++] = x; break;
+  default: ESL_XEXCEPTION(eslEINVAL, "bad flag; wanted something like eslRSQ_SAMPLE_ALPHA");
+  }
+
+  if (!s) ESL_ALLOC(s, sizeof(char) * (L+1)); /* +\0 */
+
+  for (i = 0; i < L; i++)
+    s[i] = c[ esl_rnd_Roll(rng, n) ];
+  s[L] = '\0';
+  
+  *ret_s = s;   // if using caller-provided space, this is a no-op, passing back the same *ret_s we got.
+  return eslOK;
+
+ ERROR:
+  if (! *ret_s && s) free(s);  // if we allocated s here, clean up.
+  return status;
+}
+
 
 /*****************************************************************
  *# 1. Generating iid sequences.
@@ -628,7 +745,6 @@ esl_rsq_CMarkov1(ESL_RANDOMNESS *r, const char *s, char *markoved)
 /*****************************************************************
  *# 4. Generating iid sequences (digital mode).
  *****************************************************************/
-#ifdef eslAUGMENT_ALPHABET
 
 /* Function: esl_rsq_xIID()
  * Synopsis: Generate an iid random digital sequence.
@@ -678,7 +794,89 @@ esl_rsq_xfIID(ESL_RANDOMNESS *r, const float *p, int K, int L, ESL_DSQ *dsq)
   return eslOK;
 }
 
-#endif /*eslAUGMENT_ALPHABET*/
+
+/* Function:  esl_rsq_SampleDirty()
+ * Synopsis:  Sample a digital sequence, including noncanonicals.
+ * Incept:    SRE, Wed Feb 17 10:57:28 2016 [H1/76]
+ *
+ * Purpose:   Using random number generator <rng>, use probability
+ *            vector <p> to sample an iid digital sequence in alphabet
+ *            <abc> of length <L>. Store it in <dsq>. 
+ * 
+ *            The <dsq> space, allocated by the caller, has room for
+ *            at least <L+2> residues, counting the digital
+ *            sentinels. 
+ * 
+ *            Probability vector <p> has <Kp> terms, and sums to 1.0
+ *            over them. The probabilities in <p> for residues <K>,
+ *            <Kp-2>, and <Kp-1> (gap, nonresidue, missing) are
+ *            typically zero, to generate a standard unaligned digital
+ *            sequence with degenerate residues. To sample a random
+ *            "alignment", <p[K]> is nonzero.
+ *            
+ *            If <p> is <NULL>, then we sample a probability vector
+ *            according to the following rules. 
+ *               1. Sample pc, the probability of canonical
+ *                  vs. noncanonical residues, uniformly on [0,1).
+ *               2. Sample a p[] uniformly for canonical residues
+ *                  <0..K-1>, and renormalize by multiplying by pc.
+ *                  Sample a different p[] uniformly for noncanonical
+ *                  residues <K+1..Kp-3>, and renormalize by (1-pc).
+ *               3. p[] = 0 for gap residue K, nonresidue Kp-2, 
+ *                  missing residue Kp-1.
+ *            This usage is mainly intended to make it easy to
+ *            sample dirty edge cases for automated tests.
+ *
+ * Args:      rng  :  random number generator 
+ *            abc  :  digital alphabet
+ *            p    :  OPTIONAL: p[0..Kp-1] probability vector, or NULL
+ *            L    :  length of digital sequence to sample
+ *            dsq  :  resulting digital seq sample, caller-provided space
+ * 
+ * Returns:   <eslOK> on success
+ *
+ * Throws:    <eslEMEM> on allocation failure.
+ */
+int
+esl_rsq_SampleDirty(ESL_RANDOMNESS *rng, ESL_ALPHABET *abc, double **byp_p, int L, ESL_DSQ *dsq)
+{
+  double *p = NULL;    
+  int     i;
+  int     status;
+  
+  /* If p isn't provided, sample one. */
+  if ( esl_byp_IsProvided(byp_p)) 
+    p = *byp_p;
+  else
+    {
+      double pc = esl_random(rng); /* [0,1) */
+      int    x;
+      
+      ESL_ALLOC(p, sizeof(double) * abc->Kp);
+
+      esl_rnd_Dirichlet(rng, NULL /* i.e. uniform */, abc->K, p);
+      esl_rnd_Dirichlet(rng, NULL, (abc->Kp - abc->K - 3), (p + abc->K +1));  /* K+1..Kp-3 range of alphabet */
+      for (x = 0;        x <  abc->K;    x++) p[x] = p[x] * pc;
+      for (x = abc->K+1; x <= abc->Kp-3; x++) p[x] = p[x] * (1.-pc);
+      p[abc->K]    = 0.;
+      p[abc->Kp-2] = 0.;
+      p[abc->Kp-1] = 0.;
+    }
+
+  dsq[0]   = eslDSQ_SENTINEL;
+  for (i = 1; i <= L; i++)
+    dsq[i] = esl_rnd_DChoose(rng, p, abc->Kp);
+  dsq[L+1] = eslDSQ_SENTINEL;
+
+  if      (esl_byp_IsReturned(byp_p)) *byp_p = p;
+  else if (esl_byp_IsInternal(byp_p)) free(p); 
+  return eslOK;
+
+ ERROR:
+  if (! esl_byp_IsProvided(byp_p) && p) free(p);
+  if (  esl_byp_IsReturned(byp_p))     *byp_p = NULL;
+  return status;
+}
 /*--------------------- end, digital generation ---------------- */
 
 
@@ -686,7 +884,6 @@ esl_rsq_xfIID(ESL_RANDOMNESS *r, const float *p, int K, int L, ESL_DSQ *dsq)
 /*****************************************************************
  *# 5. Shuffling sequences (digital mode)
  *****************************************************************/
-#ifdef eslAUGMENT_ALPHABET
 
 /* Function:  esl_rsq_XShuffle()
  * Synopsis:  Shuffle a digital sequence.
@@ -749,9 +946,10 @@ esl_rsq_XShuffleDP(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ
   ESL_DSQ **E  = NULL;      /* edge lists: E[0] is the edge list from vertex A */
   int     *nE  = NULL;      /* lengths of edge lists */
   int     *iE  = NULL;      /* positions in edge lists */
+  ESL_DSQ *Z   = NULL;      /* connectivity in last edge graph Z */ 
   int      n;	            /* tmp: remaining length of an edge list to be shuffled */
   ESL_DSQ  sf;              /* last character in shuffled */
-  ESL_DSQ *Z;               /* connectivity in last edge graph Z */ 
+
   int      keep_connecting; /* flag used in Z connectivity algorithm */
   int      is_eulerian;	    /* flag used for when we've got a good Z */
   
@@ -973,7 +1171,6 @@ esl_rsq_XShuffleWindows(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int w, ESL
   return eslOK;
 }
 
-#endif /*eslAUGMENT_ALPHABET*/
 /*------------------- end, digital shuffling  -------------------*/
 
 
@@ -981,7 +1178,6 @@ esl_rsq_XShuffleWindows(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int w, ESL
 /*****************************************************************
  *# 6. Randomizing sequences (digital mode)
  *****************************************************************/
-#ifdef eslAUGMENT_ALPHABET
 
 /* Function:  esl_rsq_XMarkov0()
  * Synopsis:  Generate new digital sequence of same 0th order Markov properties.
@@ -1079,12 +1275,12 @@ esl_rsq_XMarkov0(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ *m
 int 
 esl_rsq_XMarkov1(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ *markoved) 
 {
-  int      status;
+  double **p  = NULL;	/* conditional probabilities p[x][y] = P(y | x) */
+  double  *p0 = NULL;	/* marginal probabilities P(x), just for initial residue. */
   int      i; 
   ESL_DSQ  x,y;
   ESL_DSQ  i0;		/* initial symbol */
-  double **p;		/* conditional probabilities p[x][y] = P(y | x) */
-  double  *p0;		/* marginal probabilities P(x), just for initial residue. */
+  int      status;
 
   /* validate the input string */
   for (i = 1; i <= L; i++)
@@ -1144,7 +1340,6 @@ esl_rsq_XMarkov1(ESL_RANDOMNESS *r, const ESL_DSQ *dsq, int L, int K, ESL_DSQ *m
   return status;
 }
 
-#endif /*eslAUGMENT_ALPHABET*/
 /*------------------ end, digital randomizing -------------------*/
 
 /*****************************************************************
@@ -1585,7 +1780,7 @@ utest_XShufflers(ESL_RANDOMNESS *r, int L, int K)
   /* allocations */
   ESL_ALLOC(dsq, sizeof(ESL_DSQ) * (L+2));
   ESL_ALLOC(ds2, sizeof(ESL_DSQ) * (L+2));
-  ESL_ALLOC(p,   sizeof(double)  * K);
+  ESL_ALLOC(p,   sizeof(float)   * K);
   if (composition_allocate(K, &m1, &di1) != eslOK) esl_fatal(logmsg);
   if (composition_allocate(K, &m2, &di2) != eslOK) esl_fatal(logmsg);
 
@@ -1695,7 +1890,7 @@ utest_XMarkovs(ESL_RANDOMNESS *r, int L, int K)
   /* allocations */
   ESL_ALLOC(dsq, sizeof(ESL_DSQ) * (L+2));
   ESL_ALLOC(ds2, sizeof(ESL_DSQ) * (L+2));
-  ESL_ALLOC(p,   sizeof(double)  * K);
+  ESL_ALLOC(p,   sizeof(float)   * K);
   if (composition_allocate(K, &m1, &di1) != eslOK) esl_fatal(logmsg);
   if (composition_allocate(K, &m2, &di2) != eslOK) esl_fatal(logmsg);
 
@@ -1917,13 +2112,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_randomseq.c 715 2011-08-03 21:04:24Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_randomseq.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
