@@ -40,6 +40,7 @@
 #include <math.h>
 
 #include "easel.h"
+#include "esl_composition.h"
 #include "esl_dmatrix.h"
 #include "esl_vectorops.h"
 #include "esl_ratematrix.h"
@@ -815,13 +816,13 @@ main(void)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_ratematrix.c 685 2011-05-23 14:27:52Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_ratematrix.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/ 
