@@ -40,6 +40,14 @@
  *    If you do not want invalid input regex syntax to halt your application,
  *    you can install a custom error handler that can handle
  *    the eslESYNTAX errors as you wish.
+ *****************************************************************
+ * TODO:
+ *  - would be great to have an esl_regexp_Sample(), which sampled
+ *    strings from a regexp. We could use this in unit tests that 
+ *    need to stress edge cases (generating strings with unusual
+ *    but legal characters, for example). We would probably want
+ *    to implement some artificial limits on repeat operators,
+ *    to keep length of sampled seq reasonable.
  */
 #include "esl_config.h"
 
@@ -337,6 +345,46 @@ esl_regexp_SubmatchCoords(ESL_REGEXP *machine, char *origin, int elem,
   *ret_end   = 0;
   return status;
 }
+
+
+
+/* Function:  esl_regexp_ParseCoordString()
+ *
+ * Purpose:   Given a string <cstring> of the format required for a
+ *            range (<from>..<to>, e.g. 10..23  or 39-91) parse out
+ *            the start and end, and return them within the variables
+ *            <ret_start> and <ret_end>.
+ *
+ * Returns:   <eslOK> on success, and <ret_start> and <ret_end>
+ *            are set to the start/end coordinates of the parse.
+ *
+ * Throws:    <eslESYNTAX> if a regexp match is not made, and
+ *            <eslFAIL> if the start or end values are not parsed.
+ */
+int
+esl_regexp_ParseCoordString(const char *cstring, uint32_t *ret_start, uint32_t *ret_end)
+{
+  ESL_REGEXP *re = esl_regexp_Create();
+  char        tok1[32];
+  char        tok2[32];
+  int         status;
+
+  if (esl_regexp_Match(re, "^(\\d+)\\D+(\\d*)$", cstring) != eslOK) { status = eslESYNTAX; goto ERROR; }
+  if (esl_regexp_SubmatchCopy(re, 1, tok1, 32)            != eslOK) { status = eslFAIL;    goto ERROR; }
+  if (esl_regexp_SubmatchCopy(re, 2, tok2, 32)            != eslOK) { status = eslFAIL;    goto ERROR; }
+
+  *ret_start = atol(tok1);
+  *ret_end   = (tok2[0] == '\0') ? 0 : atol(tok2);
+
+  esl_regexp_Destroy(re);
+  return eslOK;
+
+ ERROR:
+  esl_regexp_Destroy(re);
+  return status;
+}
+
+
 /*=================== end of the exposed API ==========================================*/
 
 
@@ -618,12 +666,12 @@ regcomp(const char *exp)
 static char *
 reg(register struct comp *cp, int paren, int *flagp)
 {
-  int status;
   register char *ret = NULL;   /* SRE: NULL init added to silence gcc */
   register char *br;
   register char *ender;
   register int parno = 0;	/* SRE: init added to silence gcc */
-  int flags;
+  int   flags;
+  int   status;
 
   *flagp = HASWIDTH;	/* Tentatively. */
 
@@ -677,7 +725,7 @@ reg(register struct comp *cp, int paren, int *flagp)
   return(ret);
 
  ERROR:
-  return NULL;
+  return (status == eslOK ? ret : NULL);  // fake out: status always non-OK here, this is solely to use <status> and silence compiler warning
 }
 
 /*
@@ -730,11 +778,11 @@ regbranch(register struct comp *cp, int *flagp)
 static char *
 regpiece(register struct comp *cp, int *flagp)
 {
-        int status;
 	register char *ret;
 	register char op;
 	register char *next;
 	int flags;
+        int status;
 
 	ret = regatom(cp, &flags);
 	if (ret == NULL)
@@ -787,7 +835,7 @@ regpiece(register struct comp *cp, int *flagp)
 	return(ret);
 
  ERROR:
-	return NULL;
+	return (status == eslOK ? ret : NULL);  // status is not OK; this construction serves solely to silence compiler warning about unused <status>
 }
 
 /*
@@ -804,7 +852,7 @@ regpiece(register struct comp *cp, int *flagp)
 static char *
 regatom(register struct comp *cp, int *flagp)
 {
-  register char *ret;
+  register char *ret = NULL;
   int flags;
   int status;
 
@@ -932,7 +980,7 @@ regatom(register struct comp *cp, int *flagp)
   return(ret);
   
  ERROR:
-  return NULL;
+  return (status == eslOK ? ret : NULL);  // status is not OK. Construction serves to silence compiler warning about unused <status>.
 }
 
 /*
@@ -1563,9 +1611,9 @@ regsub(const esl__regexp *rp, const char *source, char *dest)
 static char *
 regescape(struct comp *cp, char c)
 {
+  char *ret = NULL;
+  char  x;
   int   status;
-  char *ret;
-  char x;
 
   switch (c) {
     /* escapes: */
@@ -1631,7 +1679,7 @@ regescape(struct comp *cp, char c)
   return ret;
 
  ERROR:
-  return NULL;
+  return (status == eslOK ? ret : NULL);  // status is not OK; construction serves to silence compiler warning about unused <status>
 }
 
 
@@ -1639,7 +1687,7 @@ regescape(struct comp *cp, char c)
  * 3 code examples, and the test driver 
  *****************************************************************/
 
-#ifdef eslREGEXP_EXAMPLE1
+#ifdef eslREGEXP_EXAMPLE
 /* Single match example.
  * Find first match of <pattern> in <string>; 
  * print coords of complete match.
@@ -1681,7 +1729,7 @@ main(int argc, char **argv)
   esl_regexp_Destroy(m);
   exit(0);
 }
-#endif /* eslREGEXP_EXAMPLE1*/
+#endif /* eslREGEXP_EXAMPLE*/
 
 
 #ifdef eslREGEXP_EXAMPLE2
@@ -1858,13 +1906,13 @@ main(void)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_regexp.c 685 2011-05-23 14:27:52Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_regexp.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
