@@ -1,7 +1,7 @@
 /* Unaligned ascii sequence file i/o.
  * 
- * SVN $Id: esl_sqio_ascii.h 715 2011-08-03 21:04:24Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sqio_ascii.h $
+ * SVN $Id$
+ * SVN $URL$
  */
 #ifndef eslSQIO_ASCII_INCLUDED
 #define eslSQIO_ASCII_INCLUDED
@@ -66,7 +66,7 @@ typedef struct esl_sqascii_s {
 
   /* MSA augmentation confers reading MSA files as sequential seq files. */
 #if defined(eslAUGMENT_MSA)
-  ESLX_MSAFILE *afp;	      /* open ESLX_MSAFILE for reading           */
+  ESL_MSAFILE  *afp;	      /* open ESL_MSAFILE for reading           */
   ESL_MSA      *msa;	      /* preloaded alignment to draw seqs from  */
   int           idx;	      /* index of next seq to return, 0..nseq-1 */
 #else
@@ -99,10 +99,10 @@ extern int  esl_sqascii_Parse(char *buf, int size, ESL_SQ *s, int format);
 #endif /*eslSQIO_ASCII_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
