@@ -1,7 +1,7 @@
 /* Unaligned ncbi sequence file i/o.
  * 
- * SVN $Id: esl_sqio_ncbi.h 664 2011-02-27 17:08:36Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sqio_ncbi.h $
+ * SVN $Id$
+ * SVN $URL$
  */
 #ifndef eslSQIO_NCBI_INCLUDED
 #define eslSQIO_NCBI_INCLUDED
@@ -30,7 +30,7 @@ typedef struct esl_sqncbi_vol_s {
   char      *name;                 /* name of the volume                       */
 
   uint32_t   start_seq;            /* starting sequence number                 */
-  uint32_t   end_seq;              /* starting sequence number                 */
+  uint32_t   end_seq;              /* ending sequence number                   */
 
   uint32_t   hdr_off;              /* disk offset in .pin to header index      */
   uint32_t   seq_off;              /* disk offset to .pin to sequence index    */
@@ -107,10 +107,10 @@ extern int  esl_sqncbi_Open(char *seqfile, int format, struct esl_sqio_s *sqfp);
 #endif /*eslSQIO_NCBI_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
