@@ -370,9 +370,9 @@ utest_expf(ESL_GETOPTS *go)
     esl_sse_dump_ps(stdout, r.v);  printf("\n");
   }
   if (r.x[0] != 0.0f)   esl_fatal("expf(-inf) should be 0");
-  if (r.x[1] != 1.0f)   esl_fatal("logf(-0)   should be 1");
-  if (r.x[2] != 1.0f)   esl_fatal("logf(0)    should be 1");
-  if (! isinf(r.x[3]))  esl_fatal("logf(inf)  should be inf");
+  if (r.x[1] != 1.0f)   esl_fatal("expf(-0)   should be 1");
+  if (r.x[2] != 1.0f)   esl_fatal("expf(0)    should be 1");
+  if (! isinf(r.x[3]))  esl_fatal("expf(inf)  should be inf");
 
   /* exp(NaN) = NaN    exp(large)  = inf   exp(-large) = 0  exp(1) = exp(1) */
   x = _mm_set_ps(1.0f, -666.0f, 666.0f, eslNaN); /* set_ps() is in order 3 2 1 0 */
@@ -576,15 +576,15 @@ int main(void) { return 0; }
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_sse.c 807 2012-09-19 17:51:00Z nawrockie $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_sse.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 /* Additionally, esl_sse_logf() and esl_sse_expf() are 
