@@ -56,15 +56,16 @@ esl_stack_ICreate(void)
   ns->nalloc   = ESL_STACK_INITALLOC;
   ns->pdata    = NULL;
   ns->cdata    = NULL;
-  ESL_ALLOC(ns->idata, sizeof(int) * ns->nalloc);
+  ns->idata    = NULL;
   ns->n        = 0;
-
 #ifdef HAVE_PTHREAD
   ns->do_mutex = FALSE;
   ns->do_cond  = FALSE;
   ns->mutex    = NULL;
   ns->cond     = NULL;
 #endif  
+
+  ESL_ALLOC(ns->idata, sizeof(int) * ns->nalloc);
   return ns;
 
  ERROR:
@@ -92,15 +93,16 @@ esl_stack_CCreate(void)
   cs->nalloc   = ESL_STACK_INITALLOC;
   cs->idata    = NULL;
   cs->pdata    = NULL;
-  ESL_ALLOC(cs->cdata, sizeof(char) * cs->nalloc);
+  cs->cdata    = NULL;
   cs->n        = 0;
-
 #ifdef HAVE_PTHREAD
   cs->do_mutex = FALSE;
   cs->do_cond  = FALSE;
   cs->mutex    = NULL;
   cs->cond     = NULL;
 #endif  
+
+  ESL_ALLOC(cs->cdata, sizeof(char) * cs->nalloc);
   return cs;
 
  ERROR:
@@ -128,9 +130,8 @@ esl_stack_PCreate(void)
   ps->nalloc   = ESL_STACK_INITALLOC;
   ps->idata    = NULL;
   ps->cdata    = NULL;
-  ESL_ALLOC(ps->pdata, sizeof(void *) * ps->nalloc);
+  ps->pdata    = NULL;
   ps->n        = 0;
-
 #ifdef HAVE_PTHREAD
   ps->do_mutex = FALSE;
   ps->do_cond  = FALSE;
@@ -138,6 +139,7 @@ esl_stack_PCreate(void)
   ps->cond     = NULL;
 #endif  
 
+  ESL_ALLOC(ps->pdata, sizeof(void *) * ps->nalloc);
   return ps;
 
  ERROR:
@@ -172,15 +174,17 @@ esl_stack_Reuse(ESL_STACK *s)
 void
 esl_stack_Destroy(ESL_STACK *s)
 {
-  if (s->idata) free(s->idata);
-  if (s->cdata) free(s->cdata);
-  if (s->pdata) free(s->pdata);
-
+  if (s) 
+    {
+       if (s->idata) free(s->idata);
+       if (s->cdata) free(s->cdata);
+       if (s->pdata) free(s->pdata);
 #ifdef HAVE_PTHREAD
-  if (s->mutex) { pthread_mutex_destroy(s->mutex); free(s->mutex); }
-  if (s->cond)  { pthread_cond_destroy(s->cond);   free(s->cond);  }
+       if (s->mutex) { pthread_mutex_destroy(s->mutex); free(s->mutex); }
+       if (s->cond)  { pthread_cond_destroy(s->cond);   free(s->cond);  }
 #endif
-  free(s);
+       free(s);
+    }
 }
 /*------------------ end, ESL_STACK object ----------------------*/
 
@@ -556,7 +560,8 @@ esl_stack_DiscardTopN(ESL_STACK *s, int n)
 int
 esl_stack_DiscardSelected(ESL_STACK *s, int (*discard_func)(void *, void *), void *param)
 {
-  int opos, npos;
+  int opos;
+  int npos = 0;
 
 #ifdef HAVE_PTHREAD
   if (s->do_mutex && pthread_mutex_lock(s->mutex) != 0) ESL_EXCEPTION(eslESYS, "pthread_mutex_lock() failure");
@@ -741,10 +746,11 @@ esl_stack_ReleaseCond(ESL_STACK *s)
   if (pthread_mutex_lock(s->mutex)    != 0) ESL_EXCEPTION(eslESYS, "pthread_mutex_lock() failure");
   if (pthread_cond_broadcast(s->cond) != 0) ESL_EXCEPTION(eslESYS, "pthread_cond_broadcast() failure");
 
-  pthread_cond_destroy(s->cond);
+  /* You can't free s->cond yet; you can only set the flag. 
+   *  Reason: you may have workers that are ALREADY in a wait state, in pthread_cond_wait(), 
+   *  and that function depends on s->cond.
+   */
   s->do_cond = FALSE;
-  free(s->cond);
-  s->cond    = NULL;
 
   if (pthread_mutex_unlock(s->mutex)  != 0) ESL_EXCEPTION(eslESYS, "pthread_mutex_unlock() failure");  
   return eslOK;
@@ -759,6 +765,8 @@ esl_stack_ReleaseCond(ESL_STACK *s)
  *****************************************************************/
 #ifdef eslSTACK_TESTDRIVE
 
+#include "esl_random.h"
+
 static void
 utest_integer(void)
 {
@@ -867,19 +875,15 @@ utest_convert2string(void)
   free(result);	/* after Convert2String, only the string itself remains to be free'd */
 }
 
-
-#ifdef eslAUGMENT_RANDOM
 static void
-utest_shuffle(void)
+utest_shuffle(ESL_RANDOMNESS *r)
 {
   char           *msg  = "stack shuffle unit test failed";
-  ESL_RANDOMNESS *r    = esl_randomness_Create(42);
   ESL_STACK      *s    = esl_stack_ICreate();
   int             n    = ESL_STACK_INITALLOC*2+1;      /* exercises reallocation */
   int            *seen = malloc(sizeof(int) * n);
   int             i;
   int             val;
-  int             appears_shuffled = FALSE;
 
   for (i = 0; i < n; i++) esl_stack_IPush(s, i);
   esl_stack_Shuffle(r, s);
@@ -888,15 +892,52 @@ utest_shuffle(void)
   i = n-1;
   while (esl_stack_IPop(s, &val) != eslEOD) {
     seen[val]++;
-    if (val != i--) appears_shuffled = TRUE;
   }
   for (i = 0; i < n; i++) if (seen[i] != 1) esl_fatal(msg);
   
   free(seen);
   esl_stack_Destroy(s);
-  esl_randomness_Destroy(r);
 }
-#endif /*eslAUGMENT_RANDOM*/
+
+/* discard all elems in the stack > thresh */
+static int
+discard_function(void *elemp, void *paramp)
+{
+  int elem   =  * (int *) elemp;
+  int thresh =  * (int *) paramp;
+  return (elem > thresh) ? TRUE : FALSE;
+}
+
+static void
+utest_discard_selected(ESL_RANDOMNESS *r)
+{
+  char *msg = "stack: DiscardSelected() unit test failed";
+  ESL_STACK      *ns = esl_stack_ICreate();
+  int              n = 1000;
+  int         thresh = 42;
+  int          npass = 0;
+  int            val;
+  int              i;
+
+  for (i = 0; i < n; i++)
+    {
+      val = esl_rnd_Roll(r, 100) + 1;
+      if (val <= thresh) npass++;
+      esl_stack_IPush(ns, val);
+    }
+  
+  if (esl_stack_DiscardSelected(ns, discard_function, &thresh) != eslOK) esl_fatal(msg);
+
+  if (esl_stack_ObjectCount(ns) != npass) esl_fatal(msg);
+  while (esl_stack_IPop(ns, &val) == eslOK)
+    {
+      if (val > thresh) esl_fatal(msg);
+      npass--;
+    }
+  if (npass != 0) esl_fatal(msg);
+  esl_stack_Destroy(ns);
+}
+
 
 #ifdef HAVE_PTHREAD
 /* Unit test for using a stack as part of an idiom
@@ -921,9 +962,9 @@ utest_shuffle(void)
  *   nonthreaded code in main()). 
  */
 struct threadtest_s {
-  ESL_STACK *input;
-  ESL_STACK *working;
-  ESL_STACK *output;
+  ESL_STACK *input;		/* faux "work unit" queue that the pusher_thread() processes*/
+  ESL_STACK *working;		/* interthread communication: pusher puts work on this stack, popper pulls it off */
+  ESL_STACK *output;		/* popper_thread() puts "finished" units on this stack */
 };
 
 static void *
@@ -1013,52 +1054,6 @@ utest_interthread_comm(void)
   return;
 }
 #endif /* HAVE_PTHREAD -- pthread-specific utests */
-
-
-#ifdef HAVE_PTHREAD
-
-/* discard all elems in the stack > thresh */
-static int
-discard_function(void *elemp, void *paramp)
-{
-  int elem   =  * (int *) elemp;
-  int thresh =  * (int *) paramp;
-  return (elem > thresh) ? TRUE : FALSE;
-}
-
-static void
-utest_DiscardSelected(void)
-{
-  char *msg = "stack: DiscardSelected() unit test failed";
-  ESL_STACK      *ns = esl_stack_ICreate();
-  ESL_RANDOMNESS  *r = esl_randomness_CreateFast(0);
-  int              n = 1000;
-  int              thresh = 42;
-  int              npass = 0;
-  int              val;
-  int              i;
-
-  for (i = 0; i < n; i++)
-    {
-      val = esl_rnd_Roll(r, 100) + 1;
-      if (val <= thresh) npass++;
-      esl_stack_IPush(ns, val);
-    }
-  
-  if (esl_stack_DiscardSelected(ns, discard_function, &thresh) != eslOK) esl_fatal(msg);
-
-  if (esl_stack_ObjectCount(ns) != npass) esl_fatal(msg);
-  while (esl_stack_IPop(ns, &val) == eslOK)
-    {
-      if (val > thresh) esl_fatal(msg);
-      npass--;
-    }
-  if (npass != 0) esl_fatal(msg);
-
-  esl_randomness_Destroy(r);
-  esl_stack_Destroy(ns);
-}
-#endif /* HAVE_PTHREAD*/
 #endif /*eslSTACK_TESTDRIVE*/
 /*---------------- end of unit tests ----------------------------*/
 
@@ -1068,16 +1063,15 @@ utest_DiscardSelected(void)
 /*****************************************************************
  * 6. Test driver.
  *****************************************************************/
-
-/*****************************************************************
+#ifdef eslSTACK_TESTDRIVE
+/*
  * Test driver and API example for the pushdown stack module.
  * To compile:
  *    gcc -g -Wall -I. -L. -DeslSTACK_TESTDRIVE -o testdrive esl_stack.c -leasel -lm
  * To run:
  *    ./testdrive
- * Returns 0 (success) w/ no output, or returns nonzero and says why.
- *****************************************************************/
-
+ * Returns 0 (success), or returns nonzero and says why.
+ */
 /* why Pop() into a void *obj_p, instead of directly into int *obj, in
  * the test of the pointer stack? On PowerPC/Linux, using gcc -O3,
  * trying to Pop() into *obj causes a "dereferencing type-punned
@@ -1088,24 +1082,45 @@ utest_DiscardSelected(void)
  * suspicious that it's a gcc optimizer bug. Pop()'ing into a void *
  * avoids the issue altogether. (SRE, Feb 22 2008 J2/119)
  */
-#ifdef eslSTACK_TESTDRIVE
+#include "easel.h"
+#include "esl_getopts.h"
+#include "esl_random.h"
+#include "esl_stack.h"
+
+static ESL_OPTIONS options[] = {
+  /* name           type      default  env  range toggles reqs incomp  help                                       docgroup*/
+  { "-h",        eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",           0 },
+  { "-s",        eslARG_INT,      "0", NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                  0 },
+  {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+};
+static char usage[]  = "[-options]";
+static char banner[] = "unit test driver for esl_stack module";
+
 int 
-main(void)
+main(int argc, char **argv)
 {
+  ESL_GETOPTS    *go  = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *rng = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+
+  fprintf(stderr, "## %s\n", argv[0]);
+  fprintf(stderr, "#  rng seed = %" PRIu32 "\n", esl_randomness_GetSeed(rng));
+
   utest_integer();
   utest_char();
   utest_pointer();
   utest_convert2string();
-
-#ifdef eslAUGMENT_RANDOM
-  utest_shuffle();
-#endif
+  utest_shuffle(rng);
+  utest_discard_selected(rng);
 
 #ifdef HAVE_PTHREAD
   utest_interthread_comm();
-  utest_DiscardSelected();
 #endif
 
+  esl_randomness_Destroy(rng);
+  esl_getopts_Destroy(go);
+  
+
+  fprintf(stderr, "#  status = ok\n");
   return eslOK;
 }
 #endif /*eslSTACK_TESTDRIVE*/
@@ -1149,13 +1164,13 @@ main(void)
 
 /*****************************************************************  
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_stack.c 777 2012-07-06 12:29:15Z nawrockie $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stack.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
