@@ -246,6 +246,7 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
   int    iter;			/* iteration counter */
   double pax;			/* P(a,x) */
   double qax;			/* Q(a,x) */
+  int    status;
 
   if (a <= 0.) ESL_EXCEPTION(eslERANGE, "esl_stats_IncompleteGamma(): a must be > 0");
   if (x <  0.) ESL_EXCEPTION(eslERANGE, "esl_stats_IncompleteGamma(): x must be >= 0");
@@ -292,7 +293,7 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
 				/* check for convergence */
 	  if (fabs((nu1-oldp)/nu1) < 1.e-7)
 	    {
-	      esl_stats_LogGamma(a, &qax);	      
+	      if ((status = esl_stats_LogGamma(a, &qax)) != eslOK) return status;	      
 	      qax = nu1 * exp(a * log(x) - x - qax);
 
 	      if (ret_pax != NULL) *ret_pax = 1 - qax;
@@ -327,7 +328,7 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
 	  
 	  if (fabs(val/p) < 1.e-7)
 	    {
-	      esl_stats_LogGamma(a, &pax);
+	      if ((status = esl_stats_LogGamma(a, &pax)) != eslOK) return status;
 	      pax = p * exp(a * log(x) - x - pax);
 
 	      if (ret_pax != NULL) *ret_pax = pax;
@@ -341,6 +342,191 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
   /*NOTREACHED*/
   return eslOK;
 }
+
+
+/* Function:  esl_stats_erfc()
+ * Synopsis:  Complementary error function.
+ *
+ * Purpose:   Calculate and return the complementary error function, 
+ *            erfc(x).
+ *            
+ *            erfc(x) is mandated by the ANSI C99 standard but that
+ *            doesn't mean it's available on supposedly modern systems
+ *            (looking at you here, Microsoft).
+ *            
+ *            Used for cumulative distribution function calculations
+ *            for the normal (Gaussian) distribution. See <esl_normal>
+ *            module.
+ *            
+ *            erfc(-inf) = 2.0
+ *            erfc(0)    = 1.0
+ *            erfc(inf)  = 0.0
+ *            erfc(NaN)  = NaN
+ *
+ * Args:      x : any real-numbered value -inf..inf
+ *
+ * Returns:   erfc(x)
+ *
+ * Throws:    (no abnormal error conditions)
+ *
+ * Source:
+ *    Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved. 
+ *    Developed at SunPro, a Sun Microsystems, Inc. business.
+ *    Permission to use, copy, modify, and distribute this software is
+ *    freely granted, provided that this notice is preserved.
+ *    [as posted by eggcrook at stackexchange.com, 21 Dec 2012]
+ *    
+ *    Removed arcane incantations for runtime detection of endianness,
+ *    and for treating IEEE754 doubles as two adjacent uint32_t;
+ *    replaced with ANSI-compliant macros and compile-time detection
+ *    of endianness. [Apr 2015]
+ */
+double
+esl_stats_erfc(double x)
+{
+  static const double tiny = 1e-300;
+  static const double half = 5.00000000000000000000e-01; /* 0x3FE00000, 0x00000000 */
+  static const double one  = 1.00000000000000000000e+00; /* 0x3FF00000, 0x00000000 */
+  static const double two  = 2.00000000000000000000e+00; /* 0x40000000, 0x00000000 */
+  static const double erx  = 8.45062911510467529297e-01; /* 0x3FEB0AC1, 0x60000000 */
+  /*
+   * Coefficients for approximation to erf on [0,0.84375]
+   */
+  static const double pp0 =  1.28379167095512558561e-01; /* 0x3FC06EBA, 0x8214DB68 */
+  static const double pp1 = -3.25042107247001499370e-01; /* 0xBFD4CD7D, 0x691CB913 */
+  static const double pp2 = -2.84817495755985104766e-02; /* 0xBF9D2A51, 0xDBD7194F */
+  static const double pp3 = -5.77027029648944159157e-03; /* 0xBF77A291, 0x236668E4 */
+  static const double pp4 = -2.37630166566501626084e-05; /* 0xBEF8EAD6, 0x120016AC */
+  static const double qq1 =  3.97917223959155352819e-01; /* 0x3FD97779, 0xCDDADC09 */
+  static const double qq2 =  6.50222499887672944485e-02; /* 0x3FB0A54C, 0x5536CEBA */
+  static const double qq3 =  5.08130628187576562776e-03; /* 0x3F74D022, 0xC4D36B0F */
+  static const double qq4 =  1.32494738004321644526e-04; /* 0x3F215DC9, 0x221C1A10 */
+  static const double qq5 = -3.96022827877536812320e-06; /* 0xBED09C43, 0x42A26120 */
+  /*
+   * Coefficients for approximation to erf in [0.84375,1.25]
+   */
+  static const double pa0 = -2.36211856075265944077e-03; /* 0xBF6359B8, 0xBEF77538 */
+  static const double pa1 =  4.14856118683748331666e-01; /* 0x3FDA8D00, 0xAD92B34D */
+  static const double pa2 = -3.72207876035701323847e-01; /* 0xBFD7D240, 0xFBB8C3F1 */
+  static const double pa3 =  3.18346619901161753674e-01; /* 0x3FD45FCA, 0x805120E4 */
+  static const double pa4 = -1.10894694282396677476e-01; /* 0xBFBC6398, 0x3D3E28EC */
+  static const double pa5 =  3.54783043256182359371e-02; /* 0x3FA22A36, 0x599795EB */
+  static const double pa6 = -2.16637559486879084300e-03; /* 0xBF61BF38, 0x0A96073F */
+  static const double qa1 =  1.06420880400844228286e-01; /* 0x3FBB3E66, 0x18EEE323 */
+  static const double qa2 =  5.40397917702171048937e-01; /* 0x3FE14AF0, 0x92EB6F33 */
+  static const double qa3 =  7.18286544141962662868e-02; /* 0x3FB2635C, 0xD99FE9A7 */
+  static const double qa4 =  1.26171219808761642112e-01; /* 0x3FC02660, 0xE763351F */
+  static const double qa5 =  1.36370839120290507362e-02; /* 0x3F8BEDC2, 0x6B51DD1C */
+  static const double qa6 =  1.19844998467991074170e-02; /* 0x3F888B54, 0x5735151D */
+  /*
+   * Coefficients for approximation to erfc in [1.25,1/0.35]
+   */
+  static const double ra0 = -9.86494403484714822705e-03; /* 0xBF843412, 0x600D6435 */
+  static const double ra1 = -6.93858572707181764372e-01; /* 0xBFE63416, 0xE4BA7360 */
+  static const double ra2 = -1.05586262253232909814e+01; /* 0xC0251E04, 0x41B0E726 */
+  static const double ra3 = -6.23753324503260060396e+01; /* 0xC04F300A, 0xE4CBA38D */
+  static const double ra4 = -1.62396669462573470355e+02; /* 0xC0644CB1, 0x84282266 */
+  static const double ra5 = -1.84605092906711035994e+02; /* 0xC067135C, 0xEBCCABB2 */
+  static const double ra6 = -8.12874355063065934246e+01; /* 0xC0545265, 0x57E4D2F2 */
+  static const double ra7 = -9.81432934416914548592e+00; /* 0xC023A0EF, 0xC69AC25C */
+  static const double sa1 =  1.96512716674392571292e+01; /* 0x4033A6B9, 0xBD707687 */
+  static const double sa2 =  1.37657754143519042600e+02; /* 0x4061350C, 0x526AE721 */
+  static const double sa3 =  4.34565877475229228821e+02; /* 0x407B290D, 0xD58A1A71 */
+  static const double sa4 =  6.45387271733267880336e+02; /* 0x40842B19, 0x21EC2868 */
+  static const double sa5 =  4.29008140027567833386e+02; /* 0x407AD021, 0x57700314 */
+  static const double sa6 =  1.08635005541779435134e+02; /* 0x405B28A3, 0xEE48AE2C */
+  static const double sa7 =  6.57024977031928170135e+00; /* 0x401A47EF, 0x8E484A93 */
+  static const double sa8 = -6.04244152148580987438e-02; /* 0xBFAEEFF2, 0xEE749A62 */
+  /*
+   * Coefficients for approximation to erfc in [1/.35,28]
+   */
+  static const double rb0 = -9.86494292470009928597e-03; /* 0xBF843412, 0x39E86F4A */
+  static const double rb1 = -7.99283237680523006574e-01; /* 0xBFE993BA, 0x70C285DE */
+  static const double rb2 = -1.77579549177547519889e+01; /* 0xC031C209, 0x555F995A */
+  static const double rb3 = -1.60636384855821916062e+02; /* 0xC064145D, 0x43C5ED98 */
+  static const double rb4 = -6.37566443368389627722e+02; /* 0xC083EC88, 0x1375F228 */
+  static const double rb5 = -1.02509513161107724954e+03; /* 0xC0900461, 0x6A2E5992 */
+  static const double rb6 = -4.83519191608651397019e+02; /* 0xC07E384E, 0x9BDC383F */
+  static const double sb1 =  3.03380607434824582924e+01; /* 0x403E568B, 0x261D5190 */
+  static const double sb2 =  3.25792512996573918826e+02; /* 0x40745CAE, 0x221B9F0A */
+  static const double sb3 =  1.53672958608443695994e+03; /* 0x409802EB, 0x189D5118 */
+  static const double sb4 =  3.19985821950859553908e+03; /* 0x40A8FFB7, 0x688C246A */
+  static const double sb5 =  2.55305040643316442583e+03; /* 0x40A3F219, 0xCEDF3BE6 */
+  static const double sb6 =  4.74528541206955367215e+02; /* 0x407DA874, 0xE79FE763 */
+  static const double sb7 = -2.24409524465858183362e+01; /* 0xC03670E2, 0x42712D62 */
+
+  int hx,ix;
+  double R,S,P,Q,s,y,z,r;
+
+  ESL_GET_HIGHWORD(hx, x);  // SRE: replaced original Sun incantation here.
+  ix = hx & 0x7fffffff;
+  if (ix>=0x7ff00000) /* erfc(nan)=nan; erfc(+-inf)=0,2 */
+    return (double)(((unsigned)hx>>31)<<1)+one/x;
+
+  if (ix < 0x3feb0000)  /* |x|<0.84375 */
+    {
+      if (ix < 0x3c700000) return one-x; /* |x|<2**-56 */
+      z = x*x;
+      r = pp0+z*(pp1+z*(pp2+z*(pp3+z*pp4)));
+      s = one+z*(qq1+z*(qq2+z*(qq3+z*(qq4+z*qq5))));
+      y = r/s;
+      if (hx < 0x3fd00000) /* x<1/4 */
+	{ 
+	  return one-(x+x*y);
+	} 
+      else 
+	{
+	  r = x*y;
+	  r += (x-half);
+	  return half - r ;
+	}
+    }
+
+  if (ix < 0x3ff40000) /* 0.84375 <= |x| < 1.25 */
+    { 
+      s = fabs(x)-one;
+      P = pa0+s*(pa1+s*(pa2+s*(pa3+s*(pa4+s*(pa5+s*pa6)))));
+      Q = one+s*(qa1+s*(qa2+s*(qa3+s*(qa4+s*(qa5+s*qa6)))));
+      if (hx>=0) 
+	{
+	  z = one-erx; 
+	  return z - P/Q;
+	}
+      else 
+	{
+	  z = erx+P/Q;
+	  return one+z;
+	}
+    }
+
+  if (ix < 0x403c0000) /* |x|<28 */
+    { 
+      x = fabs(x);
+      s = one/(x*x);
+      if (ix< 0x4006DB6D) /* |x| < 1/.35 ~ 2.857143*/ 
+	{ 
+	  R = ra0+s*(ra1+s*(ra2+s*(ra3+s*(ra4+s*(ra5+s*(ra6+s*ra7))))));
+	  S = one+s*(sa1+s*(sa2+s*(sa3+s*(sa4+s*(sa5+s*(sa6+s*(sa7+s*sa8)))))));
+	}
+      else  /* |x| >= 1/.35 ~ 2.857143 */
+	{
+	  if (hx < 0 && ix >= 0x40180000) return two-tiny; /* x < -6 */
+	  R = rb0+s*(rb1+s*(rb2+s*(rb3+s*(rb4+s*(rb5+s*rb6)))));
+	  S = one+s*(sb1+s*(sb2+s*(sb3+s*(sb4+s*(sb5+s*(sb6+s*sb7))))));
+	}
+      z = x;
+      ESL_SET_LOWWORD(z, 0);  // SRE: replaced original Sun incantation here.
+      r = exp(-z*z-0.5625) * exp((z-x)*(z+x)+R/S);
+
+      if (hx>0) return r/x;
+      else      return two-r/x;
+    } 
+  else 
+    {
+      if (hx>0) return tiny*tiny; 
+      else      return two-tiny;
+    }
+}
 /*----------------- end, special functions ----------------------*/
 
 
@@ -361,8 +547,9 @@ esl_stats_IncompleteGamma(double a, double x, double *ret_pax, double *ret_qax)
  *            experiments a,b have different binomial probabilities
  *            $p_a,p_b$. The G-test is a log-likelihood-ratio statistic,
  *            assuming maximum likelihood values for $p,p_a,p_b$. 
- *            $2G$ is distributed approximately as $\Chi^2(1)$, which
- *            we use to calculate a P-value for the G statistic. 
+ *            $2G$ is distributed approximately as $X^2(1)$,
+ *            %"X" is "Chi"
+ *            which we use to calculate a P-value for the G statistic.
  *            
  * Args:      ca    - number of positives in experiment a
  *            na    - total number in experiment a
@@ -432,7 +619,7 @@ esl_stats_GTest(int ca, int na, int cb, int nb, double *ret_G, double *ret_P)
 int
 esl_stats_ChiSquaredTest(int v, double x, double *ret_answer)
 {
-  return esl_stats_IncompleteGamma((double)v/2, x/2, NULL, ret_answer);
+  return esl_stats_IncompleteGamma((double)v/2., x/2., NULL, ret_answer);
 }
 /*----------------- end, statistical tests  ---------------------*/
 
@@ -642,6 +829,37 @@ esl_stats_LinearRegression(const double *x, const double *y, const double *sigma
 #include <gsl/gsl_sf_gamma.h>
 #endif
 
+
+/* Macros for treating IEEE754 double as two uint32_t halves, with
+ * compile-time handling of endianness; see esl_stats.h.
+ */
+static void
+utest_doublesplitting(ESL_RANDOMNESS *rng)
+{
+  char     msg[] = "esl_stats:: doublesplitting unit test failed";
+  uint32_t ix0, ix1;
+  double   x;
+  double   x2;
+  int      iteration;  // iteration 0 uses x = 2; iteration 1 uses random x = [0,1).
+
+  for (iteration = 0; iteration < 2; iteration++)
+    {
+      x = (iteration == 0 ? 2.0 : esl_random(rng));
+      ESL_GET_WORDS(ix0, ix1, x);
+      ESL_SET_WORDS(x2, ix0, ix1);
+      if (x2 != x) esl_fatal(msg);
+
+      ESL_GET_HIGHWORD(ix0, x);
+      ESL_SET_HIGHWORD(x2,  ix0);
+      if (x2 != x) esl_fatal(msg);
+
+      ESL_GET_LOWWORD(ix0, x);
+      ESL_SET_LOWWORD(x2,  ix0);
+      if (iteration == 0 && ix0 != 0)   esl_fatal(msg);
+      if (x2  != x) esl_fatal(msg);
+    }
+}
+  
 /* The LogGamma() function is rate-limiting in hmmbuild, because it is
  * used so heavily in mixture Dirichlet calculations.
  *    ./configure --with-gsl; [compile test driver]
@@ -757,6 +975,43 @@ utest_LinearRegression(ESL_RANDOMNESS *r, int use_sigma, int be_verbose)
   free(y);
   free(sigma);
 }
+
+static void
+utest_erfc(ESL_RANDOMNESS *rng, int be_verbose)
+{
+  char   msg[] = "esl_stats:: erfc unit test failed";
+  double x;
+  double result;
+  int    i;
+
+  if (be_verbose) {
+    printf("#--------------------------\n");
+    printf("# erfc unit testing...\n");
+  }
+
+  result = esl_stats_erfc( eslNaN);
+  if (! isnan(result)) esl_fatal(msg);
+  if (esl_stats_erfc(-eslINFINITY) != 2.0)    esl_fatal(msg);
+  if (esl_stats_erfc( 0.0)         != 1.0)    esl_fatal(msg);
+  if (esl_stats_erfc( eslINFINITY) != 0.0)    esl_fatal(msg);
+
+  for (i = 0; i < 42; i++)
+    {
+      x      = esl_random(rng) * 10. - 5.;
+      result = esl_stats_erfc(x);
+      if (!isfinite(result)) esl_fatal(msg);
+#ifdef HAVE_ERFC
+      if (esl_DCompare(result, erfc(x), 1e-6) != eslOK) esl_fatal(msg);
+      if (be_verbose)
+	printf("%15f %15f %15f\n", x, result, erfc(x));
+#endif
+    }
+  
+  if (be_verbose)
+    printf("#--------------------------\n");
+  return;
+}
+
 #endif /*eslSTATS_TESTDRIVE*/
 /*-------------------- end of unit tests ------------------------*/
 
@@ -797,6 +1052,8 @@ main(int argc, char **argv)
 
   if (be_verbose) printf("seed = %" PRIu32 "\n", esl_randomness_GetSeed(r));
 
+  utest_doublesplitting(r);
+  utest_erfc(r, be_verbose);
   utest_LogGamma(r, N, be_verbose);
   utest_LinearRegression(r, TRUE,  be_verbose);
   utest_LinearRegression(r, FALSE, be_verbose);
@@ -908,13 +1165,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_stats.c 771 2012-06-11 12:53:13Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stats.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
