@@ -1,6 +1,23 @@
 /* Statistical routines for stretched exponential distributions.
  * 
- * xref STL9/146
+ * Contents:
+ *   1. Evaluating densities and distributions
+ *   2. Generic API routines: for general interface w/ histogram module
+ *   3. Dumping plots for files
+ *   4. Sampling                    (augmentation: random)
+ *   5. ML fitting to complete data (augmentation: minimizer)  
+ *   6. ML fitting to binned data   (augmentation: histogram, minimizer)
+ *   7. Test driver
+ *   8. Example
+ *   9. Copyright and license information
+ *   
+ * Xrefs:
+ *    STL9/146 : original implementation
+ *    
+ * To-do:
+ *   - Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *     on failure due to small n. Compare esl_gumbel. xref J12/93.    
+ *     SRE, Wed Nov 27 11:07:44 2013
  */
 #include <esl_config.h>
 
@@ -23,7 +40,7 @@
 #endif
 
 /****************************************************************************
- * Routines for evaluating densities and distributions
+ * 1. Evaluating densities and distributions
  ****************************************************************************/ 
 /* mu <= x < infinity   
  *    [x=mu is no problem, but watch out for evaluating log(0) when it is]
@@ -163,11 +180,10 @@ double
 esl_sxp_invcdf(double p, double mu, double lambda, double tau)
 {
   double x1, x2, xm;		/* low, high guesses at x */
-  double f1, f2, fm;
+  double f2, fm;
   double tol = 1e-6;
 
   x1 = mu;
-  f1 = 0.;
   x2 = mu + 1.;
   do {				/* bracket */
     x2 = x2 + 2.*(x2-x1);
@@ -192,7 +208,7 @@ esl_sxp_invcdf(double p, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Generic API routines: for general interface w/ histogram module
+ * 2. Generic API routines: for general interface w/ histogram module
  ****************************************************************************/ 
 
 /* Function:  esl_sxp_generic_pdf()
@@ -251,7 +267,7 @@ esl_sxp_generic_invcdf(double p, void *params)
 
 
 /****************************************************************************
- * Routines for dumping plots for files
+ * 3. Dumping plots for files
  ****************************************************************************/ 
 
 /* Function:  esl_sxp_Plot()
@@ -282,7 +298,7 @@ esl_sxp_Plot(FILE *fp, double mu, double lambda, double tau,
 
 
 /****************************************************************************
- * Routines for sampling (requires augmentation w/ random, dirichlet modules)
+ * 4. Sampling (augmentation: random)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 /* Function:  esl_sxp_Sample()
@@ -305,7 +321,7 @@ esl_sxp_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Maximum likelihood fitting
+ * 5. ML fitting to complete data (augmentation: minimizer)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_MINIMIZER
 /* This structure is used to sneak the data into minimizer's generic
@@ -378,10 +394,16 @@ esl_sxp_FitComplete(double *x, int n,
   *ret_mu     = mu;
   *ret_lambda = exp(p[0]);
   *ret_tau    = exp(p[1]);
-  return eslOK;
+  return status;
 }
+#endif /*eslAUGMENT_MINIMIZER*/
 
+
+/****************************************************************************
+ * 6. ML fitting to binned data (augmentation: histogram, minimizer)
+ ****************************************************************************/ 
 #ifdef eslAUGMENT_HISTOGRAM
+#ifdef eslAUGMENT_MINIMIZER
 struct sxp_binned_data {
   ESL_HISTOGRAM *g;	/* contains the binned data    */
   double mu;		/* mu is not a learnable param */
@@ -478,75 +500,16 @@ esl_sxp_FitCompleteBinned(ESL_HISTOGRAM *g,
   *ret_mu     = mu;
   *ret_lambda = exp(p[0]);
   *ret_tau    = exp(p[1]);
-  return eslOK;
+  return status;
 }
 #endif /*eslAUGMENT_HISTOGRAM*/
 #endif /*eslAUGMENT_MINIMIZER*/
 
-/****************************************************************************
- * Example main()
- ****************************************************************************/ 
-#ifdef eslSTRETCHEXP_EXAMPLE
-/*::cexcerpt::sxp_example::begin::*/
-/* compile:
-   gcc -g -Wall -I. -o example -DeslSTRETCHEXP_EXAMPLE\
-     -DeslAUGMENT_HISTOGRAM -DeslAUGMENT_RANDOM -DeslAUGMENT_MINIMIZER\
-      esl_stretchexp.c esl_histogram.c esl_random.c esl_minimizer.c esl_stats.c esl_vectorops.c easel.c -lm
- */
-#include <stdio.h>
-#include "easel.h"
-#include "esl_random.h"
-#include "esl_histogram.h"
-#include "esl_stretchexp.h"
-
-int
-main(int argc, char **argv)
-{
-  double mu         = -50.0;
-  double lambda     = 2.5;
-  double tau        = 0.7;
-  ESL_HISTOGRAM  *h = esl_histogram_CreateFull(mu, 100., 0.1);
-  ESL_RANDOMNESS *r = esl_randomness_Create(0);
-  int    n          = 10000;
-  double *data;
-  int     ndata;
-  double emu, elam, etau;
-  int    i;
-  double x;
-
-  for (i = 0; i < n; i++)
-    {
-      x  =  esl_sxp_Sample(r, mu, lambda, tau);
-      esl_histogram_Add(h, x);
-    }
-  esl_histogram_GetData(h, &data, &ndata);
-
-  /* Plot the empirical (sampled) and expected survivals */
-  esl_histogram_PlotSurvival(stdout, h);
-  esl_sxp_Plot(stdout, mu, lambda, tau,
-	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
-
-  /* ML fit to complete data, and plot fitted survival curve */
-  esl_sxp_FitComplete(data, ndata, &emu, &elam, &etau);
-  esl_sxp_Plot(stdout, emu, elam, etau,
-	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
-
-  /* ML fit to binned data, plot fitted survival curve  */
-  esl_sxp_FitCompleteBinned(h, &emu, &elam, &etau);
-  esl_sxp_Plot(stdout, emu, elam, etau,
-	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
-
-  esl_randomness_Destroy(r);
-  esl_histogram_Destroy(h);
-  return 0;
-}
-/*::cexcerpt::sxp_example::end::*/
-#endif /*eslSTRETCHEXP_EXAMPLE*/
 
 
 
 /****************************************************************************
- * Test driver
+ * 7. Test driver
  ****************************************************************************/ 
 #ifdef eslSTRETCHEXP_TESTDRIVE
 /* gcc -g -Wall -I. -L . -o stretchexp_utest -DeslSTRETCHEXP_TESTDRIVE esl_stretchexp.c -leasel -lm
@@ -621,7 +584,7 @@ main(int argc, char **argv)
 
   if (plotfile != NULL) {
     if ((pfp = fopen(plotfile, "w")) == NULL) 
-      ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
+      esl_fatal("Failed to open plotfile");
   }
 
   for (i = 0; i < n; i++)
@@ -635,23 +598,17 @@ main(int argc, char **argv)
   if (be_verbose)
     printf("Complete data fit:  mu = %f   lambda = %f   tau = %f\n", 
 	   emu, elambda, etau);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 10%\n");
-  if (fabs( (etau-tau)/tau ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted tau > 10%\n");
+  if (fabs( (emu-mu)/mu )             > 0.01) esl_fatal("Error in (complete) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (complete) fitted lambda > 10%\n");
+  if (fabs( (etau-tau)/tau )          > 0.10) esl_fatal("Error in (complete) fitted tau > 10%\n");
 
   esl_sxp_FitCompleteBinned(h, &emu, &elambda, &etau);
   if (be_verbose)
     printf("Binned data fit:  mu = %f   lambda = %f   tau = %f\n", 
 	   emu, elambda, etau);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
-  if (fabs( (etau-tau)/tau ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted tau > 10%\n");
+  if (fabs( (emu-mu)/mu )             > 0.01) esl_fatal("Error in (binned) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) esl_fatal("Error in (binned) fitted lambda > 10%\n");
+  if (fabs( (etau-tau)/tau )          > 0.10) esl_fatal("Error in (binned) fitted tau > 10%\n");
 
   if (plot_pdf)     esl_sxp_Plot(pfp, mu, lambda, tau, &esl_sxp_pdf,     xmin, xmax, xstep);
   if (plot_logpdf)  esl_sxp_Plot(pfp, mu, lambda, tau, &esl_sxp_logpdf,  xmin, xmax, xstep);
@@ -668,16 +625,75 @@ main(int argc, char **argv)
 }
 #endif /*eslSTRETCHEXP_TESTDRIVE*/
 
+/****************************************************************************
+ * Example main()
+ ****************************************************************************/ 
+#ifdef eslSTRETCHEXP_EXAMPLE
+/*::cexcerpt::sxp_example::begin::*/
+/* compile:
+   gcc -g -Wall -I. -o example -DeslSTRETCHEXP_EXAMPLE\
+     -DeslAUGMENT_HISTOGRAM -DeslAUGMENT_RANDOM -DeslAUGMENT_MINIMIZER\
+      esl_stretchexp.c esl_histogram.c esl_random.c esl_minimizer.c esl_stats.c esl_vectorops.c easel.c -lm
+ */
+#include <stdio.h>
+#include "easel.h"
+#include "esl_random.h"
+#include "esl_histogram.h"
+#include "esl_stretchexp.h"
+
+int
+main(int argc, char **argv)
+{
+  double mu         = -50.0;
+  double lambda     = 2.5;
+  double tau        = 0.7;
+  ESL_HISTOGRAM  *h = esl_histogram_CreateFull(mu, 100., 0.1);
+  ESL_RANDOMNESS *r = esl_randomness_Create(0);
+  int    n          = 10000;
+  double *data;
+  int     ndata;
+  double emu, elam, etau;
+  int    i;
+  double x;
+
+  for (i = 0; i < n; i++)
+    {
+      x  =  esl_sxp_Sample(r, mu, lambda, tau);
+      esl_histogram_Add(h, x);
+    }
+  esl_histogram_GetData(h, &data, &ndata);
+
+  /* Plot the empirical (sampled) and expected survivals */
+  esl_histogram_PlotSurvival(stdout, h);
+  esl_sxp_Plot(stdout, mu, lambda, tau,
+	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
+
+  /* ML fit to complete data, and plot fitted survival curve */
+  esl_sxp_FitComplete(data, ndata, &emu, &elam, &etau);
+  esl_sxp_Plot(stdout, emu, elam, etau,
+	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
+
+  /* ML fit to binned data, plot fitted survival curve  */
+  esl_sxp_FitCompleteBinned(h, &emu, &elam, &etau);
+  esl_sxp_Plot(stdout, emu, elam, etau,
+	       &esl_sxp_surv,  h->xmin, h->xmax, 0.1);
+
+  esl_randomness_Destroy(r);
+  esl_histogram_Destroy(h);
+  return 0;
+}
+/*::cexcerpt::sxp_example::end::*/
+#endif /*eslSTRETCHEXP_EXAMPLE*/
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_stretchexp.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_stretchexp.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
