@@ -59,23 +59,6 @@ esl_tree_Create(int ntaxa)
   T->ld     = NULL;
   T->rd     = NULL;
   
-  /* 2nd allocation round */
-  T->N    = ntaxa;
-  ESL_ALLOC(T->parent, sizeof(int)    * (ntaxa-1));
-  ESL_ALLOC(T->left,   sizeof(int)    * (ntaxa-1));
-  ESL_ALLOC(T->right,  sizeof(int)    * (ntaxa-1));
-  ESL_ALLOC(T->ld,     sizeof(double) * (ntaxa-1));
-  ESL_ALLOC(T->rd,     sizeof(double) * (ntaxa-1));
-  
-  for (i = 0; i < ntaxa-1; i++)
-    {
-      T->parent[i] = 0;
-      T->left[i  ] = 0;
-      T->right[i]  = 0;
-      T->ld[i]   = 0.;
-      T->rd[i]   = 0.;
-    }
-
   /* Optional info starts NULL
    */
   T->taxaparent  = NULL;
@@ -95,6 +78,22 @@ esl_tree_Create(int ntaxa)
   T->show_quoted_labels       = FALSE;
   T->show_numeric_taxonlabels = TRUE;
 
+  /* 2nd allocation round */
+  T->N    = ntaxa;
+  ESL_ALLOC(T->parent, sizeof(int)    * (ntaxa-1));
+  ESL_ALLOC(T->left,   sizeof(int)    * (ntaxa-1));
+  ESL_ALLOC(T->right,  sizeof(int)    * (ntaxa-1));
+  ESL_ALLOC(T->ld,     sizeof(double) * (ntaxa-1));
+  ESL_ALLOC(T->rd,     sizeof(double) * (ntaxa-1));
+  
+  for (i = 0; i < ntaxa-1; i++)
+    {
+      T->parent[i] = 0;
+      T->left[i  ] = 0;
+      T->right[i]  = 0;
+      T->ld[i]   = 0.;
+      T->rd[i]   = 0.;
+    }
   T->nalloc = ntaxa;
   return T;
   
@@ -259,8 +258,9 @@ esl_tree_SetTaxaParents(ESL_TREE *T)
   int i;
   int status;
 
-  if (T->taxaparent != NULL) return eslOK; /* map already exists. */
+  if (T->taxaparent != NULL) return eslOK;         // map already exists. 
   ESL_ALLOC(T->taxaparent, sizeof(int) * T->N);
+  for (i = 0; i < T->N; i++) T->taxaparent[i] = 0; // solely to calm static analyzers.
 
   for (i = 0; i < T->N-1; i++)	/* traversal order doesn't matter */
     {
@@ -341,7 +341,7 @@ esl_tree_SetTaxonlabels(ESL_TREE *T, char **names)
   int status;
   
   if (T->taxonlabel != NULL) esl_Free2D((void **) T->taxonlabel, T->N);
-  ESL_ALLOC(T->taxonlabel, sizeof(char **) * T->nalloc);
+  ESL_ALLOC(T->taxonlabel, sizeof(char *) * T->nalloc);
   for (i = 0; i < T->nalloc; i++) T->taxonlabel[i] = NULL;
 
   if (names != NULL) 
@@ -389,7 +389,6 @@ esl_tree_RenumberNodes(ESL_TREE *T)
   int        status;
   int        needs_rearranging = FALSE;
 
-
   /* Pass 1. Preorder traverse of T by its children links;
    *         construct map[old] -> new.
    */
@@ -410,9 +409,8 @@ esl_tree_RenumberNodes(ESL_TREE *T)
    *         (traversal order doesn't matter here)
    */
   if (( T2 = esl_tree_Create(T->nalloc)) == NULL) { status = eslEMEM; goto ERROR; };
-  T2->N = T->N;
   if (T->nodelabel   != NULL) {
-    ESL_ALLOC(T2->nodelabel,   sizeof(char *) * (T2->nalloc-1));
+    ESL_ALLOC(T2->nodelabel,  sizeof(char *) * (T2->nalloc-1));
     for (v = 0; v < T2->nalloc-1; v++) T2->nodelabel[v] = NULL;
   }
   if (T->taxaparent != NULL)  {
@@ -431,15 +429,15 @@ esl_tree_RenumberNodes(ESL_TREE *T)
       T2->rd[map[v]]     = T->rd[v];
   
       if (T->taxaparent != NULL) {
-	if (T->left[v]  <= 0) T2->taxaparent[T->left[v]]  = map[v];
-	if (T->right[v] <= 0) T2->taxaparent[T->right[v]] = map[v];
+	if (T->left[v]  <= 0) T2->taxaparent[-(T->left[v])]  = map[v];
+	if (T->right[v] <= 0) T2->taxaparent[-(T->right[v])] = map[v];
       }
 
-      if (T->nodelabel != NULL)
-	T2->nodelabel[map[v]] = T2->nodelabel[v];
+      if (T->nodelabel != NULL) 
+	esl_strdup(T->nodelabel[v], -1, &(T2->nodelabel[map[v]]));
     }
 
-  /* Finally, swap the new guts of T2 with the old guts of T;
+ /* Finally, swap the new guts of T2 with the old guts of T;
    * destroy T2 and return. T is now renumbered.
    */
   ESL_SWAP(T->parent,     T2->parent,      int *);
@@ -486,7 +484,6 @@ esl_tree_VerifyUltrametric(ESL_TREE *T)
   for (i = 0; i < T->N; i++)
     {
       d[i]   = 0.0;
-      child  = i;
       parent = T->taxaparent[i];
       if       (T->left[parent]  == -i) d[i] += T->ld[parent];
       else if  (T->right[parent] == -i) d[i] += T->rd[parent];
@@ -648,6 +645,7 @@ static int
 newick_validate_unquoted(char *label)
 {
   char *sptr;
+
   for (sptr = label; *sptr != '\0'; sptr++)
     {
       if (! isprint(*sptr))                  return eslFAIL;
@@ -756,7 +754,7 @@ newick_write_nodelabel(FILE *fp, ESL_TREE *T, int v)
   else if (newick_validate_quoted(T->nodelabel[v]) == eslOK)
     status = newick_write_quoted(fp, T->nodelabel[v]);
   else
-    ESL_EXCEPTION(eslECORRUPT, "bad node label");
+    ESL_EXCEPTION(eslECORRUPT, "bad node label\n");
 
   return status;
 }
@@ -1028,7 +1026,7 @@ newick_parse_quoted_label(FILE *fp, char *buf, int *pos, int *nc, char **ret_lab
   }
   
   /* backtrack over any trailing whitespace and nul-terminate. */
-  while (isspace(label[n-1]) && n > 0) n--; 
+  while (n>0 && isspace(label[n-1])) n--; 
   label[n] = '\0';
   *ret_label = label;
   return eslOK;
@@ -1107,7 +1105,7 @@ newick_parse_branchlength(FILE *fp, char *buf, int *pos, int *nc, double *ret_d)
   ESL_ALLOC(branch, sizeof(char) * nalloc);
 
   if (buf[*pos] != ':') { status = eslEFORMAT; goto ERROR; }
-  if (newick_advance_buffer(fp, buf, pos, nc) != eslOK)  goto ERROR;
+  if (( status = newick_advance_buffer(fp, buf, pos, nc)) != eslOK)  goto ERROR;
 
   while (1) {
     if (strchr("(]",          buf[*pos]) != NULL) { status = eslEFORMAT; goto ERROR; }
@@ -1248,6 +1246,7 @@ esl_tree_ReadNewick(FILE *fp, char *errbuf, ESL_TREE **ret_T)
    */
   while ((status = esl_stack_CPop(cs, &c)) == eslOK)
     {
+ 
       if (newick_skip_whitespace(fp, buf, &pos, &nc) != eslOK) 
 	ESL_XFAIL(eslEFORMAT, errbuf, "file ended prematurely.");
 
@@ -1264,8 +1263,6 @@ esl_tree_ReadNewick(FILE *fp, char *errbuf, ESL_TREE **ret_T)
 	{
 	  if (buf[pos] != ';')
 	    ESL_XFAIL(eslEFORMAT, errbuf, "expected a semicolon, saw %c.", buf[pos]);
-	  if (newick_advance_buffer(fp, buf, &pos, &nc) == eslEOF)
-	    ESL_XFAIL(eslEFORMAT, errbuf, "file ended prematurely.");
 	  break;		/* end of the Newick file */
 	}
 
@@ -1396,6 +1393,7 @@ esl_tree_ReadNewick(FILE *fp, char *errbuf, ESL_TREE **ret_T)
     }
 
   esl_tree_RenumberNodes(T);
+
   esl_stack_Destroy(cs);
   esl_stack_Destroy(vs);
   *ret_T = T;
@@ -1651,8 +1649,8 @@ cluster_engine(ESL_DMATRIX *D_original, int mode, ESL_TREE **ret_T)
        */
       T->ld[N-2] = T->rd[N-2] = height[N-2];
       if (! T->is_linkage_tree) {
-	if (idx[i] > 0) T->ld[N-2] -= height[idx[i]];
-	if (idx[j] > 0) T->rd[N-2] -= height[idx[j]];      
+	if (idx[i] > 0) T->ld[N-2] = ESL_MAX(0., T->ld[N-2] - height[idx[i]]);  // max to 0, to avoid fp roundoff giving us negative length
+	if (idx[j] > 0) T->rd[N-2] = ESL_MAX(0., T->rd[N-2] - height[idx[j]]);      
       }
       
       /* If either node was an internal node, record parent in it.
@@ -1846,7 +1844,7 @@ esl_tree_Simulate(ESL_RANDOMNESS *r, int N, ESL_TREE **ret_T)
    * node (in <branchpapa>) and a 0/1 flag (in <branchside>)
    * for the branch to the left vs. right child.
    */
-  if ((T = esl_tree_Create(N)) == NULL)  goto ERROR;
+  if ((T = esl_tree_Create(N)) == NULL)  { status = eslEMEM; goto ERROR; }
   ESL_ALLOC(branchpapa, sizeof(int) * N);
   ESL_ALLOC(branchside, sizeof(int) * N);
   
@@ -2136,13 +2134,13 @@ main(int argc, char **argv)
 int main(int argc, char **argv)
 {
   ESL_TREE     *tree = NULL;
-  ESLX_MSAFILE *afp  = NULL;
+  ESL_MSAFILE  *afp  = NULL;
   ESL_MSA      *msa  = NULL;
   ESL_DMATRIX  *D    = NULL;
 
-  eslx_msafile_Open(NULL, argv[1], NULL, eslMSAFILE_UNKNOWN, NULL, &afp);
-  eslx_msafile_Read(afp, &msa);
-  eslx_msafile_Close(afp);
+  esl_msafile_Open(NULL, argv[1], NULL, eslMSAFILE_UNKNOWN, NULL, &afp);
+  esl_msafile_Read(afp, &msa);
+  esl_msafile_Close(afp);
 
   esl_dst_CDiffMx(msa->aseq, msa->nseq, &D);
   esl_tree_UPGMA(D, &tree);
@@ -2190,13 +2188,10 @@ int main(int argc, char **argv)
 
 /*****************************************************************  
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- * 
- * SVN $Id: esl_tree.c 728 2011-10-24 21:32:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_tree.c $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
