@@ -115,21 +115,38 @@ esl_vec_IIncrement(int *v, int n, int x)
  *            
  *            <esl_vec_FSum()> and <esl_vec_ISum()> do the same,
  *            but for float and integer vectors.
+ *            
+ *            The floating point summations use Kahan compensated
+ *            summation, in order to minimize roundoff error
+ *            accumulation.  Additionally, I believe they are most
+ *            accurate if vec[] is sorted in increasing order, from
+ *            small to large, so you may consider sorting <vec> before
+ *            summing it.
  */
 double 
 esl_vec_DSum(double *vec, int n)
 {
   double sum = 0.;
+  double y,t,c; 
   int    x;
-  for (x = 0; x < n; x++) sum += vec[x];
+
+  c = 0.0;
+  for (x = 0; x < n; x++) {
+    y = vec[x] - c; t = sum + y; c = (t-sum)-y; sum = t; 
+  }
   return sum;
 }
 float 
 esl_vec_FSum(float *vec, int n)
 {
   float sum = 0.;
+  float y,t,c;
   int   x;
-  for (x = 0; x < n; x++) sum += vec[x];
+
+  c = 0.0;
+  for (x = 0; x < n; x++) {
+    y = vec[x] - c; t = sum + y; c = (t-sum)-y; sum = t; 
+  }
   return sum;
 }
 int
@@ -246,6 +263,11 @@ esl_vec_ICopy(const int *src, const int n, int *dest)
  *            for floating point comparisons, and strict equality
  *            for integer comparisons. Return <eslOK>
  *            if the vectors are equal, and <eslFAIL> if not.
+ *            
+ *            If <n=0>, the test always succeeds. In this case, either
+ *            <vec1> and <vec2> (or both) may be <NULL>.  This
+ *            accommodates an occasional convention of leaving empty
+ *            vectors <NULL>.
  *
  *            <esl_vec_FCompare()> and <esl_vec_ICompare()> do the same,
  *            for float and integer vectors.
@@ -327,8 +349,17 @@ esl_vec_ISwap(int *vec1, int *vec2, int n)
  *            <vec> and <rev> can be the same, in which case <vec> is
  *            reversed in place.
  *            
- *            <esl_vec_FReverse()> and <esl_vec_IReverse()>
- *            do the same, for float and integer values.
+ *            <esl_vec_FReverse()>, <esl_vec_IReverse()>, and 
+ *            <esl_vec_CReverse()> do the same, for float, integer,
+ *            and char arrays. 
+ *            
+ *            <esl_vec_CReverse()> needs to be used carefully if
+ *            <vec> is a NUL-terminated string, instead of an array.
+ *            If you reverse a string <s> in place (i.e. 
+ *              <esl_vec_CReverse(s, s, n)>), the trailing NUL will
+ *            still be there, and you're fine. If you reverse string
+ *            <s> into new storage <s2>, you'll need to NUL-terminate
+ *            <s2> yourself.
  */
 void
 esl_vec_DReverse(double *vec, double *rev, int n)
@@ -372,7 +403,20 @@ esl_vec_IReverse(int *vec, int *rev, int n)
     }
   if (n%2) rev[i] = vec[i];
 }
+void
+esl_vec_CReverse(char *vec, char *rev, int n)
+{
+  int i;
+  char x;
 
+  for (i = 0; i < n/2; i++)
+    {
+      x          = vec[n-i-1];
+      rev[n-i-1] = vec[i];
+      rev[i]     = x;
+    }
+  if (n%2) rev[i] = vec[i];
+}
 
 
 
@@ -507,6 +551,9 @@ esl_vec_IMin(const int *vec, int n)
  *            in <vec>. In case of ties, the element with the smallest index
  *            is returned. 
  *            
+ *            <n> can be 0 and <vec> can be <NULL>, in which case the
+ *            function returns 0.
+ *            
  *            <esl_vec_FArgMax()> and <esl_vec_IArgMax()> do the same,
  *            for float and integer vectors.
  *            
@@ -1407,13 +1454,13 @@ int main(void)
 
 /*****************************************************************  
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *
- * SVN $Id: esl_vectorops.c 762 2012-05-11 21:01:15Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_vectorops.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
