@@ -1,4 +1,20 @@
 /* Statistical routines for Weibull distributions.
+ *
+ * Contents:
+ *   1. Routines for evaluating densities and distributions.
+ *   2. Generic API routines, for general interface w/ histogram module
+ *   3. Dumping plots to files
+ *   4. Sampling                    (augmentation: random)
+ *   5. ML fitting to complete data (augmentation: minimizer)
+ *   6. ML fitting to binned data   (augmentation: histogram, minimizer)
+ *   7. Test driver
+ *   8. Example 
+ *   9. Copyright and licence information.
+ *   
+ * To-do:
+ *    - Fit*() functions should return eslEINVAL on n=0, eslENORESULT
+ *      on failure due to small n. Compare esl_gumbel. xref J12/93.  
+ *      SRE, Wed Nov 27 11:13:48 2013
  */
 #include "esl_config.h"
 
@@ -22,7 +38,7 @@
 
 
 /****************************************************************************
- * Routines for evaluating densities and distributions
+ * 1. Routines for evaluating densities and distributions
  ****************************************************************************/ 
 /* mu <= x < infinity   
  *    However, x=mu can be a problem: 
@@ -94,7 +110,7 @@ esl_wei_cdf(double x, double mu, double lambda, double tau)
   double tly = tau * log(y);
 
   if      (x <= mu)                return 0.0;
-  else if (fabs(tly) < eslSMALLX1) return exp(tly);
+  else if (fabs(tly) < eslSMALLX1) return exp(tly); 
   else                             return 1 - exp(-exp(tly));
 }
 
@@ -171,7 +187,7 @@ esl_wei_invcdf(double p, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Generic API routines: for general interface w/ histogram module
+ * 2. Generic API routines: for general interface w/ histogram module
  ****************************************************************************/ 
 
 /* Function:  esl_wei_generic_pdf()
@@ -230,7 +246,7 @@ esl_wei_generic_invcdf(double p, void *params)
 
 
 /****************************************************************************
- * Routines for dumping plots for files
+ * 3. Dumping plots for files
  ****************************************************************************/ 
 
 /* Function:  esl_wei_Plot()
@@ -263,7 +279,7 @@ esl_wei_Plot(FILE *fp, double mu, double lambda, double tau,
 
 
 /****************************************************************************
- * Routines for sampling (requires augmentation w/ random module)
+ * 4. Sampling (augmentation: random)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_RANDOM
 
@@ -284,7 +300,7 @@ esl_wei_Sample(ESL_RANDOMNESS *r, double mu, double lambda, double tau)
 
 
 /****************************************************************************
- * Maximum likelihood fitting
+ * 5. ML fitting to complete data (augmentation: minimizer)
  ****************************************************************************/ 
 #ifdef eslAUGMENT_MINIMIZER
 /* Easel's conjugate gradient descent code allows a single void ptr to
@@ -393,8 +409,14 @@ esl_wei_FitComplete(double *x, int n, double *ret_mu,
   *ret_tau    = exp(p[1]);
   return status;
 }
+#endif /*eslAUGMENT_MINIMIZER*/
+
+/*****************************************************************
+ * 6. ML fitting to binned data (augmentation: histogram, minimizer)
+ *****************************************************************/
 
 #ifdef eslAUGMENT_HISTOGRAM
+#ifdef eslAUGMENT_MINIMIZER
 struct wei_binned_data {
   ESL_HISTOGRAM *h;	/* contains the binned observed data        */
   double  mu;		/* mu is considered to be known, not fitted */
@@ -432,8 +454,10 @@ wei_binned_func(double *p, int nparam, void *dptr)
       tmp = esl_wei_cdf(bi, data->mu, lambda, tau) -
             esl_wei_cdf(ai, data->mu, lambda, tau);
 
-      if (tmp == 0.) return eslINFINITY;
-      ESL_DASSERT1( (tmp > 0.)); 
+      /* for cdf~1.0, numerical roundoff error can create tmp<0 by a
+       * teensy amount; tolerate that, but catch anything worse */
+      ESL_DASSERT1( (tmp + 1e-7 > 0.)); 
+      if (tmp <= 0.) return eslINFINITY;
 
       logL += h->obs[i] * log(tmp);
     }
@@ -527,8 +551,129 @@ esl_wei_FitCompleteBinned(ESL_HISTOGRAM *h, double *ret_mu,
 /*--------------------------- end fitting ----------------------------------*/
 
 
+
+
 /****************************************************************************
- * Example main()
+ * 7. Test driver
+ ****************************************************************************/ 
+#ifdef eslWEIBULL_TESTDRIVE
+/* Compile:
+   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test -DeslWEIBULL_TESTDRIVE\
+      esl_weibull.c -leasel -lm
+*/
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "easel.h"
+#include "esl_getopts.h"
+#include "esl_random.h"
+#include "esl_histogram.h"
+#include "esl_weibull.h"
+
+static ESL_OPTIONS options[] = {
+  /* name           type      default  env  range toggles reqs incomp  help                                                  docgroup*/
+  { "-h",        eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",                     0 }, 
+  { "-l",        eslARG_REAL,   "1.0", NULL, NULL,  NULL,  NULL, NULL, "set slope of sampled variates (lambda parameter) to <x> ", 0 }, 
+  { "-m",        eslARG_REAL,  "10.0", NULL, NULL,  NULL,  NULL, NULL, "set location of sampled variates (mu parameter) to <x>",   0 }, 
+  { "-n",        eslARG_INT,  "10000", NULL, NULL,  NULL,  NULL, NULL, "set # of sampled variates to <n>",                         0 }, 
+  { "-o",    eslARG_OUTFILE,     NULL, NULL, NULL,  NULL,  NULL, NULL, "output histogram to file <f>",                             0 }, 
+  { "-s",        eslARG_INT,      "0", NULL, NULL,  NULL,  NULL, NULL, "set random number seed to <n>",                            0 },
+  { "-t",        eslARG_REAL,   "0.7", NULL, NULL,  NULL,  NULL, NULL, "set shape of sampled variates (tau parameter) to <x>",     0 }, 
+  { "-v",        eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "be more verbose in output",                                0 }, 
+  { "-w",        eslARG_REAL,   "0.1", NULL, NULL,  NULL,  NULL, NULL, "set width of histogram bins to <x>",                       0 }, 
+  { "--cdf",     eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of cumulative distribution",                     0 }, 
+  { "--logcdf",  eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of log cumulative distribution",                 0 }, 
+  { "--pdf",     eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of probability density",                         0 }, 
+  { "--logpdf",  eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of log probability density",                     0 }, 
+  { "--surv",    eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of survival P(s>x)",                             0 }, 
+  { "--logsurv", eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "dump plot of log survival, log(P(s>x))",                   0 }, 
+  { "--xL",      eslARG_REAL,    NULL, NULL, NULL,  NULL,  NULL, NULL, "set minimum x-axis value on dumped plots to <x>",          0 }, 
+  { "--xH",      eslARG_REAL,    NULL, NULL, NULL,  NULL,  NULL, NULL, "set maximum x-axis value on dumped plots to <x>",          0 }, 
+  { "--xS",      eslARG_REAL,    NULL, NULL, NULL,  NULL,  NULL, NULL, "set x-axis increment value on dumped plots to <x>",        0 }, 
+  {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
+};
+static char usage[]  = "[-options]";
+static char banner[] = "test driver for Easel's Weibull distribution module";
+
+int
+main(int argc, char **argv)
+{
+  ESL_GETOPTS    *go   = esl_getopts_CreateDefaultApp(options, 0, argc, argv, banner, usage);
+  ESL_RANDOMNESS *rng  = esl_randomness_Create(esl_opt_GetInteger(go, "-s"));
+  double  mu           = esl_opt_GetReal   (go, "-m");
+  double  lambda       = esl_opt_GetReal   (go, "-l");
+  double  tau          = esl_opt_GetReal   (go, "-t");
+  int     n            = esl_opt_GetInteger(go, "-n");
+  double  binwidth     = esl_opt_GetReal   (go, "-w");
+  int     plot_cdf     = esl_opt_GetBoolean(go, "--cdf");
+  int     plot_logcdf  = esl_opt_GetBoolean(go, "--logcdf");
+  int     plot_pdf     = esl_opt_GetBoolean(go, "--pdf");
+  int     plot_logpdf  = esl_opt_GetBoolean(go, "--logpdf");
+  int     plot_surv    = esl_opt_GetBoolean(go, "--surv");
+  int     plot_logsurv = esl_opt_GetBoolean(go, "--logsurv");
+  int     be_verbose   = esl_opt_GetBoolean(go, "-v");
+  char   *plotfile     = esl_opt_GetString (go, "-o");
+  ESL_HISTOGRAM  *h    = NULL;
+  int     xmin_set     = esl_opt_IsOn(go, "--xL");
+  double  xmin         = xmin_set ? esl_opt_GetReal(go, "--xL") : mu;
+  int     xmax_set     = esl_opt_IsOn(go, "--xH");
+  double  xmax         = xmax_set ? esl_opt_GetReal(go, "--xH") : mu+40*(1./lambda);
+  int     xstep_set    = esl_opt_IsOn(go, "--xH");
+  double  xstep        = xstep_set ? esl_opt_GetReal(go, "--xS") : 0.1;
+  FILE   *pfp          = stdout;
+  double  emu, elambda, etau;
+  int     i;
+  double  x;
+  double *data;
+  int     ndata;
+
+  fprintf(stderr, "## %s\n", argv[0]);
+  fprintf(stderr, "#  rng seed = %" PRIu32 "\n", esl_randomness_GetSeed(rng));
+
+  if (be_verbose) printf("Parametric:  mu = %f   lambda = %f    tau = %f\n", mu, lambda, tau);
+
+  h = esl_histogram_CreateFull(mu, 100., binwidth);
+  if (plotfile && (pfp = fopen(plotfile, "w")) == NULL) ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
+
+  for (i = 0; i < n; i++)
+    {
+      x = esl_wei_Sample(rng, mu, lambda, tau);
+      esl_histogram_Add(h, x);
+    }
+  esl_histogram_GetData(h, &data, &ndata);
+
+  esl_wei_FitComplete(data, ndata, &emu, &elambda, &etau);
+  if (be_verbose) printf("Complete data fit:  mu = %f   lambda = %f   tau = %f\n", emu, elambda, etau);
+  if (fabs( (emu-mu)/mu ) > 0.01)             ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 10%\n");
+  if (fabs( (etau-tau)/tau ) > 0.10)          ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted tau > 10%\n");
+
+  esl_wei_FitCompleteBinned(h, &emu, &elambda, &etau);
+  if (be_verbose)    printf("Binned data fit:  mu = %f   lambda = %f   tau = %f\n", emu, elambda, etau);
+  if (fabs( (emu-mu)/mu ) > 0.01)             ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted mu > 1%\n");
+  if (fabs( (elambda-lambda)/lambda ) > 0.10) ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
+  if (fabs( (etau-tau)/tau ) > 0.10)          ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
+
+  if (plot_pdf)     esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_pdf,     xmin, xmax, xstep);
+  if (plot_logpdf)  esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logpdf,  xmin, xmax, xstep);
+  if (plot_cdf)     esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_cdf,     xmin, xmax, xstep);
+  if (plot_logcdf)  esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logcdf,  xmin, xmax, xstep);
+  if (plot_surv)    esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_surv,    xmin, xmax, xstep);
+  if (plot_logsurv) esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logsurv, xmin, xmax, xstep);
+
+  if (plotfile) fclose(pfp);
+  esl_histogram_Destroy(h);
+  esl_randomness_Destroy(rng);
+  esl_getopts_Destroy(go);
+
+  fprintf(stderr, "#  status = ok\n");
+  return 0;
+}
+#endif /*eslWEIBULL_TESTDRIVE*/
+
+/****************************************************************************
+ * 8. Example 
  ****************************************************************************/ 
 #ifdef eslWEIBULL_EXAMPLE
 /*::cexcerpt::wei_example::begin::*/
@@ -590,145 +735,15 @@ main(int argc, char **argv)
 #endif /*eslWEIBULL_EXAMPLE*/
 
 
-
-
-
-/****************************************************************************
- * Test driver
- ****************************************************************************/ 
-#ifdef eslWEIBULL_TESTDRIVE
-/* Compile:
-   gcc -g -Wall -I. -I ~/src/easel -L ~/src/easel -o test -DeslWEIBULL_TESTDRIVE\
-    esl_weibull.c -leasel -lm
-*/
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "easel.h"
-#include "esl_random.h"
-#include "esl_histogram.h"
-#include "esl_weibull.h"
-
-int
-main(int argc, char **argv)
-{
-  ESL_HISTOGRAM  *h;
-  ESL_RANDOMNESS *r;
-  double  mu        = 10.0;
-  double  lambda    =  1.0;  
-  double  tau       =  0.7;
-  int     n         = 10000;
-  double  binwidth  = 0.1;
-  double  emu, elambda, etau;
-  int     i;
-  double  x;
-  double *data;
-  int     ndata;
-
-  int     opti;
-  int     be_verbose   = FALSE;
-  char   *plotfile     = NULL;
-  FILE   *pfp          = stdout;
-  int     plot_pdf     = FALSE;
-  int     plot_logpdf  = FALSE;
-  int     plot_cdf     = FALSE;
-  int     plot_logcdf  = FALSE;
-  int     plot_surv    = FALSE;
-  int     plot_logsurv = FALSE;
-  int     xmin_set     = FALSE;
-  double  xmin;
-  int     xmax_set     = FALSE;
-  double  xmax;
-  int     xstep_set    = FALSE;
-  double  xstep;
-
-  for (opti = 1; opti < argc && *(argv[opti]) == '-'; opti++)
-    {
-      if      (strcmp(argv[opti], "-m")  == 0) mu           = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-l")  == 0) lambda       = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-n")  == 0) n            = atoi(argv[++opti]);
-      else if (strcmp(argv[opti], "-o")  == 0) plotfile     = argv[++opti];
-      else if (strcmp(argv[opti], "-v")  == 0) be_verbose   = TRUE;
-      else if (strcmp(argv[opti], "-t")  == 0) tau          = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-w")  == 0) binwidth     = atof(argv[++opti]);
-      else if (strcmp(argv[opti], "-C")  == 0) plot_cdf     = TRUE;
-      else if (strcmp(argv[opti], "-LC") == 0) plot_logcdf  = TRUE;
-      else if (strcmp(argv[opti], "-P")  == 0) plot_pdf     = TRUE;
-      else if (strcmp(argv[opti], "-LP") == 0) plot_logpdf  = TRUE;
-      else if (strcmp(argv[opti], "-S")  == 0) plot_surv    = TRUE;
-      else if (strcmp(argv[opti], "-LS") == 0) plot_logsurv = TRUE;
-      else if (strcmp(argv[opti], "-XL") == 0) { xmin_set  = TRUE; xmin  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XH") == 0) { xmax_set  = TRUE; xmax  = atof(argv[++opti]); }
-      else if (strcmp(argv[opti], "-XS") == 0) { xstep_set = TRUE; xstep = atof(argv[++opti]); }
-      else ESL_EXCEPTION(eslEINVAL, "bad option");
-    }
-
-  if (be_verbose)
-    printf("Parametric:  mu = %f   lambda = %f    tau = %f\n", mu, lambda, tau);
-
-  r = esl_randomness_Create(0);
-  h = esl_histogram_CreateFull(mu, 100., binwidth);
-  if (plotfile != NULL) {
-    if ((pfp = fopen(plotfile, "w")) == NULL) 
-      ESL_EXCEPTION(eslFAIL, "Failed to open plotfile");
-  }
-  if (! xmin_set)  xmin  = mu;
-  if (! xmax_set)  xmax  = mu+40*(1./lambda);
-  if (! xstep_set) xstep = 0.1;
-
-  for (i = 0; i < n; i++)
-    {
-      x = esl_wei_Sample(r, mu, lambda, tau);
-      esl_histogram_Add(h, x);
-    }
-  esl_histogram_GetData(h, &data, &ndata);
-
-  esl_wei_FitComplete(data, ndata, &emu, &elambda, &etau);
-  if (be_verbose)
-    printf("Complete data fit:  mu = %f   lambda = %f   tau = %f\n", 
-	   emu, elambda, etau);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted lambda > 10%\n");
-  if (fabs( (etau-tau)/tau ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (complete) fitted tau > 10%\n");
-
-  esl_wei_FitCompleteBinned(h, &emu, &elambda, &etau);
-  if (be_verbose)
-    printf("Binned data fit:  mu = %f   lambda = %f   tau = %f\n", 
-	   emu, elambda, etau);
-  if (fabs( (emu-mu)/mu ) > 0.01)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted mu > 1%\n");
-  if (fabs( (elambda-lambda)/lambda ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
-  if (fabs( (etau-tau)/tau ) > 0.10)
-     ESL_EXCEPTION(eslFAIL, "Error in (binned) fitted lambda > 10%\n");
-
-  if (plot_pdf)     esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_pdf,     xmin, xmax, xstep);
-  if (plot_logpdf)  esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logpdf,  xmin, xmax, xstep);
-  if (plot_cdf)     esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_cdf,     xmin, xmax, xstep);
-  if (plot_logcdf)  esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logcdf,  xmin, xmax, xstep);
-  if (plot_surv)    esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_surv,    xmin, xmax, xstep);
-  if (plot_logsurv) esl_wei_Plot(pfp, mu, lambda, tau, &esl_wei_logsurv, xmin, xmax, xstep);
-
-  if (plotfile != NULL) fclose(pfp);
-  esl_histogram_Destroy(h);
-  esl_randomness_Destroy(r);
-  return 0;
-}
-#endif /*eslWEIBULL_TESTDRIVE*/
-
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  * 
- * SVN $Id: esl_weibull.c 727 2011-10-24 17:17:32Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_weibull.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
