@@ -1,27 +1,25 @@
 /* RNA secondary structure markup in WUSS notation.
  * 
- * SRE, Tue Feb 15 10:15:28 2005
- * SVN $Id: esl_wuss.h 664 2011-02-27 17:08:36Z eddys $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/esl_wuss.h $
  */
 #ifndef eslWUSS_INCLUDED
 #define eslWUSS_INCLUDED
 
 extern int esl_wuss2ct(char *ss, int len, int *ct);
 extern int esl_ct2wuss(int *ct, int n, char *ss);
+extern int esl_ct2simplewuss(int *ct, int n, char *ss);
 extern int esl_wuss2kh(char *ss, char *kh);
 extern int esl_kh2wuss(char *kh, char *ss);
 extern int esl_wuss_full(char *oldss, char *newss);
 extern int esl_wuss_nopseudo(char *ss1, char *ss2);
-
+extern int esl_wuss_reverse(char *ss, char *new);
 
 #endif /*eslWUSS_INCLUDED*/
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
