@@ -84,7 +84,7 @@ main(int argc, char **argv)
   char         *alifile = NULL;	               /* alignment file name             */
   int           infmt   = eslMSAFILE_UNKNOWN;  /* format code for alifile         */
   int           outfmt  = eslMSAFILE_UNKNOWN;  /* format code for output ali      */
-  ESLX_MSAFILE *afp     = NULL;	               /* open alignment file  normal version           */
+  ESL_MSAFILE  *afp     = NULL;	               /* open alignment file  normal version           */
   ESL_MSAFILE2 *afp2    = NULL;		       /* open alignment file: legacy small-mem version */
   FILE         *ofp;		               /* output file (default is stdout) */
   ESL_MSA      *msa     = NULL;	               /* one multiple sequence alignment */
@@ -259,12 +259,12 @@ main(int argc, char **argv)
   
   /* determine input/output formats */
   if (esl_opt_IsOn(go, "--informat")) {
-    infmt = eslx_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
+    infmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
     if (infmt == eslMSAFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --informat", esl_opt_GetString(go, "--informat")); 
     if (do_small && infmt != eslMSAFILE_PFAM) esl_fatal("small memory mode requires Pfam formatted alignments"); 
   }
   if (esl_opt_IsOn(go, "--outformat")) {
-    outfmt = eslx_msafile_EncodeFormat(esl_opt_GetString(go, "--outformat"));
+    outfmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--outformat"));
     if (outfmt == eslMSAFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --outformat", esl_opt_GetString(go, "--outformat")); 
     if (do_small && outfmt != eslMSAFILE_PFAM) esl_fatal("we can only output Pfam formatted alignments in small memory mode"); 
   }
@@ -308,8 +308,8 @@ main(int argc, char **argv)
     }
   else
     {
-      status = eslx_msafile_Open(NULL, alifile, NULL, infmt, NULL, &afp);
-      if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+      status = esl_msafile_Open(NULL, alifile, NULL, infmt, NULL, &afp);
+      if (status != eslOK) esl_msafile_OpenFailure(afp, status);
     }
 
   /* If nec, read mask from the mask file */
@@ -333,8 +333,8 @@ main(int argc, char **argv)
     }
   else
     {
-      status = eslx_msafile_Read(afp, &msa); /* if ! do_small, we read full aln into memory */
-      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
+      status = esl_msafile_Read(afp, &msa); /* if ! do_small, we read full aln into memory */
+      if (status != eslOK) esl_msafile_ReadFailure(afp, status);
 
       orig_alen = msa->alen;
       if(do_postprob && msa->pp == NULL && (! esl_opt_IsOn(go, "--ppcons"))) esl_fatal("-p was enabled, but the MSA has no posterior probability (#=GR PP) annotation.");
@@ -500,7 +500,6 @@ main(int argc, char **argv)
   if(do_gapthresh && do_postprob) { 
     for(apos = 0; apos < msa->alen; apos++) 
       useme_final[apos] = (useme_g[apos] && useme_pp[apos]) ? TRUE : FALSE;
-    nkept = esl_vec_ISum(useme_final, (int) msa->alen);
     if(be_verbose) { 
       nkept    = esl_vec_ISum(useme_final, (int) msa->alen);
       if(msa->rf == NULL) fprintf(stdout, "  %-19s  %7" PRId64 "  %7s  %7d  %7d  %7s  %7s  %13s\n", "gapfreq&postprobs", msa->alen, "-",   nkept, (int) msa->alen - nkept, "-", "-", "-");
@@ -522,7 +521,9 @@ main(int argc, char **argv)
    * Unless --small enabled, mask the alignment *
    ************************************************/
   if(! do_small) { 
-    if((status = esl_msa_ColumnSubset(msa, errbuf, useme_final)) != eslOK) esl_fatal(errbuf);
+    if (abc && (abc->type == eslRNA || abc->type == eslDNA) &&
+	(status = esl_msa_RemoveBrokenBasepairs(msa, errbuf, useme_final)) != eslOK) esl_fatal(errbuf);
+    if ((status = esl_msa_ColumnSubset         (msa, errbuf, useme_final)) != eslOK) esl_fatal(errbuf);
   } /* else we'll do it as we regurgitate it upon rereading below */
 
   /************************
@@ -530,7 +531,7 @@ main(int argc, char **argv)
    ************************/
   if (! do_small) 
     {
-      eslx_msafile_Write(ofp, msa, outfmt);
+      esl_msafile_Write(ofp, msa, outfmt);
     }
   else { 
     /* do_small==TRUE, we don't have the full msa stored, 
@@ -614,7 +615,7 @@ main(int argc, char **argv)
   if (msa)           esl_msa_Destroy(msa);
   if (abc)           esl_alphabet_Destroy(abc);
   if (w)             esl_stopwatch_Destroy(w);
-  if (afp)           eslx_msafile_Close(afp);
+  if (afp)           esl_msafile_Close(afp);
   if (afp2)          esl_msafile2_Close(afp2);
   esl_getopts_Destroy(go);
   return 0;
@@ -1208,13 +1209,10 @@ parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- *
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/miniapps/esl-alimask.c $
- * SVN $Id: esl-alistat.c 393 2009-09-27 12:04:55Z eddys $
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
