@@ -5,6 +5,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 
 #include "easel.h"
 #include "esl_getopts.h"
@@ -21,7 +22,7 @@ static char usage[] = "[-options] <format> <seqfile>\n\
   Output format choices: Unaligned      Aligned    \n\
                          -----------    -------    \n\
                          fasta          a2m        \n\
-                                        afa        \n\
+                         hmmpgmd        afa        \n\
                                         clustal    \n\
                                         clustallike\n\
                                         pfam       \n\
@@ -44,7 +45,7 @@ static ESL_OPTIONS options[] = {
   { "-r",         eslARG_NONE,   FALSE, NULL, NULL, NULL, NULL,       "-d",                  "convert to RNA alphabet (T->U)",                     0 }, 
   { "-u",         eslARG_NONE,   FALSE, NULL, NULL, NULL, NULL,       "-l",                  "convert to upper case",                              0 },
   { "-x",         eslARG_NONE,   FALSE, NULL, NULL, NULL, NULL,       "-n",                  "convert non-IUPAC chars (e.g. X) in DNA to N",       0 },
-  { "--gapsym",   eslARG_STRING,  NULL, NULL, NULL, NULL, NULL,       "--mingap,--nogap",    "convert all gaps to character <c>",                  0 },
+  { "--gapsym",   eslARG_STRING,  NULL, NULL, NULL, NULL, NULL,       "--mingap,--nogap",    "convert all gaps to character <s>",                  0 },
   { "--informat", eslARG_STRING,  NULL, NULL, NULL, NULL, NULL,       NULL,                  "input sequence file is in format <s>",               0 },
   { "--mingap",   eslARG_NONE,   FALSE, NULL, NULL, NULL, NULL,       "--nogap",             "remove columns containing all gaps (seqfile=MSA)",   0 },
   { "--keeprf",   eslARG_NONE,   FALSE, NULL, NULL, NULL, "--mingap", NULL,                  "with --mingap, keep all nongap #=GC RF columns",     0 },
@@ -57,14 +58,15 @@ static ESL_OPTIONS options[] = {
   { "--rename",   eslARG_STRING, FALSE, NULL, NULL, NULL, NULL,       NULL,                  "rename and number each sequence <s>.<n>",            0 },
   { "--replace",  eslARG_STRING, FALSE, NULL, NULL, NULL, NULL,       NULL,                  "<s> = <s1>:<s2> replace characters in <s1> with those in <s2>", 0},
   { "--small",    eslARG_NONE,   FALSE, NULL, NULL, NULL, NULL,       INCOMPATWITHSMALLOPT,  "use minimal RAM, input must be pfam, output must be afa or pfam",0 },
+  { "--id_map",   eslARG_STRING, FALSE, NULL, NULL, NULL, NULL,       NULL,                  "if format is hmmpgmd, put the id map into file <s>", 0 },
   { 0,0,0,0,0,0,0,0 },
 };
 
 static void symconvert(char *s, char *oldsyms, char *newsyms);
-static void regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsym, int force_lower, int force_upper, 
+static void regurgitate_pfam_as_afa(ESL_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsym, int force_lower, int force_upper, 
 				    int force_rna, int force_dna, int iupac_to_n, int x_is_bad, char *rename, char *rfrom, 
 				    char *rto, int *ret_reached_eof);
-static int  regurgitate_pfam_as_pfam(ESLX_MSAFILE *afp, FILE *ofp, char *gapsym, int force_lower, int force_upper, int force_rna, 
+static int  regurgitate_pfam_as_pfam(ESL_MSAFILE *afp, FILE *ofp, char *gapsym, int force_lower, int force_upper, int force_rna, 
 				     int force_dna, int iupac_to_n, int x_is_bad, int wussify, int dewuss, int fullwuss, 
 				     char *rfrom, char *rto);
 static int  parse_replace_string(const char *rstring, char **ret_from, char **ret_to);
@@ -79,6 +81,7 @@ main(int argc, char **argv)
   int          status;		                        /* return code from an Easel call          */
   FILE        *ofp;		                        /* output stream                           */
 
+
   char  *outfile;		/* output file, or NULL                      */
   int    force_rna;		/* TRUE to force RNA alphabet                */
   int    force_dna;		/* TRUE to force DNA alphabet                */
@@ -94,6 +97,7 @@ main(int argc, char **argv)
   int    fullwuss;		/* TRUE to convert simple WUSS to full WUSS  */
   char  *rename; 		/* if non-NULL rename seqs to <s>.<n>        */
   int    do_small;		/* TRUE to operate in small memory mode      */
+  int    do_fixbps;		/* TRUE to assume SS/SS_cons are for WUSS RNA*/
   char  *rstring;               /* <s> from --replace <s>                    */
   char  *rfrom, *rto;           /* <s1> and <s2> from --replace <s>=<s1>:<s2>*/
   int    reached_eof;           /* reached EOF? used only in small mem mode  */
@@ -101,6 +105,8 @@ main(int argc, char **argv)
   int    nali;                  /* number of alignments read                 */
   char   errbuf[eslERRBUFSIZE]; /* for error messages                        */
 
+
+
   /*****************************************************************
    * Parse the command line
    *****************************************************************/
@@ -158,6 +164,7 @@ main(int argc, char **argv)
   rename      = esl_opt_GetString (go, "--rename");
   do_small    = esl_opt_GetBoolean(go, "--small");
   rstring     = esl_opt_GetString( go, "--replace");
+  do_fixbps   = (force_rna || force_dna || wussify || dewuss || fullwuss) ? TRUE : FALSE;
 
   /* if --small, make sure infmt == pfam and (outfmt == afa || outfmt == pfam) */
   if(do_small && (infmt != eslMSAFILE_PFAM || (outfmt != eslMSAFILE_AFA && outfmt != eslMSAFILE_PFAM)))  
@@ -187,12 +194,11 @@ main(int argc, char **argv)
    */
   if (esl_sqio_IsAlignment(outfmt))
     {
-      ESLX_MSAFILE *afp;
+      ESL_MSAFILE *afp;
       ESL_MSA     *msa;
 
-      /* use text mode, deliberately, for maximum generality of conversion */
-      status = eslx_msafile_Open(NULL, infile, NULL, infmt, NULL, &afp);
-      if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+      status = esl_msafile_Open(NULL, infile, NULL, infmt, NULL, &afp);
+      if (status != eslOK) esl_msafile_OpenFailure(afp, status);
 
       if ( esl_opt_IsOn(go, "--ignore"))  esl_fatal("The --ignore option is unimplemented for alignment reformatting.");
       if ( esl_opt_IsOn(go, "--acceptx")) esl_fatal("The --acceptx option is unimplemented for alignment reformatting.");
@@ -212,23 +218,23 @@ main(int argc, char **argv)
 	    else if (status == eslEINVAL)  esl_fatal("--small alignment file parse error:\n%s\n", afp->errmsg);
 	    else if (status != eslOK)      esl_fatal("--small alignment file read failed with error code %d\n", status);
 	  }
-	  eslx_msafile_Close(afp);
+	  esl_msafile_Close(afp);
 	}
 	else { /* do_small enabled, but neither (infmt==pfam && outfmt=afa) nor (infmt==pfam && outfmt==pfam) */
 	  esl_fatal("--small requires '--informat pfam' and output format of either 'afa' or 'pfam'");
 	}
       }
       else { /* normal mode, --small not enabled */
-	while ((status = eslx_msafile_Read(afp, &msa)) != eslEOF)
+	while ((status = esl_msafile_Read(afp, &msa)) != eslEOF)
 	  {
-	    if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
+	    if (status != eslOK) esl_msafile_ReadFailure(afp, status);
 	    nali++;
 
-	    if (nali > 1 && ! eslx_msafile_IsMultiRecord(outfmt))
-	      esl_fatal("Input file contains >1 alignments, but %s formatted output file can only contain 1", eslx_msafile_DecodeFormat(outfmt));
+	    if (nali > 1 && ! esl_msafile_IsMultiRecord(outfmt))
+	      esl_fatal("Input file contains >1 alignments, but %s formatted output file can only contain 1", esl_msafile_DecodeFormat(outfmt));
 
-	    if (do_mingap)    if((status = esl_msa_MinimGaps(msa, errbuf, "-_.~", esl_opt_GetBoolean(go, "--keeprf"))) != eslOK) esl_fatal(errbuf);
-	    if (do_nogap)     if((status = esl_msa_NoGaps   (msa, errbuf, "-_.~"))                                     != eslOK) esl_fatal(errbuf);
+	    if (do_mingap)    if((status = esl_msa_MinimGapsText(msa, errbuf, "-_.~", esl_opt_GetBoolean(go, "--keeprf"), do_fixbps)) != eslOK) esl_fatal(errbuf);
+	    if (do_nogap)     if((status = esl_msa_NoGapsText   (msa, errbuf, "-_.~", do_fixbps))                                     != eslOK) esl_fatal(errbuf);
 	    if (rfrom)        esl_msa_SymConvert(msa, rfrom, rto);
 	    if (gapsym)       esl_msa_SymConvert(msa, "-_.", gapsym);
 	    if (force_lower)  esl_msa_SymConvert(msa,
@@ -284,73 +290,131 @@ main(int argc, char **argv)
 		      }
 	      }
 
-	    eslx_msafile_Write(ofp, msa, outfmt);
+	    esl_msafile_Write(ofp, msa, outfmt);
 	    esl_msa_Destroy(msa);
 	  }
-	eslx_msafile_Close(afp);
-      } 
+      }
+      esl_msafile_Close(afp);
     } /* end of alignment->alignment conversion */
   else
     { /* else: conversion to unaligned file formats */
       ESL_SQFILE  *sqfp;	/* open input sequence file                */
       ESL_SQ      *sq;		/* an input sequence                       */
+      char        *mapfile;  /* file name into which an hmmpgmd map file should be written */
+      FILE        *mapfp;    /* output stream for the map file                       */
 
       status = esl_sqfile_Open(infile, infmt, NULL, &sqfp);
       if (status == eslENOTFOUND)
-	esl_fatal("Couldn't open seqfile %s\n", infile);
+        esl_fatal("Couldn't open seqfile %s\n", infile);
       else if (status == eslEFORMAT)
-	esl_fatal("Couldn't determine format of seqfile %s\n", infile);      
+        esl_fatal("Couldn't determine format of seqfile %s\n", infile);
       else if (status == eslEINVAL)
-	esl_fatal("Can't autodetect format of stdin or .gz; use --informat\n");
+        esl_fatal("Can't autodetect format of stdin or .gz; use --informat\n");
       else if (status != eslOK)
-	esl_fatal("Open of seqfile %s failed, code %d\n", infile, status);
+        esl_fatal("Open of seqfile %s failed, code %d\n", infile, status);
       
       if ( esl_opt_IsOn(go, "--ignore"))  esl_sqio_Ignore  (sqfp, esl_opt_GetString(go, "--ignore"));
       if ( esl_opt_IsOn(go, "--acceptx")) esl_sqio_AcceptAs(sqfp, esl_opt_GetString(go, "--acceptx"), 'X');
 
       sq  = esl_sq_Create();
+
+      if ( outfmt == eslSQFILE_HMMPGMD ) {
+        int64_t res_cnt = 0;
+        char    timestamp[32];
+        time_t  date;
+
+        //will need to make two passes through the file, one to get sequence count, one to convert sequences
+        if (! esl_sqfile_IsRewindable(sqfp))
+          esl_fatal("Target sequence file %s isn't rewindable; can't produce a file in hmmpgmd format", infile);
+
+
+        //pick map file name, and open the file
+        if (esl_opt_IsUsed(go, "--id_map")) {
+          mapfile =  esl_opt_GetString( go, "--id_map");
+        } else {
+          esl_strdup(infile, -1, &mapfile);  // there will be an infile, because stdin was restricted above
+          esl_strcat(&mapfile, -1, ".map", 4);
+        }
+        if ((mapfp = fopen(mapfile, "w")) == NULL)
+            esl_fatal("Failed to open map output file %s\n", mapfile);
+
+        //get counts, and write out the map file
+        idx = 0;
+        while ((status = esl_sqio_Read(sqfp, sq)) == eslOK) {
+          res_cnt += sq->n;
+          esl_sq_Reuse(sq);
+          idx++;
+        }
+
+        /* status should be eslEOF on normal end; if it isn't, deal w/ error */
+        if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+                   sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+        else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
+                   status, sqfp->filename);
+
+        /* Print the first line of the hmmpgmd format, which contains database information.
+         * #<res_count> <seq_count> <db_count> <db_sequences_1> <db_sequences_before_removing_duplicates_1> <db_sequences_2> <db_sequences_before_removing_duplicates_2>  ... <date_stamp>
+         * It shoves all sequences into a single database, #1.
+         */
+        date = time(NULL);
+        ctime_r(&date, timestamp);
+        fprintf(mapfp, "%d\n", idx);
+        fprintf(ofp, "#%" PRId64 " %d %d %d %d %s", res_cnt, idx, 1, idx, idx, timestamp);
+
+        esl_sqfile_Position(sqfp, 0);
+      }
+
+
       idx = 0;
       while ((status = esl_sqio_Read(sqfp, sq)) == eslOK)
-	{
-	  if (rfrom!=NULL) symconvert(sq->seq, rfrom, rto);
-	  if (force_lower) symconvert(sq->seq, 
-				      "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
-				      "abcdefghijklmnopqrstuvwxyz");
-	  if (force_upper) symconvert(sq->seq, 
-				      "abcdefghijklmnopqrstuvwxyz",
-				      "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
-	  if (force_rna)   symconvert(sq->seq, "Tt", "Uu");
-	  if (force_dna)   symconvert(sq->seq, "Uu", "Tt");
-	  if (iupac_to_n)  symconvert(sq->seq, 
-				      "RYMKSWHBVDrymkswhbvd",
-				      "NNNNNNNNNNnnnnnnnnnn");
-	  if (x_is_bad)    symconvert(sq->seq, "Xx", "Nn");
-	  
-	  if (wussify && sq->ss != NULL) esl_kh2wuss(sq->ss, sq->ss);	    
-	  if (dewuss  && sq->ss != NULL) esl_wuss2kh(sq->ss, sq->ss);	    
+      {
+        if (rfrom!=NULL) symconvert(sq->seq, rfrom, rto);
+        if (force_lower) symconvert(sq->seq,
+                  "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
+                  "abcdefghijklmnopqrstuvwxyz");
+        if (force_upper) symconvert(sq->seq,
+                  "abcdefghijklmnopqrstuvwxyz",
+                  "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
+        if (force_rna)   symconvert(sq->seq, "Tt", "Uu");
+        if (force_dna)   symconvert(sq->seq, "Uu", "Tt");
+        if (iupac_to_n)  symconvert(sq->seq,
+                  "RYMKSWHBVDrymkswhbvd",
+                  "NNNNNNNNNNnnnnnnnnnn");
+        if (x_is_bad)    symconvert(sq->seq, "Xx", "Nn");
 
-	  if (fullwuss && sq->ss != NULL)
-	    {
-	      status = esl_wuss_full(sq->ss, sq->ss);
-	      if (status == eslESYNTAX) 
-		esl_fatal("Bad SS for %s: not in WUSS format\n", sq->name);
-	      else if (status != eslOK)
-		esl_fatal("Conversion of SS for %s failed, code %d\n", 
-			  sq->name, status);
-	    }
+        if (wussify && sq->ss != NULL) esl_kh2wuss(sq->ss, sq->ss);
+        if (dewuss  && sq->ss != NULL) esl_wuss2kh(sq->ss, sq->ss);
 
-	  if (rename) esl_sq_FormatName(sq, "%s.%d", rename, idx+1);
+        if (fullwuss && sq->ss != NULL)
+        {
+            status = esl_wuss_full(sq->ss, sq->ss);
+            if (status == eslESYNTAX)
+              esl_fatal("Bad SS for %s: not in WUSS format\n", sq->name);
+            else if (status != eslOK)
+              esl_fatal("Conversion of SS for %s failed, code %d\n",
+            sq->name, status);
+        }
 
-	  esl_sqio_Write(ofp, sq, outfmt, FALSE);
-	  esl_sq_Reuse(sq);
-	  idx++;
-	}
+        if ( outfmt == eslSQFILE_HMMPGMD ) {
+          fprintf(mapfp, "%d %s %s\n", idx+1, sq->name?sq->name:"", sq->desc?sq->desc:"");
+          esl_sq_FormatName(sq, "%d 1", idx+1);
+          esl_sq_FormatDesc(sq, "");
+        } else {
+          if (rename) esl_sq_FormatName(sq, "%s.%d", rename, idx+1);
+        }
+        esl_sqio_Write(ofp, sq, outfmt, FALSE);
+        esl_sq_Reuse(sq);
+        idx++;
+      }
       /* status should be eslEOF on normal end; if it isn't, deal w/ error */
       if      (status == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
 					       sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
       else if (status != eslEOF)     esl_fatal("Unexpected error %d reading sequence file %s",
 					       status, sqfp->filename);
       
+      if ( outfmt == eslSQFILE_HMMPGMD )
+        fclose(mapfp);
+
       esl_sq_Destroy(sq);
       esl_sqfile_Close(sqfp);
     } /* end of unaligned seq conversion */
@@ -441,7 +505,7 @@ symconvert(char *s, char *oldsyms, char *newsyms)
  * Returns void. Dies upon any input error.
  */
 static void
-regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsym, int force_lower, int force_upper, int force_rna, int force_dna, int iupac_to_n, int x_is_bad, char *rename, char *rfrom, char *rto, int *ret_reached_eof)
+regurgitate_pfam_as_afa(ESL_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsym, int force_lower, int force_upper, int force_rna, int force_dna, int iupac_to_n, int x_is_bad, char *rename, char *rfrom, char *rto, int *ret_reached_eof)
 {
   char      *p = NULL;
   esl_pos_t  n = 0;
@@ -487,7 +551,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
 
   /* Check the magic Stockholm header line, allowing blank lines */
   do { 
-    status = eslx_msafile_GetLine(afp, &p, &n);
+    status = esl_msafile_GetLine(afp, &p, &n);
     if      (status == eslEOF) return; 
     else if (status != eslOK)  esl_fatal("small mem parse error. problem reading line %d of msafile", (int) afp->linenumber);
   } while (esl_memspn(afp->line, afp->n, " \t") == afp->n  ||                 /* skip blank lines             */
@@ -496,7 +560,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
 
   if (! esl_memstrpfx(afp->line, afp->n, "# STOCKHOLM 1.")) esl_fatal("small mem parse failed (line %d): missing \"# STOCKHOLM\" header", (int) afp->linenumber);
 
-  while ((status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK) 
+  while ((status = esl_msafile_GetLine(afp, &p, &n)) == eslOK) 
     {
       while (n && ( *p == ' ' || *p == '\t')) { p++; n--; } /* skip leading whitespace */
 
@@ -510,12 +574,12 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
 	  if (esl_memstrcmp(tag, taglen, "AC"))
 	    { 
 	      if (! ac_fp && esl_tmpfile(ac_tmpfile, &ac_fp) != eslOK) esl_fatal("small mem parse failed, unable to open accession tmpfile");
-	      fprintf(ac_fp, "%.*s %.*s\n", (int) seqnamelen, seqname, (int) n, p);
+	      fprintf(ac_fp, "%.*s %.*s\n", (int) seqnamelen, seqname, (int) n, p);  // cppcheck thinks ac_fp is a "possible null pointer deference"; I think it's wrong.
 	    }
 	  if (esl_memstrcmp(tag, taglen, "DE"))
 	    { 
 	      if (! de_fp && esl_tmpfile(de_tmpfile, &de_fp) != eslOK) esl_fatal("small mem parse failed, unable to open description tmpfile");
-	      fprintf(de_fp, "%.*s %.*s\n", (int) seqnamelen, seqname, (int) n, p);
+	      fprintf(de_fp, "%.*s %.*s\n", (int) seqnamelen, seqname, (int) n, p); // cppcheck thinks de_fp is a "possible null pointer deference"; I think it's wrong.
 	    }
 	}
       else if (esl_memstrpfx(p, n, "//")) break;
@@ -526,7 +590,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
   /* The regurgitate_*() functions are limited, and only deal with single-record Pfam files. 
    * If there appears to be more data in the file, drop the reached_eof flag.
    */
-  while ((status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK) 
+  while ((status = esl_msafile_GetLine(afp, &p, &n)) == eslOK) 
     {
       while (n && ( *p == ' ' || *p == '\t')) { p++; n--; } /* skip leading whitespace */
       if    (esl_memstrpfx(p, n, "# STOCKHOLM 1.")) break;
@@ -540,8 +604,8 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
    * Pass 1 complete; rewind (close/reopen) all files
    *****************************************************************/
 
-  eslx_msafile_Close(afp);
-  if ((status = eslx_msafile_Open(NULL, alifile, NULL, eslMSAFILE_PFAM, NULL, &afp)) != eslOK)
+  esl_msafile_Close(afp);
+  if ((status = esl_msafile_Open(NULL, alifile, NULL, eslMSAFILE_PFAM, NULL, &afp)) != eslOK)
     esl_fatal("--small, second pass, unable to open file %s for reading", alifile);
   
   if (ac_fp) { /* open the tmpfile with the seq accessions */
@@ -564,7 +628,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
    ******************************************************************************************/
 
   do { 
-    status = eslx_msafile_GetLine(afp, &p, &n);
+    status = esl_msafile_GetLine(afp, &p, &n);
     if      (status == eslEOF) return; 
     else if (status != eslOK)  esl_fatal("small mem parse pass 2 error. problem reading line %d of msafile", (int) afp->linenumber);
   } while (esl_memspn(afp->line, afp->n, " \t") == afp->n  ||                 /* skip blank lines             */
@@ -573,7 +637,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
 
   if (! esl_memstrpfx(afp->line, afp->n, "# STOCKHOLM 1.")) esl_fatal("small mem parse pass 2 failed (line %d): missing \"# STOCKHOLM\" header", (int) afp->linenumber);
 
-  while ((status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK) 
+  while ((status = esl_msafile_GetLine(afp, &p, &n)) == eslOK) 
     {
       while (n && ( *p == ' ' || *p == '\t')) { p++; n--; } /* skip leading whitespace */
 
@@ -655,7 +719,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
 
   if (ac_fp) fclose(ac_fp);
   if (de_fp) fclose(de_fp);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
 
   if (first_seqname) free(first_seqname);
   if (ac_buf)        free(ac_buf);
@@ -678,7 +742,7 @@ regurgitate_pfam_as_afa(ESLX_MSAFILE *afp, FILE *ofp, char *alifile, char *gapsy
  * message that indicates the cause of the problem.
  */
 static int
-regurgitate_pfam_as_pfam(ESLX_MSAFILE *afp, FILE *ofp, char *gapsym, int force_lower, int force_upper, int force_rna, int force_dna, int iupac_to_n, int x_is_bad, int wussify, int dewuss, int fullwuss, char *rfrom, char *rto)
+regurgitate_pfam_as_pfam(ESL_MSAFILE *afp, FILE *ofp, char *gapsym, int force_lower, int force_upper, int force_rna, int force_dna, int iupac_to_n, int x_is_bad, int wussify, int dewuss, int fullwuss, char *rfrom, char *rto)
 {
   char      *p;
   esl_pos_t  n;
@@ -707,7 +771,7 @@ regurgitate_pfam_as_pfam(ESLX_MSAFILE *afp, FILE *ofp, char *gapsym, int force_l
    */
   /* Check the magic Stockholm header line, allowing blank lines */
   do { 
-    status = eslx_msafile_GetLine(afp, &p, &n);
+    status = esl_msafile_GetLine(afp, &p, &n);
     if      (status == eslEOF) return eslEOF; 
     else if (status != eslOK)  esl_fatal("small mem parse error. problem reading line %d of msafile", (int) afp->linenumber);
     fprintf(ofp, "%.*s\n", (int) afp->n, afp->line);
@@ -718,7 +782,7 @@ regurgitate_pfam_as_pfam(ESLX_MSAFILE *afp, FILE *ofp, char *gapsym, int force_l
   if (! esl_memstrpfx(afp->line, afp->n, "# STOCKHOLM 1.")) esl_fatal("small mem parse failed (line %d): missing \"# STOCKHOLM\" header", (int) afp->linenumber);
 
   /* Read the alignment file one line at a time.  */
-  while ((status = eslx_msafile_GetLine(afp, &p, &n)) == eslOK) 
+  while ((status = esl_msafile_GetLine(afp, &p, &n)) == eslOK) 
     {
       if ((int) afp->linenumber % flushpoint == 0) fflush(ofp);
       while (n && ( *p == ' ' || *p == '\t')) { p++; n--; } /* skip leading whitespace */
@@ -880,14 +944,11 @@ parse_replace_string(const char *rstring, char **ret_from, char **ret_to)
 
 /*****************************************************************
  * Easel - a library of C functions for biological sequence analysis
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 HHMI Janelia Farm Research Campus
- * Other copyrights also apply. See the COPYRIGHT file for a full list.
+ * Version 0.43; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute
+ * Other copyrights also apply. See the LICENSE file for a full list.
  * 
- * Easel is distributed under the Janelia Farm Software License, a BSD
- * license. See the LICENSE file for more details.
- *
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/easel/branches/infernal/1.1/miniapps/esl-reformat.c $
- * SVN $Id: esl-reformat.c 711 2011-07-27 20:06:15Z eddys $            
+ * Easel is open source software, distributed under the BSD license. See
+ * the LICENSE file for more details.
  *****************************************************************/
 
