@@ -1,16 +1,24 @@
 /* Simple statistics on a sequence file
  * 
  * SRE, Sun Feb 24 15:33:53 2008 [UA5315 to St. Louis]
- * SVN $Id: esl-seqstat.c 263 2008-06-11 14:25:16Z eddys $  
+ * SVN $Id$  
  * from squid's seqstat (1994)
+ * 
+ * Wish list:
+ *   - add an option for printing sequence names only.
+ *     This would facilitate using esl-seqstat in incantations (with
+ *     esl-selectn and esl-sfetch) to extract subsets of sequences
+ *     from a large file.
  */
 #include "esl_config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <math.h>
 
 #include "easel.h"
+#include "esl_composition.h"
 #include "esl_getopts.h"
 #include "esl_sq.h"
 #include "esl_sqio.h"
@@ -19,6 +27,8 @@
 static char banner[] = "show simple statistics on a sequence file";
 static char usage1[] = "   [options] <seqfile>";
 
+static void show_overall_composition(const ESL_ALPHABET *abc, const double *monoc_all, int64_t nres);
+
 #define ALPH_OPTS "--rna,--dna,--amino" /* toggle group, alphabet type options          */
 
 static ESL_OPTIONS options[] = {
@@ -30,6 +40,8 @@ static ESL_OPTIONS options[] = {
   { "--rna",      eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, ALPH_OPTS, "specify that <seqfile> contains RNA sequence",        1 },
   { "--dna",      eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, ALPH_OPTS, "specify that <seqfile> contains DNA sequence",        1 },
   { "--amino",    eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL, ALPH_OPTS, "specify that <seqfile> contains protein sequence",    1 },
+  { "--comptbl",  eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL,      NULL, "alternative output: a table of residue compositions per seq", 1 },
+  { "--stall",    eslARG_NONE,    FALSE, NULL, NULL, NULL, NULL,      NULL, "arrest after start: for debugging under gdb",        99 },  
   { 0,0,0,0,0,0,0,0,0,0 },
 };
 
@@ -74,10 +86,12 @@ main(int argc, char **argv)
   double         *monoc     = NULL; /* monoresidue composition per sequence  */
   double         *monoc_all = NULL; /* monoresidue composition over all seqs */
   int             do_comp   = FALSE;
+  int             do_comptbl = FALSE;
   int             status    = eslOK;
   int             wstatus;
-  int             i;
-  int             x;
+  int             i, x;
+  int             do_stall;       /* used to stall when debugging     */
+
 
   /* Parse command line */
   go = esl_getopts_Create(options);
@@ -86,14 +100,19 @@ main(int argc, char **argv)
   if (esl_opt_GetBoolean(go, "-h") )                   cmdline_help(argv[0], go);
   if (esl_opt_ArgNumber(go) != 1)                      cmdline_failure(argv[0], "Incorrect number of command line arguments.\n");
 
-  seqfile = esl_opt_GetArg(go, 1);
-  do_comp = esl_opt_GetBoolean(go, "-c");
+  seqfile    = esl_opt_GetArg(go, 1);
+  do_comp    = esl_opt_GetBoolean(go, "-c");
+  do_comptbl = esl_opt_GetBoolean(go, "--comptbl");
 
   if (esl_opt_GetString(go, "--informat") != NULL) {
-    infmt = esl_sqio_FormatCode(esl_opt_GetString(go, "--informat"));
+    infmt = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--informat"));
     if (infmt == eslSQFILE_UNKNOWN) esl_fatal("%s is not a valid input sequence file format for --informat"); 
   }
 
+  do_stall = esl_opt_GetBoolean(go, "--stall"); /* a stall point for attaching gdb */
+  while (do_stall); 
+
+  /* open input file */
   status = esl_sqfile_Open(seqfile, infmt, NULL, &sqfp);
   if      (status == eslENOTFOUND) esl_fatal("No such file %s", seqfile);
   else if (status == eslEFORMAT)   esl_fatal("Format of seqfile %s unrecognized.", seqfile);
@@ -104,9 +123,9 @@ main(int argc, char **argv)
   else if (esl_opt_GetBoolean(go, "--amino")) alphatype = eslAMINO;
   else {
     status = esl_sqfile_GuessAlphabet(sqfp, &alphatype);
-    if      (status == eslEAMBIGUOUS) esl_fatal("Couldn't guess alphabet from first sequence in %s", seqfile);
-    else if (status == eslEFORMAT)    esl_fatal("Sequence file parse error, line %d of file %s:\n%s\n",
-					       sqfp->linenumber, seqfile, sqfp->errbuf);
+    if      (status == eslENOALPHABET) esl_fatal("Couldn't guess alphabet from first sequence in %s", seqfile);
+    else if (status == eslEFORMAT)    esl_fatal("Parse failed (sequence file %s):\n%s\n",
+						sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));     
     else if (status == eslENODATA)    esl_fatal("Sequence file %s contains no data?", seqfile);
     else if (status != eslOK)         esl_fatal("Failed to guess alphabet (error code %d)\n", status);
   }
@@ -114,18 +133,28 @@ main(int argc, char **argv)
   sq  = esl_sq_CreateDigital(abc);
   esl_sqfile_SetDigital(sqfp, abc);
 
-  if (do_comp) {
+  if (do_comp || do_comptbl) {
     ESL_ALLOC(monoc,     (abc->Kp) * sizeof(double));  
     ESL_ALLOC(monoc_all, (abc->Kp) * sizeof(double));  
     esl_vec_DSet(monoc_all, abc->Kp, 0.0);
     esl_vec_DSet(monoc,     abc->Kp, 0.0);
   }
 
+  /* Output header, if any */
+  if (do_comptbl) {
+    printf("#%-29s %6s", " Sequence name", "Length");
+    for (x = 0; x < abc->K; x++) printf("      %c", (char) abc->sym[x]);
+    fputc('\n', stdout);
+    printf("#%-29s %6s", "-----------------------------", "------");
+    for (x = 0; x < abc->K; x++) printf(" %6s", "------");
+    fputc('\n', stdout);
+  }
+
   while ((wstatus = esl_sqio_ReadWindow(sqfp, 0, 4096, sq)) != eslEOF)
     {
       if (wstatus == eslOK)
 	{
-	  if (do_comp) 
+	  if (do_comp || do_comptbl)
 	    for (i = 1; i <= sq->n; i++) 
 	      monoc[sq->dsq[i]]++;
 	}
@@ -137,44 +166,45 @@ main(int argc, char **argv)
 	    large = ESL_MAX(large, sq->L);
 	  }
 
-	  if (esl_opt_GetBoolean(go, "-a")) {
-	    printf("= %-20s %8" PRId64 " %s\n", sq->name, sq->L, (sq->desc != NULL) ? sq->desc : "");
+	  if (!do_comptbl && esl_opt_GetBoolean(go, "-a")) {
+	    printf("= %-25s %8" PRId64 " %s\n", sq->name, sq->L, (sq->desc != NULL) ? sq->desc : "");
+	  }
+	  
+	  if (do_comptbl) {
+	    printf("%-30s %6" PRId64, sq->name, sq->L);
+	    for (x = 0; x < abc->K; x++) printf(" %6.0f", monoc[x]);
+	    fputc('\n', stdout);
 	  }
 
 	  nres += sq->L;
 	  nseq++;
 	  esl_sq_Reuse(sq);
-	  if (do_comp) {
+	  if (do_comp || do_comptbl) {
 	    esl_vec_DAdd(monoc_all, monoc, abc->Kp);
 	    esl_vec_DSet(monoc, abc->Kp, 0.0);
 	  }
 	}
-      else if (wstatus == eslEFORMAT)
-	{
-	  esl_fatal("Failed to parse sequence at line %ld, file %s:\n%s", 
-		    (long) sqfp->linenumber, sqfp->filename, sqfp->errbuf);
-	}
-      else 
-	esl_fatal("Failed in reading sequence:\n%s\n", sqfp->errbuf);
+      else if  (wstatus == eslEFORMAT) esl_fatal("Parse failed (sequence file %s):\n%s\n",
+						 sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+      else                             esl_fatal("Unexpected error %d reading sequence file %s",
+					         wstatus, sqfp->filename);
     }
 
-  printf("Format:              %s\n",   esl_sqio_DescribeFormat(sqfp->format));
-  printf("Alphabet type:       %s\n",   esl_abc_DescribeType(abc->type));
-  printf("Number of sequences: %" PRId64 "\n", nseq);
-  printf("Total # residues:    %" PRId64 "\n", nres);
-  printf("Smallest:            %" PRId64 "\n", small);
-  printf("Largest:             %" PRId64 "\n", large);
-  printf("Average length:      %.1f\n", (float) nres / (float) nseq);
+  if (! do_comptbl)
+    {
+      printf("Format:              %s\n",   esl_sqio_DecodeFormat(sqfp->format));
+      printf("Alphabet type:       %s\n",   esl_abc_DecodeType(abc->type));
+      printf("Number of sequences: %" PRId64 "\n", nseq);
+      printf("Total # residues:    %" PRId64 "\n", nres);
+      printf("Smallest:            %" PRId64 "\n", small);
+      printf("Largest:             %" PRId64 "\n", large);
+      printf("Average length:      %.1f\n", (float) nres / (float) nseq);
 
-  if (do_comp) {
-    printf("\nResidue composition:\n");
-    for (x = 0; x < abc->Kp; x++)
-      if (x < abc->K || monoc_all[x] > 0)
-	printf("residue: %c   %10.0f  %.4f\n", abc->sym[x], monoc_all[x], monoc_all[x] / (double) nres);
-    free(monoc);
-    free(monoc_all);
-  }
+      if (do_comp) show_overall_composition(abc, monoc_all, nres);
+    }
 
+  if (monoc)     free(monoc);
+  if (monoc_all) free(monoc_all);
   esl_alphabet_Destroy(abc);
   esl_sq_Destroy(sq);
   esl_sqfile_Close(sqfp);
@@ -184,3 +214,45 @@ main(int argc, char **argv)
  ERROR:
   return status;
 }
+
+
+static void
+show_overall_composition(const ESL_ALPHABET *abc, const double *monoc_all, int64_t nres)
+{
+  double *iid_bg = NULL;;
+  int x;
+  int status;
+
+  printf("\nResidue composition:\n");
+
+  if (abc->type == eslAMINO)
+    {
+      ESL_DASSERT1(( abc->K == 20 )); // esl_composition_*() functions assume this
+      ESL_ALLOC(iid_bg, sizeof(double) * abc->K);
+      esl_composition_SW50(iid_bg);
+
+      for (x = 0; x < abc->K; x++)
+	printf("residue: %c   %10.0f  %6.4f  %8.4f\n",
+	       abc->sym[x], monoc_all[x], monoc_all[x] / (double) nres,
+	       log((monoc_all[x] / (double) nres) / iid_bg[x]) * eslCONST_LOG2R);
+      for ( ;     x < abc->Kp; x++)
+	if (monoc_all[x] > 0)
+	  printf("residue: %c   %10.0f  %6.4f\n",
+		 abc->sym[x], monoc_all[x], monoc_all[x] / (double) nres);
+      free(iid_bg);
+    }
+  else
+    {
+      for (x = 0; x < abc->Kp; x++)
+	if (x < abc->K || monoc_all[x] > 0)
+	  printf("residue: %c   %10.0f  %.4f\n", abc->sym[x], monoc_all[x], monoc_all[x] / (double) nres);
+    }
+    
+  return;
+
+ ERROR:
+  if (iid_bg) free(iid_bg);
+  return;
+}
+
+
