@@ -2,7 +2,7 @@
  * 
  * From squid's sfetch and ffetch
  * SRE, Mon Mar 31 16:12:50 2008 [Janelia] 
- * SVN $Id: esl-sfetch.c 769 2012-06-05 17:50:33Z nawrockie $
+ * SVN $Id$
  */
 #include "esl_config.h"
 
@@ -51,7 +51,8 @@ cmdline_help(char *argv0, ESL_GETOPTS *go)
   esl_opt_DisplayHelp(stdout, go, 2, 2, 80);
   puts("\n  On command line, subseq coords are separated by any nonnumeric, nonspace character(s).");
   puts("  for example, -c 23..100 or -c 23/100 or -c 23-100 all work.\n");
-  puts("  Additionally, to retrieve a suffix to the end, omit the end coord; -c 23: will work.");
+  puts("  Additionally, to retrieve a suffix to the end, omit the end coord or set it to zero; -c 23.. ");
+  puts("  will work, as will -c 23..0\n");
   puts("  By default, the subseq will be named <source name>/<from>-<to>. To assign a name of");
   puts("  your choice, use -n <newname>.\n");
   puts("  In retrieving subsequences listed in a file (-C -f, or just -Cf), each line of the file");
@@ -60,8 +61,6 @@ cmdline_help(char *argv0, ESL_GETOPTS *go)
   puts("  retrieved; in protein sequence, this is an error. The -r option is another way to revcomp.");
   puts("\n other options:");
   esl_opt_DisplayHelp(stdout, go, 3, 2, 80);
-  puts("\n options for retreiving subsequences from cmsearch tab file (require -C and -f):");
-  esl_opt_DisplayHelp(stdout, go, 4, 2, 80);
   exit(0);
 }
 
@@ -73,6 +72,7 @@ static ESL_OPTIONS options[] = {
   { "-n",          eslARG_STRING, FALSE,  NULL, NULL, NULL, NULL,              "-f,--index",         "rename the sequence <s>",                           1 },
   { "-r",          eslARG_NONE,   FALSE,  NULL, NULL, NULL, NULL,              "--index",            "reverse complement the seq(s)",                     1 },
 
+
   { "-c",          eslARG_STRING, FALSE,  NULL, NULL, NULL, NULL,              "-f,--index",         "retrieve subsequence coords <from>..<to>",          2 },
   { "-C",          eslARG_NONE,   FALSE,  NULL, NULL, NULL, "-f",              "--index",            "<namefile> in <f> contains subseq coords too",      2 },
 
@@ -91,7 +91,6 @@ static void onefetch(ESL_GETOPTS *go, FILE *ofp, char *key, ESL_SQFILE *sqfp);
 static void multifetch_subseq(ESL_GETOPTS *go, FILE *ofp, char *keyfile, ESL_SQFILE *sqfp);
 static void onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, 
 			    char *key, uint32_t given_start, uint32_t given_end);
-static int  parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end);
 
 int
 main(int argc, char **argv)
@@ -188,7 +187,10 @@ main(int argc, char **argv)
 	{
 	  uint32_t start, end;
 
-	  parse_coord_string(cstring, &start, &end);
+	  status = esl_regexp_ParseCoordString(cstring, &start, &end);
+	  if (status == eslESYNTAX) esl_fatal("-c takes arg of subseq coords <from>..<to>; %s not recognized", cstring);
+	  if (status == eslFAIL)    esl_fatal("Failed to find <from> or <to> coord in %s", cstring);
+
 	  onefetch_subseq(go, ofp, sqfp, newname, key, start, end);
 	  if (ofp != stdout) printf("\n\nRetrieved subsequence %s/%d-%d.\n",  key, start, end);
 	}
@@ -431,12 +433,19 @@ multifetch_subseq(ESL_GETOPTS *go, FILE *ofp, char *gdffile, ESL_SQFILE *sqfp)
     {
       if (esl_fileparser_GetTokenOnLine(efp, &newname, &n1) != eslOK)
 	esl_fatal("Failed to read subseq name on line %d of file %s\n", efp->linenumber, gdffile);
+
       if (esl_fileparser_GetTokenOnLine(efp, &s, NULL) != eslOK)
 	esl_fatal("Failed to read start coord on line %d of file %s\n", efp->linenumber, gdffile);
       start = atoi(s);
+      if(start <= 0) 
+	esl_fatal("Read invalid start coord %d on line %d of file %s (must be positive integer)\n", start, efp->linenumber, gdffile);
+
       if (esl_fileparser_GetTokenOnLine(efp, &s, NULL) != eslOK)
 	esl_fatal("Failed to read end coord on line %d of file %s\n", efp->linenumber, gdffile);
       end   = atoi(s);
+      if(end < 0)
+	esl_fatal("Read invalid end coord %d on line %d of file %s (must be positive integer, or 0 for full length)\n", end, efp->linenumber, gdffile);
+
       if (esl_fileparser_GetTokenOnLine(efp, &source, &n2) != eslOK)
 	esl_fatal("Failed to read source seq name on line %d of file %s\n", efp->linenumber, gdffile);
 
@@ -478,20 +487,3 @@ onefetch_subseq(ESL_GETOPTS *go, FILE *ofp, ESL_SQFILE *sqfp, char *newname, cha
 }
 
 
-static int
-parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end)
-{
-  ESL_REGEXP *re = esl_regexp_Create();
-  char        tok1[32];
-  char        tok2[32];
-
-  if (esl_regexp_Match(re, "^(\\d+)\\D+(\\d*)$", cstring) != eslOK) esl_fatal("-c takes arg of subseq coords <from>..<to>; %s not recognized", cstring);
-  if (esl_regexp_SubmatchCopy(re, 1, tok1, 32)            != eslOK) esl_fatal("Failed to find <from> coord in %s", cstring);
-  if (esl_regexp_SubmatchCopy(re, 2, tok2, 32)            != eslOK) esl_fatal("Failed to find <to> coord in %s",   cstring);
-  
-  *ret_start = atol(tok1);
-  *ret_end   = (tok2[0] == '\0') ? 0 : atol(tok2);
-  
-  esl_regexp_Destroy(re);
-  return eslOK;
-}
