@@ -33,7 +33,7 @@
 #include "esl_sqio.h"
 #include "esl_stack.h"
 #include "esl_vectorops.h"
-
+#include "esl_composition.h"
 
 static char banner[] = "construct a benchmark profile training/test set";
 static char usage[]  = "[options] <basename> <msafile> <seqdb>\n";
@@ -158,7 +158,7 @@ main(int argc, char **argv)
   char          outfile[256];	/* name of an output file          */
   int           alifmt;		/* format code for alifile         */
   int           dbfmt;		/* format code for dbfile          */
-  ESLX_MSAFILE  *afp    = NULL;	/* open alignment file             */
+  ESL_MSAFILE   *afp    = NULL;	/* open alignment file             */
   ESL_MSA      *origmsa = NULL;	/* one multiple sequence alignment */
   ESL_MSA      *msa     = NULL;	/* MSA after frags are removed     */
   ESL_MSA      *trainmsa= NULL;	/* training set, aligned           */
@@ -216,8 +216,8 @@ main(int argc, char **argv)
   else if (esl_opt_GetBoolean(go, "--dna"))     cfg.abc = esl_alphabet_Create(eslDNA);
   else if (esl_opt_GetBoolean(go, "--rna"))     cfg.abc = esl_alphabet_Create(eslRNA);
 
-  status = eslx_msafile_Open(&(cfg.abc), alifile, NULL, alifmt, NULL, &afp);
-  if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+  status = esl_msafile_Open(&(cfg.abc), alifile, NULL, alifmt, NULL, &afp);
+  if (status != eslOK) esl_msafile_OpenFailure(afp, status);
 
   if (cfg.abc->type == eslAMINO) esl_composition_SW34(cfg.fq);
   else                           esl_vec_DSet(cfg.fq, cfg.abc->K, 1.0 / (double) cfg.abc->K);
@@ -227,9 +227,9 @@ main(int argc, char **argv)
 
   /* Read and process MSAs one at a time  */
   nali = 0;
-  while ((status = eslx_msafile_Read(afp, &origmsa)) != eslEOF)
+  while ((status = esl_msafile_Read(afp, &origmsa)) != eslEOF)
     {
-      if (status != eslOK) eslx_msafile_ReadFailure(afp, status);
+      if (status != eslOK) esl_msafile_ReadFailure(afp, status);
       esl_msa_ConvertDegen2X(origmsa); 
       esl_msa_Hash(origmsa);
 
@@ -252,7 +252,7 @@ main(int argc, char **argv)
 
 	  synthesize_positives(go, &cfg, msa->name, teststack, &ntest);
 
-	  eslx_msafile_Write(cfg.out_msafp, trainmsa, eslMSAFILE_STOCKHOLM);
+	  esl_msafile_Write(cfg.out_msafp, trainmsa, eslMSAFILE_STOCKHOLM);
 
 	  esl_dst_XAverageId(cfg.abc, trainmsa->ax, trainmsa->nseq, 10000, &avgid); /* 10000 is max_comparisons, before sampling kicks in */
 	  fprintf(cfg.tblfp, "%-20s  %3.0f%% %6d %6d %6d %6d %6d %6d\n", msa->name, 100.*avgid, (int) trainmsa->alen, msa->nseq, nfrags, trainmsa->nseq, ntestdom, ntest);
@@ -275,7 +275,7 @@ main(int argc, char **argv)
   if (cfg.pidfp) fclose(cfg.pidfp);
   esl_randomness_Destroy(cfg.r);
   esl_alphabet_Destroy(cfg.abc);
-  eslx_msafile_Close(afp);
+  esl_msafile_Close(afp);
   esl_getopts_Destroy(go);
   return 0;
 }
@@ -324,7 +324,6 @@ remove_fragments(struct cfg_s *cfg, ESL_MSA *msa, ESL_MSA **ret_filteredmsa, int
   int     *useme    = NULL;
   double   len      = 0.0;
   int      i;
-  int      nfrags;
   int      status;
 
   for (i = 0; i < msa->nseq; i++) 
@@ -332,7 +331,7 @@ remove_fragments(struct cfg_s *cfg, ESL_MSA *msa, ESL_MSA **ret_filteredmsa, int
   len *= cfg->fragfrac / (double) msa->nseq;
 
   ESL_ALLOC(useme, sizeof(int) * msa->nseq);
-  for (nfrags = 0, i = 0; i < msa->nseq; i++) 
+  for (i = 0; i < msa->nseq; i++) 
     useme[i] = (esl_abc_dsqrlen(msa->abc, msa->ax[i]) < len) ? 0 : 1;
 
   if ((status = esl_msa_SequenceSubset(msa, useme, ret_filteredmsa)) != eslOK) goto ERROR;
@@ -362,7 +361,6 @@ separate_sets(struct cfg_s *cfg, ESL_MSA *msa, ESL_MSA **ret_trainmsa, ESL_STACK
   int  nc         = 0;
   int  c;
   int  ctrain;			/* index of the cluster that becomes the training alignment */
-  int  ntrain;			/* number of seqs in the training alignment */
   int  nskip;
   int  i;
   int  status;
@@ -372,7 +370,7 @@ separate_sets(struct cfg_s *cfg, ESL_MSA *msa, ESL_MSA **ret_trainmsa, ESL_STACK
 
   if ((status = esl_msacluster_SingleLinkage(msa, cfg->idthresh1, &assignment, &nin, &nc)) != eslOK) goto ERROR;
   ctrain = esl_vec_IArgMax(nin, nc);
-  ntrain = esl_vec_IMax(nin, nc);
+  //ntrain = esl_vec_IMax(nin, nc);   // We don't need <ntrain> for anything, but this is how you'd get it.
 
   for (i = 0; i < msa->nseq; i++) useme[i] = (assignment[i] == ctrain) ? 1 : 0;
   if ((status = esl_msa_SequenceSubset(msa, useme, &trainmsa)) != eslOK) goto ERROR;
@@ -634,7 +632,7 @@ set_random_segment(ESL_GETOPTS *go, struct cfg_s *cfg, FILE *logfp, ESL_DSQ *dsq
   char    *pkey         = NULL;
   int      start, end;
   int64_t  Lseq;
-  int      status;
+  int      status = eslOK;
 
   if (L==0) return eslOK;
   if (L > cfg->db_maxL) esl_fatal("can't fetch a segment of length %d; database max is %d\n", L, cfg->db_maxL);
@@ -750,15 +748,12 @@ write_pids(FILE *pidfp, ESL_MSA *origmsa, ESL_MSA *trainmsa, ESL_STACK *teststac
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
- *
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/profmark/create-profmark.c $
- * SVN $Id: create-profmark.c 3267 2010-05-14 17:27:36Z eddys $
  *****************************************************************/
 
   
