@@ -57,11 +57,14 @@ static ESL_OPTIONS options[] = {
 /* mask ranges */
   { "--modelrange", eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "range(s) for mask(s) in model coordinates", 5 },
   { "--alirange",   eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "range(s) for mask(s) in alignment coordinates", 5 },
-  { "--model2ali",  eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "print model ranges corresponding to input alignment ranges", 5 },
-  { "--ali2model",  eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "print alignment ranges corresponding to input model ranges", 5 },
+  { "--apendmask",    eslARG_NONE, NULL, NULL, NULL,  NULL,  WGTOPTS,         NULL,  "add to existing mask (default ignores to existing mask)",    5 },
+  { "--model2ali",  eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "given model ranges, print corresp. input alignment ranges", 5 },
+  { "--ali2model",  eslARG_STRING, NULL, NULL, NULL,  NULL,    NULL,     RANGEOPTS,  "given alignment ranges, print corresp. model ranges", 5 },
+
 /* Other options */
-  { "--informat", eslARG_STRING, NULL, NULL, NULL,      NULL,      NULL,    NULL, "assert input alifile is in format <s> (no autodetect)", 8 },
-  { "--seed",     eslARG_INT,   "42", NULL, "n>=0",     NULL,      NULL,    NULL, "set RNG seed to <n> (if 0: one-time arbitrary seed)",   8 },
+  { "--informat", eslARG_STRING,      NULL,    NULL, NULL,   NULL,   NULL,  NULL, "assert input alifile is in format <s> (no autodetect)", 8 },
+  { "--outformat", eslARG_STRING, "Stockholm", NULL, NULL,   NULL,   NULL,  NULL, "output alignment in format <s>",                                    2 },
+  { "--seed",     eslARG_INT,        "42",     NULL, "n>=0", NULL,   NULL,  NULL, "set RNG seed to <n> (if 0: one-time arbitrary seed)",   8 },
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
@@ -138,13 +141,21 @@ static int
 output_header(const ESL_GETOPTS *go, FILE *ofp, char *alifile, char *postmsafile)
 {
   p7_banner(ofp, go->argv[0], banner);
-  
+
   if (fprintf(ofp, "# input alignment file:             %s\n", alifile) < 0)     ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 
   if (esl_opt_IsUsed(go, "--alirange") || esl_opt_IsUsed(go, "--modelrange") ) {
     if (fprintf(ofp, "# output alignment file:            %s\n", postmsafile) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   }
 
+  if (esl_opt_IsUsed(go, "--alirange")   && fprintf(ofp, "# alignment range:                  %s\n",        esl_opt_GetString(go, "--alirange"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--modelrange") && fprintf(ofp, "# model range:                      %s\n",        esl_opt_GetString(go, "--modelrange"))< 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--apendmask")  && fprintf(ofp, "# add to existing mask:             [on]\n"                                            )< 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+
+  if (esl_opt_IsUsed(go, "--model2ali")   && fprintf(ofp, "# ali ranges for model range:      %s\n",        esl_opt_GetString(go, "--model2ali"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--ali2model")   && fprintf(ofp, "# model ranges for ali range:      %s\n",        esl_opt_GetString(go, "--ali2model"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+
+
   if (esl_opt_IsUsed(go, "-o")           && fprintf(ofp, "# output directed to file:          %s\n",        esl_opt_GetString(go, "-o"))         < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--amino")      && fprintf(ofp, "# input alignment is asserted as:   protein\n")                                        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--dna")        && fprintf(ofp, "# input alignment is asserted as:   DNA\n")                                            < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
@@ -167,26 +178,61 @@ output_header(const ESL_GETOPTS *go, FILE *ofp, char *alifile, char *postmsafile
   return eslOK;
 }
 
-/* lifted from esl-sfetch */
-static int
-parse_coord_string(const char *cstring, uint32_t *ret_start, uint32_t *ret_end)
+
+/* Function:  p7_Alimask_MakeModel2AliMap()
+ * Synopsis:  Compute map of coordinate in the alignment corresponding to each model position.
+ *
+ * Args:      msa     - The alignment for which the mapped model is to be computed. We assume
+ *                      the MSA has already been manipulated to account for model building
+ *                      flags (e.g. weighting).
+ *            do_hand - TRUE when the model is to follow a hand-build RF line (which must be
+ *                      part of the file.
+ *            symfraq - if weighted occupancy exceeds this value, include the column in the model.
+ *            map     - int array into which the map values will be stored. Calling function
+ *                      must allocate (msa->alen+1) ints.
+ *
+ * Returns:   The number of mapped model positions.
+ */
+int
+p7_Alimask_MakeModel2AliMap(ESL_MSA *msa, int do_hand, float symfrac, int *map )
 {
-  ESL_REGEXP *re = esl_regexp_Create();
-  char        tok1[32];
-  char        tok2[32];
+  int      i = 0;
+  int      apos, idx;
+  float    r;            /* weighted residue count              */
+  float    totwgt;       /* weighted residue+gap count          */
 
-  if (esl_regexp_Match(re, "^(\\d+)\\D+(\\d*)$", cstring) != eslOK) esl_fatal("-c takes arg of subseq coords <from>..<to>; %s not recognized", cstring);
-  if (esl_regexp_SubmatchCopy(re, 1, tok1, 32)            != eslOK) esl_fatal("Failed to find <from> coord in %s", cstring);
-  if (esl_regexp_SubmatchCopy(re, 2, tok2, 32)            != eslOK) esl_fatal("Failed to find <to> coord in %s",   cstring);
+  i = 0;
+  if ( do_hand ) {
+     if (msa->rf == NULL)      p7_Fail("Model file does not contain an RF line, required for --hand.\n");
+     /* Watch for off-by-one. rf is [0..alen-1]*/
+     for (apos = 1; apos <= msa->alen; apos++) {
+       if (!esl_abc_CIsGap(msa->abc, msa->rf[apos-1]) ) {
+         map[i] = apos;
+         i++;
+       }
+     }
 
-  *ret_start = atol(tok1);
-  *ret_end   = (tok2[0] == '\0') ? 0 : atol(tok2);
+  } else {
 
-  esl_regexp_Destroy(re);
-  return eslOK;
+    for (apos = 1; apos <= msa->alen; apos++)
+    {
+        r = totwgt = 0.;
+        for (idx = 0; idx < msa->nseq; idx++)
+        {
+          if       (esl_abc_XIsResidue(msa->abc, msa->ax[idx][apos])) { r += msa->wgt[idx]; totwgt += msa->wgt[idx]; }
+          else if  (esl_abc_XIsGap(msa->abc,     msa->ax[idx][apos])) {                     totwgt += msa->wgt[idx]; }
+          else if  (esl_abc_XIsMissing(msa->abc, msa->ax[idx][apos])) continue;
+        }
+
+        if (r > 0. && r / totwgt >= symfrac) {
+          map[i] = apos;
+          i++;
+        }
+    }
+  }
+  return i;
 }
 
-
 int
 main(int argc, char **argv)
 {
@@ -201,9 +247,11 @@ main(int argc, char **argv)
   FILE         *ofp         = NULL;    /* output file (default is stdout) */
   ESL_ALPHABET *abc         = NULL;    /* digital alphabet */
 
-  char         *alifile;  /* name of the alignment file we're building HMMs from  */
-  ESLX_MSAFILE *afp         = NULL;            /* open alifile  */
-  int           fmt;    /* format code for alifile */
+  char         *alifile;                             /* name of the alignment file we're building HMMs from  */
+  ESL_MSAFILE  *afp         = NULL;                  /* open alifile  */
+  int           infmt       = eslMSAFILE_UNKNOWN;    /* autodetect alignment format by default. */
+  int           outfmt      = eslMSAFILE_STOCKHOLM;
+
 
   char         *postmsafile;  /* optional file to resave annotated, modified MSAs to  */
   FILE         *postmsafp = NULL;  /* open <postmsafile>, or NULL */
@@ -218,59 +266,65 @@ main(int argc, char **argv)
 
   int     *map = NULL; /* map[i]=j,  means model position i comes from column j of the alignment; 1..alen */
 
+  int    keep_mm;
+
   /* Set processor specific flags */
   impl_Init();
-
   alifile     = NULL;
   postmsafile = NULL;
 
   /* Parse the command line
    */
   process_commandline(argc, argv, &go, &alifile, &postmsafile);
-
+  keep_mm = esl_opt_IsUsed(go, "--apendmask");
 
   /* Initialize what we can in the config structure (without knowing the alphabet yet).
    * Fields controlled by masters are set up in usual_master() or mpi_master()
    * Fields used by workers are set up in mpi_worker()
    */
   ofp         = NULL;
-  fmt         = eslMSAFILE_UNKNOWN;    /* autodetect alignment format by default. */
+  infmt         = eslMSAFILE_UNKNOWN;
   afp         = NULL;
   abc         = NULL;
 
   if (esl_opt_IsOn(go, "--informat")) {
-    fmt = eslx_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
-    if (fmt == eslMSAFILE_UNKNOWN) p7_Fail("%s is not a recognized input sequence file format\n", esl_opt_GetString(go, "--informat"));
+    infmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
+    if (infmt == eslMSAFILE_UNKNOWN) p7_Fail("%s is not a recognized input sequence file format\n", esl_opt_GetString(go, "--informat"));
   }
 
+  /* Determine output alignment file format */
+  outfmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--outformat"));
+  if (outfmt == eslMSAFILE_UNKNOWN)    p7_Fail(argv[0], "%s is not a recognized output MSA file format\n", esl_opt_GetString(go, "--outformat"));
+
+
 
   /* Parse the ranges */
 
   if (esl_opt_IsUsed(go, "--alirange")) {
-    rangestr = esl_opt_GetString(go, "--alirange");
+    esl_strdup(esl_opt_GetString(go, "--alirange"), -1, &rangestr) ;
   } else if (esl_opt_IsUsed(go, "--modelrange")) {
-    rangestr = esl_opt_GetString(go, "--modelrange");
+    esl_strdup(esl_opt_GetString(go, "--modelrange"), -1, &rangestr) ;
   } else if (esl_opt_IsUsed(go, "--model2ali")) {
-    rangestr = esl_opt_GetString(go, "--model2ali");
+    esl_strdup(esl_opt_GetString(go, "--model2ali"), -1, &rangestr) ;
   } else if (esl_opt_IsUsed(go, "--ali2model")) {
-    rangestr = esl_opt_GetString(go, "--ali2model");
+    esl_strdup(esl_opt_GetString(go, "--ali2model"), -1, &rangestr) ;
   } else {
     if (puts("Must specify mask range with --modelrange, --alirange, --model2ali, or --ali2model\n") < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed"); goto ERROR;
   }
 
-
   while ( (status = esl_strtok(&rangestr, ",", &range) ) == eslOK) {
-    parse_coord_string(range, mask_starts + mask_range_cnt, mask_ends + mask_range_cnt );
+    status = esl_regexp_ParseCoordString(range, mask_starts + mask_range_cnt, mask_ends + mask_range_cnt );
+    if (status == eslESYNTAX) esl_fatal("range flags take coords <from>..<to>; %s not recognized", range);
+    if (status == eslFAIL)    esl_fatal("Failed to find <from> or <to> coord in %s", range);
+
     mask_range_cnt++;
   }
 
 
-
   /* Start timing. */
   esl_stopwatch_Start(w);
 
 
-
   /* Open files, set alphabet.
    *   afp       - open alignment file for input
    *   abc       - alphabet expected or guessed in ali file
@@ -282,8 +336,8 @@ main(int argc, char **argv)
   else if (esl_opt_GetBoolean(go, "--rna"))     abc = esl_alphabet_Create(eslRNA);
   else                                          abc = NULL;
   
-  status = eslx_msafile_Open(&abc, alifile, NULL, fmt, NULL, &afp);
-  if (status != eslOK) eslx_msafile_OpenFailure(afp, status);
+  status = esl_msafile_Open(&abc, alifile, NULL, infmt, NULL, &afp);
+  if (status != eslOK) esl_msafile_OpenFailure(afp, status);
 
   if (esl_opt_IsUsed(go, "--alirange") || esl_opt_IsUsed(go, "--modelrange") ) {
     postmsafp = fopen(postmsafile, "w");
@@ -297,31 +351,33 @@ main(int argc, char **argv)
     } 
   else ofp = stdout;
 
+
   /* Looks like the i/o is set up successfully...
    * Initial output to the user
    */
   output_header(go, ofp, alifile, postmsafile);                                  /* cheery output header                                */
 
   /* read the alignment */
-  if ((status = eslx_msafile_Read(afp, &msa)) != eslOK)  eslx_msafile_ReadFailure(afp, status);
+  if ((status = esl_msafile_Read(afp, &msa)) != eslOK)  esl_msafile_ReadFailure(afp, status);
 
 
   if (esl_opt_IsUsed(go, "--alirange") || esl_opt_IsUsed(go, "--modelrange") ) {
     /* add/modify mmline for the mask */
     if (msa->mm == NULL) {
-      // if it doesn't already exist, allocate it and set is to all '.'
       ESL_ALLOC(msa->mm, msa->alen);
-      for (i=0; i<msa->alen; i++)
-        msa->mm[i] = '.';
+      keep_mm = FALSE;
     }
+
+    if (!keep_mm)
+      for (i=0; i<msa->alen; i++) msa->mm[i] = '.';
+
   }
 
   // convert model coordinates to alignment coordinates, if necessary
   if (esl_opt_IsUsed(go, "--modelrange") || esl_opt_IsUsed(go, "--model2ali") || esl_opt_IsUsed(go, "--ali2model") ) {
-    int      apos, idx;
-    float    r;            /* weighted residue count              */
-    float    totwgt;       /* weighted residue+gap count          */
-    float    symfrac;
+
+    float symfrac = esl_opt_GetReal(go, "--symfrac");
+    int do_hand  =  esl_opt_IsOn(go, "--hand");
 
     //same as p7_builder relative_weights
     if      (esl_opt_IsOn(go, "--wnone")  )                  { esl_vec_DSet(msa->wgt, msa->nseq, 1.); }
@@ -330,35 +386,24 @@ main(int argc, char **argv)
     else if (esl_opt_IsOn(go, "--wgsc")   )                  status = esl_msaweight_GSC(msa);
     else if (esl_opt_IsOn(go, "--wblosum"))                  status = esl_msaweight_BLOSUM(msa, esl_opt_GetReal(go, "--wid"));
 
-
-
-    symfrac = esl_opt_GetReal(go, "--symfrac");
-
     if ((status =  esl_msa_MarkFragments(msa, esl_opt_GetReal(go, "--fragthresh")))           != eslOK) goto ERROR;
 
-    // Determine weighted sym freq in each column, build a map of model mask coordinates to alignment coords
+    //build a map of model mask coordinates to alignment coords
     ESL_ALLOC(map, sizeof(int)     * (msa->alen+1));
-    i = 0;
-    for (apos = 1; apos <= msa->alen; apos++)
-    {
-        r = totwgt = 0.;
-        for (idx = 0; idx < msa->nseq; idx++)
-        {
-          if       (esl_abc_XIsResidue(msa->abc, msa->ax[idx][apos])) { r += msa->wgt[idx]; totwgt += msa->wgt[idx]; }
-          else if  (esl_abc_XIsGap(msa->abc,     msa->ax[idx][apos])) {                     totwgt += msa->wgt[idx]; }
-          else if  (esl_abc_XIsMissing(msa->abc, msa->ax[idx][apos])) continue;
-        }
-        map[i] = apos;
-
-        if (r > 0. && r / totwgt >= symfrac)
-          i++;
-    }
+    p7_Alimask_MakeModel2AliMap(msa, do_hand, symfrac, map );  // Returns <L>, which we don't need.
 
     if ( esl_opt_IsUsed(go, "--model2ali") ) {
       //print mapping
       printf ("model coordinates     alignment coordinates\n");
       for (i=0; i<mask_range_cnt; i++)
         printf ("%8d..%-8d -> %8d..%-8d\n", mask_starts[i], mask_ends[i], map[mask_starts[i]-1], map[mask_ends[i]-1]);
+      /* If I wanted to, I could print all the map values independently:
+        printf("\n\n-----------\n");
+        printf("Map\n");
+        printf("---\n");
+        for (i=0; i<L; i++)
+          printf("%d -> %d\n", i+1, map[i]);
+      */
     } else if ( esl_opt_IsUsed(go, "--ali2model") ) {
       //print mapping  (requires scanning the inverted map
       int alistart = 0;
@@ -389,15 +434,15 @@ main(int argc, char **argv)
       for (j=mask_starts[i]; j<=mask_ends[i]; j++)
         msa->mm[j-1] = 'm';
 
-    if ((status = eslx_msafile_Write(postmsafp, msa, eslMSAFILE_STOCKHOLM))  != eslOK) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed");
+    if ((status = esl_msafile_Write(postmsafp, msa, outfmt))  != eslOK) ESL_XEXCEPTION_SYS(eslEWRITE, "write failed");
   }
 
   esl_stopwatch_Stop(w);
 
   if (esl_opt_IsOn(go, "-o"))  fclose(ofp);
   if (postmsafp) fclose(postmsafp);
-  if (afp)   eslx_msafile_Close(afp);
-  if (abc)   esl_alphabet_Destroy(abc);
+  if (afp)       esl_msafile_Close(afp);
+  if (abc)       esl_alphabet_Destroy(abc);
 
   esl_getopts_Destroy(go);
   esl_stopwatch_Destroy(w);
@@ -412,8 +457,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
