@@ -79,7 +79,7 @@ p7_seqcache_Open(char *seqfile, P7_SEQCACHE **ret_cache, char *errbuf)
 
   /* This is a bit of a hack.  The first line contains database information.
    *
-   * #<res_count> <seq_count> <db_count> <db_entries> ... <id>
+   * #<res_count> <seq_count> <db_count> <db_sequences_1> <db_sequences_before_removing_duplicates_1> <db_sequences_2> <db_sequences_before_removing_duplicates_2>  ... <date_stamp>
    *
    * The rest of the file is a fasta format.  The fasta header is just
    * sequence number followed by a binary number indicating which
@@ -250,10 +250,10 @@ p7_seqcache_Open(char *seqfile, P7_SEQCACHE **ret_cache, char *errbuf)
   }
 
   for (i = 0; i < cache->db_cnt; ++i) {
-    printf("final (%d):: %d %d\n", i, cache->db[i].count, db_inx[i]);
+    printf("sequence database (%d):: %d %d\n", i, cache->db[i].count, db_inx[i]);
   }
 
-  printf("\nTotal memory %" PRId64 "\n", total_mem);
+  printf("\nLoaded sequence db file %s; total memory %" PRId64 "\n", seqfile, total_mem);
 
   esl_sqfile_Close(sqfp);
   esl_sq_Destroy(sq);
@@ -659,13 +659,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  *
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/cachedb.c $
- * SVN $Id: cachedb.c 3754 2011-11-21 14:25:31Z eddys $
+ * SVN $URL$
+ * SVN $Id$
  *****************************************************************/
