@@ -12,11 +12,12 @@
  *  # 7. Copyright and license.
  *
  */
+#include "p7_config.h"
+
+#include "easel.h"
+#include "esl_getopts.h"
 #include "hmmer.h"
 
-
-
-
 /* Function:  fm_initSeeds()
  *
  * Synopsis:  initialize the object used to store a list of seed diagonals
@@ -62,21 +63,78 @@ ERROR:
 
 
 
+/* Function:  fm_initAmbiguityList()
+ *
+ * Synopsis:  initialize the object used to store a list of ambiguity ranges
+ *
+ * Returns:   eslEMEM in event of allocation failure, otherwise eslOK
+ */
+int
+fm_initAmbiguityList (FM_AMBIGLIST *list) {
+
+  int status;
+
+  list->size = 1000;
+
+  ESL_ALLOC(list->ranges, list->size * sizeof(FM_INTERVAL));
+  if (list->ranges == NULL  )
+    esl_fatal("unable to allocate memory to store FM ambiguity data\n");
+
+  list->count = 0;
+
+  return eslOK;
+
+ERROR:
+  return eslEMEM;
+
+}
+
+/* Function:  fm_addAmbiguityRange()
+ *
+ * Synopsis:  return a pointer to the next seed element on the list,
+ *            increasing the size of the list, if necessary.
+ *
+ * Returns:   NULL in event of allocation failure, otherwise pointer to
+ *            the next seed diagonal
+ */
+int
+fm_addAmbiguityRange (FM_AMBIGLIST *list, uint32_t start, uint32_t stop) {
+  int status;
+
+  if (list->count == list->size) {
+    list->size *= 4;
+    ESL_REALLOC(list->ranges, list->size * sizeof(FM_INTERVAL));
+  }
+
+  list->ranges[list->count].lower = start;
+  list->ranges[list->count].upper = stop;
+
+  list->count++;
+
+  return eslOK;
+
+ERROR:
+  return eslFAIL;
+}
+
+
+
+
+
 /*********************************************************************
  *# 2. Interval / range computation
  *********************************************************************/
 
 /* Function:  fm_updateIntervalForward()
  *
- * Synopsis:  return a pointer to the next window element on the list,
- *            increasing the size of the list, if necessary.
+ * Synopsis:  Implement Algorithm 4 (i371) of Simpson (Bioinformatics 2010)
  *
- * Purpose:   Implement Algorithm 4 (i371) of Simpson (Bioinformatics 2010)
+ * Purpose:
  *
  * Returns:   eslOK
  */
 int
-fm_updateIntervalForward( const FM_DATA *fm, FM_CFG *cfg, char c, FM_INTERVAL *interval_bk, FM_INTERVAL *interval_f) {
+fm_updateIntervalForward( const FM_DATA *fm, const FM_CFG *cfg, char c, FM_INTERVAL *interval_bk, FM_INTERVAL *interval_f) {
   uint32_t occLT_l, occLT_u, occ_l, occ_u;
 
   fm_getOccCountLT (fm, cfg, interval_bk->lower - 1, c, &occ_l, &occLT_l);
@@ -113,7 +171,8 @@ fm_getSARangeForward( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
   interval->lower  = interval_bk.lower = abs(fm->C[c]);
   interval->upper  = interval_bk.upper = abs(fm->C[c+1])-1;
 
-  while (interval_bk.lower>0 && interval_bk.lower <= interval_bk.upper) {
+
+  while (interval_bk.lower>=0 && interval_bk.lower <= interval_bk.upper) {
     c = query[++i];
     if (c == '\0')  // end of query - the current range defines the hits
       break;
@@ -128,7 +187,7 @@ fm_getSARangeForward( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
 
 
 int
-fm_updateIntervalReverse( const FM_DATA *fm, FM_CFG *cfg, char c, FM_INTERVAL *interval) {
+fm_updateIntervalReverse( const FM_DATA *fm, const FM_CFG *cfg, char c, FM_INTERVAL *interval) {
   int count1, count2;
   //TODO: counting in these calls will often overlap
     // - might get acceleration by merging to a single redundancy-avoiding call
@@ -161,9 +220,7 @@ fm_getSARangeReverse( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
   interval->lower  = abs(fm->C[(int)c]);
   interval->upper  = abs(fm->C[(int)c+1])-1;
 
-//  fprintf (stderr, "%d : %12d..%12d\n", c, interval->lower, interval->upper );
-
-  while (interval->lower>0 && interval->lower <= interval->upper) {
+  while (interval->lower>=0 && interval->lower <= interval->upper) {
     c = query[++i];
     if (c == '\0')  // end of query - the current range defines the hits
       break;
@@ -172,8 +229,6 @@ fm_getSARangeReverse( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
 
     fm_updateIntervalReverse(fm, cfg, c, interval);
 
-//    fprintf (stderr, "%d : %12d..%12d\n", c, interval->lower, interval->upper );
-
     cfg->occCallCnt+=2;
   }
 
@@ -189,126 +244,177 @@ fm_getSARangeReverse( const FM_DATA *fm, FM_CFG *cfg, char *query, char *inv_alp
 
 /* Function:  getChar()
  * Synopsis:  Find the character c residing at a given position in the BWT.
- * Purpose:   The returned char is used by getFMHits(), to seed a call to
- *            bwt_getOccCount().
+ * Purpose:   This method must account for possible string compression, either
+ *            4 characters in one byte for a 4-letter DNA/RNA alphabet, 2 chars
+ *            per byte for a 15-letter alphabet of DNA/RNA with ambiguity codes,
+ *            or one char per byte for amino acids.
  */
-//#ifndef FMDEBUG
-//inline
-//#endif
 uint8_t
 fm_getChar(uint8_t alph_type, int j, const uint8_t *B )
 {
   uint8_t c = -1;
 
-  if (alph_type == fm_DNA) {
+  if (alph_type == fm_DNA /* || alph_type == fm_RNA */) {
     /*
-     *  B[j>>2] is the byte of B in which j is found (j/4)
+     *  B[j/4] is the byte of B in which j is found
      *
      *  Let j' be the final two bits of j (j&0x2)
      *  The char bits are the two starting at position 2*j'.
-     *  Without branching, grab them by shifting B[j>>2] right 6-2*j' bits,
+     *  Without branching, grab them by shifting B[j/4] right 6-2*j' bits,
      *  then masking to keep the final two bits
      */
-    c = (B[j>>2] >> ( 0x6 - ((j&0x3)<<1) ) & 0x3);
-  } else if (alph_type == fm_DNA_full) {
-    c = (B[j>>1] >> (((j&0x1)^0x1)<<2) ) & 0xf;  //unpack the char: shift 4 bits right if it's odd, then mask off left bits in any case
-  } else {
-    esl_fatal("Invalid alphabet type\n");
+    c = (B[j/4] >> ( 0x6 - ((j&0x3)*2) ) & 0x3);
+/*
+  } else if (alph_type == fm_DNA_full ) {
+    c = (B[j/2] >> (((j&0x1)^0x1)*4) ) & 0xf;  //unpack the char: shift 4 bits right if it's odd, then mask off left bits in any case
+*/
+  } else { // amino
+    c = B[j];
   }
 
   return c;
 }
 
 
-/* Function:  getChar()
- * Synopsis:  Find the character c residing at a given position in the BWT.
- * Purpose:   The returned char is used by getFMHits(), to seed a call to
- *            bwt_getOccCount().
- */
-//#ifndef FMDEBUG
-//inline
-//#endif
-int
-fm_convertRange2DSQ(FM_METADATA *meta, int id, int first, int length, const uint8_t *B, ESL_SQ *sq )
-{
-  int i;
-  uint8_t c;
 
+/* Function:  fm_findOverlappingAmbiguityBlock()
+ * Synopsis:  Search in the meta->ambig_list array for the first
+ *            ambiguity range starting after <start>. If that index
+ *            comes before <end>, return it. Otherwise, return -1.
+ */
+int32_t
+fm_findOverlappingAmbiguityBlock (const FM_DATA *fm, const FM_METADATA *meta, uint32_t start, uint32_t end)
+{
+
+  int lo = fm->ambig_offset;
+  int hi = lo + fm->ambig_cnt - 1;
+  int mid;
+  FM_INTERVAL *ranges = meta->ambig_list->ranges;
+
+  // (1) Search in the meta->ambig_list array for the last ambiguity range
+  // ending before <start>, using binary search, first handling edge cases:
+  if (hi <= lo)                   return hi; // either 0 or -1
+  if (ranges[lo].lower > end)     return -1;
+  if (ranges[hi].upper < start)   return -1;
+
+  while (lo < hi) {
+    mid = (lo + hi) / 2;  // round down
+    if      (ranges[mid].lower   < start)  lo = mid + 1; // too far left
+    else                                   hi = mid;    // might be too far right
+  }
+
+  //the range test above may have pushed the target one too far to the right
+  if      (lo>0 &&  ranges[lo-1].upper   >= start && ranges[lo-1].lower <= end) return lo-1;
+  else if (         ranges[lo].upper     >= start && ranges[lo].lower   <= end) return lo;
+  else return -1;
+
+}
+
+
+/* Function:  fm_convertRange2DSQ()
+ * Synopsis:  Convert the BWT range into a DSQ.
+ *
+ * Purpose:   The input value of <first> is the 0-based position at which
+ *            the requested range starts on either FM->T or revcomp(FM->T),
+ *            depending on <complementarity>. Since only FM->T is stored,
+ *            the necessary work is done to correct positions in the case
+ *            that the positions are relative to the revcomp.
+ */
+int
+fm_convertRange2DSQ(const FM_DATA *fm, const FM_METADATA *meta, uint64_t first, int length, int complementarity, ESL_SQ *sq, int fix_ambiguities)
+{
+  uint64_t i, j;
+
+  if (complementarity == p7_COMPLEMENT)
+    first = fm->N-(first+length)-1;
+
+  //All the "+1" dsq offsets below are because the dsq characters are 1-based.
   esl_sq_GrowTo(sq, length);
   sq->n = length;
 
-  if (meta->alph_type == fm_DNA || meta->alph_type == fm_RNA) {
+  if (meta->alph_type == fm_DNA ) {
     /*
      *  B[j>>2] is the byte of B in which j is found (j/4)
      *
      *  Let j' be the final two bits of j (j&0x2)
      *  The char bits are the two starting at position 2*j'.
-     *  Without branching, grab them by shifting B[j>>2] right 6-2*j' bits,
+     *  Without branching, grab them by shifting B[j/4] right 6-2*j' bits,
      *  then masking to keep the final two bits
      */
-    for (i = first; i<= first+length-1; i++)
-      sq->dsq[i-first+1] = (B[i>>2] >> ( 0x6 - ((i&0x3)<<1) ) & 0x3);
-
+    for (i = first; i <= first+length-1; i++)
+      sq->dsq[i-first+1] = (fm->T[i/4] >> ( 0x6 - ((i&0x3)*2) ) & 0x3);
     sq->dsq[length+1] = eslDSQ_SENTINEL;
 
-  } else if (meta->alph_type == fm_DNA_full || meta->alph_type == fm_RNA_full ) {
+    if (fix_ambiguities) {
+      /*  Account for the fact that in the DNA alphabet without ambiguity codes,
+       *  makehmmerdb turns ambiguity codes into one of the nucleotides. Need
+       *  to replace with an N.
+       */
+      int32_t pos = fm_findOverlappingAmbiguityBlock (fm, meta, first, first+length-1 );
+      if (pos != -1) {
+        while (pos <= fm->ambig_offset + fm->ambig_cnt -1 && meta->ambig_list->ranges[pos].lower <= first+length-1) {
+          int start = ESL_MAX(first,          meta->ambig_list->ranges[pos].lower);
+          int end =   ESL_MIN(first+length-1, meta->ambig_list->ranges[pos].upper);
+          for (j= start; j<=end; j++)
+              sq->dsq[j-first+1] = sq->abc->Kp-3; //'N'
+          pos++;
+        }
+      }
+    }
+
+/*
+  } else if (meta->alph_type == fm_DNA_full) {
     for (i = first; i<= first+length-1; i++) {
-      c = (B[i>>1] >> (((i&0x1)^0x1)<<2) ) & 0xf;  //unpack the char: shift 4 bits right if it's odd, then mask off left bits in any case
-      sq->dsq[i-first+1] = c + (c < 4 ? 0 : 1);
+      c = (fm->T[i/2] >> (((i&0x1)^0x1)*4) ) & 0xf;  //unpack the char: shift 4 bits right if it's odd, then mask off left bits in any case
+      sq->dsq[i-first+1] = c + (c < 4 ? 0 : 1); //increment by one for ambiguity codes
     }
     sq->dsq[length+1] = eslDSQ_SENTINEL;
-  } else {
-    esl_fatal("Invalid alphabet type\n");
+*/
+  } else { // amino
+    for (i = first; i<= first+length-1; i++)
+      sq->dsq[i-first+1] = fm->T[i] + (fm->T[i] < 20 ? 0 : 1); //increment by one for ambiguity codes
+
+    sq->dsq[length+1] = eslDSQ_SENTINEL;
   }
 
+  if (complementarity == p7_COMPLEMENT)
+    esl_sq_ReverseComplement(sq);
+
   return eslOK;
 }
 
 
 
-/* Function:  computeSequenceOffset()
+/* Function:  fm_computeSequenceOffset()
  * Synopsis:  Search in the meta->seq_data array for the sequence id corresponding to the
  *            requested position. The matching entry is the one with the largest index i
  *            such that seq_data[i].offset < pos
- *
- *
- * Input: file pointer to binary file
- * Output: return filled meta struct
  */
-//inline
-uint32_t
-fm_computeSequenceOffset (const FM_DATA *fms, FM_METADATA *meta, int block, int pos)
+uint64_t
+fm_computeSequenceOffset (const FM_DATA *fms, const FM_METADATA *meta, int block, uint64_t pos)
 {
 
-  uint32_t lo = fms[block].seq_offset;
-  uint32_t hi  = lo + fms[block].seq_cnt - 1;
-  uint32_t mid;
-
-  /*  //linear scan
-  for (mid=lo+1; i<=hi; i++) {
-    if (meta->seq_data[i].offset > pos) // the position of interest belongs to the previous sequence
-      break;
-  }
-  return i-1;
-    */
+  uint64_t lo = fms[block].seq_offset;
+  uint64_t hi  = lo + fms[block].seq_cnt - 1;
+  uint64_t mid;
 
   //binary search, first handling edge cases
-  if (lo==hi)                           return lo;
-  if (meta->seq_data[hi].offset <= pos) return hi;
+  if (lo==hi)                             return lo;
+  if (meta->seq_data[hi].fm_start <= pos) return hi;
 
   while (1) {
     mid = (lo + hi + 1) / 2;  /* round up */
-    if      (meta->seq_data[mid].offset <= pos) lo = mid; /* too far left  */
-    else if (meta->seq_data[mid-1].offset > pos) hi = mid; /* too far right */
+    if      (meta->seq_data[mid].fm_start <= pos) lo = mid; /* too far left  */
+    else if (meta->seq_data[mid-1].fm_start > pos) hi = mid; /* too far right */
     else return mid-1;                 /* found it */
   }
 
-
 }
 
 
-/* Function:  FM_getOriginalPosition()
- * Synopsis:  find
+/* Function:  fm_getOriginalPosition()
+ * Synopsis:  Find the id of the sequence in the original input corresponding
+ *            to a given hit, and the position of that hit in the original
  * Purpose:   Given:
  *            fms       - an array of FM-indexes
  *            meta      - the fm metadata
@@ -319,28 +425,29 @@ fm_computeSequenceOffset (const FM_DATA *fms, FM_METADATA *meta, int block, int
  *
  *            Returns
  *            *segment_id - index of the sequence segment captured in the FM-index
- *            *seg_pos    - position in the original sequence, as compressed in the FM binary data structure
+ *            *seg_pos    - position in the original sequence, as compressed in the FM binary data structure (zero based)
+ *            int         - eslERANGE if the range in question crosses the boundary between two target sequences. Otherwise eslOK.
  */
 int
-fm_getOriginalPosition (const FM_DATA *fms, FM_METADATA *meta, int fm_id, int length, int direction, uint32_t fm_pos,
-                  uint32_t *segment_id, uint32_t *seg_pos
+fm_getOriginalPosition (const FM_DATA *fms, const FM_METADATA *meta, int fm_id, int length, int complementarity,
+                        uint64_t fm_pos, uint32_t *segment_id, uint64_t *seg_pos
 ) {
+
+  // if complementarity == p7_NOCOMPLEMENT, the end positions are in context of FM->T
+  // otherwise, they're in context of revcomp(FM->T).
+  if (complementarity == p7_COMPLEMENT)  // need location in forward context:
+    fm_pos = fms->N - fm_pos - 1;
+
   *segment_id = fm_computeSequenceOffset( fms, meta, fm_id, fm_pos);
-  *seg_pos    =  ( fm_pos - meta->seq_data[ *segment_id ].offset) + meta->seq_data[ *segment_id ].start - 1;
+  *seg_pos    =  ( fm_pos - meta->seq_data[ *segment_id ].fm_start) + 1;
+
+  if (complementarity == p7_COMPLEMENT) // now reverse orientation
+    *seg_pos    =  meta->seq_data[ *segment_id ].length - *seg_pos + 1;
+
 
   //verify that the hit doesn't extend beyond the bounds of the target sequence
-  if (direction == fm_forward) {
-    if (*seg_pos + length > meta->seq_data[ *segment_id ].start + meta->seq_data[ *segment_id ].length ) {
-      *segment_id  = *seg_pos  = -1;  // goes into the next sequence, so it should be ignored
-      return eslOK;
-    }
-  } else { //backward
-    if ((int)*seg_pos - length + 1 < 0 ) {
-      *segment_id  = *seg_pos  = -1; // goes into the previous sequence, so it should be ignored
-      return eslOK;
-    }
-  }
-
+  if (*seg_pos + length - 1  > meta->seq_data[ *segment_id ].length )
+    return eslERANGE;
 
   return eslOK;
 }
@@ -352,59 +459,47 @@ fm_getOriginalPosition (const FM_DATA *fms, FM_METADATA *meta, int fm_id, int le
 int
 fm_initConfigGeneric( FM_CFG *cfg, ESL_GETOPTS *go ) {
 
-  cfg->maskSA       =  cfg->meta->freq_SA - 1;
-  cfg->shiftSA      =  cfg->meta->SA_shift;
+  cfg->ssv_length        = (go ? esl_opt_GetInteger(go, "--seed_ssv_length") : -1);
+  cfg->max_depth         = (go ? esl_opt_GetInteger(go, "--seed_max_depth") :  -1);
+  cfg->drop_max_len      = (go ? esl_opt_GetInteger(go, "--seed_drop_max_len") : -1);
+  cfg->consec_pos_req    = (go ? esl_opt_GetInteger(go, "--seed_req_pos") : -1);
+  cfg->consensus_match_req = (go ? esl_opt_GetInteger(go, "--seed_consens_match") : 10);
+  cfg->drop_lim          = eslCONST_LOG2 * (go ? esl_opt_GetReal(go, "--seed_drop_lim") : -1.0);  // convert from bits to nats
+  cfg->score_density_req = eslCONST_LOG2 * (go ? esl_opt_GetReal(go, "--seed_sc_density") : -1.0);// convert from bits to nats
+  cfg->scthreshFM        = eslCONST_LOG2 * (go ? esl_opt_GetReal(go, "--seed_sc_thresh") : -1.0); // convert from bits to nats
 
-  cfg->msv_length      = (go ? esl_opt_GetInteger(go, "--fm_msv_length") : -1);
-  cfg->max_depth       = (go ? esl_opt_GetInteger(go, "--fm_max_depth") :  -1);
-  cfg->neg_len_limit   = (go ? esl_opt_GetInteger(go, "--fm_max_neg_len") : -1);
-  cfg->consec_pos_req  = (go ? esl_opt_GetInteger(go, "--fm_req_pos") : -1);
-  cfg->score_ratio_req = (go ? esl_opt_GetReal(go, "--fm_sc_ratio") : -1.0);
-  cfg->max_scthreshFM  = (go ? esl_opt_GetReal(go, "--fm_max_scthresh") : -1.0);
-
-
-/*
-
-  //bounding cutoffs
-  cfg->max_scthreshFM  = 10.5;
-
-  cfg->max_depth       = 18;
-  cfg->neg_len_limit   = 4;
-  cfg->consec_pos_req  = 4;
-  cfg->score_ratio_req = 0.40;
-  cfg->msv_length      = 50;
-
-*/
   return eslOK;
 }
 
-/* Function:  freeFM()
+/* Function:  fm_FM_free()
  * Synopsis:  release the memory required to store an individual FM-index
  */
 void
-fm_freeFM ( FM_DATA *fm, int isMainFM)
+fm_FM_destroy ( FM_DATA *fm, int isMainFM)
 {
 
-  if (fm->BWT_mem)      free (fm->BWT_mem);
-  if (fm->C)            free (fm->C);
-  if (fm->occCnts_b)    free (fm->occCnts_b);
-  if (fm->occCnts_sb)   free (fm->occCnts_sb);
+  free (fm->BWT_mem);
+  free (fm->C);
+  free (fm->occCnts_b);
+  free (fm->occCnts_sb);
 
-  if (isMainFM && fm->T)  free (fm->T);
-  if (isMainFM && fm->SA) free (fm->SA);
+  if (isMainFM) {
+     free (fm->T);
+     free (fm->SA);
+  }
 }
 
-/* Function:  readFM()
+/* Function:  fm_FM_read()
  * Synopsis:  Read the FM index off disk
  * Purpose:   Read the FM-index as written by fmbuild.
  *            First read the metadata header, then allocate space for the full index,
  *            then read it in.
  */
 int
-fm_readFM( FM_DATA *fm, FM_METADATA *meta, int getAll )
+fm_FM_read( FM_DATA *fm, FM_METADATA *meta, int getAll )
 {
   //shortcut variables
-  int *C               = NULL;
+  int64_t *C               = NULL;
 
   int status;
   int i;
@@ -412,45 +507,48 @@ fm_readFM( FM_DATA *fm, FM_METADATA *meta, int getAll )
   uint16_t *occCnts_b  = NULL;  //convenience variables, used to simplify macro calls
   uint32_t *occCnts_sb = NULL;
 
-  int compressed_bytes;
+  int32_t compressed_bytes;
   int num_freq_cnts_b;
   int num_freq_cnts_sb;
   int num_SA_samples;
-  int prevC;
+  int64_t prevC;
   int cnt;
   int chars_per_byte = 8/meta->charBits;
 
 
-  if(fread(&(fm->N), sizeof(fm->N), 1, meta->fp) !=  1)
+  if(fread(&(fm->N), sizeof(uint64_t), 1, meta->fp) !=  1)
     esl_fatal( "%s: Error reading block_length in FM index.\n", __FILE__);
-  if(fread(&(fm->term_loc), sizeof(fm->term_loc), 1, meta->fp) !=  1)
+  if(fread(&(fm->term_loc), sizeof(uint32_t), 1, meta->fp) !=  1)
     esl_fatal( "%s: Error reading terminal location in FM index.\n", __FILE__);
-  if(fread(&(fm->seq_offset), sizeof(fm->seq_offset), 1, meta->fp) !=  1)
+  if(fread(&(fm->seq_offset), sizeof(uint32_t), 1, meta->fp) !=  1)
     esl_fatal( "%s: Error reading seq_offset in FM index.\n", __FILE__);
-  if(fread(&(fm->overlap), sizeof(fm->overlap), 1, meta->fp) !=  1)
+  if(fread(&(fm->ambig_offset), sizeof(uint32_t), 1, meta->fp) !=  1)
+    esl_fatal( "%s: Error reading ambig_offset in FM index.\n", __FILE__);
+  if(fread(&(fm->overlap), sizeof(uint32_t), 1, meta->fp) !=  1)
     esl_fatal( "%s: Error reading overlap in FM index.\n", __FILE__);
-  if(fread(&(fm->seq_cnt), sizeof(fm->seq_cnt), 1, meta->fp) !=  1)
+  if(fread(&(fm->seq_cnt), sizeof(uint32_t), 1, meta->fp) !=  1)
     esl_fatal( "%s: Error reading seq_cnt in FM index.\n", __FILE__);
-
+  if(fread(&(fm->ambig_cnt), sizeof(uint32_t), 1, meta->fp) !=  1)
+    esl_fatal( "%s: Error reading ambig_cnt in FM index.\n", __FILE__);
 
   compressed_bytes =   ((chars_per_byte-1+fm->N)/chars_per_byte);
-  num_freq_cnts_b  = 1+ceil((float)fm->N/meta->freq_cnt_b);
-  num_freq_cnts_sb = 1+ceil((float)fm->N/meta->freq_cnt_sb);
-  num_SA_samples   = 1+floor((float)fm->N/meta->freq_SA);
+  num_freq_cnts_b  = 1+ceil((double)fm->N/meta->freq_cnt_b);
+  num_freq_cnts_sb = 1+ceil((double)fm->N/meta->freq_cnt_sb);
+  num_SA_samples   = floor((double)fm->N/meta->freq_SA);
 
   // allocate space, then read the data
-  if (getAll) ESL_ALLOC (fm->T, compressed_bytes );
-  ESL_ALLOC (fm->BWT_mem, compressed_bytes + 31 ); // +31 for manual 16-byte alignment  ( typically only need +15, but this allows offset in memory, plus offset in case of <16 bytes of characters at the end)
+  if (getAll) ESL_ALLOC (fm->T, sizeof(uint8_t) * compressed_bytes );
+  ESL_ALLOC (fm->BWT_mem,  sizeof(uint8_t) * (compressed_bytes + 31) ); // +31 for manual 16-byte alignment  ( typically only need +15, but this allows offset in memory, plus offset in case of <16 bytes of characters at the end)
      fm->BWT =   (uint8_t *) (((unsigned long int)fm->BWT_mem + 15) & (~0xf));   // align vector memory on 16-byte boundaries
   if (getAll) ESL_ALLOC (fm->SA, num_SA_samples * sizeof(uint32_t));
-  ESL_ALLOC (fm->C, 1+meta->alph_size * sizeof(uint32_t));
+  ESL_ALLOC (fm->C, (1+meta->alph_size) * sizeof(int64_t));
   ESL_ALLOC (fm->occCnts_b,  num_freq_cnts_b *  (meta->alph_size ) * sizeof(uint16_t)); // every freq_cnt positions, store an array of ints
   ESL_ALLOC (fm->occCnts_sb,  num_freq_cnts_sb *  (meta->alph_size ) * sizeof(uint32_t)); // every freq_cnt positions, store an array of ints
 
 
   if(getAll && fread(fm->T, sizeof(uint8_t), compressed_bytes, meta->fp) != compressed_bytes)
     esl_fatal( "%s: Error reading T in FM index.\n", __FILE__);
-  if(fread(fm->BWT, sizeof(uint8_t), compressed_bytes, meta->fp) != compressed_bytes)
+  if( fread(fm->BWT, sizeof(uint8_t), compressed_bytes, meta->fp)  != compressed_bytes)
     esl_fatal( "%s: Error reading BWT in FM index.\n", __FILE__);
   if(getAll && fread(fm->SA, sizeof(uint32_t), (size_t)num_SA_samples, meta->fp) != (size_t)num_SA_samples)
     esl_fatal( "%s: Error reading SA in FM index.\n", __FILE__);
@@ -490,7 +588,7 @@ fm_readFM( FM_DATA *fm, FM_METADATA *meta, int getAll )
   return eslOK;
 
 ERROR:
-  fm_freeFM(fm, getAll);
+  fm_FM_destroy(fm, getAll);
   esl_fatal("Error allocating memory in %s\n", "readFM");
   return eslFAIL;
 }
@@ -508,6 +606,9 @@ fm_readFMmeta( FM_METADATA *meta)
   int status;
   int i;
 
+
+  fm_initAmbiguityList(meta->ambig_list);
+
   if( fread(&(meta->fwd_only),     sizeof(meta->fwd_only),     1, meta->fp) != 1 ||
       fread(&(meta->alph_type),    sizeof(meta->alph_type),    1, meta->fp) != 1 ||
       fread(&(meta->alph_size),    sizeof(meta->alph_size),    1, meta->fp) != 1 ||
@@ -515,11 +616,9 @@ fm_readFMmeta( FM_METADATA *meta)
       fread(&(meta->freq_SA),      sizeof(meta->freq_SA),      1, meta->fp) != 1 ||
       fread(&(meta->freq_cnt_sb),  sizeof(meta->freq_cnt_sb),  1, meta->fp) != 1 ||
       fread(&(meta->freq_cnt_b),   sizeof(meta->freq_cnt_b),   1, meta->fp) != 1 ||
-      fread(&(meta->SA_shift),     sizeof(meta->SA_shift),     1, meta->fp) != 1 ||
-      fread(&(meta->cnt_shift_sb), sizeof(meta->cnt_shift_sb), 1, meta->fp) != 1 ||
-      fread(&(meta->cnt_shift_b),  sizeof(meta->cnt_shift_b),  1, meta->fp) != 1 ||
       fread(&(meta->block_count),  sizeof(meta->block_count),  1, meta->fp) != 1 ||
       fread(&(meta->seq_count),    sizeof(meta->seq_count),    1, meta->fp) != 1 ||
+      fread(&(meta->ambig_list->count), sizeof(meta->ambig_list->count),    1, meta->fp) != 1 ||
       fread(&(meta->char_count),   sizeof(meta->char_count),   1, meta->fp) != 1
   )
     esl_fatal( "%s: Error reading meta data for FM index.\n", __FILE__);
@@ -531,10 +630,10 @@ fm_readFMmeta( FM_METADATA *meta)
 
 
   for (i=0; i<meta->seq_count; i++) {
-    if( fread(&(meta->seq_data[i].id),           sizeof(meta->seq_data[i].id),           1, meta->fp) != 1 ||
-        fread(&(meta->seq_data[i].start),        sizeof(meta->seq_data[i].start),        1, meta->fp) != 1 ||
+    if( fread(&(meta->seq_data[i].target_id),    sizeof(meta->seq_data[i].target_id),           1, meta->fp) != 1 ||
+        fread(&(meta->seq_data[i].target_start), sizeof(meta->seq_data[i].target_start),        1, meta->fp) != 1 ||
+        fread(&(meta->seq_data[i].fm_start),     sizeof(meta->seq_data[i].fm_start),        1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].length),       sizeof(meta->seq_data[i].length),       1, meta->fp) != 1 ||
-        fread(&(meta->seq_data[i].offset),       sizeof(meta->seq_data[i].offset),       1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].name_length),  sizeof(meta->seq_data[i].name_length),  1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].acc_length),   sizeof(meta->seq_data[i].acc_length),   1, meta->fp) != 1 ||
         fread(&(meta->seq_data[i].source_length),sizeof(meta->seq_data[i].source_length),1, meta->fp) != 1 ||
@@ -556,6 +655,20 @@ fm_readFMmeta( FM_METADATA *meta)
 
   }
 
+  if (meta->ambig_list->count > meta->ambig_list->size) {
+    ESL_REALLOC(meta->ambig_list->ranges,  meta->ambig_list->count  * sizeof(FM_INTERVAL));
+    if (meta->ambig_list->ranges == NULL  )
+      esl_fatal("unable to allocate memory to store FM ambiguity data\n");
+    meta->ambig_list->size = meta->ambig_list->count;
+  }
+
+  for (i=0; i<meta->ambig_list->count; i++) {
+    if( fread(&(meta->ambig_list->ranges[i].lower),   sizeof(meta->ambig_list->ranges[i].lower),       1, meta->fp) != 1 ||
+        fread(&(meta->ambig_list->ranges[i].upper),   sizeof(meta->ambig_list->ranges[i].upper),       1, meta->fp) != 1
+    )
+      esl_fatal( "%s: Error reading ambiguity data for FM index.\n", __FILE__);
+  }
+
   return eslOK;
 
 ERROR:
@@ -572,46 +685,98 @@ ERROR:
 }
 
 
-
-
-
 /* Function:  fm_configAlloc()
- * Synopsis:  Allocate a 16-byte-aligned <FM_CFG> model object, and it's FM_METADATA
- *
+ * Synopsis:  Allocate a <FM_CFG> model object, and its FM_METADATA
  */
 int
-fm_configAlloc(void **mem, FM_CFG **cfg)
+fm_configAlloc(FM_CFG **cfg)
 {
   int status;
 
-  if (mem == NULL || cfg == NULL)
+  if ( cfg == NULL)
     esl_fatal("null pointer when allocating FM configuration\n");
 
-  *mem = NULL;
   *cfg = NULL;
 
-  ESL_ALLOC(*mem, sizeof(FM_CFG)+ 15 );
-    *cfg =   (FM_CFG *) (((unsigned long int)(*mem) + 15) & (~0xf));   /* align vector memory on 16-byte boundaries */
+  ESL_ALLOC(*cfg, sizeof(FM_CFG) );
+  if ((*cfg) == NULL)
+    esl_fatal("unable to allocate memory to store FM config data\n");
 
   ESL_ALLOC((*cfg)->meta, sizeof(FM_METADATA));
+  if ((*cfg)->meta == NULL)
+    esl_fatal("unable to allocate memory to store FM meta data\n");
+
+  ESL_ALLOC ((*cfg)->meta->ambig_list, sizeof(FM_AMBIGLIST));
+  if ((*cfg)->meta->ambig_list == NULL)
+      esl_fatal("unable to allocate memory to store FM ambiguity data\n");
+
+
+
 
   return eslOK;
 
 ERROR:
-  if (*cfg != NULL)
-   if ((*cfg)->meta != NULL) free ((*cfg)->meta);
-
-  if (*mem != NULL)
-    free (*mem);
-
+  if (*cfg != NULL) {
+    if ((*cfg)->meta != NULL) free ((*cfg)->meta);
+    free (*cfg);
+  }
   return eslEMEM;
 }
 
 
+
+/* Function:  fm_configDestroy()
+ * Synopsis:  Destroy various memory items used for the FMindex implementation
+ *
+ * Purpose:   Destroy the masks used by the FM index, the metadata for
+ *            the FM index, and the config itself.
+ */
+int
+fm_configDestroy(FM_CFG *cfg ) {
+  if (cfg) {
+#if   defined (p7_IMPL_SSE)
+    if (cfg->fm_chars_mem)         free(cfg->fm_chars_mem);
+    if (cfg->fm_masks_mem)         free(cfg->fm_masks_mem);
+    if (cfg->fm_reverse_masks_mem) free(cfg->fm_reverse_masks_mem);
+#endif
+    fm_metaDestroy(cfg->meta);
+    free(cfg);
+  }
+  return eslOK;
+}
+
+/* Function:  fm_metaDestroy()
+ * Synopsis:  Destroy various metadata for the FMindex implementation
+ */
+int
+fm_metaDestroy(FM_METADATA *meta ) {
+  int i;
+  if (meta != NULL) {
+    for (i=0; i<meta->seq_count; i++) {
+      if(meta->seq_data[i].name)   free(meta->seq_data[i].name);
+      if(meta->seq_data[i].acc)    free(meta->seq_data[i].acc);
+      if(meta->seq_data[i].source) free(meta->seq_data[i].source);
+      if(meta->seq_data[i].desc)   free(meta->seq_data[i].desc);
+
+    }
+    free(meta->seq_data);
+
+    if (meta->ambig_list) {
+      if (meta->ambig_list->ranges) free(meta->ambig_list->ranges);
+      free(meta->ambig_list);
+    }
+
+    fm_alphabetDestroy(meta);
+    free (meta);
+  }
+
+  return eslOK;
+}
+
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
