@@ -9,7 +9,7 @@
  *   6. Copyright and license information.
  * 
  * SRE, Fri Aug 15 10:38:21 2008 [Janelia]
- * SVN $Id: generic_msv.c 4103 2012-06-24 02:09:43Z wheelert $
+ * SVN $Id$
  */
 
 #include "p7_config.h"
@@ -165,7 +165,7 @@ p7_GMSV_longtarget(const ESL_DSQ *dsq, int L, P7_PROFILE *gm, P7_GMX *gx, float
 
   float 	   tloop_total = tloop * gm->max_length;
 
-  int target_end;
+  //int target_end;
   int target_start;
 
 
@@ -226,12 +226,12 @@ p7_GMSV_longtarget(const ESL_DSQ *dsq, int L, P7_PROFILE *gm, P7_GMX *gx, float
 	  if (XMX(i,p7G_C) > sc_thresh)
 	  {
 	    target_start =  ESL_MAX(1, i - gm->max_length + 1);
-	    target_end   =  ESL_MIN(L, i + gm->max_length - 1);
+	    //target_end   =  ESL_MIN(L, i + gm->max_length - 1);
 
 	    //TODO: this is wrong - it just pretends the hit was to the middle of the model, to get dummy to
 	    // quit crashing.  Results are definitely wrong.
 	    // Need to get diagonal backtracking implemented in dummy
-	    p7_hmmwindow_new(windowlist, 0, target_start, 0, gm->M/2, 1, XMX(i,p7G_C), fm_nocomplement );
+	    p7_hmmwindow_new(windowlist, 0, target_start, 0, gm->M/2, 1, XMX(i,p7G_C), p7_NOCOMPLEMENT, L);
 
 
 		  //start the search all over again
@@ -574,8 +574,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
