@@ -281,7 +281,7 @@ p7_GViterbi_longtarget(const ESL_DSQ *dsq, int L, const P7_PROFILE *gm, P7_GMX *
 
         for (k = 1; k <= gm->M; k++) {
           if (MMX(i,k) == XMX(i,p7G_E)) {
-            p7_hmmwindow_new(windowlist, 0, i, 0, k, 1, 0.0, fm_nocomplement );
+            p7_hmmwindow_new(windowlist, 0, i, 0, k, 1, 0.0, p7_NOCOMPLEMENT, L);
           }
           MMX(i,0) = IMX(i,0) = DMX(i,0) = -eslINFINITY;
         }
@@ -713,13 +713,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  * 
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/generic_viterbi.c $
+ * SVN $URL$
  * SVN $Id: generic_viterbi.c 3569 2011-06-16 16:16:13Z eddys $
  *****************************************************************/
