@@ -45,7 +45,7 @@ typedef struct {
   ESL_MSA    *msa;
   P7_HMM     *hmm;
   double      entropy;
-  int         force_single; /* FALSE by default,  TRUE if esl_opt_IsUsed(go, "--single") ;  only matters for single sequences */
+  int         force_single; /* FALSE by default,  TRUE if esl_opt_IsUsed(go, "--singlemx") ;  only matters for single sequences */
 } WORK_ITEM;
 
 typedef struct _pending_s {
@@ -60,7 +60,7 @@ typedef struct _pending_s {
 
 #define ALPHOPTS "--amino,--dna,--rna"                         /* Exclusive options for alphabet choice */
 #define CONOPTS "--fast,--hand"                                /* Exclusive options for model construction                    */
-#define EFFOPTS "--eent,--eclust,--eset,--enone"               /* Exclusive options for effective sequence number calculation */
+#define EFFOPTS "--eent,--eentexp,--eclust,--eset,--enone"               /* Exclusive options for effective sequence number calculation */
 #define WGTOPTS "--wgsc,--wblosum,--wpb,--wnone,--wgiven"      /* Exclusive options for relative weighting                    */
 
 static ESL_OPTIONS options[] = {
@@ -88,21 +88,35 @@ static ESL_OPTIONS options[] = {
   { "--wid",     eslARG_REAL, "0.62",  NULL,"0<=x<=1",   NULL,"--wblosum",   NULL, "for --wblosum: set identity cutoff",                   4 },
 /* Alternative effective sequence weighting strategies */
   { "--eent",    eslARG_NONE,"default",NULL, NULL,    EFFOPTS,    NULL,      NULL, "adjust eff seq # to achieve relative entropy target",  5 },
+  { "--eentexp", eslARG_NONE,"default",NULL, NULL,    EFFOPTS,    NULL,      NULL, "adjust eff seq # to reach rel. ent. target using exp scaling",  5 },
   { "--eclust",  eslARG_NONE,  FALSE,  NULL, NULL,    EFFOPTS,    NULL,      NULL, "eff seq # is # of single linkage clusters",            5 },
   { "--enone",   eslARG_NONE,  FALSE,  NULL, NULL,    EFFOPTS,    NULL,      NULL, "no effective seq # weighting: just use nseq",          5 },
   { "--eset",    eslARG_REAL,   NULL,  NULL, NULL,    EFFOPTS,    NULL,      NULL, "set eff seq # for all models to <x>",                  5 },
-  { "--ere",     eslARG_REAL,   NULL,  NULL,"x>0",       NULL, "--eent",     NULL, "for --eent: set minimum rel entropy/position to <x>",  5 },
-  { "--esigma",  eslARG_REAL, "45.0",  NULL,"x>0",       NULL, "--eent",     NULL, "for --eent: set sigma param to <x>",                   5 },
+  { "--ere",     eslARG_REAL,   NULL,  NULL,"x>0",       NULL,    NULL,     NULL, "for --eent[exp]: set minimum rel entropy/position to <x>",  5 },
+  { "--esigma",  eslARG_REAL, "45.0",  NULL,"x>0",       NULL,    NULL,     NULL, "for --eent[exp]: set sigma param to <x>",                   5 },
   { "--eid",     eslARG_REAL, "0.62",  NULL,"0<=x<=1",   NULL,"--eclust",    NULL, "for --eclust: set fractional identity cutoff to <x>",  5 },
 /* Alternative prior strategies */
   { "--pnone",   eslARG_NONE,  FALSE,  NULL, NULL,       NULL,  NULL,"--plaplace", "don't use any prior; parameters are frequencies",      9 },
   { "--plaplace",eslARG_NONE,  FALSE,  NULL, NULL,       NULL,  NULL,   "--pnone", "use a Laplace +1 prior",                               9 },
+  { "--popen",    eslARG_REAL,  NULL,  NULL,"0<=x<0.5",NULL, NULL,           "",   "force gap open prob. (w/ --singlemx, aa default 0.02, nt 0.031)",       9 },
+  { "--pextend",  eslARG_REAL,  NULL,  NULL, "0<=x<1", NULL, NULL,           "",   "force gap extend prob. (w/ --singlemx, aa default 0.4, nt 0.75)",      9 },
+
+  { "--tmm",  eslARG_REAL,"2.0", NULL, NULL,      NULL,      NULL,    NULL, "MM transition",   9 },
+  { "--tmi",  eslARG_REAL,"0.1", NULL, NULL,      NULL,      NULL,    NULL, "MI transition",   9 },
+  { "--tmd",  eslARG_REAL,"0.1", NULL, NULL,      NULL,      NULL,    NULL, "MD transition",   9 },
+
+  { "--tim",  eslARG_REAL,"0.12", NULL, NULL,      NULL,      NULL,    NULL, "IM transition",   9 },
+  { "--tii",  eslARG_REAL,"0.4", NULL, NULL,      NULL,      NULL,    NULL,  "II transition",   9 },
+
+  { "--tdm",  eslARG_REAL,"0.5", NULL, NULL,      NULL,      NULL,    NULL, "DM transition",   9 },
+  { "--tdd",  eslARG_REAL,"1.0", NULL, NULL,      NULL,      NULL,    NULL, "DD transition",   9 },
+
+
+
 /* Single sequence methods */
-  { "--single",  eslARG_NONE,   FALSE, NULL,   NULL,   NULL,  NULL,            "",   "use substitution score matrix for single-sequence protein inputs",     10 },
-  { "--popen",    eslARG_REAL,   "0.02", NULL,"0<=x<0.5",NULL, NULL,           "",   "gap open probability (with --single)",                         10 },
-  { "--pextend",  eslARG_REAL,    "0.4", NULL, "0<=x<1", NULL, NULL,           "",   "gap extend probability (with --single)",                       10 },
-  { "--mx",     eslARG_STRING, "BLOSUM62", NULL, NULL,   NULL, NULL,   "--mxfile",   "substitution score matrix (built-in matrices, with --single)", 10 },
-  { "--mxfile", eslARG_INFILE,     NULL, NULL,   NULL,   NULL, NULL,       "--mx",   "read substitution score matrix from file <f> (with --single)", 10 },
+  { "--singlemx", eslARG_NONE,   FALSE, NULL,   NULL,   NULL,  NULL,           "",   "use substitution score matrix for single-sequence inputs",     10 },
+  { "--mx",     eslARG_STRING, "BLOSUM62", NULL, NULL,   NULL, NULL,   "--mxfile",   "substitution score matrix (built-in matrices, with --singlemx)", 10 },
+  { "--mxfile", eslARG_INFILE,     NULL, NULL,   NULL,   NULL, NULL,       "--mx",   "read substitution score matrix from file <f> (with --singlemx)", 10 },
 
   /* Control of E-value calibration */
   { "--EmL",     eslARG_INT,    "200", NULL,"n>0",       NULL,    NULL,      NULL, "length of sequences for MSV Gumbel mu fit",            6 },   
@@ -120,12 +134,18 @@ static ESL_OPTIONS options[] = {
 #ifdef HAVE_MPI
   { "--mpi",     eslARG_NONE,   FALSE, NULL, NULL,      NULL,      NULL,  NULL,  "run as an MPI parallel program",                        8 },
 #endif
-  { "--stall",   eslARG_NONE,   FALSE, NULL, NULL,      NULL,      NULL,    NULL, "arrest after start: for attaching debugger to process", 8 },  
-  { "--informat", eslARG_STRING, NULL, NULL, NULL,      NULL,      NULL,    NULL, "assert input alifile is in format <s> (no autodetect)", 8 },
-  { "--seed",     eslARG_INT,   "42", NULL, "n>=0",     NULL,      NULL,    NULL, "set RNG seed to <n> (if 0: one-time arbitrary seed)",   8 },
-  { "--w_beta",   eslARG_REAL,  NULL, NULL, NULL,       NULL,      NULL,    NULL, "tail mass at which window length is determined",        8 },
-  { "--w_length", eslARG_INT,   NULL, NULL, NULL,       NULL,      NULL,    NULL, "window length ",                                        8 },
-  { "--uniforminsert", eslARG_NONE,   FALSE, NULL, NULL, NULL,     NULL,    NULL, "learn uniform insert parameters (not position-specific)", 8 },
+  { "--stall",   eslARG_NONE,       FALSE, NULL, NULL,    NULL,     NULL,    NULL, "arrest after start: for attaching debugger to process", 8 },
+  { "--informat", eslARG_STRING,     NULL, NULL, NULL,    NULL,     NULL,    NULL, "assert input alifile is in format <s> (no autodetect)", 8 },
+  { "--seed",     eslARG_INT,        "42", NULL, "n>=0",  NULL,     NULL,    NULL, "set RNG seed to <n> (if 0: one-time arbitrary seed)",   8 },
+  { "--w_beta",   eslARG_REAL,       NULL, NULL, NULL,    NULL,     NULL,    NULL, "tail mass at which window length is determined",        8 },
+  { "--w_length", eslARG_INT,        NULL, NULL, NULL,    NULL,     NULL,    NULL, "window length ",                                        8 },
+  { "--maxinsertlen",  eslARG_INT,   NULL, NULL, "n>=5",  NULL,     NULL,    NULL, "pretend all inserts are length <= <n>",   8 },
+
+  /* expert-only option (for now), hidden from view. May not keep. */
+  { "--seq_weights_r",  eslARG_OUTFILE,FALSE, NULL, NULL,      NULL,      NULL,    NULL, "write seq weights after relative seq weighting to file <f>",   99 },
+  { "--seq_weights_e",  eslARG_OUTFILE,FALSE, NULL, NULL,      NULL,      NULL,    NULL, "write seq weights after entropy weighting to file <f>",   99 },
+
+
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
@@ -140,7 +160,7 @@ struct cfg_s {
 
   char         *alifile;	/* name of the alignment file we're building HMMs from  */
   int           fmt;		/* format code for alifile */
-  ESLX_MSAFILE *afp;            /* open alifile  */
+  ESL_MSAFILE  *afp;            /* open alifile  */
   ESL_ALPHABET *abc;		/* digital alphabet */
 
   char         *hmmName;        /* hmm file name supplied from -n          */
@@ -150,6 +170,12 @@ struct cfg_s {
   char         *postmsafile;	/* optional file to resave annotated, modified MSAs to  */
   FILE         *postmsafp;	/* open <postmsafile>, or NULL */
 
+  char         *seqweights_r_file;  /* optional file to write sequence weights after relative seq weighting */
+  FILE         *seqweights_r_fp;  /* open <seqweights_r_file>, or NULL */
+
+  char         *seqweights_e_file;  /* optional file to write sequence weights after entropy weighting */
+  FILE         *seqweights_e_fp;  /* open <seqweights_e_file>, or NULL */
+
   int           nali;		/* which # alignment this is in file (only valid in serial mode)   */
   int           nnamed;		/* number of alignments that had their own names */
 
@@ -173,7 +199,7 @@ static void pipeline_thread(void *arg);
 #ifdef HAVE_MPI
 static void  mpi_master    (const ESL_GETOPTS *go, struct cfg_s *cfg);
 static void  mpi_worker    (const ESL_GETOPTS *go, struct cfg_s *cfg);
-static void  mpi_init_open_failure(ESLX_MSAFILE *afp, int status);
+static void  mpi_init_open_failure(ESL_MSAFILE *afp, int status);
 static void  mpi_init_other_failure(char *format, ...);
 #endif
 
@@ -292,6 +318,7 @@ output_header(const ESL_GETOPTS *go, const struct cfg_s *cfg)
   if (esl_opt_IsUsed(go, "--wnone")      && fprintf(cfg->ofp, "# relative weighting scheme:        none\n")                                           < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--wid")        && fprintf(cfg->ofp, "# frac id cutoff for BLOSUM wgts:   %f\n",        esl_opt_GetReal(go, "--wid"))        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--eent")       && fprintf(cfg->ofp, "# effective seq number scheme:      entropy weighting\n")                              < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--eentexp")    && fprintf(cfg->ofp, "# effective seq number scheme:      entropy weighting using exponent-based scaling\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--eclust")     && fprintf(cfg->ofp, "# effective seq number scheme:      single linkage clusters\n")                        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--enone")      && fprintf(cfg->ofp, "# effective seq number scheme:      none\n")                                           < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--eset")       && fprintf(cfg->ofp, "# effective seq number:             set to %f\n", esl_opt_GetReal(go, "--eset"))       < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
@@ -307,10 +334,13 @@ output_header(const ESL_GETOPTS *go, const struct cfg_s *cfg)
   if (esl_opt_IsUsed(go, "--EfL")        && fprintf(cfg->ofp, "# seq length for Fwd exp tau fit:   %d\n",        esl_opt_GetInteger(go, "--EfL"))     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--EfN")        && fprintf(cfg->ofp, "# seq number for Fwd exp tau fit:   %d\n",        esl_opt_GetInteger(go, "--EfN"))     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--Eft")        && fprintf(cfg->ofp, "# tail mass for Fwd exp tau fit:    %f\n",        esl_opt_GetReal(go, "--Eft"))        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-  if (esl_opt_IsUsed(go, "--popen")      && fprintf(cfg->ofp, "# gap open probability:            %f\n",         esl_opt_GetReal   (go, "--popen"))   < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-  if (esl_opt_IsUsed(go, "--pextend")    && fprintf(cfg->ofp, "# gap extend probability:          %f\n",         esl_opt_GetReal   (go, "--pextend")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-  if (esl_opt_IsUsed(go, "--mx")         && fprintf(cfg->ofp, "# subst score matrix (built-in):   %s\n",         esl_opt_GetString (go, "--mx"))      < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-  if (esl_opt_IsUsed(go, "--mxfile")     && fprintf(cfg->ofp, "# subst score matrix (file):       %s\n",         esl_opt_GetString (go, "--mxfile"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--singlemx")   && fprintf(cfg->ofp, "# use score matrix for 1-seq MSAs:  on\n")                                              < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--popen")      && fprintf(cfg->ofp, "# gap open probability:             %f\n",         esl_opt_GetReal   (go, "--popen"))   < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--pextend")    && fprintf(cfg->ofp, "# gap extend probability:           %f\n",         esl_opt_GetReal   (go, "--pextend")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--mx")         && fprintf(cfg->ofp, "# subst score matrix (built-in):    %s\n",         esl_opt_GetString (go, "--mx"))      < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--mxfile")     && fprintf(cfg->ofp, "# subst score matrix (file):        %s\n",         esl_opt_GetString (go, "--mxfile"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--maxinsertlen")  && fprintf(cfg->ofp, "# max insert length:                %d\n",         esl_opt_GetInteger (go, "--maxinsertlen"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+
 
 #ifdef HMMER_THREADS
   if (esl_opt_IsUsed(go, "--cpu")        && fprintf(cfg->ofp, "# number of worker threads:         %d\n",        esl_opt_GetInteger(go, "--cpu"))     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");  
@@ -322,13 +352,44 @@ output_header(const ESL_GETOPTS *go, const struct cfg_s *cfg)
     if (esl_opt_GetInteger(go, "--seed") == 0  && fprintf(cfg->ofp,"# random number seed:               one-time arbitrary\n")                        < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
     else if                              (  fprintf(cfg->ofp,"# random number seed set to:        %d\n",         esl_opt_GetInteger(go, "--seed"))    < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   }
-  if (esl_opt_IsUsed(go, "--w_beta")     && fprintf(cfg->ofp, "# window length beta value:         %g bits\n",   esl_opt_GetReal(go, "--w_beta"))     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--w_beta")     && fprintf(cfg->ofp, "# window length tail mass:          %g bits\n",   esl_opt_GetReal(go, "--w_beta"))     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--w_length")   && fprintf(cfg->ofp, "# window length :                   %d\n",        esl_opt_GetInteger(go, "--w_length"))< 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 
+
   if (fprintf(cfg->ofp, "# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   return eslOK;
 }
 
+void
+apply_fixed_gap_params(P7_HMM *hmm, double popen, double pextend){
+  int k;
+  for (k = 0; k <= hmm->M; k++)
+  {
+     if (popen != -1) {
+        hmm->t[k][p7H_MM] = 1.0 - 2 * popen;
+        hmm->t[k][p7H_MI] = popen;
+        hmm->t[k][p7H_MD] = popen;
+     }
+     if (pextend != -1) {
+        hmm->t[k][p7H_IM] = 1.0 - pextend;
+        hmm->t[k][p7H_II] = pextend;
+        hmm->t[k][p7H_DM] = 1.0 - pextend;
+        hmm->t[k][p7H_DD] = pextend;
+     }
+  }
+
+  /* Deal w/ special stuff at node M, overwriting a little of what we
+   * just did.
+   */
+  if (popen != -1) {
+    hmm->t[hmm->M][p7H_MM] = 1.0 - popen;
+  }
+  hmm->t[hmm->M][p7H_MD] = 0.;
+  hmm->t[hmm->M][p7H_DM] = 1.0;
+  hmm->t[hmm->M][p7H_DD] = 0.;
+
+}
+
 int
 main(int argc, char **argv)
 {
@@ -356,7 +417,13 @@ main(int argc, char **argv)
   cfg.abc         = NULL;	           
   cfg.hmmfp       = NULL;	           
   cfg.postmsafile = esl_opt_GetString(go, "-O"); /* NULL by default */
-  cfg.postmsafp   = NULL;                  
+  cfg.postmsafp         = NULL;
+
+  cfg.seqweights_r_file = esl_opt_GetString(go, "--seq_weights_r"); /* NULL by default */
+  cfg.seqweights_r_fp   = NULL;
+
+  cfg.seqweights_e_file = esl_opt_GetString(go, "--seq_weights_e"); /* NULL by default */
+  cfg.seqweights_e_fp   = NULL;
 
   cfg.nali       = 0;		           /* this counter is incremented in masters */
   cfg.nnamed     = 0;		           /* 0 or 1 if a single MSA; == nali if multiple MSAs */
@@ -367,7 +434,7 @@ main(int argc, char **argv)
   cfg.hmmName    = esl_opt_GetString(go, "-n"); /* NULL by default */
 
   if (esl_opt_IsOn(go, "--informat")) {
-    cfg.fmt = eslx_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
+    cfg.fmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--informat"));
     if (cfg.fmt == eslMSAFILE_UNKNOWN) p7_Fail("%s is not a recognized input sequence file format\n", esl_opt_GetString(go, "--informat"));
   }
 
@@ -415,7 +482,7 @@ main(int argc, char **argv)
    */
   if (cfg.my_rank == 0) {
     if (esl_opt_IsOn(go, "-o")) { fclose(cfg.ofp); }
-    if (cfg.afp)   eslx_msafile_Close(cfg.afp);
+    if (cfg.afp)   esl_msafile_Close(cfg.afp);
     if (cfg.abc)   esl_alphabet_Destroy(cfg.abc);
     if (cfg.hmmfp) fclose(cfg.hmmfp);
   }
@@ -460,8 +527,8 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
   else if (esl_opt_GetBoolean(go, "--rna"))     cfg->abc = esl_alphabet_Create(eslRNA);
   else                                          cfg->abc = NULL;
   
-  status = eslx_msafile_Open(&(cfg->abc), cfg->alifile, NULL, cfg->fmt, NULL, &(cfg->afp));
-  if (status != eslOK) eslx_msafile_OpenFailure(cfg->afp, status);
+  status = esl_msafile_Open(&(cfg->abc), cfg->alifile, NULL, cfg->fmt, NULL, &(cfg->afp));
+  if (status != eslOK) esl_msafile_OpenFailure(cfg->afp, status);
 
   cfg->hmmfp = fopen(cfg->hmmfile, "w");
   if (cfg->hmmfp == NULL) p7_Fail("Failed to open HMM file %s for writing", cfg->hmmfile);
@@ -476,10 +543,11 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
   if (cfg->postmsafile) 
     {
       cfg->postmsafp = fopen(cfg->postmsafile, "w");
-      if (cfg->postmsafp == NULL) p7_Fail("Failed to MSA resave file %s for writing", cfg->postmsafile);
-    } 
+      if (cfg->postmsafp == NULL) p7_Fail("Failed to resave MSA file %s", cfg->postmsafile);
+    }
   else cfg->postmsafp = NULL;
 
+
   /* Looks like the i/o is set up successfully...
    * Initial output to the user
    */
@@ -498,27 +566,88 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
     }
 #endif
 
+
+  if (cfg->seqweights_r_file)
+  {
+      if (ncpus != 0)
+        p7_Fail("--seq_weights_r flag only valid with --cpu=0", NULL);
+      cfg->seqweights_r_fp = fopen(cfg->seqweights_r_file, "w");
+      if (cfg->seqweights_r_fp == NULL) p7_Fail("Failed to write sequence weight (W) file %s", cfg->seqweights_r_file);
+  }
+  else cfg->seqweights_r_fp = NULL;
+
+  if (cfg->seqweights_e_file)
+  {
+      if (ncpus != 0)
+        p7_Fail("--seq_weights_e flag only valid with --cpu=0", NULL);
+      cfg->seqweights_e_fp = fopen(cfg->seqweights_e_file, "w");
+      if (cfg->seqweights_e_fp == NULL) p7_Fail("Failed to write sequence weight (W) file %s", cfg->seqweights_e_file);
+  }
+  else cfg->seqweights_e_fp = NULL;
+
+
+
   infocnt = (ncpus == 0) ? 1 : ncpus;
   ESL_ALLOC(info, sizeof(*info) * infocnt);
 
   for (i = 0; i < infocnt; ++i)
-    {
+  {
       info[i].bg = p7_bg_Create(cfg->abc);
       info[i].bld = p7_builder_Create(go, cfg->abc);
 
       if (info[i].bld == NULL)  p7_Fail("p7_builder_Create failed");
 
-      //do this here instead of in p7_builder_Create(), because it's an hmmbuild-specific option
-      info[i].bld->do_uniform_insert = esl_opt_IsOn(go, "--uniforminsert");
 
 
+      //do this here instead of in p7_builder_Create(), because it's an hmmbuild-specific option
+
+      if ( esl_opt_IsOn(go, "--maxinsertlen") )
+        info[i].bld->max_insert_len    = esl_opt_GetInteger(go, "--maxinsertlen");
+
+      if( !esl_opt_GetBoolean(go, "--pnone") && !esl_opt_GetBoolean(go, "--plaplace") )
+      {
+           if (esl_opt_IsUsed(go, "--tmm"))  info[i].bld->prior->tm->alpha[0][0] = esl_opt_GetReal(go, "--tmm"); // TMM
+           if (esl_opt_IsUsed(go, "--tmi"))  info[i].bld->prior->tm->alpha[0][1] = esl_opt_GetReal(go, "--tmi"); // TMM
+           if (esl_opt_IsUsed(go, "--tmd"))  info[i].bld->prior->tm->alpha[0][2] = esl_opt_GetReal(go, "--tmd"); // TMM
+
+           if (esl_opt_IsUsed(go, "--tim"))  info[i].bld->prior->ti->alpha[0][0] = esl_opt_GetReal(go, "--tim"); // TMM
+           if (esl_opt_IsUsed(go, "--tii"))  info[i].bld->prior->ti->alpha[0][1] = esl_opt_GetReal(go, "--tii"); // TMM
+
+           if (esl_opt_IsUsed(go, "--tdm"))  info[i].bld->prior->td->alpha[0][0] = esl_opt_GetReal(go, "--tdm"); // TMM
+           if (esl_opt_IsUsed(go, "--tdd"))  info[i].bld->prior->td->alpha[0][1] = esl_opt_GetReal(go, "--tdd"); // TMM
+
+      }
+
+
+
+      double popen;
+      double pextend;
+      if ( cfg->abc->type == eslDNA || cfg->abc->type == eslRNA ) {
+        //If user hasn't overridden defaults, assign the nucleotide defaults
+        popen   = esl_opt_IsUsed(go, "--popen")   ? esl_opt_GetReal(go, "--popen") : 0.03125;
+        pextend = esl_opt_IsUsed(go, "--pextend") ? esl_opt_GetReal(go, "--pextend") : 0.75;
+      } else {
+        //protein defaults
+        popen   = esl_opt_IsUsed(go, "--popen")   ? esl_opt_GetReal(go, "--popen") : 0.02;
+        pextend = esl_opt_IsUsed(go, "--pextend") ? esl_opt_GetReal(go, "--pextend") : 0.4;
+      }
+
       /* Default matrix is stored in the --mx option, so it's always IsOn().
        * Check --mxfile first; then go to the --mx option and the default.
        */
-      if ( cfg->abc->type == eslAMINO && esl_opt_IsUsed(go, "--single")) {
-        if (esl_opt_IsOn(go, "--mxfile")) status = p7_builder_SetScoreSystem (info[i].bld, esl_opt_GetString(go, "--mxfile"), NULL, esl_opt_GetReal(go, "--popen"), esl_opt_GetReal(go, "--pextend"), info[i].bg);
-        else                              status = p7_builder_LoadScoreSystem(info[i].bld, esl_opt_GetString(go, "--mx"),           esl_opt_GetReal(go, "--popen"), esl_opt_GetReal(go, "--pextend"), info[i].bg);
+      if ( esl_opt_IsUsed(go, "--singlemx") ) {
+        char  *mx      = esl_opt_GetString(go, "--mx");
+
+        if ( cfg->abc->type == eslDNA || cfg->abc->type == eslRNA ) {
+          //If user hasn't overridden defaults, assign the nucleotide defaults
+          if ( !esl_opt_IsUsed(go, "--mx") )       mx      = "DNA1";
+        }
+        if (esl_opt_IsOn(go, "--mxfile")) status = p7_builder_SetScoreSystem (info[i].bld, esl_opt_GetString(go, "--mxfile"), NULL, popen, pextend, info[i].bg);
+        else                              status = p7_builder_LoadScoreSystem(info[i].bld, mx,                                      popen, pextend, info[i].bg);
         if (status != eslOK) p7_Fail("Failed to set single query seq score system:\n%s\n", info[i].bld->errbuf);
+      } else {
+        if (esl_opt_IsUsed(go, "--popen") )  info[i].bld->popen   = popen;
+        if (esl_opt_IsUsed(go, "--pextend")) info[i].bld->pextend = pextend;
       }
 
       /* special arguments for hmmbuild */
@@ -530,11 +659,11 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
       info[i].queue = queue;
       if (ncpus > 0) esl_threads_AddThread(threadObj, &info[i]);
 #endif
-    }
+  }
 
 #ifdef HMMER_THREADS
   for (i = 0; i < ncpus * 2; ++i)
-    {
+  {
       ESL_ALLOC(item, sizeof(*item));
 
       item->nali      = 0;
@@ -546,7 +675,7 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
 
       status = esl_workqueue_Init(queue, item);
       if (status != eslOK) esl_fatal("Failed to add block to work queue");
-    }
+  }
 #endif
 
 #ifdef HMMER_THREADS
@@ -557,22 +686,22 @@ usual_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
 #endif
 
   for (i = 0; i < infocnt; ++i)
-    {
+  {
       p7_bg_Destroy(info[i].bg);
       p7_builder_Destroy(info[i].bld);
-    }
+  }
 
 #ifdef HMMER_THREADS
   if (ncpus > 0)
-    {
+  {
       esl_workqueue_Reset(queue);
       while (esl_workqueue_Remove(queue, (void **) &item) == eslOK)
-	{
-	  free(item);
-	}
+      {
+        free(item);
+      }
       esl_workqueue_Destroy(queue);
       esl_threads_Destroy(threadObj);
-    }
+  }
 #endif
 
   free(info);
@@ -638,7 +767,7 @@ mpi_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
   else if (esl_opt_GetBoolean(go, "--rna"))     cfg->abc = esl_alphabet_Create(eslRNA);
   else                                          cfg->abc = NULL;
 
-  status = eslx_msafile_Open(&(cfg->abc), cfg->alifile, NULL, cfg->fmt, NULL, &(cfg->afp));
+  status = esl_msafile_Open(&(cfg->abc), cfg->alifile, NULL, cfg->fmt, NULL, &(cfg->afp));
   if (status != eslOK) mpi_init_open_failure(cfg->afp, status);
 
   cfg->hmmfp = fopen(cfg->hmmfile, "w");
@@ -705,9 +834,9 @@ mpi_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
     {
       if (have_work) 
 	{
-	  rstatus = eslx_msafile_Read(cfg->afp, &msa);
+	  rstatus = esl_msafile_Read(cfg->afp, &msa);
 	  if      (rstatus == eslOK)  {  cfg->nali++;                            ESL_DPRINTF1(("MPI master read MSA %s\n", msa->name == NULL? "" : msa->name));  } 
-	  else if (rstatus == eslEOF) {  have_work  = FALSE;                     ESL_DPRINTF1(("MPI master has run out of MSAs (having read %d)\n", cfg->nali)); }
+	  else if (rstatus == eslEOF) {  have_work  = FALSE;  rstatus = eslOK;   ESL_DPRINTF1(("MPI master has run out of MSAs (having read %d)\n", cfg->nali)); }
 	  else                        {  have_work  = FALSE;  xstatus = rstatus; ESL_DPRINTF1(("MPI master msa read has failed... start to shut down\n")); }
 	}
 
@@ -785,17 +914,17 @@ mpi_master(const ESL_GETOPTS *go, struct cfg_s *cfg)
   free(msalist);
   p7_bg_Destroy(bg);
 
-  if      (rstatus != eslOK) { MPI_Finalize(); eslx_msafile_ReadFailure(cfg->afp, rstatus); }
+  if      (rstatus != eslOK) { MPI_Finalize(); esl_msafile_ReadFailure(cfg->afp, rstatus); }
   else if (xstatus != eslOK) { MPI_Finalize(); p7_Fail(errmsg); }
   else                        return;
 }
 
 static void
-mpi_init_open_failure(ESLX_MSAFILE *afp, int status)
+mpi_init_open_failure(ESL_MSAFILE *afp, int status)
 {
   MPI_Bcast(&status, 1, MPI_INT, 0, MPI_COMM_WORLD);
   MPI_Finalize();
-  eslx_msafile_OpenFailure(afp, status);
+  esl_msafile_OpenFailure(afp, status);
 }
 
 static void
@@ -877,16 +1006,16 @@ mpi_worker(const ESL_GETOPTS *go, struct cfg_s *cfg)
       /* Build the HMM */
       ESL_DPRINTF2(("worker %d: has received MSA %s (%d columns, %d seqs)\n", cfg->my_rank, msa->name, msa->alen, msa->nseq));
 
-      if ( msa->nseq > 1 || cfg->abc->type != eslAMINO || !esl_opt_IsUsed(go, "--single")) {
-        if ((status = p7_Builder(bld, msa, bg, &hmm, NULL, NULL, NULL, postmsa_ptr)) != eslOK) { strcpy(errmsg, bld->errbuf); goto ERROR; }
-      } else {
-        //for protein, single sequence, use blosum matrix:
+      if ( msa->nseq == 1 && esl_opt_IsUsed(go, "--singlemx")) {
+        //for, single sequence, use blosum matrix:
         sq = esl_sq_CreateDigital(cfg->abc);
         if ((status = esl_sq_FetchFromMSA(msa, 0, &sq)) != eslOK) { strcpy(errmsg, bld->errbuf); goto ERROR; }
         if ((status = p7_SingleBuilder(bld, sq, bg, &hmm, NULL, NULL, NULL)) != eslOK) { strcpy(errmsg, bld->errbuf); goto ERROR; }
         esl_sq_Destroy(sq);
         sq = NULL;
         hmm->eff_nseq = 1;
+      } else {
+        if ((status = p7_Builder(bld, msa, bg, &hmm, NULL, NULL, NULL, postmsa_ptr, NULL, NULL)) != eslOK) { strcpy(errmsg, bld->errbuf); goto ERROR; }
       }
 
 
@@ -947,25 +1076,28 @@ serial_loop(WORKER_INFO *info, struct cfg_s *cfg, const ESL_GETOPTS *go)
   double      entropy;
 
   cfg->nali = 0;
-  while ((status = eslx_msafile_Read(cfg->afp, &msa)) != eslEOF)
+  while ((status = esl_msafile_Read(cfg->afp, &msa)) != eslEOF)
     {
-      if (status != eslOK) eslx_msafile_ReadFailure(cfg->afp, status);
+      if (status != eslOK) esl_msafile_ReadFailure(cfg->afp, status);
       cfg->nali++;  
 
       if ((status = set_msa_name(cfg, errmsg, msa)) != eslOK) p7_Fail("%s\n", errmsg); /* cfg->nnamed gets incremented in this call */
 
 
       /*         bg   new-HMM trarr gm   om  */
-      if ( msa->nseq > 1 || cfg->abc->type != eslAMINO || !esl_opt_IsUsed(go, "--single")) {
-        if ((status = p7_Builder(info->bld, msa, info->bg, &hmm, NULL, NULL, NULL, postmsa_ptr)) != eslOK) p7_Fail("build failed: %s", bld->errbuf);
-      } else {
-        //for protein, single sequence, use blosum matrix:
-        sq = esl_sq_CreateDigital(cfg->abc);
+      if ( msa->nseq == 1 && esl_opt_IsUsed(go, "--singlemx")) {
         if ((status = esl_sq_FetchFromMSA(msa, 0, &sq)) != eslOK) p7_Fail("build failed: %s", bld->errbuf);
         if ((status = p7_SingleBuilder(info->bld, sq, info->bg, &hmm, NULL, NULL, NULL)) != eslOK) p7_Fail("build failed: %s", bld->errbuf);
         esl_sq_Destroy(sq);
         sq = NULL;
         hmm->eff_nseq = 1;
+      } else {
+        if ((status = p7_Builder(info->bld, msa, info->bg, &hmm, NULL, NULL, NULL, postmsa_ptr, cfg->seqweights_r_fp,  cfg->seqweights_e_fp )) != eslOK) p7_Fail("build failed: %s", bld->errbuf);
+
+        //if not --singlemx, but the user set the popen/pextend flags, override the computed gap params now:
+        if (info->bld->popen != -1 || info->bld->pextend != -1) {
+          apply_fixed_gap_params(hmm, info->bld->popen, info->bld->pextend);
+        }
       }
       entropy = p7_MeanMatchRelativeEntropy(hmm, info->bg);
       if ((status = output_result(cfg, errmsg, cfg->nali, msa, hmm, postmsa, entropy))         != eslOK) p7_Fail(errmsg);
@@ -1004,17 +1136,17 @@ thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, struct cfg_s *cfg, const ES
   /* Main loop: */
   item = (WORK_ITEM *) newItem;
   while (sstatus == eslOK) {
-    sstatus = eslx_msafile_Read(cfg->afp, &item->msa);
+    sstatus = esl_msafile_Read(cfg->afp, &item->msa);
     if (sstatus == eslOK) {
       item->nali = ++cfg->nali;
       if (set_msa_name(cfg, errmsg, item->msa) != eslOK) p7_Fail("%s\n", errmsg);
     }
     else if (sstatus == eslEOF && processed < cfg->nali) sstatus = eslOK;
     else if (sstatus != eslEOF) 
-      eslx_msafile_ReadFailure(cfg->afp, sstatus);
+      esl_msafile_ReadFailure(cfg->afp, sstatus);
 	  
     if (sstatus == eslOK) {
-      item->force_single = esl_opt_IsUsed(go, "--single");
+      item->force_single = esl_opt_IsUsed(go, "--singlemx");
       status = esl_workqueue_ReaderUpdate(queue, item, &newItem);
       if (status != eslOK) esl_fatal("Work queue reader failed");
 
@@ -1143,12 +1275,7 @@ pipeline_thread(void *arg)
   while (item->msa != NULL)
     {
 
-      if ( item->msa->nseq > 1 || info->bg->abc->type != eslAMINO || !item->force_single) {
-        status = p7_Builder(info->bld, item->msa, info->bg, &item->hmm, NULL, NULL, NULL, &item->postmsa);
-        if (status != eslOK) p7_Fail("build failed: %s", info->bld->errbuf);
-      } else {
-        //for protein, single sequence, use blosum matrix:
-        sq = esl_sq_CreateDigital(info->bg->abc);
+      if ( item->msa->nseq == 1 && item->force_single) {
         status = esl_sq_FetchFromMSA(item->msa, 0, &sq);
         if (status != eslOK) p7_Fail("build failed: %s", info->bld->errbuf);
 
@@ -1158,6 +1285,14 @@ pipeline_thread(void *arg)
         esl_sq_Destroy(sq);
         sq = NULL;
         item->hmm->eff_nseq = 1;
+      } else {
+        status = p7_Builder(info->bld, item->msa, info->bg, &item->hmm, NULL, NULL, NULL, &item->postmsa, NULL, NULL);
+        if (status != eslOK) p7_Fail("build failed: %s", info->bld->errbuf);
+
+        //if not --singlemx, but the user set the popen/pextend flags, override the computed gap params now:
+        if (info->bld->popen != -1 || info->bld->pextend != -1) {
+          apply_fixed_gap_params(item->hmm, info->bld->popen, info->bld->pextend);
+        }
       }
 
       item->entropy   = p7_MeanMatchRelativeEntropy(item->hmm, info->bg);
@@ -1190,30 +1325,47 @@ output_result(const struct cfg_s *cfg, char *errbuf, int msaidx, ESL_MSA *msa, P
    * so we can keep the data and labels properly sync'ed.
    */
   if (msa == NULL)
-    {
+  {
+    if (cfg->abc->type == eslAMINO) {
+      if (fprintf(cfg->ofp, "#%4s %-20s %5s %5s %5s %8s %6s %s\n", " idx", "name",                 "nseq",  "alen",  "mlen",  "eff_nseq",  "re/pos",  "description")     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
+      if (fprintf(cfg->ofp, "#%4s %-20s %5s %5s %5s %8s %6s %s\n", "----", "--------------------", "-----", "-----", "-----", "--------",  "------",  "-----------") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
+    } else {
       if (fprintf(cfg->ofp, "#%4s %-20s %5s %5s %5s %5s %8s %6s %s\n", " idx", "name",                 "nseq",  "alen",  "mlen",  "W", "eff_nseq",  "re/pos",  "description")     < 0) ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
       if (fprintf(cfg->ofp, "#%4s %-20s %5s %5s %5s %5s %8s %6s %s\n", "----", "--------------------", "-----", "-----", "-----", "-----", "--------",  "------",  "-----------") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
-      return eslOK;
     }
-
+    return eslOK;
+  }
 //  if ((status = p7_hmm_Validate(hmm, errbuf, 0.0001))       != eslOK) return status;
   if ((status = p7_hmmfile_WriteASCII(cfg->hmmfp, -1, hmm)) != eslOK) ESL_FAIL(status, errbuf, "HMM save failed");
-  
+
 	             /* #   name nseq alen M max_length eff_nseq re/pos description */
-  if (fprintf(cfg->ofp, "%-5d %-20s %5d %5" PRId64 " %5d %5d %8.2f %6.3f %s\n",
-	      msaidx,
-	      (msa->name != NULL) ? msa->name : "",
-	      msa->nseq,
-	      msa->alen,
-	      hmm->M,
-	      hmm->max_length,
-	      hmm->eff_nseq,
-	      entropy,
-	      (msa->desc != NULL) ? msa->desc : "") < 0)
-    ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
-  
+  if (cfg->abc->type == eslAMINO) {
+    if (fprintf(cfg->ofp, "%-5d %-20s %5d %5" PRId64 " %5d %8.2f %6.3f %s\n",
+          msaidx,
+          (msa->name != NULL) ? msa->name : "",
+          msa->nseq,
+          msa->alen,
+          hmm->M,
+          hmm->eff_nseq,
+          entropy,
+          (msa->desc != NULL) ? msa->desc : "") < 0)
+      ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
+  } else {
+    if (fprintf(cfg->ofp, "%-5d %-20s %5d %5" PRId64 " %5d %5d %8.2f %6.3f %s\n",
+          msaidx,
+          (msa->name != NULL) ? msa->name : "",
+          msa->nseq,
+          msa->alen,
+          hmm->M,
+          hmm->max_length,
+          hmm->eff_nseq,
+          entropy,
+          (msa->desc != NULL) ? msa->desc : "") < 0)
+      ESL_EXCEPTION_SYS(eslEWRITE, "output_result: write failed");
+  }
+
   if (cfg->postmsafp != NULL && postmsa != NULL) {
-    eslx_msafile_Write(cfg->postmsafp, postmsa, eslMSAFILE_STOCKHOLM);
+    esl_msafile_Write(cfg->postmsafp, postmsa, eslMSAFILE_STOCKHOLM);
   }
 
   return eslOK;
@@ -1287,8 +1439,8 @@ set_msa_name(struct cfg_s *cfg, char *errbuf, ESL_MSA *msa)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
