@@ -14,10 +14,10 @@
 #include <ctype.h>
 
 #include <sys/socket.h>
-#include <arpa/inet.h>
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>	    /* On FreeBSD, you need netinet/in.h for struct sockaddr_in */
-#endif
+#include <netinet/in.h>     /* On FreeBSD, you need netinet/in.h for struct sockaddr_in            */
+#endif                      /* On OpenBSD, netinet/in.h is required for (must precede) arpa/inet.h */
+#include <arpa/inet.h>
 
 #include "easel.h"
 #include "esl_alphabet.h"
@@ -39,7 +39,8 @@
 #define THRESHOPTS  "-E,-T,--domE,--domT,--incE,--incT,--incdomE,--incdomT,--cut_ga,--cut_nc,--cut_tc"
 
 static ESL_OPTIONS searchOpts[] = {
-  /* Control of output */
+      /* name           type          default  env  range toggles  reqs   incomp                         help                                           docgroup*/
+    /* Control of output */
   { "--acc",        eslARG_NONE,        FALSE, NULL, NULL,    NULL,  NULL,  NULL,            "prefer accessions over names in output",                       2 },
   { "--noali",      eslARG_NONE,        FALSE, NULL, NULL,    NULL,  NULL,  NULL,            "don't output alignments, so output is smaller",                2 },
   { "--notextw",    eslARG_NONE,         NULL, NULL, NULL,    NULL,  NULL, "--textw",        "unlimit ASCII text output line width",                         2 },
@@ -82,8 +83,19 @@ static ESL_OPTIONS searchOpts[] = {
   { "--nonull2",    eslARG_NONE,        NULL,  NULL, NULL,    NULL,  NULL,  NULL,            "turn off biased composition score corrections",               12 },
   { "-Z",           eslARG_REAL,        FALSE, NULL, "x>0",   NULL,  NULL,  NULL,            "set # of comparisons done, for E-value calculation",          12 },
   { "--domZ",       eslARG_REAL,        FALSE, NULL, "x>0",   NULL,  NULL,  NULL,            "set # of significant seqs, for domain E-value calculation",   12 },
-  { "--seqdb",      eslARG_INT,         NULL,  NULL, "n>0",   NULL,  NULL,  "--hmmdb",       "protein database to search",                                  12 },
   { "--hmmdb",      eslARG_INT,         NULL,  NULL, "n>0",   NULL,  NULL,  "--seqdb",       "hmm database to search",                                      12 },
+  { "--seqdb",      eslARG_INT,         NULL,  NULL, "n>0",   NULL,  NULL,  "--hmmdb",       "protein database to search",                                  12 },
+  { "--seqdb_ranges",eslARG_STRING,     NULL,  NULL,  NULL,   NULL, "--seqdb", NULL,         "range(s) of sequences within --seqdb that will be searched",  12 },
+
+  /* name           type        default  env  range toggles reqs incomp  help                                          docgroup*/
+  { "-c",         eslARG_INT,       "1", NULL, NULL, NULL,  NULL, "--seqdb",  "use alt genetic code of NCBI transl table <n>", 15 },
+  { "-l",         eslARG_INT,      "20", NULL, NULL, NULL,  NULL, "--seqdb",  "minimum ORF length",                            15 },
+  { "-m",         eslARG_NONE,    FALSE, NULL, NULL, NULL,  NULL, "-M,--seqdb",  "ORFs must initiate with AUG only",              15 },
+  { "-M",         eslARG_NONE,    FALSE, NULL, NULL, NULL,  NULL, "-m,--seqdb",  "ORFs must start with allowed initiation codon", 15 },
+  { "--informat", eslARG_STRING,  FALSE, NULL, NULL, NULL,  NULL, "--seqdb",  "specify that input file is in format <s>",      15 },
+  { "--watson",   eslARG_NONE,    FALSE, NULL, NULL, NULL,  NULL, "--seqdb",  "only translate top strand",                     15 },
+  { "--crick",    eslARG_NONE,    FALSE, NULL, NULL, NULL,  NULL, "--seqdb",  "only translate bottom strand",                  15 },
+
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
@@ -170,7 +182,7 @@ int main(int argc, char *argv[])
   int              size;
 
   char            *seq;
-  char            *opts;
+  char             opts[MAX_READ_LEN];
   int              seqlen;
 
   int              ali;
@@ -243,11 +255,6 @@ int main(int argc, char *argv[])
     exit(1);
   }
 
-  if ((opts = malloc(MAX_READ_LEN)) == NULL) {
-    fprintf(stderr, "[%s:%d] malloc error %d - %s\n", __FILE__, __LINE__, errno, strerror(errno));
-    exit(1);
-  }
-
   /* set up a signal handler for broken pipes */
   if (signal(SIGINT, sig_int) == SIG_ERR) {
     fprintf(stderr, "[%s:%d] signal error %d - %s\n", __FILE__, __LINE__, errno, strerror(errno));
@@ -359,14 +366,11 @@ int main(int argc, char *argv[])
       /* create a commandline string with dummy program name for
        * the esl_opt_ProcessSpoof() function to parse.
        */
-      strncpy(opts, "X ", MAX_READ_LEN);
-      strncat(opts, s,    MAX_READ_LEN);
-      strncat(opts, "\n", MAX_READ_LEN);
-      opts[MAX_READ_LEN-1] = 0;
+      snprintf(opts, sizeof(opts), "X %s\n", s);
 
       if (esl_getopts_Reuse(go) != eslOK) p7_Die("Internal failure reusing options object");
       if (esl_opt_ProcessSpoof(go, opts) != eslOK) { 
-        printf("Failed to parse options string: %s\n", go->errbuf); 
+        printf("Failed to parse options string: %s\n", go->errbuf);
         continue;
       }
       if (esl_opt_VerifyConfig(go) != eslOK) { 
@@ -385,6 +389,7 @@ int main(int argc, char *argv[])
       while (*ptr && isspace(*ptr)) ++ptr;
     }
 
+
     if (*ptr && strncmp(ptr, "//", 2) != 0) {
       P7_HMM          *hmm     = NULL;        /* query HMM                       */
       //     P7_HMMFILE      *hfp     = NULL;        /* open input HMM file             */
@@ -510,6 +515,19 @@ int main(int argc, char *argv[])
           char   *base;
           P7_HIT *hit = th->unsrt + i;
 
+
+          /* Given the sequence header:
+           * >1 000101001 12343829483298 1234
+           * hmmpgmd hijacks the desc and acc fields to pass back domain
+           * architecture and taxonomy id information to the hmmer webserver.
+           * This means the fields are non-NULL, but don't point to real
+           * addresses, so will break any attempt to print non-NULL desc
+           * or acc fields. So we simply clear them out here
+           */
+          hit->desc = NULL;
+          hit->acc  = NULL;
+
+
           hit->dcl = (P7_DOMAIN *)(data + ((char *)hit->dcl - (char *)NULL));
 
           /* the hit string pointers contain the length of the string including
@@ -538,6 +556,7 @@ int main(int argc, char *argv[])
             if (ad->model   != NULL) ad->model   = base + (ad->model   - (char *)NULL);
             if (ad->mline   != NULL) ad->mline   = base + (ad->mline   - (char *)NULL);
             if (ad->aseq    != NULL) ad->aseq    = base + (ad->aseq    - (char *)NULL);
+            if (ad->ntseq   != NULL) ad->ntseq   = base + (ad->ntseq   - (char *)NULL);
             if (ad->ppline  != NULL) ad->ppline  = base + (ad->ppline  - (char *)NULL);
             if (ad->hmmname != NULL) ad->hmmname = base + (ad->hmmname - (char *)NULL);
             if (ad->hmmacc  != NULL) ad->hmmacc  = base + (ad->hmmacc  - (char *)NULL);
@@ -548,13 +567,14 @@ int main(int argc, char *argv[])
 
             ptr += sizeof(P7_ALIDISPLAY) + ad->memsize;
             ++dcl;
+			
           }
         }
 
         /* adjust the reported and included hits */
         //th->is_sorted = FALSE;
         //p7_tophits_Sort(th);
-
+		
         /* Print the results.  */
         if (scores) p7_tophits_Targets(stdout, th, pli, 120); fprintf(stdout, "\n\n");
         if (ali)    p7_tophits_Domains(stdout, th, pli, 120); fprintf(stdout, "\n\n");
@@ -576,12 +596,11 @@ int main(int argc, char *argv[])
       if (abc) esl_alphabet_Destroy(abc);
       if (hmm) p7_hmm_Destroy(hmm);
       if (sq)  esl_sq_Destroy(sq);
-    }
+
+	  }
   }
 
   free(seq);
-  free(opts);
-
   esl_getopts_Destroy(go);
   esl_stopwatch_Destroy(w);
 
@@ -593,15 +612,15 @@ int main(int argc, char *argv[])
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  *
- * SVN $Id: hmmc2.c 4314 2012-11-26 14:53:12Z nawrockie $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmc2.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
 
