@@ -1,8 +1,8 @@
 /* master side of the hmmpgmd daemon
  * 
  * MSF, Thu Aug 12, 2010 [Janelia]
- * SVN $Id: hmmdmstr.c 4326 2012-12-05 19:02:25Z nawrockie $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmdmstr.c $
+ * SVN $Id$
+ * SVN $URL$
  */
 #include "p7_config.h"
 
@@ -18,8 +18,8 @@
 #include <setjmp.h>
 #include <sys/socket.h>
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>	    /* On FreeBSD, you need netinet/in.h for struct sockaddr_in            */
-#endif			    /* On OpenBSD, netinet/in.h is required for (must precede) arpa/inet.h */
+#include <netinet/in.h>     /* On FreeBSD, you need netinet/in.h for struct sockaddr_in            */
+#endif                      /* On OpenBSD, netinet/in.h is required for (must precede) arpa/inet.h */
 #include <arpa/inet.h>
 #include <syslog.h>
 #include <assert.h>
@@ -94,6 +94,8 @@ typedef struct {
   int              idle_cnt;
   struct worker_s *idling;
 
+  RANGE_LIST       *range_list;  /* (optional) list of ranges searched within the seqdb */
+
   int              completed;
 } WORKERSIDE_ARGS;
 
@@ -121,6 +123,7 @@ typedef struct worker_s {
   struct worker_s      *prev;
 } WORKER_DATA;
 
+
 static void setup_clientside_comm(ESL_GETOPTS *opts, CLIENTSIDE_ARGS  *args);
 static void setup_workerside_comm(ESL_GETOPTS *opts, WORKERSIDE_ARGS  *args);
 
@@ -298,8 +301,10 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
   int n;
   int cnt;
   int inx;
-  int rem;
+  int ready_workers;    /* counter variable used to track the number of workers currently available to receive work; short for "remaining", I imagine */
   int tries;
+  int i;
+
 
   memset(&results, 0, sizeof(SEARCH_RESULTS)); /* avoid valgrind bitching about uninit bytes; remove, if we ever serialize structs properly */
 
@@ -307,7 +312,6 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
   esl_stopwatch_Start(w);
 
   /* figure out the size of the database we are searching */
-  inx = 0;
   if (query->cmd_type == HMMD_CMD_SEARCH) {
     cnt = args->seq_db->db[query->dbx].count;
   } else {
@@ -315,8 +319,20 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
   }
 
   init_results(&results);
-  tries = 0;
 
+  //if range(s) are given, count how many of the seqdb's sequences are within supplied range(s)
+  if (args->range_list) { // can only happen in HMMD_CMD_SEARCH case
+    int range_cnt = 0; // this will now count how many of the seqs in the db are within the range
+    for (i=0; i<cnt; i++) {
+      if ( hmmpgmd_IsWithinRanges(args->seq_db->list[i].idx, args->range_list ) )
+        range_cnt++;
+    }
+    cnt = range_cnt;
+  }
+
+
+  inx = 0;
+  tries = 0;
   do {
     /* process any changes to the available workers */
     if ((n = pthread_mutex_lock (&args->work_mutex)) != 0) LOG_FATAL_MSG("mutex lock", n);
@@ -326,10 +342,11 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
 
     /* if there are no workers, report an error */
     if (args->ready > 0) {
-      rem = args->ready;
+      ready_workers = args->ready;
 
       /* update the workers search information */
       worker = args->head;
+
       while (worker != NULL) {
         worker->cmd        = query->cmd;
         worker->completed  = 0;
@@ -337,12 +354,26 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
 
         /* assign each worker a portion of the database */
         worker->srch_inx = inx;
-        worker->srch_cnt = cnt / rem;
-
-        inx += worker->srch_cnt;
-        cnt -= worker->srch_cnt;
-        --rem;
+        if (args->range_list) {
+          // if ranges are given, need to split the db list based on which elements in the list are within the given range(s)
+          int goal = cnt / ready_workers; //how many within-range sequences do I want to ask this worker to handle
+          int curr = 0;                   //how many within-range sequences have I seen since the start of this full-db range
+          worker->srch_cnt = 0;
+          while (curr < goal) {
+            if ( hmmpgmd_IsWithinRanges (args->seq_db->list[inx].idx, args->range_list ) )
+                curr++;
+            worker->srch_cnt++;
+            inx++;
+          }
+          cnt -= curr;
+        } else {
+          // default - split evenly among workers
+          worker->srch_cnt = cnt / ready_workers;
+          inx += worker->srch_cnt;
+          cnt -= worker->srch_cnt;
+        }
 
+        --ready_workers;
         worker            = worker->next;
       }
 
@@ -351,7 +382,7 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
       /* notify all the worker threads of the new query */
       if ((n = pthread_cond_broadcast(&args->start_cond)) != 0) LOG_FATAL_MSG("cond broadcast", n);
     }
-    
+
     if ((n = pthread_mutex_unlock (&args->work_mutex)) != 0)  LOG_FATAL_MSG("mutex unlock", n);
 
     if (args->ready > 0) {
@@ -377,6 +408,7 @@ process_search(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
 
   } while (args->ready > 0 && results.errors == 1 && tries < 2);
 
+
   esl_stopwatch_Stop(w);
 
   /* copy the search stats */
@@ -514,7 +546,7 @@ process_load(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
 
     if ( (status = p7_hmmcache_SetNumericNames(hmm_db)) != eslOK) goto ERROR;
 
-    client_msg(query->sock, eslOK, "Loaded profile db %s;  models: %d  memory: %" PRId64 "\n", 
+    client_msg(query->sock, eslOK, "Loaded profile db %s;  models: %d  memory: %" PRId64 "\n",
 	       name, hmm_db->n, p7_hmmcache_Sizeof(hmm_db));
   }
 
@@ -665,6 +697,7 @@ process_shutdown(WORKERSIDE_ARGS *args, QUEUE_DATA *query)
   if ((n = pthread_mutex_unlock (&args->work_mutex)) != 0) LOG_FATAL_MSG("mutex unlock", n);
 }
 
+
 void
 master_process(ESL_GETOPTS *go)
 {
@@ -686,6 +719,7 @@ master_process(ESL_GETOPTS *go)
     char *name = esl_opt_GetString(go, "--seqdb");
     if ((status = p7_seqcache_Open(name, &seq_db, errbuf)) != eslOK) 
       p7_Fail("Failed to cache %s (%d)", name, status);
+
   }
 
   if (esl_opt_IsUsed(go, "--hmmdb")) {
@@ -703,6 +737,14 @@ master_process(ESL_GETOPTS *go)
 	   name, hmm_db->n, (uint64_t) p7_hmmcache_Sizeof(hmm_db));
   }
 
+  /* if stdout is redirected at the commandline, it causes printf's to be buffered,
+   * which means status logging isn't printed. This line strongly requests unbuffering,
+   * which should be ok, given the low stdout load of hmmpgmd
+   */
+  setvbuf (stdout, NULL, _IONBF, BUFSIZ);
+  printf("Data loaded into memory. Master is ready.\n");
+  setvbuf (stdout, NULL, _IOFBF, BUFSIZ);
+
   /* initialize the search stack, set it up for interthread communication  */
   cmdstack = esl_stack_PCreate();
   esl_stack_UseMutex(cmdstack);
@@ -741,6 +783,13 @@ master_process(ESL_GETOPTS *go)
     printf("Processing command %d from %s\n", query->cmd_type, query->ip_addr);
     fflush(stdout);
 
+    worker_comm.range_list = NULL;
+    if (esl_opt_IsUsed(query->opts, "--seqdb_ranges")) {
+      ESL_ALLOC(worker_comm.range_list, sizeof(RANGE_LIST));
+      hmmpgmd_GetRanges(worker_comm.range_list, esl_opt_GetString(query->opts, "--seqdb_ranges"));
+    }
+
+
     switch(query->cmd_type) {
     case HMMD_CMD_SEARCH:      process_search(&worker_comm, query); break;
     case HMMD_CMD_SCAN:        process_search(&worker_comm, query); break;
@@ -770,7 +819,19 @@ master_process(ESL_GETOPTS *go)
   pthread_cond_destroy(&worker_comm.start_cond);
   pthread_cond_destroy(&worker_comm.complete_cond);
 
+
+  if (worker_comm.range_list) {
+    if (worker_comm.range_list->starts)  free(worker_comm.range_list->starts);
+    if (worker_comm.range_list->ends)    free(worker_comm.range_list->ends);
+    free (worker_comm.range_list);
+  }
+
   return;
+
+
+ERROR:
+  p7_Fail("Memory allocation error. Code: %d\n",    status);
+
 }
 
 static int
@@ -1250,7 +1311,7 @@ clientside_loop(CLIENTSIDE_ARGS *data)
 
   char              *ptr;
   char              *buffer;
-  char              *opt_str;
+  char               opt_str[MAX_BUFFER];
 
   int                dbx;
   int                buf_size;
@@ -1273,10 +1334,10 @@ clientside_loop(CLIENTSIDE_ARGS *data)
   time_t             date;
   char               timestamp[32];
 
+  ESL_ALPHABET      *abcDNA = NULL;       /* DNA sequence alphabet         */
+
   buf_size = MAX_BUFFER;
   if ((buffer  = malloc(buf_size))   == NULL) LOG_FATAL_MSG("malloc", errno);
-  if ((opt_str = malloc(MAX_BUFFER)) == NULL) LOG_FATAL_MSG("malloc", errno);
-
   ptr = buffer;
   remaining = buf_size;
   amount = 0;
@@ -1329,7 +1390,6 @@ clientside_loop(CLIENTSIDE_ARGS *data)
   if (*ptr == '!') {
     process_ServerCmd(ptr, data);
     free(buffer);
-    free(opt_str);
     return 0;
   } else if (*ptr == '@') {
     char *s = ++ptr;
@@ -1341,24 +1401,19 @@ clientside_loop(CLIENTSIDE_ARGS *data)
     /* create a commandline string with dummy program name for
      * the esl_opt_ProcessSpoof() function to parse.
      */
-    strncpy(opt_str, "hmmpgmd ", MAX_BUFFER);
-    strncat(opt_str, s, MAX_BUFFER);
-    strncat(opt_str, "\n", MAX_BUFFER);
-    opt_str[MAX_BUFFER-1] = 0;
+    snprintf(opt_str, sizeof(opt_str), "hmmpgmd %s\n", s);
 
     /* skip remaining white spaces */
     while (*ptr && isspace(*ptr)) ++ptr;
   } else {
     client_msg(data->sock_fd, eslEFORMAT, "Missing options string");
     free(buffer);
-    free(opt_str);
     return 0;
   }
 
   if (strncmp(ptr, "//", 2) == 0) {
     client_msg(data->sock_fd, eslEFORMAT, "Missing search sequence/hmm");
     free(buffer);
-    free(opt_str);
     return 0;
   }
 
@@ -1391,6 +1446,7 @@ clientside_loop(CLIENTSIDE_ARGS *data)
     if (*ptr == '>') {
       /* try to parse the input buffer as a FASTA sequence */
       seq = esl_sq_CreateDigital(abc);
+      /* try to parse the input buffer as a FASTA sequence */
       status = esl_sqio_Parse(ptr, strlen(ptr), seq, eslSQFILE_DAEMON);
       if (status != eslOK) client_msg_longjmp(data->sock_fd, status, &jmp_env, "Error parsing FASTA sequence");
       if (seq->n < 1) client_msg_longjmp(data->sock_fd, eslEFORMAT, &jmp_env, "Error zero length FASTA sequence");
@@ -1421,7 +1477,6 @@ clientside_loop(CLIENTSIDE_ARGS *data)
     if (seq  != NULL) esl_sq_Destroy(seq);
     if (sco  != NULL) esl_scorematrix_Destroy(sco);
 
-    free(opt_str);
     free(buffer);
     return 0;
   }
@@ -1542,10 +1597,10 @@ clientside_loop(CLIENTSIDE_ARGS *data)
   esl_stack_PPush(cmdstack, parms);
 
   free(buffer);
-  free(opt_str);
   return 0;
 }
 
+
 /* discard_function()
  * function handed to esl_stack_DiscardSelected() to remove
  * all commands in the stack that are associated with a
@@ -1835,7 +1890,7 @@ workerside_thread(void *arg)
   WORKERSIDE_ARGS  *parent  = (WORKERSIDE_ARGS *)worker->parent;
   HMMD_HEADER       hdr;
   int               n;
-  int               fd;
+  int               fd = 0;
   int               version;
   int               updated;
   int               status = eslOK;
@@ -1860,7 +1915,8 @@ workerside_thread(void *arg)
     if (parent->seq_db != NULL) n += strlen(parent->seq_db->name) + 1;
     if (parent->hmm_db != NULL) n += strlen(parent->hmm_db->name) + 1;
 
-    if ((cmd = malloc(n)) == NULL) {
+    cmd = malloc(n);
+    if (cmd == NULL) {
       p7_syslog(LOG_ERR,"[%s:%d] - malloc %d - %s\n", __FILE__, __LINE__, errno, strerror(errno));
       goto EXIT;
     }
@@ -2077,11 +2133,14 @@ setup_workerside_comm(ESL_GETOPTS *opts, WORKERSIDE_ARGS *args)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
+ *
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
