@@ -14,8 +14,8 @@
 #include <setjmp.h>
 #include <sys/socket.h>
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>	    /* On FreeBSD, you need netinet/in.h for struct sockaddr_in            */
-#endif			    /* On OpenBSD, netinet/in.h is required for (must precede) arpa/inet.h */
+#include <netinet/in.h>     /* On FreeBSD, you need netinet/in.h for struct sockaddr_in            */
+#endif                      /* On OpenBSD, netinet/in.h is required for (must precede) arpa/inet.h */
 #include <arpa/inet.h>
 #include <syslog.h>
 #include <time.h>
@@ -60,6 +60,8 @@ typedef struct {
   ESL_ALPHABET     *abc;         /* digital alphabet                 */
   ESL_GETOPTS      *opts;        /* search specific options          */
 
+  RANGE_LIST       *range_list;  /* (optional) list of ranges searched within the seqdb */
+
   double            elapsed;     /* elapsed search time              */
 
   /* Structure created and populated by the individual threads.
@@ -78,14 +80,14 @@ typedef struct {
 } WORKER_ENV;
 
 static void process_InitCmd(HMMD_COMMAND *cmd, WORKER_ENV *env);
-static void process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env);
+static void process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env, QUEUE_DATA *query);
 static void process_Shutdown(HMMD_COMMAND *cmd, WORKER_ENV *env);
 
 static QUEUE_DATA *process_QueryCmd(HMMD_COMMAND *cmd, WORKER_ENV *env);
 
 static int  setup_masterside_comm(ESL_GETOPTS *opts);
 
-static void send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info);
+static void send_results(int fd, ESL_STOPWATCH *w, P7_TOPHITS *th, P7_PIPELINE *pli);
 
 #define BLOCK_SIZE 1000
 static void search_thread(void *arg);
@@ -144,7 +146,9 @@ worker_process(ESL_GETOPTS *go)
   int           shutdown = 0;
   WORKER_ENV    env;
   int           status;
-    
+   
+  QUEUE_DATA      *query      = NULL;   
+  
   /* Initializations */
   impl_Init();
   p7_FLogsumInit();      /* we're going to use table-driven Logsum() approximations at times */
@@ -160,10 +164,20 @@ worker_process(ESL_GETOPTS *go)
     {
       if ((status = read_Command(&cmd, &env)) != eslOK) break;
 
+
       switch (cmd->hdr.command) {
       case HMMD_CMD_INIT:      process_InitCmd  (cmd, &env);                break;
-      case HMMD_CMD_SCAN:      process_SearchCmd(cmd, &env);                break;
-      case HMMD_CMD_SEARCH:    process_SearchCmd(cmd, &env);                break;
+      case HMMD_CMD_SCAN: 
+	  {	  
+ 		   query = process_QueryCmd(cmd, &env);
+ 		   process_SearchCmd(cmd, &env, query);
+ 		   free_QueueData(query);
+	  }
+		 break;
+      case HMMD_CMD_SEARCH:
+		   query = process_QueryCmd(cmd, &env);
+	     process_SearchCmd(cmd, &env, query);
+         break;
       case HMMD_CMD_SHUTDOWN:  process_Shutdown (cmd, &env);  shutdown = 1; break;
       default: p7_syslog(LOG_ERR,"[%s:%d] - unknown command %d (%d)\n", __FILE__, __LINE__, cmd->hdr.command, cmd->hdr.length);
       }
@@ -178,8 +192,9 @@ worker_process(ESL_GETOPTS *go)
   return;
 }
 
+
 static void 
-process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
+process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env, QUEUE_DATA *query)
 { 
   int              i;
   int              cnt;
@@ -192,7 +207,6 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
   ESL_THREADS     *threadObj  = NULL;
   pthread_mutex_t  inx_mutex;
   int              current_index;
-  QUEUE_DATA      *query      = NULL;
   time_t           date;
   char             timestamp[32];
 
@@ -208,9 +222,15 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
   printf("\n%s", timestamp);	/* note that ctime_r() leaves \n on end of timestamp  */
 
   /* initialize thread data */
-  query = process_QueryCmd(cmd, env);
   esl_stopwatch_Start(w);
 
+  info->range_list = NULL;
+  if (esl_opt_IsUsed(query->opts, "--seqdb_ranges")) {
+    ESL_ALLOC(info->range_list, sizeof(RANGE_LIST));
+    hmmpgmd_GetRanges(info->range_list, esl_opt_GetString(query->opts, "--seqdb_ranges"));
+  }
+
+
   if (query->cmd_type == HMMD_CMD_SEARCH) threadObj = esl_threads_Create(&search_thread);
   else                                    threadObj = esl_threads_Create(&scan_thread);
 
@@ -219,10 +239,15 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
   } else {
     fprintf(stdout, "Search hmm %s  [M=%d]", query->hmm->name, query->hmm->M);
   }
-  fprintf(stdout, " vs %s DB %d [%d - %d]\n", 
+  fprintf(stdout, " vs %s DB %d [%d - %d]",
           (query->cmd_type == HMMD_CMD_SEARCH) ? "SEQ" : "HMM", 
           query->dbx, query->inx, query->inx + query->cnt - 1);
 
+  if (info->range_list)
+    fprintf(stdout, " in range(s) %s", esl_opt_GetString(query->opts, "--seqdb_ranges"));
+
+  fprintf(stdout, "\n");
+
   /* Create processing pipeline and hit list */
   for (i = 0; i < env->ncpus; ++i) {
     info[i].abc   = query->abc;
@@ -230,6 +255,8 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
     info[i].seq   = query->seq;
     info[i].opts  = query->opts;
 
+    info[i].range_list  = info[0].range_list;
+
     info[i].th    = NULL;
     info[i].pli   = NULL;
 
@@ -293,17 +320,22 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
   }
 
   print_timings(99, w->elapsed, info[0].pli);
-  send_results(env->fd, w, info);
+  send_results(env->fd, w, info[0].th, info[0].pli);
 
   /* free the last of the pipeline data */
   p7_pipeline_Destroy(info->pli);
   p7_tophits_Destroy(info->th);
 
-  free_QueueData(query);
-
   esl_threads_Destroy(threadObj);
 
   pthread_mutex_destroy(&inx_mutex);
+
+  if (info->range_list) {
+    if (info->range_list->starts)  free(info->range_list->starts);
+    if (info->range_list->ends)    free(info->range_list->ends);
+    free (info->range_list);
+  }
+
   free(info);
 
   esl_stopwatch_Destroy(w);
@@ -315,7 +347,6 @@ process_SearchCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
   LOG_FATAL_MSG("malloc", errno);
 }
 
-
 static QUEUE_DATA *
 process_QueryCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
 {
@@ -351,6 +382,7 @@ process_QueryCmd(HMMD_COMMAND *cmd, WORKER_ENV *env)
 
   query->hmm = NULL;
   query->seq = NULL;
+
   query->abc = esl_alphabet_Create(eslAMINO);
 
   /* check if we are processing a sequence or hmm */
@@ -484,8 +516,21 @@ process_InitCmd(HMMD_COMMAND *cmd, WORKER_ENV  *env)
     }
 
     env->hmm_db = hcache;
+
+    printf("Loaded profile db %s;  models: %d  memory: %" PRId64 "\n",
+         p, hcache->n, (uint64_t) p7_hmmcache_Sizeof(hcache));
+
   }
 
+  /* if stdout is redirected at the commandline, it causes printf's to be buffered,
+   * which means status logging isn't printed. This line strongly requests unbuffering,
+   * which should be ok, given the low stdout load of hmmpgmd
+   */
+  setvbuf (stdout, NULL, _IONBF, BUFSIZ);
+  printf("Data loaded into memory. Worker is ready.\n");
+  setvbuf (stdout, NULL, _IOFBF, BUFSIZ);
+
+
   /* write back to the master that we are on line */
   n = MSG_SIZE(cmd);
   cmd->hdr.status = eslOK;
@@ -595,20 +640,20 @@ search_thread(void *arg)
 
     /* Main loop: */
     for (i = 0; i < count; ++i, ++sq) {
+      if ( !(info->range_list) || hmmpgmd_IsWithinRanges ((*sq)->idx, info->range_list)) {
+        dbsq.name  = (*sq)->name;
+        dbsq.dsq   = (*sq)->dsq;
+        dbsq.n     = (*sq)->n;
+        dbsq.idx   = (*sq)->idx;
+        if((*sq)->desc != NULL) dbsq.desc  = (*sq)->desc;
 
-      dbsq.name  = (*sq)->name;
-      dbsq.dsq   = (*sq)->dsq;
-      dbsq.n     = (*sq)->n;
-      dbsq.idx   = (*sq)->idx;
-      if((*sq)->desc != NULL) dbsq.desc  = (*sq)->desc;
+        p7_bg_SetLength(bg, dbsq.n);
+        p7_oprofile_ReconfigLength(om, dbsq.n);
 
-      //p7_pli_NewSeq(pli, &dbsq);
-      p7_bg_SetLength(bg, dbsq.n);
-      p7_oprofile_ReconfigLength(om, dbsq.n);
+        p7_Pipeline(pli, om, bg, &dbsq, NULL, th);
 
-      p7_Pipeline(pli, om, bg, &dbsq, th);
-
-      p7_pipeline_Reuse(pli);
+        p7_pipeline_Reuse(pli);
+      }
     }
   }
 
@@ -664,7 +709,6 @@ scan_thread(void *arg)
   pli = p7_pipeline_Create(info->opts, 100, 100, FALSE, p7_SCAN_MODELS);
 
   p7_pli_NewSeq(pli, info->seq);
-  p7_bg_SetLength(bg, info->seq->n);
 
   /* loop until all sequences have been processed */
   count = 1;
@@ -696,9 +740,10 @@ scan_thread(void *arg)
     /* Main loop: */
     for (i = 0; i < count; ++i, ++om) {
       p7_pli_NewModel(pli, *om, bg);
+      p7_bg_SetLength(bg, info->seq->n);
       p7_oprofile_ReconfigLength(*om, info->seq->n);
 	      
-      p7_Pipeline(pli, *om, bg, info->seq, th);
+      p7_Pipeline(pli, *om, bg, info->seq, NULL, th);
       p7_pipeline_Reuse(pli);
     }
   }
@@ -721,8 +766,9 @@ scan_thread(void *arg)
   return;
 }
 
+
 static void
-send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
+send_results(int fd, ESL_STOPWATCH *w, P7_TOPHITS *th, P7_PIPELINE *pli)
 {
   HMMD_SEARCH_STATS   stats;
   HMMD_SEARCH_STATUS  status;
@@ -730,6 +776,8 @@ send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
   P7_DOMAIN          *dcl;
   int                 i, j, n;
   esl_pos_t           offset;
+  char               *pEnd; /* pointer used by strtol to locate the taxonomy id on the description line. */
+
 
   memset(&status, 0, sizeof(HMMD_SEARCH_STATUS)); /* silence valgrind errors - zero out entire structure including its padding */
   status.status     = eslOK;
@@ -740,21 +788,21 @@ send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
   stats.user        = w->user;
   stats.sys         = w->sys;
 
-  stats.nmodels     = info->pli->nmodels;
-  stats.nseqs       = info->pli->nseqs;
-  stats.n_past_msv  = info->pli->n_past_msv;
-  stats.n_past_bias = info->pli->n_past_bias;
-  stats.n_past_vit  = info->pli->n_past_vit;
-  stats.n_past_fwd  = info->pli->n_past_fwd;
+  stats.nmodels     = pli->nmodels;
+  stats.nseqs       = pli->nseqs;
+  stats.n_past_msv  = pli->n_past_msv;
+  stats.n_past_bias = pli->n_past_bias;
+  stats.n_past_vit  = pli->n_past_vit;
+  stats.n_past_fwd  = pli->n_past_fwd;
 
-  stats.Z           = info->pli->Z;
-  stats.domZ        = info->pli->domZ;
-  stats.Z_setby     = info->pli->Z_setby;
-  stats.domZ_setby  = info->pli->domZ_setby;
+  stats.Z           = pli->Z;
+  stats.domZ        = pli->domZ;
+  stats.Z_setby     = pli->Z_setby;
+  stats.domZ_setby  = pli->domZ_setby;
 
-  stats.nhits       = info->th->N;
-  stats.nreported   = info->th->nreported;
-  stats.nincluded   = info->th->nincluded;
+  stats.nhits       = th->N;
+  stats.nreported   = th->nreported;
+  stats.nincluded   = th->nincluded;
 
   n = sizeof(P7_HIT) * stats.nhits;
 
@@ -765,23 +813,37 @@ send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
   /* get the data in the right format before we send it */
   for (i = 0; i < stats.nhits; ++i) {
     P7_HIT *h1 = &hit[i];
-    P7_HIT *h2 = &info->th->unsrt[i];
+    P7_HIT *h2 = &th->unsrt[i];
 
     memcpy(h1, h2, sizeof(P7_HIT));
 
     /* the name will be an integer value of the sequence index */
     h1->name = (char *) strtol(h2->name, NULL, 10);
-    h1->acc  = NULL;
+//    h1->acc  = NULL;
 
     /* We want to override the sequence description as a container
      * for the domain architecture.
      * We have already assigned the architecture when generating
      * the sequence database, and comparing it to the latest version
      * of Pfam. Copying the example above, convert the string to
-     * a long and cast back to the a char. Nasty hack, but atleast
+     * a long and cast back to a char. Nasty hack, but at least
      * then it gets written back out to the socket. rdf
      */
-    if(h2->desc != NULL) h1->desc = (char *) strtol(h2->desc, NULL, 10);
+    //if(h2->desc != NULL) h1->desc = (char *) strtol(h2->desc, NULL, 10);
+    if (h2->desc != NULL) {
+      h1->desc = (char *) strtol(h2->desc, &pEnd, 10);
+      /* Given the sequence header:
+       * >1 000101001 12343829483298 1234
+       * This will grab the last value (1234) if present and place it in
+       * the acc entry as it is not being used by hmmpgmd. This is the same
+       * way that we hijacked the desc field to pass the architecture
+       * information back. Currently used to pass the taxonomy id to the
+       * hmmer website.
+       */
+       h1->acc  = (char *) strtol(pEnd, &pEnd, 10);
+    }
+
+
     h1->offset = offset;
 
     /* figure out how big the domains are and their offset */
@@ -810,7 +872,7 @@ send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
   /* loop through the hit list sending the domains */
   for (i = 0; i < stats.nhits; ++i) {
     char *base;
-    P7_HIT *h2 = &info->th->unsrt[i];
+    P7_HIT *h2 = &th->unsrt[i];
 
     dcl = h2->dcl;
 
@@ -834,6 +896,7 @@ send_results(int fd, ESL_STOPWATCH *w, WORKER_INFO *info)
       if (ad->model   != NULL) ad->model   = base + (ad->model   - ad->mem);
       if (ad->mline   != NULL) ad->mline   = base + (ad->mline   - ad->mem);
       if (ad->aseq    != NULL) ad->aseq    = base + (ad->aseq    - ad->mem);
+      if (ad->ntseq   != NULL) ad->ntseq   = base + (ad->ntseq   - ad->mem);
       if (ad->ppline  != NULL) ad->ppline  = base + (ad->ppline  - ad->mem);
       if (ad->hmmname != NULL) ad->hmmname = base + (ad->hmmname - ad->mem);
       if (ad->hmmacc  != NULL) ad->hmmacc  = base + (ad->hmmacc  - ad->mem);
@@ -903,11 +966,14 @@ setup_masterside_comm(ESL_GETOPTS *opts)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
+ *
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
 
