@@ -19,7 +19,7 @@
 
 #include "hmmer.h"
 
-#define EMITOPTS "-a,-c,-C,-p,--simplerepeats"
+#define EMITOPTS "-a,-c,-C,-p"
 #define MODEOPTS "--local,--unilocal,--glocal,--uniglocal"
 
 static ESL_OPTIONS options[] = {
@@ -32,7 +32,6 @@ static ESL_OPTIONS options[] = {
   { "-c",          eslARG_NONE,   FALSE, NULL, NULL,      NULL,      NULL, EMITOPTS, "emit simple majority-rule consensus sequence",           2 },
   { "-C",          eslARG_NONE,   FALSE, NULL, NULL,      NULL,      NULL, EMITOPTS, "emit fancier consensus sequence (req's --minl, --minu)", 2 },
   { "-p",          eslARG_NONE,   FALSE, NULL, NULL,      NULL,      NULL, EMITOPTS, "sample sequences from profile, not core model",          2 },
-  { "--simplerepeats",eslARG_NONE,FALSE, NULL, NULL,      NULL,      NULL, EMITOPTS, "find locations with simple repeat signal",               2 },
 /* options controlling emission from profiles with -p  */
   { "-L",          eslARG_INT,    "400", NULL, NULL,      NULL,      "-p",    NULL, "set expected length from profile to <n>",               3 },
   { "--local",     eslARG_NONE,"default",NULL, NULL,    MODEOPTS,    "-p",    NULL, "configure profile in multihit local mode",              3 }, 
@@ -57,7 +56,6 @@ static void emit_consensus(ESL_GETOPTS *go, FILE *ofp, int outfmt,
 static void emit_fancycons(ESL_GETOPTS *go, FILE *ofp, int outfmt,                    P7_HMM *hmm);
 static void emit_alignment(ESL_GETOPTS *go, FILE *ofp, int outfmt, ESL_RANDOMNESS *r, P7_HMM *hmm);
 static void emit_sequences(ESL_GETOPTS *go, FILE *ofp, int outfmt, ESL_RANDOMNESS *r, P7_HMM *hmm);
-static void find_simplerepeats(ESL_GETOPTS *go, FILE *ofp, P7_HMM *hmm);
 
 
 int
@@ -99,7 +97,7 @@ main(int argc, char **argv)
 
   while ((status = p7_hmmfile_Read(hfp, &abc, &hmm)) != eslEOF)
     {
-      if      (status == eslEFORMAT)    esl_fatal("Bad file format in HMM file %s:\n:%s\n",         hfp->fname, hfp->errbuf);
+      if      (status == eslEFORMAT)    esl_fatal("Bad file format in HMM file %s:\n%s\n",          hfp->fname, hfp->errbuf);
       else if (status == eslEINCOMPAT)  esl_fatal("HMM in %s is not in the expected %s alphabet\n", hfp->fname, esl_abc_DecodeType(abc->type));
       else if (status != eslOK)         esl_fatal("Unexpected error in reading HMMs from %s\n",     hfp->fname);
       nhmms++;
@@ -107,7 +105,6 @@ main(int argc, char **argv)
       if      (esl_opt_GetBoolean(go, "-c"))  emit_consensus(go, ofp, outfmt,    hmm);
       else if (esl_opt_GetBoolean(go, "-C"))  emit_fancycons(go, ofp, outfmt,    hmm);
       else if (esl_opt_GetBoolean(go, "-a"))  emit_alignment(go, ofp, outfmt, r, hmm);
-      else if (esl_opt_IsOn(go, "--simplerepeats")) find_simplerepeats(go, ofp, hmm);
       else                                    emit_sequences(go, ofp, outfmt, r, hmm);
 
 
@@ -213,7 +210,7 @@ emit_alignment(ESL_GETOPTS *go, FILE *ofp, int outfmt, ESL_RANDOMNESS *r, P7_HMM
     }
 
   p7_tracealign_Seqs(sq, tr, N, hmm->M, optflags, hmm, &msa);
-  eslx_msafile_Write(ofp, msa, outfmt);
+  esl_msafile_Write(ofp, msa, outfmt);
   
   for (i = 0; i < N; i++) p7_trace_Destroy(tr[i]);  free(tr);
   for (i = 0; i < N; i++) esl_sq_Destroy(sq[i]);    free(sq);
@@ -231,7 +228,7 @@ emit_sequences(ESL_GETOPTS *go, FILE *ofp, int outfmt, ESL_RANDOMNESS *r, P7_HMM
   int         do_profile   = esl_opt_GetBoolean(go, "-p");
   int         N            = esl_opt_GetInteger(go, "-N");
   int         L            = esl_opt_GetInteger(go, "-L");
-  int         mode;
+  int         mode         = p7_LOCAL;
   int         nseq;
   int         status;
 
@@ -275,74 +272,12 @@ emit_sequences(ESL_GETOPTS *go, FILE *ofp, int outfmt, ESL_RANDOMNESS *r, P7_HMM
 
 
 
-/* repeat_sorter(): qsort's pawn, below */
-static int
-repeat_sorter(const void *vw1, const void *vw2)
-{
-  P7_HMM_WINDOW *w1 = (P7_HMM_WINDOW*) vw1;
-  P7_HMM_WINDOW *w2 = (P7_HMM_WINDOW*) vw2;
-
-  if      (w1->n > w2->n) return  1;
-  else if (w1->n < w2->n) return -1;
-  else                    return  0;
-}
-
-static void
-find_simplerepeats(ESL_GETOPTS *go, FILE *ofp, P7_HMM *hmm)
-{
-  int i, j;
-  int status;
-  int max_repeat_len = 5;
-  int min_repeat_cnt = 4;
-  int min_rep_len = 25;
-  int min_tot_len = 50;
-  float relent_thresh = 1.5;
-  P7_HMM_WINDOWLIST *windowlist;
-  ESL_ALLOC(windowlist, sizeof(P7_HMM_WINDOWLIST));
-  p7_hmmwindow_init(windowlist);
-
-  p7_hmm_GetSimpleRepeats(hmm, max_repeat_len, min_repeat_cnt, min_rep_len, relent_thresh, windowlist);
-
-  if (windowlist->count > 1)  qsort(windowlist->windows, windowlist->count, sizeof(P7_HMM_WINDOW), repeat_sorter);
-
-  //merge neighboring windows
-  j = 0;
-  for (i=1; i<windowlist->count; i++) {
-    if (windowlist->windows[i].n < windowlist->windows[j].n + windowlist->windows[j].length + max_repeat_len ) {
-      windowlist->windows[j].length = (windowlist->windows[i].n+windowlist->windows[i].length)-windowlist->windows[j].n;
-    } else {
-      j++;
-      windowlist->windows[j].n      = windowlist->windows[i].n;
-      windowlist->windows[j].length = windowlist->windows[i].length;
-    }
-  }
-  windowlist->count = j+1;
-
-  //print repeat ranges, if sufficiently long
-  fprintf(ofp, "Position ranges for simple repeats\n");
-  for (i=0; i<windowlist->count; i++) {
-    if (windowlist->windows[i].length >= min_tot_len) {
-      fprintf(ofp, "%d .. %d\n",windowlist->windows[i].n - min_rep_len, windowlist->windows[i].n - min_rep_len + windowlist->windows[i].length - 1 );
-    }
-  }
-
-
-ERROR:
-  if (windowlist != NULL) free(windowlist);
-
-}
-
-
-
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
- * 
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmemit.c $
- * SVN $Id: hmmemit.c 3474 2011-01-17 13:25:32Z eddys $
  *****************************************************************/
