@@ -8,7 +8,6 @@
  */
 
 #include "hmmer.h"
-//#include "esl_alphabet.h"
 
 /*****************************************************************
  * 1. logo value functions
@@ -28,30 +27,29 @@ hmmlogo_maxHeight (P7_BG *bg)
 
 /* assumes rel_ents is allocated with abc->K floats, and heights with hmm->M*abc->K floats*/
 int
-hmmlogo_emissionHeightsDivRelent (P7_HMM *hmm, P7_BG *bg, float *rel_ents, float **heights ) {
+hmmlogo_RelativeEntropy_all (P7_HMM *hmm, P7_BG *bg, float *rel_ents, float **probs, float **heights ) {
 
   int    K     = hmm->abc->K;
   int    M     = hmm->M;
   int    i, j;
 
-  float p;
   float logodds;
 
   for (i = 1; i <= M; i++) {
     // height of column, to be split among the residues
     rel_ents[i] = 0;
     for (j=0; j<K; j++) {
-      p       = hmm->mat[i][j];
-      if ( p > 0 ) {
-        logodds = eslCONST_LOG2R * log(p / bg->f[j]);  //bits
-        rel_ents[i] +=  p * logodds ;
+      probs[i][j] = hmm->mat[i][j];
+
+      if ( probs[i][j] > 0 ) {
+        logodds = eslCONST_LOG2R * log(probs[i][j] / bg->f[j]);  //bits
+        rel_ents[i] +=  probs[i][j] * logodds ;
       }
     }
 
     // height of residues
     for (j=0; j<K; j++) {
-      p             = hmm->mat[i][j];
-      heights[i][j] = p * rel_ents[i];
+      heights[i][j] = rel_ents[i] * probs[i][j];
     }
 
   }
@@ -61,36 +59,40 @@ hmmlogo_emissionHeightsDivRelent (P7_HMM *hmm, P7_BG *bg, float *rel_ents, float
 
 /* assumes rel_ents is allocated with abc->K floats, and heights with hmm->M*abc->K floats*/
 int
-hmmlogo_posScoreHeightsDivRelent (P7_HMM *hmm, P7_BG *bg, float *rel_ents, float **heights ) {
+hmmlogo_RelativeEntropy_above_bg (P7_HMM *hmm, P7_BG *bg, float *rel_ents, float **probs, float **heights ) {
 
   int    K     = hmm->abc->K;
   int    M     = hmm->M;
   int    i, j;
 
-  float p;
   float logodds;
-  float pos_scoresum;
+  float abovebg_prob_sum;
 
   for (i = 1; i <= M; i++) {
 
     // height of column, to be split among the residues; also sum of of positive scores
     rel_ents[i] = 0;
-    pos_scoresum = 0.0;
+    abovebg_prob_sum = 0.0;
     for (j=0; j<K; j++) {
-      p       = hmm->mat[i][j];
-      logodds = eslCONST_LOG2R * log(p / bg->f[j]);  //bits
-      rel_ents[i] += p==0? 0 : p * logodds ;
-      if (logodds > 0)
-        pos_scoresum += logodds;
+      probs[i][j] = hmm->mat[i][j];
+
+      if ( probs[i][j] > 0 ) {
+        logodds = eslCONST_LOG2R * log(probs[i][j] / bg->f[j]);  //bits
+        rel_ents[i] += probs[i][j] * logodds ;
+
+        if (logodds > 0)
+          abovebg_prob_sum += probs[i][j];
+      }
     }
 
     //height of residues
     for (j=0; j<K; j++) {
-      p       = hmm->mat[i][j];
-      logodds = log(p / bg->f[j]);  //bits
-      heights[i][j] = logodds<=0 ? 0.0 : (rel_ents[i] * eslCONST_LOG2R * logodds / pos_scoresum) ;
+      heights[i][j] = 0.0;
+      if ( probs[i][j] > 0 ) {
+        logodds = eslCONST_LOG2R * log(probs[i][j] / bg->f[j]);  //bits
+        heights[i][j] = logodds<=0 ? 0.0 : (rel_ents[i] * probs[i][j] / abovebg_prob_sum) ;
+      }
     }
-
   }
   return eslOK;
 }
@@ -105,18 +107,15 @@ hmmlogo_ScoreHeights (P7_HMM *hmm, P7_BG *bg, float **heights ) {
 
   float p;
   float logodds;
-  float pos_scoresum;
 
   for (i = 1; i <= M; i++) {
 
     // height of column, to be split among the residues; also sum of of positive scores
-    pos_scoresum = 0.0;
     for (j=0; j<K; j++) {
       p       = hmm->mat[i][j];
       logodds = eslCONST_LOG2R * log(p / bg->f[j]);  //bits
       heights[i][j] = logodds;
     }
-
   }
   return eslOK;
 }
@@ -124,23 +123,25 @@ hmmlogo_ScoreHeights (P7_HMM *hmm, P7_BG *bg, float **heights ) {
 
 /* assumes heights is allocated with hmm->M floats*/
 int
-hmmlogo_IndelValues (P7_HMM *hmm, float *insert_P, float *insert_expL, float *delete_P ) {
+hmmlogo_IndelValues (P7_HMM *hmm, float *insert_P, float *insert_expL, float *occupancy ) {
 
   int    i;
 
   if (insert_P != NULL)    insert_P[1]    = hmm->t[1][p7H_MI];                   //probability of inserting after this match
   if (insert_expL != NULL) insert_expL[1] =  1 / (1 - hmm->t[1][p7H_II]) ;       //expected length of the insert, if it happens
-  if (delete_P != NULL)    delete_P[1]    = 0.0;  //1st match state never deleted
+//  if (delete_P != NULL)    delete_P[1]    = 0.0;  //1st match state never deleted
 
   for (i = 2; i < hmm->M; i++) {
     if (insert_P != NULL)    insert_P[i]    = hmm->t[i][p7H_MI];                         //probability of inserting after this match
     if (insert_expL != NULL) insert_expL[i] =  1 / (1 - hmm->t[i][p7H_II]) ;          //expected length of the insert, if it happens
-    if (delete_P != NULL)    delete_P[i]    = ( (1.0-delete_P[i-1]) * hmm->t[i-1][p7H_MD] ) + ( delete_P[i-1] * hmm->t[i-1][p7H_DD]) ; //probability of missing this state, either due to DD or MD from previous position
+//    if (delete_P != NULL)    delete_P[i]    = ( (1.0-delete_P[i-1]) * hmm->t[i-1][p7H_MD] ) + ( delete_P[i-1] * hmm->t[i-1][p7H_DD]) ; //probability of missing this state, either due to DD or MD from previous position
   }
 
   if (insert_P != NULL)    insert_P[hmm->M]    = 0.0;   //no inserts after final position
   if (insert_expL != NULL) insert_expL[hmm->M] = 0.0;   //no inserts after final position
-  if (delete_P != NULL)    delete_P[hmm->M]    = ( (1.0-delete_P[hmm->M-1]) * hmm->t[hmm->M-1][p7H_MD] ) + ( delete_P[hmm->M-1] * hmm->t[hmm->M-1][p7H_DD]) ; //probability of missing this state, either due to DD or MD from previous position
+//  if (delete_P != NULL)    delete_P[hmm->M]    = ( (1.0-delete_P[hmm->M-1]) * hmm->t[hmm->M-1][p7H_MD] ) + ( delete_P[hmm->M-1] * hmm->t[hmm->M-1][p7H_DD]) ; //probability of missing this state, either due to DD or MD from previous position
+
+  if (occupancy != NULL)  p7_hmm_CalculateOccupancy(hmm, occupancy, NULL);
 
   return eslOK;
 }
@@ -154,25 +155,26 @@ hmmlogo_IndelValues (P7_HMM *hmm, float *insert_P, float *insert_expL, float *de
  * 2. hmmlogo application
  *****************************************************************/
 
-#define HMMLOGO_OPTS "--height_emission,--height_positive_score,--height_bits"
-#define HMMLOGO_HEIGHT_EMISSION   1
-#define HMMLOGO_HEIGHT_POS_SCORE  2
-#define HMMLOGO_HEIGHT_BITS       3
+#define HMMLOGO_OPTS "--height_relent_all,--height_relent_abovebg,--height_score"
+#define HMMLOGO_RELENT_ALL        1
+#define HMMLOGO_RELENT_ABOVEBG    2
+#define HMMLOGO_SCORE             3
+
 
 static ESL_OPTIONS options[] = {
   /* name                           type        defaul  env  range   toggles   reqs   incomp              help                                                      docgroup*/
   { "-h",                        eslARG_NONE,   FALSE, NULL, NULL,    NULL,  NULL,  NULL,             "show brief help on version and usage",                         1 },
   /* Control of output */
-  { "--height_emission",         eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = relative entropy ; residue height = emission  (default)",     1 },
-  { "--height_positive_score",   eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = relative entropy ; residue height = % of positive score ",    1 },
-  { "--height_bits",                    eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = sums of (pos|neg) scores; residue height = score",            1 },
+  { "--height_relent_all",       eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = relative entropy ; all letters shown (default)",     1 },
+  { "--height_relent_abovebg",   eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = relative entropy ; only letters >bg shown",    1 },
+  { "--height_score",            eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  HMMLOGO_OPTS,     "total height = sums of (pos|neg) scores; residue height = score",            1 },
   { "--no_indel",                eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL,  NULL,             "don't provide indel rate values",                                            1 },
 
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
 
-static char usage[]  = "<hmmfile> [options]";
+static char usage[]  = "[options] <hmmfile>";
 static char banner[] = "given an hmm, produce data required to build an hmm logo";
 
 int
@@ -191,12 +193,13 @@ main(int argc, char **argv)
 
   float *rel_ents  = NULL;
   float **heights  = NULL;
+  float **probs    = NULL;
   float *ins_P     = NULL;
   float *ins_expL  = NULL;
-  float *del_P     = NULL;
+  float *occupancy = NULL;
 
 
-  int mode = HMMLOGO_HEIGHT_EMISSION;  //default
+  int mode = HMMLOGO_RELENT_ALL;  //default
 
 
   go = esl_getopts_Create(options);
@@ -207,21 +210,22 @@ main(int argc, char **argv)
    p7_banner (stdout, argv[0], banner);
    esl_usage (stdout, argv[0], usage);
    puts("\nOptions:");
-   esl_opt_DisplayHelp(stdout, go, 1, 2, 80);
+   esl_opt_DisplayHelp(stdout, go, 1, 2, 100);
+   exit(0);
   }
 
   if (esl_opt_ArgNumber(go) != 1)                      esl_fatal(argv[0], "Incorrect number of command line arguments.\n");
 
   hmmfile = esl_opt_GetArg(go, 1);
 
-  if (esl_opt_IsOn(go, "--height_emission"))
-    mode = HMMLOGO_HEIGHT_EMISSION;
-  else if (esl_opt_IsOn(go, "--height_positive_score"))
-    mode = HMMLOGO_HEIGHT_POS_SCORE;
-  else if (esl_opt_IsOn(go, "--height_bits"))
-    mode = HMMLOGO_HEIGHT_BITS;
-
-
+  if (esl_opt_IsOn(go, "--height_relent_all"))
+    mode = HMMLOGO_RELENT_ALL;
+  else if (esl_opt_IsOn(go, "--height_relent_abovebg"))
+    mode = HMMLOGO_RELENT_ABOVEBG;
+  else if (esl_opt_IsOn(go, "--height_score"))
+    mode = HMMLOGO_SCORE;
+  else
+    mode = HMMLOGO_RELENT_ALL;  //default
 
   /* Open the query profile HMM file */
   status = p7_hmmfile_OpenE(hmmfile, NULL, &hfp, errbuf);
@@ -236,18 +240,22 @@ main(int argc, char **argv)
 
   ESL_ALLOC(rel_ents, (hmm->M+1) * sizeof(float));
   ESL_ALLOC(heights,  (hmm->M+1) * sizeof(float*));
-  for (i = 1; i <= hmm->M; i++)
+  ESL_ALLOC(probs,    (hmm->M+1) * sizeof(float*));
+
+  for (i = 1; i <= hmm->M; i++) {
     ESL_ALLOC(heights[i], abc->K * sizeof(float));
+    ESL_ALLOC(probs[i],   abc->K * sizeof(float));
+  }
 
   /* residue heights */
-  if (mode == HMMLOGO_HEIGHT_EMISSION) {
+  if (mode == HMMLOGO_RELENT_ALL) {
     printf ("max expected height = %.2f\n", hmmlogo_maxHeight(bg) );
-    hmmlogo_emissionHeightsDivRelent(hmm, bg, rel_ents, heights);
-  } else if (mode == HMMLOGO_HEIGHT_POS_SCORE) {
+    hmmlogo_RelativeEntropy_all(hmm, bg, rel_ents, probs, heights);
+  } else if (mode == HMMLOGO_RELENT_ABOVEBG) {
     printf ("max expected height = %.2f\n", hmmlogo_maxHeight(bg) );
-    hmmlogo_posScoreHeightsDivRelent(hmm, bg, rel_ents, heights);
-  } else if (mode == HMMLOGO_HEIGHT_BITS) {
-    hmmlogo_ScoreHeights (hmm, bg, heights );
+    hmmlogo_RelativeEntropy_above_bg(hmm, bg, rel_ents, probs, heights);
+  } else if (mode == HMMLOGO_SCORE) {
+    hmmlogo_ScoreHeights(hmm, bg, heights );
   }
 
   printf ("Residue heights\n");
@@ -256,7 +264,7 @@ main(int argc, char **argv)
     for (j=0; j<abc->K; j++)
       printf("%6.3f ", heights[i][j] );
 
-    if (mode != HMMLOGO_HEIGHT_BITS)
+    if (mode != HMMLOGO_SCORE)
       printf(" (%6.3f)", rel_ents[i]);
 
     printf("\n");
@@ -276,17 +284,17 @@ main(int argc, char **argv)
 
     ESL_ALLOC(ins_P, (hmm->M+1) * sizeof(float));
     ESL_ALLOC(ins_expL, (hmm->M+1) * sizeof(float));
-    ESL_ALLOC(del_P, (hmm->M+1) * sizeof(float));
+    ESL_ALLOC(occupancy, (hmm->M+1) * sizeof(float));
 
-    hmmlogo_IndelValues(hmm, ins_P, ins_expL, del_P);
+    hmmlogo_IndelValues(hmm, ins_P, ins_expL, occupancy);
 
     printf ("Indel values\n");
     for (i = 1; i <= hmm->M; i++)
-      printf("%d: %6.3f %6.3f %6.3f\n", i, ins_P[i], ins_expL[i], del_P[i] );
+      printf("%d: %6.3f %6.3f %6.3f\n", i, ins_P[i], ins_expL[i], occupancy[i] );
 
     free(ins_P);
     free(ins_expL);
-    free(del_P);
+    free(occupancy);
 
   }
 
@@ -310,9 +318,9 @@ main(int argc, char **argv)
   if (abc != NULL) esl_alphabet_Destroy(abc);
 
 
-  if (ins_P != NULL)    free(ins_P);
-  if (ins_expL != NULL) free(ins_expL);
-  if (del_P != NULL)    free(del_P);
+  if (ins_P != NULL)     free(ins_P);
+  if (ins_expL != NULL)  free(ins_expL);
+  if (occupancy != NULL) free(occupancy);
 
 
 }
@@ -321,8 +329,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
