@@ -1,6 +1,6 @@
 /*
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmpgmd.h $
- * SVN $Id: hmmpgmd.h 3529 2011-04-27 21:20:37Z eddys $
+ * SVN $URL$
+ * SVN $Id$
  */
 #ifndef P7_HMMPGMD_INCLUDED
 #define P7_HMMPGMD_INCLUDED
@@ -100,7 +100,7 @@ size_t writen(int fd, const void *vptr, size_t n);
 size_t readn(int fd, void *vptr, size_t n);
 
 typedef struct queue_data_s {
-  uint32_t       cmd_type;    /* type of command to preform     */
+  uint32_t       cmd_type;    /* type of command to perform     */
   uint32_t       query_type;  /* type of the query              */
   P7_HMM        *hmm;         /* query HMM                      */
   ESL_SQ        *seq;         /* query sequence                 */
@@ -117,7 +117,16 @@ typedef struct queue_data_s {
 
 } QUEUE_DATA;
 
+
+typedef struct {
+  int       N;       /* number of ranges */
+  uint32_t *starts;  /* 0..N-1  start positions */
+  uint32_t *ends;    /* 0..N-1  start positions */
+} RANGE_LIST;
+
 extern void free_QueueData(QUEUE_DATA *data);
+extern int  hmmpgmd_IsWithinRanges (int64_t sq_idx, RANGE_LIST *list );
+extern int  hmmpgmd_GetRanges (RANGE_LIST *list, char *rangestr);
 
 extern int  process_searchopts(int fd, char *cmdstr, ESL_GETOPTS **ret_opts);
 
@@ -133,8 +142,8 @@ extern void master_process(ESL_GETOPTS *go);
 
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
