@@ -62,10 +62,10 @@ static ESL_OPTIONS options[] = {
   { "-h",           eslARG_NONE,   FALSE, NULL, NULL,    NULL,  NULL,  NULL,            "show brief help on version and usage",                         1 },
   /* Control of output */
   { "-o",           eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "direct output to file <f>, not stdout",                        2 },
-  { "-A",           eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save multiple alignment of all hits to file <s>",              2 },
-  { "--tblout",     eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save parseable table of per-sequence hits to file <s>",        2 },
-  { "--domtblout",  eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save parseable table of per-domain hits to file <s>",          2 },
-  { "--pfamtblout", eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save table of hits and domains to file, in Pfam format <s>",   2 },
+  { "-A",           eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save multiple alignment of all hits to file <f>",              2 },
+  { "--tblout",     eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save parseable table of per-sequence hits to file <f>",        2 },
+  { "--domtblout",  eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save parseable table of per-domain hits to file <f>",          2 },
+  { "--pfamtblout", eslARG_OUTFILE, NULL, NULL, NULL,    NULL,  NULL,  NULL,            "save table of hits and domains to file, in Pfam format <f>",   2 },
   { "--acc",        eslARG_NONE,   FALSE, NULL, NULL,    NULL,  NULL,  NULL,            "prefer accessions over names in output",                       2 },
   { "--noali",      eslARG_NONE,   FALSE, NULL, NULL,    NULL,  NULL,  NULL,            "don't output alignments, so output is smaller",                2 },
   { "--notextw",    eslARG_NONE,    NULL, NULL, NULL,    NULL,  NULL, "--textw",        "unlimit ASCII text output line width",                         2 },
@@ -90,6 +90,7 @@ static ESL_OPTIONS options[] = {
   { "--F2",         eslARG_REAL,  "1e-3", NULL, NULL,    NULL,  NULL, "--max",          "Stage 2 (Vit) threshold: promote hits w/ P <= F2",             7 },
   { "--F3",         eslARG_REAL,  "1e-5", NULL, NULL,    NULL,  NULL, "--max",          "Stage 3 (Fwd) threshold: promote hits w/ P <= F3",             7 },
   { "--nobias",     eslARG_NONE,   NULL,  NULL, NULL,    NULL,  NULL, "--max",          "turn off composition bias filter",                             7 },
+
 /* Other options */
   { "--nonull2",    eslARG_NONE,   NULL,  NULL, NULL,    NULL,  NULL,  NULL,            "turn off biased composition score corrections",               12 },
   { "-Z",           eslARG_REAL,   FALSE, NULL, "x>0",   NULL,  NULL,  NULL,            "set # of comparisons done, for E-value calculation",          12 },
@@ -104,6 +105,16 @@ static ESL_OPTIONS options[] = {
   { "--stall",      eslARG_NONE,   FALSE, NULL, NULL,    NULL,"--mpi", NULL,            "arrest after start: for debugging MPI under gdb",             12 },  
   { "--mpi",        eslARG_NONE,   FALSE, NULL, NULL,    NULL,  NULL,  MPIOPTS,         "run as an MPI parallel program",                              12 },
 #endif
+
+  /* Restrict search to subset of database - hidden because these flags are
+   *   (a) currently for internal use
+   *   (b) probably going to change
+   * Doesn't work with MPI
+   */
+  { "--restrictdb_stkey", eslARG_STRING, "0",  NULL, NULL,    NULL,  NULL,  NULL,       "Search starts at the sequence with name <s> (not with MPI)",     99 },
+  { "--restrictdb_n",eslARG_INT,        "-1",  NULL, NULL,    NULL,  NULL,  NULL,       "Search <j> target sequences (starting at --restrictdb_stkey)",   99 },
+  { "--ssifile",    eslARG_STRING,       NULL, NULL, NULL,    NULL,  NULL,  NULL,       "restrictdb_x values require ssi file. Override default to <s>",  99 },
+
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
@@ -122,14 +133,17 @@ struct cfg_s {
   int              do_mpi;            /* TRUE if we're doing MPI parallelization         */
   int              nproc;             /* how many MPI processes, total                   */
   int              my_rank;           /* who am I, in 0..nproc-1                         */
+
+  char             *firstseq_key;     /* name of the first sequence in the restricted db range */
+  int              n_targetseq;       /* number of sequences in the restricted range */
 };
 
 static int  serial_master(ESL_GETOPTS *go, struct cfg_s *cfg);
-static int  serial_loop  (WORKER_INFO *info, ESL_SQFILE *dbfp);
+static int  serial_loop  (WORKER_INFO *info, ESL_SQFILE *dbfp, int n_targetseqs);
 #ifdef HMMER_THREADS
 #define BLOCK_SIZE 1000
 
-static int  thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp);
+static int  thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp, int n_targetseqs);
 static void pipeline_thread(void *arg);
 #endif /*HMMER_THREADS*/
 
@@ -233,6 +247,10 @@ output_header(FILE *ofp, const ESL_GETOPTS *go, char *hmmfile, char *seqfile)
   if (esl_opt_IsUsed(go, "--F2")         && fprintf(ofp, "# Vit filter P threshold:       <= %g\n",             esl_opt_GetReal(go, "--F2"))           < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--F3")         && fprintf(ofp, "# Fwd filter P threshold:       <= %g\n",             esl_opt_GetReal(go, "--F3"))           < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--nobias")     && fprintf(ofp, "# biased composition HMM filter:   off\n")                                                   < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--restrictdb_stkey") && fprintf(ofp, "# Restrict db to start at seq key: %s\n",            esl_opt_GetString(go, "--restrictdb_stkey"))  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--restrictdb_n")     && fprintf(ofp, "# Restrict db to # target seqs:    %d\n",            esl_opt_GetInteger(go, "--restrictdb_n")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+  if (esl_opt_IsUsed(go, "--ssifile")          && fprintf(ofp, "# Override ssi file to:            %s\n",            esl_opt_GetString(go, "--ssifile"))       < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+
   if (esl_opt_IsUsed(go, "--nonull2")    && fprintf(ofp, "# null2 bias corrections:          off\n")                                                   < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "-Z")           && fprintf(ofp, "# sequence search space set to:    %.0f\n",           esl_opt_GetReal(go, "-Z"))             < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--domZ")       && fprintf(ofp, "# domain search space set to:      %.0f\n",           esl_opt_GetReal(go, "--domZ"))         < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
@@ -268,9 +286,29 @@ main(int argc, char **argv)
   cfg.do_mpi     = FALSE;	           /* this gets reset below, if we init MPI */
   cfg.nproc      = 0;		           /* this gets reset below, if we init MPI */
   cfg.my_rank    = 0;		           /* this gets reset below, if we init MPI */
+  cfg.firstseq_key = NULL;
+  cfg.n_targetseq  = -1;
 
   process_commandline(argc, argv, &go, &cfg.hmmfile, &cfg.dbfile);    
 
+/* is the range restricted? */
+
+#ifndef eslAUGMENT_SSI
+  if (esl_opt_IsUsed(go, "--restrictdb_stkey") || esl_opt_IsUsed(go, "--restrictdb_n")  || esl_opt_IsUsed(go, "--ssifile")  )
+    p7_Fail("Unable to use range-control options unless an SSI index file is available. See 'esl_sfetch --index'\n");
+#else
+  if (esl_opt_IsUsed(go, "--restrictdb_stkey") )
+    if ((cfg.firstseq_key = esl_opt_GetString(go, "--restrictdb_stkey")) == NULL)  p7_Fail("Failure capturing --restrictdb_stkey\n");
+
+  if (esl_opt_IsUsed(go, "--restrictdb_n") )
+    cfg.n_targetseq = esl_opt_GetInteger(go, "--restrictdb_n");
+
+  if ( cfg.n_targetseq != -1 && cfg.n_targetseq < 1 )
+    p7_Fail("--restrictdb_n must be >= 1\n");
+
+#endif
+
+
   /* Figure out who we are, and send control there: 
    * we might be an MPI master, an MPI worker, or a serial program.
    */
@@ -363,6 +401,16 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   else if (status == eslEINVAL)    p7_Fail("Can't autodetect format of a stdin or .gz seqfile");
   else if (status != eslOK)        p7_Fail("Unexpected error %d opening sequence file %s\n", status, cfg->dbfile);  
 
+
+  if (esl_opt_IsUsed(go, "--restrictdb_stkey") || esl_opt_IsUsed(go, "--restrictdb_n")) {
+    if (esl_opt_IsUsed(go, "--ssifile"))
+      esl_sqfile_OpenSSI(dbfp, esl_opt_GetString(go, "--ssifile"));
+    else
+      esl_sqfile_OpenSSI(dbfp, NULL);
+  }
+
+
+
   /* Open the query profile HMM file */
   status = p7_hmmfile_OpenE(cfg->hmmfile, NULL, &hfp, errbuf);
   if      (status == eslENOTFOUND) p7_Fail("File existence/permissions problem in trying to open HMM file %s.\n%s\n", cfg->hmmfile, errbuf);
@@ -397,7 +445,7 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
     {
       /* One-time initializations after alphabet <abc> becomes known */
       output_header(ofp, go, cfg->hmmfile, cfg->dbfile);
-      dbfp->abc = abc; //ReadBlock requires knowledge of the alphabet to decide how best to read blocks
+      esl_sqfile_SetDigital(dbfp, abc); //ReadBlock requires knowledge of the alphabet to decide how best to read blocks
 
       for (i = 0; i < infocnt; ++i)
 	{
@@ -430,11 +478,19 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
       /* seqfile may need to be rewound (multiquery mode) */
       if (nquery > 1)
-	{
-	  if (! esl_sqfile_IsRewindable(dbfp)) 
-	    esl_fatal("Target sequence file %s isn't rewindable; can't search it with multiple queries", cfg->dbfile);
-	  esl_sqfile_Position(dbfp, 0);
-	}
+      {
+        if (! esl_sqfile_IsRewindable(dbfp))
+          esl_fatal("Target sequence file %s isn't rewindable; can't search it with multiple queries", cfg->dbfile);
+
+        if (! esl_opt_IsUsed(go, "--restrictdb_stkey") )
+          esl_sqfile_Position(dbfp, 0); //only re-set current position to 0 if we're not planning to set it in a moment
+      }
+
+      if ( cfg->firstseq_key != NULL ) { //it's tempting to want to do this once and capture the offset position for future passes, but ncbi files make this non-trivial, so this keeps it general
+        sstatus = esl_sqfile_PositionByKey(dbfp, cfg->firstseq_key);
+        if (sstatus != eslOK)
+          p7_Fail("Failure setting restrictdb_stkey to %d\n", cfg->firstseq_key);
+      }
 
       if (fprintf(ofp, "Query:       %s  [M=%d]\n", hmm->name, hmm->M)  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
       if (hmm->acc)  { if (fprintf(ofp, "Accession:   %s\n", hmm->acc)  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); }
@@ -447,47 +503,47 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
       p7_oprofile_Convert(gm, om);                  /* <om> is now p7_LOCAL, multihit */
 
       for (i = 0; i < infocnt; ++i)
-	{
-	  /* Create processing pipeline and hit list */
-	  info[i].th  = p7_tophits_Create(); 
-	  info[i].om  = p7_oprofile_Clone(om);
-	  info[i].pli = p7_pipeline_Create(go, om->M, 100, FALSE, p7_SEARCH_SEQS); /* L_hint = 100 is just a dummy for now */
-	  p7_pli_NewModel(info[i].pli, info[i].om, info[i].bg);
+      {
+        /* Create processing pipeline and hit list */
+        info[i].th  = p7_tophits_Create();
+        info[i].om  = p7_oprofile_Clone(om);
+        info[i].pli = p7_pipeline_Create(go, om->M, 100, FALSE, p7_SEARCH_SEQS); /* L_hint = 100 is just a dummy for now */
+        p7_pli_NewModel(info[i].pli, info[i].om, info[i].bg);
 
 #ifdef HMMER_THREADS
-	  if (ncpus > 0) esl_threads_AddThread(threadObj, &info[i]);
+        if (ncpus > 0) esl_threads_AddThread(threadObj, &info[i]);
 #endif
-	}
+      }
 
 #ifdef HMMER_THREADS
-      if (ncpus > 0)  sstatus = thread_loop(threadObj, queue, dbfp);
-      else            sstatus = serial_loop(info, dbfp);
+      if (ncpus > 0)  sstatus = thread_loop(threadObj, queue, dbfp, cfg->n_targetseq);
+      else            sstatus = serial_loop(info, dbfp, cfg->n_targetseq);
 #else
-      sstatus = serial_loop(info, dbfp);
+      sstatus = serial_loop(info, dbfp, cfg->n_targetseq);
 #endif
       switch(sstatus)
-	{
-	case eslEFORMAT: 
-	  esl_fatal("Parse failed (sequence file %s):\n%s\n", 
-		    dbfp->filename, esl_sqfile_GetErrorBuf(dbfp));
-	  break;
-	case eslEOF:
-	  /* do nothing */
-	  break;
-	default:
-	  esl_fatal("Unexpected error %d reading sequence file %s", sstatus, dbfp->filename);
-	}
+      {
+      case eslEFORMAT:
+        esl_fatal("Parse failed (sequence file %s):\n%s\n",
+            dbfp->filename, esl_sqfile_GetErrorBuf(dbfp));
+        break;
+      case eslEOF:
+        /* do nothing */
+        break;
+      default:
+        esl_fatal("Unexpected error %d reading sequence file %s", sstatus, dbfp->filename);
+      }
 
       /* merge the results of the search results */
       for (i = 1; i < infocnt; ++i)
-	{
-	  p7_tophits_Merge(info[0].th, info[i].th);
-	  p7_pipeline_Merge(info[0].pli, info[i].pli);
+      {
+        p7_tophits_Merge(info[0].th, info[i].th);
+        p7_pipeline_Merge(info[0].pli, info[i].pli);
 
-	  p7_pipeline_Destroy(info[i].pli);
-	  p7_tophits_Destroy(info[i].th);
-	  p7_oprofile_Destroy(info[i].om);
-	}
+        p7_pipeline_Destroy(info[i].pli);
+        p7_tophits_Destroy(info[i].th);
+        p7_oprofile_Destroy(info[i].om);
+      }
 
       /* Print the results.  */
       p7_tophits_SortBySortkey(info->th);
@@ -509,8 +565,8 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
 	if (p7_tophits_Alignment(info->th, abc, NULL, NULL, 0, p7_ALL_CONSENSUS_COLS, &msa) == eslOK)
 	  {
-	    if (textw > 0) eslx_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
-	    else           eslx_msafile_Write(afp, msa, eslMSAFILE_PFAM);
+	    if (textw > 0) esl_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
+	    else           esl_msafile_Write(afp, msa, eslMSAFILE_PFAM);
 	  
 	    if (fprintf(ofp, "# Alignment of %d hits satisfying inclusion thresholds saved to: %s\n", msa->nseq, esl_opt_GetString(go, "-A")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 	  } 
@@ -542,6 +598,7 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
    */
   if (tblfp)    p7_tophits_TabularTail(tblfp,    "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
   if (domtblfp) p7_tophits_TabularTail(domtblfp, "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
+  if (pfamtblfp) p7_tophits_TabularTail(pfamtblfp,"hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
   if (ofp)      { if (fprintf(ofp, "[ok]\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); }
 
   /* Cleanup - prepare for exit
@@ -570,6 +627,7 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   if (afp)           fclose(afp);
   if (tblfp)         fclose(tblfp);
   if (domtblfp)      fclose(domtblfp);
+  if (pfamtblfp)     fclose(pfamtblfp);
 
   return eslOK;
 
@@ -654,7 +712,7 @@ typedef struct {
  * MPI workers.  If multiple hmm's are in the query file, the
  * blocks are reused without parsing the database a second time.
  */
-int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block)
+int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block, int n_targetseqs)
 {
   int      status   = eslOK;
 
@@ -662,23 +720,23 @@ int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block)
   if (list->complete)
     {
       if (list->current == list->last)
-	{
-	  block->offset = 0;
-	  block->length = 0;
-	  block->count  = 0;
+      {
+        block->offset = 0;
+        block->length = 0;
+        block->count  = 0;
 
-	  status = eslEOF;
-	}
+        status = eslEOF;
+      }
       else
-	{
-	  int inx = list->current++;
+      {
+        int inx = list->current++;
 
-	  block->offset = list->blocks[inx].offset;
-	  block->length = list->blocks[inx].length;
-	  block->count  = list->blocks[inx].count;
+        block->offset = list->blocks[inx].offset;
+        block->length = list->blocks[inx].length;
+        block->count  = list->blocks[inx].count;
 
-	  status = eslOK;
-	}
+        status = eslOK;
+      }
 
       return status;
     }
@@ -688,7 +746,8 @@ int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block)
   block->count = 0;
 
   esl_sq_Reuse(sq);
-  while (block->length < MAX_BLOCK_SIZE && (status = esl_sqio_ReadInfo(sqfp, sq)) == eslOK)
+  if (n_targetseqs == 0) status = eslEOF; //this is to handle the end-case of a restrictdb scenario, where no more targets are required, and we want to mark the list as complete
+  while (block->length < MAX_BLOCK_SIZE && (n_targetseqs <0 || block->count < n_targetseqs) && (status = esl_sqio_ReadInfo(sqfp, sq)) == eslOK)
     {
       if (block->count == 0) block->offset = sq->roff;
       block->length = sq->eoff - block->offset + 1;
@@ -696,7 +755,9 @@ int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block)
       esl_sq_Reuse(sq);
     }
 
-  if (status == eslEOF && block->count > 0) status = eslOK;
+  if (block->count > 0)
+    if (status == eslEOF || block->count == n_targetseqs)
+      status = eslOK;
   if (status == eslEOF) list->complete = 1;
 
   /* add the block to the list of known blocks */
@@ -774,6 +835,8 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   MPI_Status       mpistatus;
   char             errbuf[eslERRBUFSIZE];
 
+  int              n_targets;
+
   w = esl_stopwatch_Create();
 
   if (esl_opt_GetBoolean(go, "--notextw")) textw = 0;
@@ -791,6 +854,14 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   else if (status == eslEINVAL)    mpi_failure("Can't autodetect format of a stdin or .gz seqfile");
   else if (status != eslOK)        mpi_failure("Unexpected error %d opening sequence file %s\n", status, cfg->dbfile);  
 
+  if (esl_opt_IsUsed(go, "--restrictdb_stkey") || esl_opt_IsUsed(go, "--restrictdb_n")) {
+      if (esl_opt_IsUsed(go, "--ssifile"))
+        esl_sqfile_OpenSSI(dbfp, esl_opt_GetString(go, "--ssifile"));
+      else
+        esl_sqfile_OpenSSI(dbfp, NULL);
+  }
+
+
   /* Open the query profile HMM file */
   status = p7_hmmfile_OpenE(cfg->hmmfile, NULL, &hfp, errbuf);
   if      (status == eslENOTFOUND) mpi_failure("File existence/permissions problem in trying to open HMM file %s.\n%s\n", cfg->hmmfile, errbuf);
@@ -830,19 +901,28 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
       bg = p7_bg_Create(abc);
     }
   
+
+  if ( cfg->firstseq_key != NULL ) { //it's tempting to want to do this once and capture the offset position for future passes, but ncbi files make this non-trivial, so this keeps it general
+    sstatus = esl_sqfile_PositionByKey(dbfp, cfg->firstseq_key);
+    if (sstatus != eslOK)
+      p7_Fail("Failure setting restrictdb_stkey to %d\n", cfg->firstseq_key);
+  }
+
   /* Outer loop: over each query HMM in <hmmfile>. */
   while (hstatus == eslOK) 
     {
-      P7_PROFILE      *gm    = NULL;
-      P7_OPROFILE     *om    = NULL;       /* optimized query profile                  */
-      P7_PIPELINE     *pli   = NULL;
-      P7_TOPHITS      *th    = NULL;
-
+      P7_PROFILE      *gm      = NULL;
+      P7_OPROFILE     *om      = NULL;       /* optimized query profile                  */
+      P7_PIPELINE     *pli     = NULL;
+      P7_TOPHITS      *th      = NULL;
+      int              seq_cnt = 0;
       nquery++;
       esl_stopwatch_Start(w);
 
+      n_targets = cfg->n_targetseq;
+
       /* seqfile may need to be rewound (multiquery mode) */
-      if (nquery > 1) list->current = 0;
+      if (nquery > 1)   list->current = 0;
 
       if (fprintf(ofp, "Query:       %s  [M=%d]\n", hmm->name, hmm->M)  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
       if (hmm->acc)  { if (fprintf(ofp, "Accession:   %s\n", hmm->acc)  < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); }
@@ -860,38 +940,44 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
       p7_pli_NewModel(pli, om, bg);
 
       /* Main loop: */
-      while ((sstatus = next_block(dbfp, dbsq, list, &block)) == eslOK)
-	{
-	  if (MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &mpistatus) != 0) 
-	    mpi_failure("MPI error %d receiving message from %d\n", mpistatus.MPI_SOURCE);
+      while ((n_targets==-1 || seq_cnt<=n_targets) && (sstatus = next_block(dbfp, dbsq, list, &block, n_targets-seq_cnt)) == eslOK )
+      {
+        seq_cnt += block.count;
 
-	  MPI_Get_count(&mpistatus, MPI_PACKED, &size);
-	  if (mpi_buf == NULL || size > mpi_size) {
-	    void *tmp;
-	    ESL_RALLOC(mpi_buf, tmp, sizeof(char) * size);
-	    mpi_size = size; 
-	  }
+        if (MPI_Probe(MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &mpistatus) != 0)
+          mpi_failure("MPI error %d receiving message from %d\n", mpistatus.MPI_SOURCE);
 
-	  dest = mpistatus.MPI_SOURCE;
-	  MPI_Recv(mpi_buf, size, MPI_PACKED, dest, mpistatus.MPI_TAG, MPI_COMM_WORLD, &mpistatus);
+        MPI_Get_count(&mpistatus, MPI_PACKED, &size);
+        if (mpi_buf == NULL || size > mpi_size) {
+          void *tmp;
+          ESL_RALLOC(mpi_buf, tmp, sizeof(char) * size);
+          mpi_size = size;
+        }
+
+        dest = mpistatus.MPI_SOURCE;
+        MPI_Recv(mpi_buf, size, MPI_PACKED, dest, mpistatus.MPI_TAG, MPI_COMM_WORLD, &mpistatus);
+
+        if (mpistatus.MPI_TAG == HMMER_ERROR_TAG)
+          mpi_failure("MPI client %d raised error:\n%s\n", dest, mpi_buf);
+        if (mpistatus.MPI_TAG != HMMER_READY_TAG)
+          mpi_failure("Unexpected tag %d from %d\n", mpistatus.MPI_TAG, dest);
+
+        MPI_Send(&block, 3, MPI_LONG_LONG_INT, dest, HMMER_BLOCK_TAG, MPI_COMM_WORLD);
+      }
+
+      if (n_targets!=-1 && seq_cnt==n_targets)
+        sstatus = eslEOF;
 
-	  if (mpistatus.MPI_TAG == HMMER_ERROR_TAG)
-	    mpi_failure("MPI client %d raised error:\n%s\n", dest, mpi_buf);
-	  if (mpistatus.MPI_TAG != HMMER_READY_TAG)
-	    mpi_failure("Unexpected tag %d from %d\n", mpistatus.MPI_TAG, dest);
-      
-	  MPI_Send(&block, 3, MPI_LONG_LONG_INT, dest, HMMER_BLOCK_TAG, MPI_COMM_WORLD);
-	}
       switch(sstatus)
-	{
-	case eslEFORMAT: 
-	  mpi_failure("Parse failed (sequence file %s):\n%s\n", dbfp->filename, esl_sqfile_GetErrorBuf(dbfp));
-	  break;
-	case eslEOF:
-	  break;
-	default:
-	  mpi_failure("Unexpected error %d reading sequence file %s", sstatus, dbfp->filename);
-	}
+      {
+      case eslEFORMAT:
+        mpi_failure("Parse failed (sequence file %s):\n%s\n", dbfp->filename, esl_sqfile_GetErrorBuf(dbfp));
+        break;
+      case eslEOF:
+        break;
+      default:
+        mpi_failure("Unexpected error %d reading sequence file %s", sstatus, dbfp->filename);
+      }
 
       block.offset = 0;
       block.length = 0;
@@ -950,7 +1036,8 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
       if (tblfp)    p7_tophits_TabularTargets(tblfp,    hmm->name, hmm->acc, th, pli, (nquery == 1));
       if (domtblfp) p7_tophits_TabularDomains(domtblfp, hmm->name, hmm->acc, th, pli, (nquery == 1));
-  
+      if (pfamtblfp) p7_tophits_TabularXfam(pfamtblfp, hmm->name, hmm->acc, th, pli);
+
       esl_stopwatch_Stop(w);
       p7_pli_Statistics(ofp, pli, w);
       if (fprintf(ofp, "//\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
@@ -961,8 +1048,8 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
 	if (p7_tophits_Alignment(th, abc, NULL, NULL, 0, p7_ALL_CONSENSUS_COLS, &msa) == eslOK)
 	  {
-	    if (textw > 0) eslx_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
-	    else           eslx_msafile_Write(afp, msa, eslMSAFILE_PFAM);
+	    if (textw > 0) esl_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
+	    else           esl_msafile_Write(afp, msa, eslMSAFILE_PFAM);
 	  
 	    if (fprintf(ofp, "# Alignment of %d hits satisfying inclusion thresholds saved to: %s\n", msa->nseq, esl_opt_GetString(go, "-A")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 	  } 
@@ -1010,8 +1097,9 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
   /* Terminate outputs... any last words?
    */
-  if (tblfp)    p7_tophits_TabularTail(tblfp,    "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
-  if (domtblfp) p7_tophits_TabularTail(domtblfp, "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
+  if (tblfp)    p7_tophits_TabularTail(tblfp,     "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
+  if (domtblfp) p7_tophits_TabularTail(domtblfp,  "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
+  if (pfamtblfp)p7_tophits_TabularTail(pfamtblfp, "hmmsearch", p7_SEARCH_SEQS, cfg->hmmfile, cfg->dbfile, go);
   if (ofp)     { if (fprintf(ofp, "[ok]\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed"); }
 
   /* Cleanup - prepare for exit
@@ -1030,6 +1118,7 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   if (afp)           fclose(afp);
   if (tblfp)         fclose(tblfp);
   if (domtblfp)      fclose(domtblfp);
+  if (pfamtblfp)     fclose(pfamtblfp);
 
   return eslOK;
 
@@ -1081,6 +1170,7 @@ mpi_worker(ESL_GETOPTS *go, struct cfg_s *cfg)
       /* One-time initializations after alphabet <abc> becomes known */
       dbsq = esl_sq_CreateDigital(abc);
       bg = p7_bg_Create(abc);
+      esl_sqfile_SetDigital(dbfp, abc);
     }
   
   /* Outer loop: over each query HMM in <hmmfile>. */
@@ -1126,7 +1216,7 @@ mpi_worker(ESL_GETOPTS *go, struct cfg_s *cfg)
 	      p7_bg_SetLength(bg, dbsq->n);
 	      p7_oprofile_ReconfigLength(om, dbsq->n);
       
-	      p7_Pipeline(pli, om, bg, dbsq, th);
+	      p7_Pipeline(pli, om, bg, dbsq, NULL, th);
 
 	      esl_sq_Reuse(dbsq);
 	      p7_pipeline_Reuse(pli);
@@ -1193,25 +1283,30 @@ mpi_worker(ESL_GETOPTS *go, struct cfg_s *cfg)
 #endif /*HAVE_MPI*/
 
 static int
-serial_loop(WORKER_INFO *info, ESL_SQFILE *dbfp)
+serial_loop(WORKER_INFO *info, ESL_SQFILE *dbfp, int n_targetseqs)
 {
   int      sstatus;
   ESL_SQ   *dbsq     = NULL;   /* one target sequence (digital)  */
+  int seq_cnt = 0;
 
   dbsq = esl_sq_CreateDigital(info->om->abc);
 
   /* Main loop: */
-  while ((sstatus = esl_sqio_Read(dbfp, dbsq)) == eslOK)
-    {
+  while ( (n_targetseqs==-1 || seq_cnt<n_targetseqs) &&  (sstatus = esl_sqio_Read(dbfp, dbsq)) == eslOK)
+  {
       p7_pli_NewSeq(info->pli, dbsq);
       p7_bg_SetLength(info->bg, dbsq->n);
       p7_oprofile_ReconfigLength(info->om, dbsq->n);
       
-      p7_Pipeline(info->pli, info->om, info->bg, dbsq, info->th);
-	  
+      p7_Pipeline(info->pli, info->om, info->bg, dbsq, NULL, info->th);
+
+      seq_cnt++;
       esl_sq_Reuse(dbsq);
       p7_pipeline_Reuse(info->pli);
-    }
+  }
+
+  if (n_targetseqs!=-1 && seq_cnt==n_targetseqs)
+    sstatus = eslEOF;
 
   esl_sq_Destroy(dbsq);
 
@@ -1220,7 +1315,7 @@ serial_loop(WORKER_INFO *info, ESL_SQFILE *dbfp)
 
 #ifdef HMMER_THREADS
 static int
-thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp)
+thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp, int n_targetseqs)
 {
   int  status  = eslOK;
   int  sstatus = eslOK;
@@ -1228,8 +1323,6 @@ thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp)
   ESL_SQ_BLOCK *block;
   void         *newBlock;
 
-  impl_ThreadInit();
-
   esl_workqueue_Reset(queue);
   esl_threads_WaitForStart(obj);
 
@@ -1237,21 +1330,30 @@ thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp)
   if (status != eslOK) esl_fatal("Work queue reader failed");
       
   /* Main loop: */
-  while (sstatus == eslOK)
+  while (sstatus == eslOK )
     {
       block = (ESL_SQ_BLOCK *) newBlock;
-      sstatus = esl_sqio_ReadBlock(dbfp, block, -1, FALSE);
+
+      if (n_targetseqs == 0)
+      {
+        block->count = 0;
+        sstatus = eslEOF;
+      } else {
+        sstatus = esl_sqio_ReadBlock(dbfp, block, -1, n_targetseqs, FALSE);
+        n_targetseqs -= block->count;
+      }
+
       if (sstatus == eslEOF)
-	{
-	  if (eofCount < esl_threads_GetWorkerCount(obj)) sstatus = eslOK;
-	  ++eofCount;
-	}
-	  
+      {
+        if (eofCount < esl_threads_GetWorkerCount(obj)) sstatus = eslOK;
+        ++eofCount;
+      }
+
       if (sstatus == eslOK)
-	{
-	  status = esl_workqueue_ReaderUpdate(queue, block, &newBlock);
-	  if (status != eslOK) esl_fatal("Work queue reader failed");
-	}
+      {
+        status = esl_workqueue_ReaderUpdate(queue, block, &newBlock);
+        if (status != eslOK) esl_fatal("Work queue reader failed");
+      }
     }
 
   status = esl_workqueue_ReaderUpdate(queue, block, NULL);
@@ -1279,6 +1381,8 @@ pipeline_thread(void *arg)
   ESL_SQ_BLOCK  *block = NULL;
   void          *newBlock;
   
+  impl_Init();
+
   obj = (ESL_THREADS *) arg;
   esl_threads_Started(obj, &workeridx);
 
@@ -1300,7 +1404,7 @@ pipeline_thread(void *arg)
 	  p7_bg_SetLength(info->bg, dbsq->n);
 	  p7_oprofile_ReconfigLength(info->om, dbsq->n);
 	  
-	  p7_Pipeline(info->pli, info->om, info->bg, dbsq, info->th);
+	  p7_Pipeline(info->pli, info->om, info->bg, dbsq, NULL, info->th);
 	  
 	  esl_sq_Reuse(dbsq);
 	  p7_pipeline_Reuse(info->pli);
@@ -1323,14 +1427,11 @@ pipeline_thread(void *arg)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
- * 
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmsearch.c $
- * SVN $Id: hmmsearch.c 3572 2011-06-20 18:06:12Z wheelert $
  *****************************************************************/
 
