@@ -194,10 +194,7 @@ main(int argc, char **argv)
    */
   cfg.hmmfile  = esl_opt_GetArg(go, 1);
   cfg.r        = esl_randomness_Create(esl_opt_GetInteger(go, "--seed"));
-  cfg.abc      = esl_alphabet_Create(eslAMINO);
-
-  if (esl_opt_GetBoolean(go, "--bgflat")) cfg.bg = p7_bg_CreateUniform(cfg.abc);
-  else                                    cfg.bg = p7_bg_Create(cfg.abc);
+  //cfg.abc      = esl_alphabet_Create(eslAMINO);
 
   cfg.my_rank  = 0;		/* MPI init will change this soon, if --mpi was set */
   cfg.nproc    = 0;		/* MPI init will change this soon, if --mpi was set */
@@ -212,9 +209,7 @@ main(int argc, char **argv)
   cfg.ffp      = NULL;
   cfg.xfp      = NULL;
   cfg.alfp     = NULL;
-
-  p7_bg_SetLength(cfg.bg, esl_opt_GetInteger(go, "-L"));  /* set the null model background length in both master and workers. */
-
+  cfg.bg       = NULL;
 
   /* This is our stall point, if we need to wait until we get a
    * debugger attached to this process for debugging (especially
@@ -376,12 +371,18 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
       (av = malloc(sizeof(int)    * cfg->N)) == NULL)       p7_Fail("allocation failed");
 
   while ((status = p7_hmmfile_Read(cfg->hfp, &(cfg->abc), &hmm)) != eslEOF) 
-    {
+  {
       if      (status == eslEOD)       p7_Fail("read failed, HMM file %s may be truncated?", cfg->hmmfile);
       else if (status == eslEFORMAT)   p7_Fail("bad file format in HMM file %s",             cfg->hmmfile);
       else if (status == eslEINCOMPAT) p7_Fail("HMM file %s contains different alphabets",   cfg->hmmfile);
       else if (status != eslOK)        p7_Fail("Unexpected error in reading HMMs from %s",   cfg->hmmfile);
 
+      if (cfg->bg == NULL) {
+        if (esl_opt_GetBoolean(go, "--bgflat")) cfg->bg = p7_bg_CreateUniform(cfg->abc);
+        else                                    cfg->bg = p7_bg_Create(cfg->abc);
+        p7_bg_SetLength(cfg->bg, esl_opt_GetInteger(go, "-L"));  /* set the null model background length in both master and workers. */
+      }
+
       if (process_workunit(go, cfg, errbuf, hmm, xv, av, &mu, &lambda) != eslOK) p7_Fail(errbuf);
       if (output_result   (go, cfg, errbuf, hmm, xv, av,  mu,  lambda) != eslOK) p7_Fail(errbuf);
 
@@ -455,6 +456,13 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 	      else if (status == eslEFORMAT)   { xstatus = status; sprintf(errbuf, "bad file format in HMM file %s",             cfg->hmmfile); }
 	      else if (status == eslEINCOMPAT) { xstatus = status; sprintf(errbuf, "HMM file %s contains different alphabets",   cfg->hmmfile); }
 	      else if (status != eslEOF)       { xstatus = status; sprintf(errbuf, "Unexpected error in reading HMMs from %s",   cfg->hmmfile); }
+
+	      if (cfg->bg == NULL) { // first time only
+          if (esl_opt_GetBoolean(go, "--bgflat")) cfg->bg = p7_bg_CreateUniform(cfg->abc);
+          else                                    cfg->bg = p7_bg_Create(cfg->abc);
+	      }
+	      //this next step is redundant, but it avoids a race condition above.
+        p7_bg_SetLength(cfg->bg, esl_opt_GetInteger(go, "-L"));  /* set the null model background length in both master and workers. */
 	    }
 	}
 
@@ -955,13 +963,13 @@ elide_length_model(P7_PROFILE *gm, P7_BG *bg)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  * 
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/hmmsim.c $
- * SVN $Id: hmmsim.c 3658 2011-08-20 17:41:31Z eddys $
+ * SVN $URL$
+ * SVN $Id$
  *****************************************************************/
