@@ -8,7 +8,7 @@
  *    5. Copyright and license information.
  *    
  * SRE, Thu Jun 14 09:59:20 2007 [Janelia] [Tom Waits, Orphans]
- * SVN $Id: mpi.c 3474 2011-01-17 13:25:32Z eddys $
+ * SVN $Id$
  */
 #include "p7_config.h"		
 
@@ -24,7 +24,7 @@
 #include "esl_getopts.h"
 
 #include "hmmer.h"
-#include "impl_dummy."
+#include "impl_dummy.h"
 
 /*****************************************************************
  * 1. Communicating P7_OPROFILE, an optimized model.
@@ -367,8 +367,8 @@ int main(void) { return 0; }
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
