@@ -263,13 +263,13 @@ p7_oprofile_ReconfigUnihit(P7_OPROFILE *om, int L)
 
 
 /* Function:  p7_oprofile_GetFwdTransitionArray()
- * Synopsis:  Retrieve full 32-bit float transition probabilities from an
- *            optimized profile into a flat array
+ * Synopsis:  Retrieve full 32-bit float transition probabilities from a
+ *            profile into a flat array
  *
  * Purpose:   Extract an array of <type> (e.g. p7O_II) transition probabilities
  *            from the underlying <om> profile. In SIMD implementations,
  *            these are striped and interleaved, making them difficult to
- *            directly access. Here, this is trivial
+ *            directly access. Here, this is trivial.
  *
  * Args:      <om>   - optimized profile, containing transition information
  *            <type> - transition type (e.g. p7O_II)
@@ -285,7 +285,7 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
   int i;
 
   for (i=0; i<om->M; i++) {
-    arr[i] = p7P_TSC(om, i, type);
+    arr[i] = exp(p7P_TSC(om, i, type));
   }
 
   return eslOK;
@@ -294,12 +294,12 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
 
 
 
-/* Function:  p7_oprofile_GetMSVEmissionArray()
- * Synopsis:  Retrieve MSV residue emission scores from an optimized
+/* Function:  p7_oprofile_GetSSVEmissionScoreArray()
+ * Synopsis:  Retrieve MSV residue emission scores from a
  *            profile into an array
  *
  * Purpose:   Extract an implicitly 2D array of 8-bit int MSV residue
- *            emission scores from an optimized profile <om>. <arr> must
+ *            emission scores from a profile <om>. <arr> must
  *            be allocated by the calling function to be of size
  *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
  *            is done as  [om->abc->Kp * i +  c ] for character c at
@@ -310,7 +310,7 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
  *            is based on code from the function mf_conversion in impl_sse's
  *            p7_oprofile.c
  *
- * Args:      <om>   - optimized profile, containing transition information
+ * Args:      <om>   - profile, containing emission information
  *            <arr>  - preallocated array into which scores will be placed
  *
  * Returns:   <eslOK> on success.
@@ -318,7 +318,7 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
  * Throws:    (no abnormal error conditions)
  */
 int
-p7_oprofile_GetMSVEmissionArray(const P7_OPROFILE *om, uint8_t *arr )
+p7_oprofile_GetSSVEmissionScoreArray(const P7_OPROFILE *om, uint8_t *arr )
 {
   int     M   = om->M;    /* length of the query                                          */
   int i, j;
@@ -346,6 +346,131 @@ p7_oprofile_GetMSVEmissionArray(const P7_OPROFILE *om, uint8_t *arr )
   return eslOK;
 }
 
+/* Function:  p7_oprofile_GetFwdEmissionArray()
+ * Synopsis:  Retrieve Fwd (float) residue emission scores from a
+ *            profile into an array
+ *
+ * Purpose:   Extract an implicitly 2D array of 32-bit float Fwd residue
+ *            emission scores from a profile <om>. <arr> must
+ *            be allocated by the calling function to be of size
+ *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
+ *            is done as  [om->abc->Kp * i +  c ] for character c at
+ *            position i.
+ *
+ *
+ * Args:      <om>   - profile
+ *            <arr>  - preallocated array into which scores will be placed
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    (no abnormal error conditions)
+ */
+int
+p7_oprofile_GetFwdEmissionScoreArray(const P7_OPROFILE *om, float *arr )
+{
+  int i, j;
+
+  for (i = 1; i <= om->M; i++) {
+    for (j=0; j<om->abc->Kp; j++) {
+      arr[i*om->abc->Kp + j] =  om->rsc[j][(i) * p7P_NR     + p7P_MSC];
+    }
+  }
+
+  return eslOK;
+}
+
+/* Function:  p7_oprofile_GetFwdEmissionArray()
+ * Synopsis:  Retrieve Fwd (float) residue emission values from an optimized
+ *            profile into an array
+ *
+ * Purpose:   Extract an implicitly 2D array of 32-bit float Fwd residue
+ *            emission values from an optimized profile <om>, converting
+ *            back to emission values based on the background. <arr> must
+ *            be allocated by the calling function to be of size
+ *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
+ *            is done as  [om->abc->Kp * i +  c ] for character c at
+ *            position i.
+ *
+ * Args:      <om>   - optimized profile, containing transition information
+ *            <bg>   - background frequencies
+ *            <arr>  - preallocated array into which scores will be placed
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    (no abnormal error conditions)
+ */
+int
+p7_oprofile_GetFwdEmissionArray(const P7_OPROFILE *om, P7_BG *bg, float *arr )
+{
+  int i, j;
+
+  for (i = 1; i <= om->M; i++) {
+    for (j=0; j<om->abc->Kp; j++) {
+      arr[i*om->abc->Kp + j] =  bg->f[j] * exp( om->rsc[j][(i) * p7P_NR     + p7P_MSC]);
+    }
+  }
+
+  return eslOK;
+}
+
+
+
+/* Function:  p7_oprofile_UpdateFwdEmissionScores()
+ * Synopsis:  Update om match emissions to account for new bg, using
+ *            preallocated sc_tmp[].
+ *
+ * Purpose:   Change scores based on updated background model
+ *
+ */
+int
+p7_oprofile_UpdateFwdEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_tmp)
+{
+  int     M   = om->M;    /* length of the query                                          */
+  int     i, j;
+  int     K   = om->abc->K;
+  int     Kp  = om->abc->Kp;
+
+  for (i = 1; i <= om->M; i++) {
+
+    for (j=0; j<K; j++) {
+      if (om->mm && om->mm[i] == 'm')
+        sc_tmp[j] = 0;
+      else
+        sc_tmp[j] = log(fwd_emissions[i*om->abc->Kp + j] / bg->f[j]);
+    }
+
+
+    esl_abc_FExpectScVec(bg->abc, sc_tmp, bg->f);
+
+    for (j=0; j<Kp; j++)
+      om->rsc[j][(i) * p7P_NR  + p7P_MSC] =  sc_tmp[j];
+
+  }
+
+  return eslOK;
+
+}
+
+/* Function:  p7_oprofile_UpdateVitEmissionScores()
+ * Synopsis:  Dummy function - no need to update Viterbi-specific scores in dummy
+ *
+ */
+int
+p7_oprofile_UpdateVitEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr)
+{
+  return eslOK;
+}
+
+
+/* Function:  p7_oprofile_UpdateMSVEmissionScores()
+ * Synopsis:  Dummy function - no need to update Viterbi-specific scores in dummy
+ */
+int
+p7_oprofile_UpdateMSVEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr)
+{
+  return eslOK;
+
+}
 /*------------ end, conversions from P7_OPROFILE ------------------*/
 
 
@@ -668,8 +793,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
