@@ -9,7 +9,7 @@
  *   6. Copyright and license information.
  *
  * SRE, Mon Aug 18 08:31:11 2008 [Janelia]
- * SVN $Id: null2.c 3474 2011-01-17 13:25:32Z eddys $
+ * SVN $Id$
  */
 #include "p7_config.h"
 
@@ -141,7 +141,7 @@ p7_Null2_ByTrace(const P7_OPROFILE *om, const P7_TRACE *tr, int zstart, int zend
   float  xfactor;
   __m128 sv;
   __m128 *rp;
-  int    q, r, s;
+  int    q, r;
   int    x;
   int    z;
 
@@ -162,7 +162,7 @@ p7_Null2_ByTrace(const P7_OPROFILE *om, const P7_TRACE *tr, int zstart, int zend
       Ld++;
       if (tr->k[z] > 0)	/* must be an M or I */
 	{ /* surely there's an easier way? but our workspace is striped, interleaved quads... */
-	  s = ( (tr->st[z] == p7T_M) ?  p7X_M : p7X_I);
+	  // s = ( (tr->st[z] == p7T_M) ?  p7X_M : p7X_I);  // We don't need the state type <s>, but this is how you'd get it.
 	  q = p7X_NSCELLS * ( (tr->k[z] - 1) % Q) + p7X_M;
 	  r = (tr->k[z] - 1) / Q;
 	  u.v            = wrk->dpf[0][q];
@@ -506,8 +506,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
