@@ -29,6 +29,11 @@
 #include "hmmer.h"
 #include "impl_sse.h"
 
+static uint8_t unbiased_byteify(P7_OPROFILE *om, float sc);
+static uint8_t biased_byteify(P7_OPROFILE *om, float sc);
+static int16_t wordify(P7_OPROFILE *om, float sc);
+static int     sf_conversion(P7_OPROFILE *om);
+
 /*****************************************************************
  * 1. The P7_OPROFILE structure: a score profile.
  *****************************************************************/
@@ -373,6 +378,7 @@ p7_oprofile_Copy(P7_OPROFILE *om1)
   om2->allocM    = om1->allocM;
   om2->mode      = om1->mode;
   om2->nj        = om1->nj;
+  om2->max_length   = om1->max_length;
 
   om2->clone     = om1->clone;
 
@@ -412,6 +418,236 @@ p7_oprofile_Clone(const P7_OPROFILE *om1)
   return NULL;
 }
 
+
+/* Function:  p7_oprofile_UpdateFwdEmissionScores()
+ * Synopsis:  Update the Forward/Backward part of the optimized profile
+ *            match emissions to account for new background distribution.
+ *
+ * Purpose:   This implementation re-orders the loops used to access/modify
+ *            the rfv array relative to how it's accessed for example in
+ *            fb_conversion(), to minimize the required size of sc_arr.
+ *
+ * Args:      om              - optimized profile to be updated.
+ *            bg              - the new bg distribution
+ *            fwd_emissions   - precomputed Fwd (float) residue emission
+ *                              probabilities in serial order (gathered from
+ *                              the optimized striped <om> with
+ *                              p7_oprofile_GetFwdEmissionArray() ).
+ *            sc_arr            Preallocated array of at least Kp*4 floats
+ */
+int
+p7_oprofile_UpdateFwdEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr)
+{
+  int     M   = om->M;    /* length of the query                                          */
+  int     k, q, x, z;
+  int     nq  = p7O_NQF(M);     /* segment length; total # of striped vectors needed            */
+  int     K   = om->abc->K;
+  int     Kp  = om->abc->Kp;
+  union   { __m128 v; float x[4]; } tmp; /* used to align and load simd minivectors               */
+
+
+  for (k = 1, q = 0; q < nq; q++, k++) {
+
+    //First compute the core characters of the alphabet
+    for (x = 0; x < K; x++) {
+      for (z = 0; z < 4; z++) {
+        if (k+ z*nq <= M)  sc_arr[z*Kp + x] =  (om->mm && om->mm[(k+z*nq)]=='m') ? 0 : log( (double)(fwd_emissions[Kp * (k+z*nq) + x])/bg->f[x]);
+        else               sc_arr[z*Kp + x] =  -eslINFINITY;
+
+        tmp.x[z] = sc_arr[z*Kp + x];
+      }
+      om->rfv[x][q] = esl_sse_expf(tmp.v);
+
+    }
+
+    /* gap, nonresidue, and missing data residue codes don't get set by FExpectScVec(), 
+     * so do them
+     */
+    for (z = 0; z < 4; z++)
+      {
+	sc_arr[z*Kp + K]        = -eslINFINITY; /* gap char -     */
+	sc_arr[z*Kp + (Kp - 2)] = -eslINFINITY; /* nonresidue *   */
+	sc_arr[z*Kp + (Kp - 1)] = -eslINFINITY; /* missing data ~ */
+      }
+
+    // Then compute corresponding scores for ambiguity codes.
+    for (z = 0; z < 4; z++)
+      esl_abc_FExpectScVec(om->abc, sc_arr+(z*Kp), bg->f);
+
+    //finish off the interleaved values
+    for (x = K; x < Kp; x++) {
+      for (z = 0; z < 4; z++)
+         tmp.x[z] = sc_arr[z*Kp + x];  // computed in FExpectScVec call above
+      om->rfv[x][q] = esl_sse_expf(tmp.v);
+    }
+  }
+
+  return eslOK;
+}
+
+
+/* Function:  p7_oprofile_UpdateVitEmissionScores()
+ * Synopsis:  Update the Viterbi part of the optimized profile match
+ *            emissions to account for new background distribution.
+ *.
+ * Purpose:   This implementation re-orders the loops used to access/modify
+ *            the rmv array relative to how it's accessed for example in
+ *            vf_conversion(), to minimize the required size of sc_arr.
+ *
+ * Args:      om              - optimized profile to be updated.
+ *            bg              - the new bg distribution
+ *            fwd_emissions   - precomputed Fwd (float) residue emission
+ *                              probabilities in serial order (gathered from
+ *                              the optimized striped <om> with
+ *                              p7_oprofile_GetFwdEmissionArray() ).
+ *            sc_arr            Preallocated array of at least Kp*8 floats
+ */
+int
+p7_oprofile_UpdateVitEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr)
+{
+  int     M   = om->M;    /* length of the query                                          */
+  int     k, q, x, z;
+  int     nq  = p7O_NQW(M);     /* segment length; total # of striped vectors needed            */
+  int     K   = om->abc->K;
+  int     Kp  = om->abc->Kp;
+  int     idx;
+  union   { __m128i v; int16_t i[8]; } tmp; /* used to align and load simd minivectors            */
+
+  for (k = 1, q = 0; q < nq; q++, k++) {
+
+    //First compute the core characters of the alphabet
+    for (x = 0; x < K; x++) {
+      for (z = 0; z < 8; z++) {
+        idx = z*Kp + x;
+        if (k+ z*nq <= M)  {
+          sc_arr[idx] = (om->mm && om->mm[(k+z*nq)]=='m') ? 0 : log( (double)(fwd_emissions[Kp * (k+z*nq) + x])/bg->f[x]);
+          tmp.i[z]    = wordify(om, sc_arr[idx]);
+        }
+        else
+        {
+          sc_arr[idx] =  -eslINFINITY;
+          tmp.i[z]    =  -32768;
+        }
+
+      }
+      om->rwv[x][q] = tmp.v;
+
+    }
+
+    // Then compute corresponding scores for ambiguity codes.
+    for (z = 0; z < 8; z++)
+      esl_abc_FExpectScVec(om->abc, sc_arr+(z*Kp), bg->f);
+
+    //finish off the interleaved values
+    for (x = K; x < Kp; x++) {
+      for (z = 0; z < 8; z++) {
+        idx = z*Kp + x;
+        if (x==K || x>Kp-3 || sc_arr[idx] == -eslINFINITY)
+          tmp.i[z]  =  -32768;
+        else
+          tmp.i[z]  = wordify(om, sc_arr[idx]);
+      }
+      om->rwv[x][q] = tmp.v;
+    }
+  }
+
+  return eslOK;
+}
+
+
+
+/* Function:  p7_oprofile_UpdateMSVEmissionScores()
+ * Synopsis:  Update the MSV part of the optimized profile match
+ *            emissions to account for new background distribution.
+ *.
+ * Purpose:   This implementation re-orders the loops used to access/modify
+ *            the rbv array relative to how it's accessed for example in
+ *            mf_conversion(), to minimize the required size of sc_arr.
+ *
+ * Args:      om              - optimized profile to be updated.
+ *            bg              - the new bg distribution
+ *            fwd_emissions   - precomputed Fwd (float) residue emission
+ *                              probabilities in serial order (gathered from
+ *                              the optimized striped <om> with
+ *                              p7_oprofile_GetFwdEmissionArray() ).
+ *            sc_arr            Preallocated array of at least Kp*16 floats
+ */
+int
+p7_oprofile_UpdateMSVEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr)
+{
+  int     M   = om->M;    /* length of the query                                          */
+  int     k, q, x, z;
+  int     nq  = p7O_NQB(M);     /* segment length; total # of striped vectors needed            */
+  int     K   = om->abc->K;
+  int     Kp  = om->abc->Kp;
+  int     idx;
+  float   max = 0.0;    /* maximum residue score: used for unsigned emission score bias */
+  union   { __m128i v; uint8_t i[16]; } tmp; /* used to align and load simd minivectors           */
+
+
+  /* First we determine the basis for the limited-precision MSVFilter scoring system.
+   * Default: 1/3 bit units, base offset 190:  range 0..255 => -190..65 => -63.3..21.7 bits
+   * See J2/66, J4/138 for analysis.
+   * This depends on having computed scores. I do this in a first pass, to get the max
+   * score ... then re-compute those scores so they can be converted to 8bit scores
+   */
+  for (k = 1, q = 0; q < nq; q++, k++) {
+    for (x = 0; x < K; x++) {
+      for (z = 0; z < 16; z++) {
+        idx = z*Kp + x;
+        if (k+ z*nq <= M && !(om->mm && om->mm[(k+z*nq)]=='m'))
+          max = ESL_MAX(max, log( (double)(fwd_emissions[Kp * (k+z*nq) + x])/bg->f[x]));
+      }
+    }
+  }
+  om->scale_b = 3.0 / eslCONST_LOG2;                    /* scores in units of third-bits */
+  om->base_b  = 190;
+  om->bias_b  = unbiased_byteify(om, -1.0 * max);
+
+  for (k = 1, q = 0; q < nq; q++, k++) {
+
+    //First compute the core characters of the alphabet
+    for (x = 0; x < K; x++) {
+      for (z = 0; z < 16; z++) {
+        idx = z*Kp + x;
+        if (k+ z*nq <= M)  {
+          sc_arr[idx] = (om->mm && om->mm[(k+z*nq)]=='m') ? 0 : log( (double)(fwd_emissions[Kp * (k+z*nq) + x])/bg->f[x]);
+          tmp.i[z]    = biased_byteify(om, sc_arr[idx]);
+        }
+        else
+        {
+          sc_arr[idx] =  -eslINFINITY;
+          tmp.i[z]    =  255;
+        }
+
+      }
+      om->rbv[x][q] = tmp.v;
+
+    }
+
+    // Then compute corresponding scores for ambiguity codes.
+    for (z = 0; z < 16; z++)
+      esl_abc_FExpectScVec(om->abc, sc_arr+(z*Kp), bg->f);
+
+    //finish off the interleaved values
+    for (x = K; x < Kp; x++) {
+      for (z = 0; z < 16; z++) {
+        idx = z*Kp + x;
+        if (x==K || x>Kp-3 || sc_arr[idx] == -eslINFINITY)
+          tmp.i[z]  =  255;
+        else
+          tmp.i[z]  = biased_byteify(om, sc_arr[idx]);
+      }
+      om->rbv[x][q] = tmp.v;
+    }
+  }
+
+  sf_conversion(om);
+
+  return eslOK;
+}
+
+
 /*----------------- end, P7_OPROFILE structure ------------------*/
 
 
@@ -559,11 +795,13 @@ mf_conversion(const P7_PROFILE *gm, P7_OPROFILE *om)
 
   /* striped match costs: start at k=1.  */
   for (x = 0; x < gm->abc->Kp; x++)
+  {
     for (q = 0, k = 1; q < nq; q++, k++)
-      {
-	for (z = 0; z < 16; z++) tmp.i[z] = ((k+ z*nq <= M) ? biased_byteify(om, p7P_MSC(gm, k+z*nq, x)) : 255);
-	om->rbv[x][q]   = tmp.v;	
-      }
+    {
+      for (z = 0; z < 16; z++) tmp.i[z] = ((k+ z*nq <= M) ? biased_byteify(om, p7P_MSC(gm, k+z*nq, x)) : 255);
+      om->rbv[x][q]   = tmp.v;
+    }
+  }
 
   /* transition costs */
   om->tbm_b = unbiased_byteify(om, logf(2.0f / ((float) gm->M * (float) (gm->M+1)))); /* constant B->Mk penalty        */
@@ -713,6 +951,7 @@ fb_conversion(const P7_PROFILE *gm, P7_OPROFILE *om)
 	om->rfv[x][q] = esl_sse_expf(tmp.v);
       }
 
+
   /* Transition scores, all but the DD's. */
   for (j = 0, k = 1, q = 0; q < nq; q++, k++)
     {
@@ -733,7 +972,7 @@ fb_conversion(const P7_PROFILE *gm, P7_OPROFILE *om)
 	}
     }
 
-  /* And finally the DD's, which are at the end of the optimized tsc vector; (j is already there) */
+  /* And finally the DD's, which are at the end of the optimized tfv vector; (j is already there) */
   for (k = 1, q = 0; q < nq; q++, k++)
     {
       for (z = 0; z < 4; z++) tmp.x[z] = (k+z*nq < M) ? p7P_TSC(gm, k+z*nq, p7P_DD) : -eslINFINITY;
@@ -987,7 +1226,8 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
 
 
   for (i=0; i<nq; i++) {
-    tmp.v = om->tfv[ type  + 7 * i ];
+    // because DD transitions are held at the end of the tfv array
+    tmp.v = om->tfv[ (type==p7O_DD ?  nq*7+i :  type+7*i) ];
     for (j=0; j<4; j++)
       if ( i+1+ j*nq < om->M+1)
         arr[i+1+ j*nq]      = tmp.x[j];
@@ -997,11 +1237,11 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
 
 }
 
-/* Function:  p7_oprofile_GetMSVEmissionArray()
+/* Function:  p7_oprofile_GetSSVEmissionScoreArray()
  * Synopsis:  Retrieve MSV residue emission scores from an optimized
  *            profile into an array
  *
- * Purpose:   Extract an implicitly 2D array of 8-bit int MSV residue
+ * Purpose:   Extract an implicitly 2D array of 8-bit int SSV residue
  *            emission scores from an optimized profile <om>. <arr> must
  *            be allocated by the calling function to be of size
  *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
@@ -1021,7 +1261,7 @@ p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr )
  * Throws:    (no abnormal error conditions)
  */
 int
-p7_oprofile_GetMSVEmissionArray(const P7_OPROFILE *om, uint8_t *arr )
+p7_oprofile_GetSSVEmissionScoreArray(const P7_OPROFILE *om, uint8_t *arr )
 {
   int x, q, z, k;
   union { __m128i v; uint8_t i[16]; } tmp; /* used to align and read simd minivectors           */
@@ -1041,6 +1281,104 @@ p7_oprofile_GetMSVEmissionArray(const P7_OPROFILE *om, uint8_t *arr )
 
   return eslOK;
 }
+
+
+/* Function:  p7_oprofile_GetFwdEmissionScoreArray()
+ * Synopsis:  Retrieve Fwd (float) residue emission scores from an optimized
+ *            profile into an array
+ *
+ * Purpose:   Extract an implicitly 2D array of 32-bit float Fwd residue
+ *            emission scores from an optimized profile <om>. <arr> must
+ *            be allocated by the calling function to be of size
+ *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
+ *            is done as  [om->abc->Kp * i +  c ] for character c at
+ *            position i.
+ *
+ *            In SIMD implementations, the residue scores are striped
+ *            and interleaved, making them somewhat difficult to
+ *            directly access.
+ *
+ * Args:      <om>   - optimized profile, containing transition information
+ *            <arr>  - preallocated array into which scores will be placed
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    (no abnormal error conditions)
+ */
+int
+p7_oprofile_GetFwdEmissionScoreArray(const P7_OPROFILE *om, float *arr )
+{
+  int x, q, z, k;
+  union { __m128 v; float f[4]; } tmp; /* used to align and read simd minivectors               */
+  int      M   = om->M;    /* length of the query                                          */
+  int      K   = om->abc->Kp;
+  int      nq  = p7O_NQF(M);     /* segment length; total # of striped vectors needed            */
+  int cell_cnt = (om->M + 1) * K;
+
+  for (x = 0; x < K; x++) {
+      for (q = 0, k = 1; q < nq; q++, k++) {
+        tmp.v = esl_sse_logf(om->rfv[x][q]);
+        for (z = 0; z < 4; z++)
+          if (  (K * (k+z*nq) + x) < cell_cnt)
+            arr[ K * (k+z*nq) + x ] = tmp.f[z];
+      }
+  }
+
+  return eslOK;
+}
+
+/* Function:  p7_oprofile_GetFwdEmissionArray()
+ * Synopsis:  Retrieve Fwd (float) residue emission values from an optimized
+ *            profile into an array
+ *
+ * Purpose:   Extract an implicitly 2D array of 32-bit float Fwd residue
+ *            emission values from an optimized profile <om>, converting
+ *            back to emission values based on the background. <arr> must
+ *            be allocated by the calling function to be of size
+ *            ( om->abc->Kp * ( om->M  + 1 )), and indexing into the array
+ *            is done as  [om->abc->Kp * i +  c ] for character c at
+ *            position i.
+ *
+ *            In SIMD implementations, the residue scores are striped
+ *            and interleaved, making them somewhat difficult to
+ *            directly access.
+ *
+ * Args:      <om>   - optimized profile, containing transition information
+ *            <bg>   - background frequencies
+ *            <arr>  - preallocated array into which scores will be placed
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    (no abnormal error conditions)
+ */
+int
+p7_oprofile_GetFwdEmissionArray(const P7_OPROFILE *om, P7_BG *bg, float *arr )
+{
+  int x, q, z, k;
+  union { __m128 v; float f[4]; } tmp; /* used to align and read simd minivectors               */
+  int      M   = om->M;    /* length of the query                                          */
+  int      Kp  = om->abc->Kp;
+  int      K   = om->abc->K;
+  int      nq  = p7O_NQF(M);     /* segment length; total # of striped vectors needed            */
+  int cell_cnt = (om->M + 1) * Kp;
+
+  for (x = 0; x < K; x++) {
+      for (q = 0, k = 1; q < nq; q++, k++) {
+        tmp.v = om->rfv[x][q];
+        for (z = 0; z < 4; z++)
+          if (  (Kp * (k+z*nq) + x) < cell_cnt)
+            arr[ Kp * (k+z*nq) + x ] = tmp.f[z] * bg->f[x];
+      }
+  }
+
+  //degeneracy emissions for each position
+  for (x = 0; x <= M; x++)
+    esl_abc_FExpectScVec(om->abc, arr+Kp*x, bg->f);
+
+  return eslOK;
+}
+
+
 /*------------ end, conversions from P7_OPROFILE ------------------*/
 
 
@@ -1821,13 +2159,13 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  *   
- * SVN $Id: p7_oprofile.c 3960 2012-03-22 21:42:50Z wheelert $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/impl_sse/p7_oprofile.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
