@@ -2,7 +2,7 @@
  * routines: structures, declarations, and macros.
  * 
  * SRE, Sun Nov 25 11:23:02 2007
- * SVN $Id: impl_vmx.h 4103 2012-06-24 02:09:43Z wheelert $
+ * SVN $Id$
  */
 #ifndef P7_IMPL_VMX_INCLUDED
 #define P7_IMPL_VMX_INCLUDED
@@ -244,207 +244,9 @@ p7_omx_FSetMDI(const P7_OMX *ox, int s, int i, int k, float val)
 }
   
 
-/*****************************************************************
- * 3. FM-index
- *****************************************************************/
-
-/* Effectively global variables, to be initialized once in fm_initConfig(),
- * then passed around among threads to avoid recomputing them
- *
- * When allocated, must be 16-byte aligned, and all vector elements
- * must precede other types
- */
-typedef struct {
-  /* mask arrays, and 16-byte-offsets into them */
-  vector unsigned char  *fm_masks_mem;
-  vector unsigned char  *fm_masks_v;
-  vector unsigned char  *fm_reverse_masks_mem;
-  vector unsigned char  *fm_reverse_masks_v;
-  vector unsigned char  *fm_chars_mem;
-  vector unsigned char  *fm_chars_v;
-
-  /*various precomputed vectors*/
-  vector unsigned char  fm_allones_v;
-  vector unsigned char  fm_zeros_v;
-  vector unsigned char  fm_neg128_v;
-  vector unsigned char  fm_m0f;  //00 00 11 11
-  vector unsigned char  fm_m01;  //01 01 01 01
-  vector unsigned char  fm_m11;  //00 00 00 11
-  vector unsigned char  fm_one;  //value of 1 in each byte, used for shifting
-  vector unsigned char  fm_two;  //value of 2 in each byte, used for shifting
-  vector unsigned char  fm_four; //value of 4 in each byte, used for shifting
-
-
-  /* no non-vector elements above this line */
-
-
-  /*suffix-array mask and offset values*/
-  int maskSA;
-  int shiftSA;
-
-  /*counter, to compute FM-index speed*/
-  int occCallCnt;
-
-  /*bounding cutoffs*/
-  int max_depth;
-  int neg_len_limit;
-  int consec_pos_req;
-  float score_ratio_req;
-  int msv_length;
-  float max_scthreshFM;
-
-  /*pointer to FM-index metadata*/
-  FM_METADATA *meta;
-
-} FM_CFG;
-
-//used to convert from a byte array to a vec
-typedef union {
-        uint8_t bytes[16];
-        int ints[4];
-        vector signed char   i8;
-        vector unsigned char u8;
-        vector signed int    i32;
-        } byte_vec;
-
-/* Gather the sum of all counts in a 16x8-bit element into a single 32-bit
- *  element of the register (the last (3rd) element)
- */
-#define FM_GATHER_8BIT_COUNTS( in_v, out_v  ) do {\
-             out_v  = vec_sums(vec_sum4s(in_v, (vector signed int)cfg->fm_zeros_v), (vector signed int)cfg->fm_zeros_v);\
-  } while (0)
-
-
-
-/* Macro for altivec operations to turn 2-bit character values into 2-bit binary
- * (00 or 01) match/mismatch values representing occurrences of a character in a
- * 4-char-per-byte packed BWT.
- *
- * Typically followed by a call to FM_COUNT_SSE_4PACKED, possibly with a
- * mask in between to handle the case where we don't want to add over all
- * positions in the vector
- *
- * tmp_v and tmp2_v are used as temporary vectors throughout, and hold meaningless values
- * at the end
- *
- * xor(in_v, c_v)        : each 2-bit value will be 00 if a match, and non-0 if a mismatch
- * and(in_v, 01010101)   : look at the right bit of each 2-bit value,
- * sr(1)+and()         : look at the left bit of each 2-bit value,
- * or()                  : if either left bit or right bit is non-0, 01, else 00 (match is 00)
- *
- * subs()                : invert, so match is 01, mismatch is 00
- *
- */
-#define FM_MATCH_2BIT(in_v, c_v, a_v, b_v, out_v) do {\
-          a_v = vec_xor(in_v, c_v);\
-                \
-    b_v  = vec_and(a_v, cfg->fm_m01);\
-    a_v  = vec_sr(a_v,  cfg->fm_one);\
-    a_v  = vec_and(a_v, cfg->fm_m01);\
-    a_v  = vec_or(a_v, b_v);\
-    \
-    out_v  = vec_subs(cfg->fm_m01,a_v);\
-  } while (0)
-
-
-/*Macro for altivec operations to count bits produced by FM_MATCH_SSE_4PACKED
- *
- * tmp_v and tmp2_v are used as temporary vectors throughout, and hold meaningless values
- * at the end
- *
- * then add up the 2-bit values:
- * sr(4)+add()         : left 4 bits shifted right, added to right 4 bits
- *
- * sr(2)+and(00000011) : left 2 bits (value 0..2) shifted right, masked, so no other bits active
- * and(00000011)         : right 2 bits (value 0..2) masked so no other bits active
- *
- * final 2 add()s        : tack current counts on to already-tabulated counts.
- */
-#define FM_COUNT_2BIT(a_v, b_v, cnts_v) do {\
-        b_v  = vec_sr(a_v, cfg->fm_four);\
-        a_v  = (vector unsigned char)vec_adds( (vector signed char)a_v, (vector signed char)b_v);\
-        \
-        b_v  = vec_sr(a_v, cfg->fm_two);\
-        a_v  = vec_and(a_v,cfg->fm_m11);\
-        b_v  = vec_and(b_v,cfg->fm_m11);\
-        \
-        cnts_v = vec_adds(cnts_v, (vector signed char)a_v);\
-        cnts_v = vec_adds(cnts_v, (vector signed char)b_v);\
-  } while (0)
-
-
-
-/* Macro for altivec operations that turns a vector of 4-bit character values into
- * 2 vectors representing matches. Each byte in the input vector consists of
- * a left half (4 bits) and a right half (4 bits). The 16 left-halves produce
- * one vector, which contains all-1s for bytes in which the left half matches
- * the c_v character (and 0s if it doesn't), while the 16 right-halves produce
- * the other vector, again with each byte either all-1s or all-0s.
- *
- * The expectation is that FM_COUNT_4BIT will be called after this, to
- * turn these binary values into sums over a series of vectors. The macros
- * are split up to allow one end or other to be trimmed in the case that
- * counting is not expected to include the full vector.
- *
- * sr(4)+and() : capture the left 4-bit value   (need the mask because 16-bit shift leaves garbage in left-4-bit chunks)
- * and()         : capture the right 4-bit value
- *
- * cmpeq()x2     : test if both left and right == c.  For each, if ==c , value = 11111111 (-1)
- */
-#define FM_MATCH_4BIT(in_v, c_v, out1_v, out2_v) do {\
-    out1_v    = vec_sr(in_v, cfg->fm_four);\
-    out2_v    = vec_and(in_v, cfg->fm_m0f);\
-    out1_v    = vec_and(out1_v, cfg->fm_m0f);\
-    \
-    out1_v    = (vector unsigned char)vec_cmpeq(out1_v, c_v);\
-    out2_v    = (vector unsigned char)vec_cmpeq(out2_v, c_v);\
-  } while (0)
-
-
-/* Macro for altivec operations that turns a vector of 4-bit character values into
-* 2 vectors representing matches. Each byte in the input vector consists of
- * a left half (4 bits) and a right half (4 bits). The 16 left-halves produce
- * one vector, which contains all-1s for bytes in which the left half is less than
- * the c_v character (and 0s if it doesn't), while the 16 right-halves produce
- * the other vector, again with each byte either all-1s or all-0s.
- *
- * The expectation is that FM_COUNT_4BIT will be called after this, to
- * turn these binary values into sums over a series of vectors. The macros
- * are split up to allow one end or other to be trimmed in the case that
- * counting is not expected to include the full vector.
- *
- * sr(4)+and() : capture the left 4-bit value   (need the mask because 16-bit shift leaves garbage in left-4-bit chunks)
- * and()         : capture the right 4-bit value
- *
- * cmpeq()x2     : test if both left and right < c.  For each, if <c , value = 11111111 (-1)
- */
-#define FM_LT_4BIT(in_v, c_v, out1_v, out2_v) do {\
-    out1_v    = vec_sr(in_v, cfg->fm_four);\
-    out2_v    = vec_and(in_v, cfg->fm_m0f);\
-    out1_v    = vec_and(out1_v, cfg->fm_m0f);\
-    \
-    out1_v    = (vector unsigned char)vec_cmplt(out1_v, c_v);\
-    out2_v    = (vector unsigned char)vec_cmplt(out2_v, c_v);\
-  } while (0)
-
-
-/* Macro for SSE operations to add occurrence counts to the tally vector counts_v,
- * in the 4-bits-per-character case
- *
- * The expectation is that in[12]_v will contain bytes that are either
- *   00000000  =  0
- *  or
- *   11111111  = -1
- * so subtracting the value of the byte is the same as adding 0 or 1.
- */
-#define FM_COUNT_4BIT(in1_v, in2_v, counts_v) do {\
-    counts_v = vec_subs(counts_v, in1_v);\
-    counts_v = vec_subs(counts_v, in2_v);\
-  } while (0)
-
 
 /*****************************************************************
- * 4. Declarations of the external API.
+ * 3. Declarations of the external API.
  *****************************************************************/
 
 /* p7_omx.c */
@@ -468,6 +270,7 @@ extern void         p7_oprofile_Destroy(P7_OPROFILE *om);
 extern size_t       p7_oprofile_Sizeof(P7_OPROFILE *om);
 extern P7_OPROFILE *p7_oprofile_Copy(P7_OPROFILE *om);
 extern P7_OPROFILE *p7_oprofile_Clone(const P7_OPROFILE *om);
+extern int          p7_oprofile_UpdateFwdEmissionScores(P7_OPROFILE *om, P7_BG *bg, float *fwd_emissions, float *sc_arr);
 
 extern int          p7_oprofile_Convert(const P7_PROFILE *gm, P7_OPROFILE *om);
 extern int          p7_oprofile_ReconfigLength    (P7_OPROFILE *om, int L);
@@ -484,7 +287,9 @@ extern int          p7_profile_SameAsMF(const P7_OPROFILE *om, P7_PROFILE *gm);
 extern int          p7_profile_SameAsVF(const P7_OPROFILE *om, P7_PROFILE *gm);
 
 extern int          p7_oprofile_GetFwdTransitionArray(const P7_OPROFILE *om, int type, float *arr );
-extern int          p7_oprofile_GetMSVEmissionArray(const P7_OPROFILE *om, uint8_t *arr );
+extern int          p7_oprofile_GetSSVEmissionScoreArray(const P7_OPROFILE *om, uint8_t *arr );
+extern int          p7_oprofile_GetFwdEmissionScoreArray(const P7_OPROFILE *om, float *arr );
+extern int          p7_oprofile_GetFwdEmissionArray(const P7_OPROFILE *om, P7_BG *bg, float *arr );
 
 /* decoding.c */
 extern int p7_Decoding      (const P7_OPROFILE *om, const P7_OMX *oxf,       P7_OMX *oxb, P7_OMX *pp);
@@ -509,7 +314,7 @@ extern void p7_oprofile_DestroyBlock(P7_OM_BLOCK *block);
 
 /* msvfilter.c */
 extern int p7_MSVFilter    (const ESL_DSQ *dsq, int L, const P7_OPROFILE *om, P7_OMX *ox, float *ret_sc);
-extern int p7_MSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox, const P7_MSVDATA *msvdata, P7_BG *bg, double P, P7_HMM_WINDOWLIST *windowlist);
+extern int p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox, const P7_SCOREDATA *msvdata, P7_BG *bg, double P, P7_HMM_WINDOWLIST *windowlist);
 
 /* null2.c */
 extern int p7_Null2_ByExpectation(const P7_OPROFILE *om, const P7_OMX *pp, float *null2);
@@ -532,16 +337,10 @@ extern int p7_ViterbiFilter_longtarget(const ESL_DSQ *dsq, int L, const P7_OPROF
 /* vitscore.c */
 extern int p7_ViterbiScore (const ESL_DSQ *dsq, int L, const P7_OPROFILE *om, P7_OMX *ox, float *ret_sc);
 
-/*fm.c */
-extern int fm_initConfig      (FM_CFG *cfg, ESL_GETOPTS *go );
-extern int fm_destroyConfig   (FM_CFG *cfg );
-extern int fm_getOccCount     (const FM_DATA *fm, FM_CFG *cfg, int pos, uint8_t c);
-extern int fm_getOccCountLT   (const FM_DATA *fm, FM_CFG *cfg, int pos, uint8_t c, uint32_t *cnteq, uint32_t *cntlt);
-
 
 
 /*****************************************************************
- * 4. Implementation specific initialization
+ * 3. Implementation specific initialization
  *****************************************************************/
 static inline void
 impl_Init(void)
@@ -559,8 +358,8 @@ impl_ThreadInit(void)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
