@@ -15,7 +15,7 @@
  *   6. Copyright and license information
  * 
  * SRE, Sun Nov 25 11:26:48 2007 [Casa de Gatos]
- * SVN $Id: msvfilter.c 4198 2012-08-27 19:33:50Z nawrockie $
+ * SVN $Id$
  */
 #include "p7_config.h"
 
@@ -205,35 +205,35 @@ p7_MSVFilter(const ESL_DSQ *dsq, int L, const P7_OPROFILE *om, P7_OMX *ox, float
 /*------------------ end, p7_MSVFilter() ------------------------*/
 
 
-
 /* Function:  p7_SSVFilter_longtarget()
  * Synopsis:  Finds windows with SSV scores above some threshold (vewy vewy fast, in limited precision)
- * Incept:    TJW, Mon Feb 22 16:27:25 2010 [Janelia] - based on p7_MSVFilter
  *
+ * Purpose:   Calculates an approximation of the SSV (single ungapped diagonal)
+ *            score for regions of sequence <dsq> of length <L> residues, using
+ *            optimized profile <om>, and a preallocated one-row DP matrix <ox>,
+ *            and captures the positions at which such regions exceed the score
+ *            required to be significant in the eyes of the calling function,
+ *            which depends on the <bg> and <p> (usually p=0.02 for nhmmer).
+ *            Note that this variant performs only SSV computations, never
+ *            passing through the J state - the score required to pass SSV at
+ *            the default threshold (or less restrictive) is sufficient to
+ *            pass MSV in essentially all DNA models we've tested.
  *
- * Purpose:   Calculates an approximation of the MSV score for regions of
- *            sequence <dsq> of length <L> residues, using optimized profile
- *            <om>, and a preallocated one-row DP matrix <ox>, and captures
- *            the positions at which such regions exceed the score required
- *            to be significant in the eyes of the calling function (usually
- *            p=0.02).  Note that this variant performs only SSV computations,
- *            never passing through the J state. See comments in
- *            p7_MSVFilter_longtarget() for details
- *
- *            Rather than simply capturing positions at which a score threshold
- *            is reached, this function establishes windows around those
- *            high-scoring positions. p7_MSVFilter_longtarget then merges
- *            overlapping windows.
+ *            Above-threshold diagonals are captured into a preallocated list
+ *            <windowlist>. Rather than simply capturing positions at which a
+ *            score threshold is reached, this function establishes windows
+ *            around those high-scoring positions, using scores in <msvdata>.
+ *            These windows can be merged by the calling function.
  *
  *
  * Args:      dsq     - digital target sequence, 1..L
  *            L       - length of dsq in residues
  *            om      - optimized profile
  *            ox      - DP matrix
- *            sc_threshv - a precomputed vector containing the score threshold required for a window to be considered "high-scoring"
- *            starts  - RETURN: array of start positions for windows surrounding above-threshold areas
- *            ends    - RETURN: array of end positions for windows surrounding above-threshold areas
- *			  hit_cnt - RETURN: count of entries in the above two arrays
+ *            msvdata    - compact representation of substitution scores, for backtracking diagonals
+ *            bg         - the background model, required for translating a P-value threshold into a score threshold
+ *            P          - p-value below which a region is captured as being above threshold
+ *            windowlist - preallocated container for all hits (resized if necessary)
  *
  *
  * Note:      We misuse the matrix <ox> here, using only a third of the
@@ -247,8 +247,9 @@ p7_MSVFilter(const ESL_DSQ *dsq, int L, const P7_OPROFILE *om, P7_OMX *ox, float
  *
  * Throws:    <eslEINVAL> if <ox> allocation is too small.
  */
-static int
-p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox, const P7_MSVDATA *msvdata, uint8_t sc_thresh, vector unsigned char sc_threshv, P7_HMM_WINDOWLIST *windowlist)
+int
+p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox, const P7_SCOREDATA *ssvdata,
+                        P7_BG *bg, double P, P7_HMM_WINDOWLIST *windowlist)
 {
 
   vector unsigned char mpv;        /* previous row values                                       */
@@ -284,16 +285,45 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
   float ret_sc;
 
   union { vector unsigned char v; uint8_t b[16]; } u;
-  uint8_t *scores = NULL;
 
-  ESL_ALLOC(scores, ((om->abc->Kp)*Q) * sizeof(uint8_t) );
 
+  /*
+   * Computing the score required to let P meet the F1 prob threshold
+   * In original code, converting from a scaled int MSV
+   * score S (the score getting to state E) to a probability goes like this:
+   *  usc =  S - om->tec_b - om->tjb_b - om->base_b;
+   *  usc /= om->scale_b;
+   *  usc -= 3.0;
+   *  P = f ( (usc - nullsc) / eslCONST_LOG2 , mu, lambda)
+   * and we're computing the threshold usc, so reverse it:
+   *  (usc - nullsc) /  eslCONST_LOG2 = inv_f( P, mu, lambda)
+   *  usc = nullsc + eslCONST_LOG2 * inv_f( P, mu, lambda)
+   *  usc += 3
+   *  usc *= om->scale_b
+   *  S = usc + om->tec_b + om->tjb_b + om->base_b
+   *
+   *  Here, I compute threshold with length model based on max_length.  Doesn't
+   *  matter much - in any case, both the bg and om models will change with roughly
+   *  1 bit for each doubling of the length model, so they offset.
+   */
+  float nullsc;
+  float invP = esl_gumbel_invsurv(P, om->evparam[p7_MMU],  om->evparam[p7_MLAMBDA]);
+  vector unsigned char sc_threshv;               /* pushes value to saturation if it's above pthresh  */
+  int sc_thresh;
 
   /* Check that the DP matrix is ok for us. */
   if (Q > ox->allocQ16)  ESL_EXCEPTION(eslEINVAL, "DP matrix allocated too small");
   ox->M   = om->M;
 
 
+  p7_bg_SetLength(bg, om->max_length);
+  p7_oprofile_ReconfigMSVLength(om, om->max_length);
+  p7_bg_NullOne  (bg, dsq, om->max_length, &nullsc);
+
+  sc_thresh = (int) ceil( ( ( nullsc  + (invP * eslCONST_LOG2) + 3.0 )  * om->scale_b ) + om->base_b +  om->tec_b  + om->tjb_b  );
+  sc_threshv = esl_vmx_set_u8( (int8_t)sc_thresh - 1);
+
+
   /* Initialization. In offset unsigned arithmetic, -infinity is 0, and 0 is om->base.
    */
   biasv = esl_vmx_set_u8(om->bias_b);
@@ -336,22 +366,22 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
       rem_sc = -1;
       for (q = 0; q < Q; q++) {  /// Unpack and unstripe, so we can find the state that exceeded pthresh
           u.v = dp[q];
-          for (k = 0; k < 16; k++) scores[q+Q*k+1] = u.b[k]; // unstripe
-          dp[q] = vec_splat_u8(0); // while we're here ... this will cause values to get reset to xB in next dp iteration
-
-      }
-      for (k = 1; k <= om->M; k++) {
-          if (scores[k] >= sc_thresh && scores[k] > rem_sc) {
-            end = k;
-            rem_sc = scores[k];
+          for (k = 0; k < 16; k++) { // unstripe
+            //(q+Q*k+1) is the model position k at which the xE score is found
+            if (u.b[k] >= sc_thresh && u.b[k] > rem_sc && (q+Q*k+1) <= om->M) {
+              end = (q+Q*k+1);
+              rem_sc = u.b[k];
+            }
           }
+          dp[q] = vec_splat_u8(0); // while we're here ... this will cause values to get reset to xB in next dp iteration
       }
+
       //recover the diagonal that hit threshold
       start = end;
       target_end = target_start = i;
       sc = rem_sc;
       while (rem_sc > om->base_b - om->tjb_b - om->tbm_b) {
-        rem_sc -= om->bias_b -  msvdata->scores[start*om->abc->Kp + dsq[target_start]];
+        rem_sc -= om->bias_b -  ssvdata->ssv_scores[start*om->abc->Kp + dsq[target_start]];
         --start;
         --target_start;
         //if ( start == 0 || target_start==0)    break;
@@ -368,7 +398,7 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
       max_sc = sc;
       pos_since_max = 0;
       while (k<om->M && n<=L) {
-        sc += om->bias_b -  msvdata->scores[k*om->abc->Kp + dsq[n]];
+        sc += om->bias_b -  ssvdata->ssv_scores[k*om->abc->Kp + dsq[n]];
         if (sc >= max_sc) {
           max_sc = sc;
           max_end = n;
@@ -389,7 +419,18 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
       ret_sc /= om->scale_b;
       ret_sc -= 3.0; // that's ~ L \log \frac{L}{L+3}, for our NN,CC,JJ
 
-      p7_hmmwindow_new(windowlist, 0, target_start, -1, end, end-start+1 , ret_sc, fm_nocomplement );
+      p7_hmmwindow_new(  windowlist,
+                         0,                  // sequence_id; used in the FM-based filter, but not here
+                         target_start,       // position in the target at which the diagonal starts
+                         0,                  // position in the target fm_index at which diagonal starts;  not used here, just in FM-based filter
+                         end,                // position in the model at which the diagonal ends
+                         end-start+1 ,       // length of diagonal
+                         ret_sc,             // score of diagonal
+                         p7_NOCOMPLEMENT,    // always p7_NOCOMPLEMENT here;  varies in FM-based filter
+                         L
+                       );
+
+
 
       i = target_end; // skip forward
 
@@ -398,8 +439,6 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
 
   } /* end loop over sequence residues 1..L */
 
-  free(scores);
-
   return eslOK;
 
 
@@ -410,268 +449,6 @@ p7_SSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox,
 /*------------------ end, p7_SSVFilter_longtarget() ------------------------*/
 
 
-/* Function:  p7_MSVFilter_longtarget()
- * Synopsis:  Finds windows with MSV scores above some threshold (vewy vewy fast, in limited precision)
- * Incept:    TJW, Mon Feb 22 16:27:25 2010 [Janelia] - based on p7_MSVFilter
- *
- * Purpose:   Calculates an approximation of the MSV score for regions of
- *            sequence <dsq> of length <L> residues, using optimized profile
- *            <om>, and a preallocated one-row DP matrix <ox>, and captures
- *            the positions at which such regions exceed the score required
- *            to be significant in the eyes of the calling function (usually
- *            p=0.02).  Note that this will typically simply call
- *            p7_SSVFilter_longtarget(), as the score threshold is nearly
- *            always lower than the score required for a pass through the
- *            J-state to yield a positive score.
- *
- *            Rather than simply capturing positions at which a score threshold
- *            is reached, this function (and SSVFilter_longtarget) establishes
- *            windows around those high-scoring positions, and merges overlapping
- *            windows.
- *
- *
- * Args:      dsq        - digital target sequence, 1..L
- *            L          - length of dsq in residues
- *            om         - optimized profile
- *            ox         - DP matrix
- *            msvdata    - compact representation of substitution scores, for backtracking diagonals
- *            bg         - the background model, required for translating a P-value threshold into a score threshold
- *            P          - p-value below which a region is captured as being above threshold
- *            windowlist - RETURN: array of hit windows (start and end of diagonal) for the above-threshold areas
- *
- * Note:      We misuse the matrix <ox> here, using only a third of the
- *            first dp row, accessing it as <dp[0..Q-1]> rather than
- *            in triplets via <{MDI}MX(q)> macros, since we only need
- *            to store M state values. We know that if <ox> was big
- *            enough for normal DP calculations, it must be big enough
- *            to hold the MSVFilter calculation.
- *
- * Returns:   <eslOK> on success.
- *
- * Throws:    <eslEINVAL> if <ox> allocation is too small.
- */
-int
-p7_MSVFilter_longtarget(const ESL_DSQ *dsq, int L, P7_OPROFILE *om, P7_OMX *ox, const P7_MSVDATA *msvdata, P7_BG *bg, double P, P7_HMM_WINDOWLIST *windowlist)
-{
-#ifdef ALLOW_MSV
-  vector unsigned char mpv;        /* previous row values                                       */
-  vector unsigned char xEv;		   /* E state: keeps max for Mk->E as we go                     */
-  vector unsigned char xBv;		   /* B state: splatted vector of B[i-1] for B->Mk calculations */
-  vector unsigned char sv;		   /* temp storage of 1 curr row value in progress              */
-  vector unsigned char biasv;	   /* emission bias in a vector                                 */
-  uint8_t  xJ;                 /* special states' scores                                    */
-  int i;			           /* counter over sequence positions 1..L                      */
-  int j;
-  int q;			           /* counter over vectors 0..nq-1                              */
-  int Q        = p7O_NQB(om->M);   /* segment length: # of vectors                              */
-  vector unsigned char *dp  = ox->dpb[0];	   /* we're going to use dp[0][0..q..Q-1], not {MDI}MX(q) macros*/
-  vector unsigned char *rsc;			        /* will point at om->rbv[x] for residue x[i]                 */
-
-  vector unsigned char zerov;	   /* vector of zeros                                           */
-  vector unsigned char xEv_prev;            	 /* E state: keeps the max seen up to the current position i, for window overlap test.  Note that this will always just be xJv + tecv    */
-  vector unsigned char xJv;                     /* vector for states score                                   */
-  vector unsigned char tecv;                    /* vector for E->C  cost                                     */
-  vector unsigned char tjbmv;                    /* vector for [JN]->B->M move cost                                  */
-  vector unsigned char basev;                   /* offset for scores                                         */
-  vector unsigned char jthreshv;                /* pushes value to saturation if it's above pthresh  */
-  vector unsigned char tempv;                   /* work vector                                               */
-#endif
-
-
-  /*
-   * Computing the score required to let P meet the F1 prob threshold
-   * In original code, converting from a scaled int MSV
-   * score S (the score getting to state E) to a probability goes like this:
-   *  usc =  S - om->tec_b - om->tjb_b - om->base_b;
-   *  usc /= om->scale_b;
-   *  usc -= 3.0;
-   *  P = f ( (usc - nullsc) / eslCONST_LOG2 , mu, lambda)
-   * and we're computing the threshold usc, so reverse it:
-   *  (usc - nullsc) /  eslCONST_LOG2 = inv_f( P, mu, lambda)
-   *  usc = nullsc + eslCONST_LOG2 * inv_f( P, mu, lambda)
-   *  usc += 3
-   *  usc *= om->scale_b
-   *  S = usc + om->tec_b + om->tjb_b + om->base_b
-   *
-   *  Here, I compute threshold with length model based on max_length.  Doesn't
-   *  matter much - in any case, both the bg and om models will change with roughly
-   *  1 bit for each doubling of the length model, so they offset.
-   */
-  float nullsc;
-  float invP = esl_gumbel_invsurv(P, om->evparam[p7_MMU],  om->evparam[p7_MLAMBDA]);
-  vector unsigned char sc_threshv;               /* pushes value to saturation if it's above pthresh  */
-  int sc_thresh;
-
-  p7_bg_SetLength(bg, om->max_length);
-  p7_oprofile_ReconfigMSVLength(om, om->max_length);
-  p7_bg_NullOne  (bg, dsq, om->max_length, &nullsc);
-
-  sc_thresh = (int) ceil( ( ( nullsc  + (invP * eslCONST_LOG2) + 3.0 )  * om->scale_b ) + om->base_b +  om->tec_b  + om->tjb_b  );
-  sc_threshv = esl_vmx_set_u8( (int8_t)sc_thresh - 1);
-
-#ifdef ALLOW_MSV
-  int jthresh = om->base_b + om->tec_b + 1;  //+1 because the score of a pass through the model must be positive to contribute to MSV score
-  jthreshv = esl_vmx_set_u8( (int8_t)jthresh - 1);
-
-  if (force_ssv || (sc_thresh < jthresh)) {
-#endif
-
-	   p7_SSVFilter_longtarget(dsq, L, om, ox, msvdata, (uint8_t)sc_thresh, sc_threshv, windowlist);
-
-#ifdef ALLOW_MSV
-
-  } else {
-	  /*e.g. if base=190, tec=3, tjb=22 then a score of 217 would be required for a
-	   * second ssv-hit to improve the score of an earlier one. Usually,
-	   * sc_thresh < base. So only bother with msv if sc thresh is surprisingly
-	   *  high. This will require mu of something in the neighborhood of -6.4 bits
-	   *  (see TJW notes Mon Jun  7 10:19:34 EDT 2010 for details)
-	   */
-	  int      status;
-
-	  //used to track neighboring MSV peaks
-	  int first_peak = -1;
-	  int last_peak = -1;
-	  int hit_jthresh = FALSE;
-
-
-	  /* Check that the DP matrix is ok for us. */
-	  if (Q > ox->allocQ16)  ESL_EXCEPTION(eslEINVAL, "DP matrix allocated too small");
-	  ox->M   = om->M;
-
-	  /* Initialization. In offset unsigned arithmetic, -infinity is 0, and 0 is om->base.
-	   */
-	  biasv = esl_vmx_set_u8(om->bias_b);
-	  for (q = 0; q < Q; q++) dp[q] = vec_splat_u8(0);
-	  xJ   = 0;
-	  zerov = vec_splat_u8(0);
-	  xEv_prev = vec_splat_u8(0);
-
-	  basev = esl_vmx_set_u8((int8_t) om->base_b);
-	  tecv = esl_vmx_set_u8((int8_t) om->tec_b);
-	  tjbmv = esl_vmx_set_u8((int8_t) om->tjb_b + (int8_t) om->tbm_b);
-
-	  xJv = vec_subs(biasv, biasv);
-	  xBv = vec_subs(basev, tjbmv);
-
-
-	#if p7_DEBUGGING
-      if (ox->debugging)
-	    {
-	  unsigned char xB;
-	  vec_ste(xBv, 0, &xB);
-	  vec_ste(xJv, 0, &xJ);
-	  p7_omx_DumpMFRow(ox, 0, 0, 0, xJ, xB, xJ);
-	    }
-	#endif
-
-
-	  for (i = 1; i <= L; i++)
-	  {
-	      rsc = om->rbv[dsq[i]];
-	      xEv = vec_splat_u8(0);
-
-		  /* Right shifts by 1 byte. 4,8,12,x becomes x,4,8,12.
-		   * Because ia32 is littlendian, this means a left bit shift.
-		   * Zeros shift on automatically, which is our -infinity.
-		   */
-	      mpv = vec_sld(zerov, dp[Q-1], 15);
-		  for (q = 0; q < Q; q++)
-		  {
-			  /* Calculate new MMXo(i,q); don't store it yet, hold it in sv. */
-			  sv   = vec_max(mpv, xBv);
-			  sv   = vec_adds(sv, biasv);
-			  sv   = vec_subs(sv, *rsc);   rsc++;
-			  xEv  = vec_max(xEv, sv);
-
-			  mpv   = dp[q];   	  /* Load {MDI}(i-1,q) into mpv */
-			  dp[q] = sv;       	  /* Do delayed store of M(i,q) now that memory is usable */
-		  }
-
-
-		  if (last_peak >= 0 || vec_any_gt(xEv, jthreshv) )
-			  hit_jthresh = TRUE; //previously saw, or just saw an above-jthresh peak recently
-		  else
-			  hit_jthresh = FALSE;
-
-
-	      /* Now the "special" states, which start from Mk->E (->C, ->J->B)
-	       * Use rotates instead of shifts so when the last max has completed,
-	       * all elements of the simd register will contain the max value.
-	       */
-	      tempv = vec_sld(xEv, xEv, 1);
-	      xEv = vec_max(xEv, tempv);
-	      tempv = vec_sld(xEv, xEv, 2);
-	      xEv = vec_max(xEv, tempv);
-	      tempv = vec_sld(xEv, xEv, 4);
-	      xEv = vec_max(xEv, tempv);
-	      tempv = vec_sld(xEv, xEv, 8);
-	      xEv = vec_max(xEv, tempv);
-
-
-		  if (hit_jthresh) {
-			  //test if xE has increased this iteration.  If we go a long way without increasing, then we'll give up on the current window
-			  if ( vec_any_gt(xEv, xEv_prev) ) { // there was an increase. Establish peaks
-				  xEv_prev = xEv;
-				  if (first_peak == -1)
-					  first_peak = i;
-				  last_peak = i;
-
-				  //check if we've hit sc_thresh.  If so, reset values and add this region to the list of hits
-				  if (vec_any_gt(xEv, sc_threshv) ) {
-				    int start = first_peak - om->max_length + 1;
-				    int length = i-first_peak+1 + 2 * om->max_length;
-				    fm_newWindow(windowlist, 0, start, -1, -1, length , -1, fm_nocomplement );
-
-					  // got one peak.  Start scanning for the next one, as though starting from scratch
-					  first_peak = last_peak = -1;
-
-					  //reset xE, xJ, and xB
-					  xJv = vec_splat_u8(0);
-					  xBv = vec_subs(basev, tjbmv);
-					  for (q = 0; q < Q; q++)
-							dp[q] = vec_splat_u8(0); // this will cause values to get reset to xB in next iteration
-
-				  }
-			  } else if (i > last_peak + om->max_length) {
-				  //been a long time since the last above-jthresh peak was seen;
-				  // give up looking for a J-state-induced extension that will drive the score above sc_thresh
-
-				  first_peak = last_peak = -1;
-
-				  //Reset values as if the previous peak hadn't existed, retaining the growth of the current peak
-				  //See end of TW notes feb 25, 2010
-				  tempv = vec_subs(xEv_prev, basev);
-				  tempv = vec_subs(tempv, tecv);
-				  for (q = 0; q < Q; q++)
-					  dp[q] = vec_subs(dp[q], tempv);
-
-				  //reset xE, xJ, and xB
-				  xEv_prev = xEv = xJv = vec_splat_u8(0);
-				  xBv = vec_subs(basev, tjbmv);
-
-			  } //else ... nothing to do, just extend another step
-
-		  }
-
-	      xEv = vec_subs(xEv, tecv);
-	      xJv = vec_max(xJv,xEv);
-
-	      xBv = vec_max(basev, xJv);
-	      xBv = vec_subs(xBv, tjbmv);
-
-	  } /* end loop over sequence residues 1..L */
-  }
-
-#endif
-
-  return eslOK;
-
-
-}
-/*------------------ end, p7_MSVFilter_longtarget() ------------------------*/
-
-
 /*****************************************************************
  * 2. Benchmark driver.
  *****************************************************************/
@@ -1106,8 +883,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
