@@ -18,7 +18,7 @@
  *   6. Copyright and license information
  * 
  * SRE, Thu Jul 31 20:32:25 2008 [Casa de Gatos]
- * SVN $Id: vitfilter.c 4103 2012-06-24 02:09:43Z wheelert $
+ * SVN $Id$
  */
 #include "p7_config.h"
 
@@ -421,7 +421,7 @@ p7_ViterbiFilter_longtarget(const ESL_DSQ *dsq, int L, const P7_OPROFILE *om, P7
           for (z = 0; z < 8; z++)  { // unstripe
             if ( tmp.i[z] == xE && (q+Q*z+1) <= om->M) {
               // (q+Q*z+1) is the model position k at which the xE score is found
-              p7_hmmwindow_new(windowlist, 0, i, 0, (q+Q*z+1), 1, 0.0, fm_nocomplement );
+              p7_hmmwindow_new(windowlist, 0, i, i-1, (q+Q*z+1), 1, 0.0, p7_NOCOMPLEMENT, L );
             }
           }
           MMXo(q) = IMXo(q) = DMXo(q) = negInfv; //reset score to start search for next vit window.
@@ -913,8 +913,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
