@@ -1,7 +1,7 @@
 /* Miscellaneous summary statistics calculated for HMMs and profiles.
  * 
  * SRE, Fri May  4 11:43:20 2007 [Janelia]
- * SVN $Id: modelstats.c 4103 2012-06-24 02:09:43Z wheelert $
+ * SVN $Id$
  */
 
 #include "p7_config.h"
@@ -256,140 +256,10 @@ p7_hmm_CompositionKLDist(P7_HMM *hmm, P7_BG *bg, float *ret_KL, float **opt_avp)
 
 
 
-
-/* Function:  p7_hmm_GetSimpleRepeats()
- * Synopsis:  Print locations of regions identified as containing
- *            simple tandem repeats
- *
- * Purpose:   Scan the model <hmm>, searching for regions with simple
- *            tandem repeats, and return their locations in the model
- *            using <ranges>.
- *
- *            Tests for repeats of up to length <maxK>, and requires
- *            at least <min_rep> consecutive (ungapped) instances,
- *            where the average relative entropy (depends on <hmm>
- *            and <bg>) of the length-K repeat must exceed <relent>
- *            bits.
- *
- *            Basic idea: build a circular list for each length
- *            1..maxK. Suppose we're dealing with list of length
- *            K=3. Then node 1 will count the number of times
- *            each character is observed in the model's consensus
- *            at positions n*3+1 over the most recent k*min_rep
- *            states. Once k*min_rep states have been added to the
- *            node counts, compute avg relative entropy, and compare
- *            to threshold. If passes, that's a repeat seed (see
- *            below), otherwise, remove the count associated with
- *            position 1, and add the next position. Repeat,
- *            removing count for position 2 and adding for
- *            k*min_rep+1, then removing 3 and adding k*min_rep+2,
- *            and so on.
- *
- *            Once complete, ignore long seeds that overlap short
- *            seeds, then for each seed, attempt to extend by lining
- *            up consecutive length-K blocks on either side of the
- *            seed, allowing up to length-2 indels.
- *
- *
- * Returns:   <eslOK> on success.
- *
- * Throws:    <eslEMEM> on allocation error.
- */
-
-int
-p7_hmm_GetSimpleRepeats(P7_HMM *hmm, int maxK, int min_rep, int min_length, float relent_thresh, P7_HMM_WINDOWLIST *ranges)
-{
-  int    K   = hmm->abc->K;
-  int    i, j, k, n, x;
-  int  **counts;
-  int    pos;  // where we are in the circular list
-  int    status;
-  float  relent;
-  P7_BG *bg;
-  int    reps;
-
-
-  // can't hope to find a repeat under this definition
-  if (hmm->M < min_rep)
-    return eslOK;
-
-  if ((bg = p7_bg_Create(hmm->abc))              == NULL)  esl_fatal("failed to create null model");
-
-  // for each circular-list length
-  for (n=2; n<=maxK; n++) {
-
-    reps = min_rep;
-    //go to the next # of reps required to reach min-length with equal # chars from each node
-    if (n*reps < min_length) {
-      reps = ceil(min_length/n);
-    }
-
-
-    ESL_ALLOC(counts, n*sizeof(int*)) ;  // there are n nodes in the list
-
-    //establish count vector for each node
-    for (i=0; i<n; i++) {
-      ESL_ALLOC(counts[i], K*sizeof(int));
-      esl_vec_ISet(counts[i], K, 0);
-    }
-
-    pos = 0;
-    //for each model position
-    for (i = 1; i <= hmm->M; i++) {
-      // add to count vector
-      x = esl_vec_FArgMax(hmm->mat[i], hmm->abc->K);
-      counts[pos][x]++;  //TODO: could use emission probabilities instead, using  hmm->mat[k]
-      pos++;
-      pos %= n;
-
-      //if we've accrued enough counts, check relent
-      if (i >= n*reps  ) {
-        //that means go through each node, computing relent, add them up, then divide by # nodes
-        relent = 0.0;
-        for (j=0; j<n; j++) { //for each node
-
-          for (k=0; k<K; k++) {
-            float p       =  (float)(counts[j][k])/(float)reps;
-            float logodds = log((p+.000000001) / bg->f[k]);  //natural log  add that tiny amount to avoid log-of-zero trouble ... just means the definition of 0*(log(0)) = 0
-            relent += p * logodds / log(2);
-          }
-        }
-
-        if (relent/n > relent_thresh) {//avg relent
-          //printf ("%d : %.2f  (%d-mer)\n", i, rel_ent, n);
-          p7_hmmwindow_new(ranges, 0, i, 0, 0, n*reps, relent/n, 0);
-        }
-
-        //remove the first character of the preceding (n*min_rep), since it'll be replaced with a new one
-        x = esl_vec_FArgMax(hmm->mat[i - (n*reps) + 1], hmm->abc->K);
-        counts[pos][x]--;
-      }
-
-    }
-
-    for (i=0; i<n; i++)
-      if (counts[i] != NULL) free(counts[i]) ;
-    free(counts);
-
-  }
-
-  return eslOK;
-
- ERROR:
-
- if (counts != NULL) {
-   for (i=0; i<n; i++)
-     if (counts[i] != NULL) free(counts[i]) ;
-   free(counts);
- }
-  return status;
-}
-
-
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
