@@ -26,97 +26,6 @@
  * 1. The P7_ALIDISPLAY object
  *****************************************************************/
 
-/* Function:  p7_alidisplay_pickHqBounds()
- * Synopsis:  Compute conservative (high-quality) alignment boundaries
- *            based aligned posterior probabilities (APP).
- *
- * Purpose:   Each hit coming from the nhmmer pipeline brings with it
- *            an APP array (in ddef->mocc) that indicates the confidence
- *            nhmmer has that each position in the alignment is supposed
- *            to be aligned to *some* state of the model. There are
- *            typically only a few bases on either end that fall below
- *             a threshold of, say 80% or 90%, though in some cases, the
- *            length of such a low-quality terminal run can be pretty
- *            long.
- *
- *            This function a scans along from the left end to the right
- *            until a position i with mocc matching <thresh> is found, then:
- *            (a) if position i represents a positive-scoring match
- *                to the model, it scans back left until the first
- *                negative-scoring "mismatch" to the model at position j,
- *                and sets ihq (the hq start) to j+1;
- *            (b) if position i is a negative-scoring "mismatch", it
- *                scans to the right until the first positive-scoring
- *                match at position j, and sets ihq to j.
- *            The symmetric computation is done from the right end of APP
- *            to compute jhq.
- *
- *            The result of the function is then to update the ihq and jhq
- *            values for all hits in the <th> object.
- *
- *            This function assumes the hits are found in th->hit, i.e.
- *            that deduplication and sorting have already taken place
- *
- *            For details about offsets into mocc, see
- *            ~wheelert/notebook/2012/0203_app_hq_bounds/, (Feb 5)
- *
- * Args:      ddef     - object housing the mocc posterior probability array
- *            ad       - object holding the mline values used for the final edge
- *                       settlement and into which the hq-bounds will be stored
- *            tr       - trace, used in the position stepping
- *            z1       - envelope start
- *            z2       - envelope end
- *            thresh   - P-value threshold used in trimming
- *
- * Returns:   <eslOK> on success.
- */
-static int
-p7_alidisplay_pickHqBounds(P7_DOMAINDEF *ddef, P7_ALIDISPLAY *ad, const P7_TRACE *tr, int z1, int z2, float thresh)
-{
-  int z, i;    /* counter over positions in the alignment */
-
-  /*strange special case, should be gone with new (as of May 2012) domain definition machinery*/
-  if (z1==z2){
-    ad->hqto = ad->hqfrom = tr->i[z1];
-    return eslOK;
-  }
-
-  /*scan left to right to find left boundary*/
-  i = z1-2;
-  for (z = z1 ; z < z2; z++) {
-    if ( tr->st[z] != p7T_D ) {  //skip delete states in the trace
-      if (ddef->mocc[i++] >= thresh) break;
-    }
-  }
-  if (z==z2) { /* no confidence!, set values to 0 */
-    ad->hqto = ad->hqfrom = 0;
-    return eslOK;
-  }
-  //walk left or right to add good terminal positions or trim bad ones.
-  i=z-z1;
-  while (i>0 && ad->mline[i] != ' ')   i--; // positive score, so move out to the left
-  while (ad->mline[i] == ' ')          i++; // negative score, move back to right
-  ad->hqfrom = tr->i[z1+i];
-
-
-
-  /*scan right to left to find right boundary*/
-  i = ddef->L + 1 - (tr->N - z2 - 3);
-  for (z = z2 ; z > z1; z--) {
-    if ( tr->st[z] != p7T_D ) {  //skip delete states in the trace
-      if (ddef->mocc[i--] >= thresh) break;
-    }
-  }
-
-  //walk left or right to add good terminal positions or trim bad ones.
-  i=z-z1;
-  while (i<z2-z1 && ad->mline[i] != ' ')   i++; // positive score, so move out to the right
-  while (ad->mline[i] == ' ')              i--; // negative score, move back to left
-  ad->hqto = tr->i[z1+i];
-
-
-  return eslOK;
-}
 
 /* Function:  p7_alidisplay_Create()
  * Synopsis:  Create an alignment display, from trace and oprofile.
@@ -139,6 +48,7 @@ p7_alidisplay_pickHqBounds(P7_DOMAINDEF *ddef, P7_ALIDISPLAY *ad, const P7_TRACE
  *            which    - domain number, 0..tr->ndom-1
  *            om       - optimized profile (query)
  *            sq       - digital sequence (target)
+ *            ntsq     - text sequence (original nucleotide target in the case of translated search)
  *            ddef_app - optional posterior prob alignment line; only nhmmer sends a not-NULL value
  *
  * Returns:   <eslOK> on success.
@@ -147,7 +57,7 @@ p7_alidisplay_pickHqBounds(P7_DOMAINDEF *ddef, P7_ALIDISPLAY *ad, const P7_TRACE
  *            in the data.
  */
 P7_ALIDISPLAY *
-p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const ESL_SQ *sq, P7_DOMAINDEF *ddef_app)
+p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const ESL_SQ *sq, const ESL_SQ *ntsq)
 {
   P7_ALIDISPLAY *ad       = NULL;
   char          *Alphabet = om->abc->sym;
@@ -157,6 +67,9 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
   int            hmm_namelen, hmm_acclen, hmm_desclen;
   int            sq_namelen,  sq_acclen,  sq_desclen;
   int            status;
+  char           n1,n2,n3;
+  int            j;
+  ESL_SQ         *ntorfseqtxt = NULL;
   
   /* First figure out which piece of the trace (from first match to last match) 
    * we're going to represent, and how big it is.
@@ -188,18 +101,19 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
    * bookkeeping.  
    */
   n = (z2-z1+2) * 3;                     /* model, mline, aseq mandatory         */
+
   if (om->rf[0]  != 0)    n += z2-z1+2;  /* optional reference line              */
   if (om->mm[0]  != 0)    n += z2-z1+2;  /* optional reference line              */
   if (om->cs[0]  != 0)    n += z2-z1+2;  /* optional structure line              */
   if (tr->pp     != NULL) n += z2-z1+2;  /* optional posterior prob line         */
-  if (ddef_app       != NULL) n += z2-z1+2;  /* optional posterior prob alignment line         */
   hmm_namelen = strlen(om->name);                           n += hmm_namelen + 1;
   hmm_acclen  = (om->acc  != NULL ? strlen(om->acc)  : 0);  n += hmm_acclen  + 1;
   hmm_desclen = (om->desc != NULL ? strlen(om->desc) : 0);  n += hmm_desclen + 1;
-  sq_namelen  = strlen(sq->name);                           n += sq_namelen  + 1;
+
+  sq_namelen  = strlen(sq->name);                           n += sq_namelen  + 1;	  
   sq_acclen   = strlen(sq->acc);                            n += sq_acclen   + 1; /* sq->acc is "\0" when unset */
   sq_desclen  = strlen(sq->desc);                           n += sq_desclen  + 1; /* same for desc              */
-  
+ 
   ESL_ALLOC(ad, sizeof(P7_ALIDISPLAY));
   ad->mem = NULL;
 
@@ -207,13 +121,14 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
   ad->memsize = sizeof(char) * n;
   ESL_ALLOC(ad->mem, ad->memsize);
   if (om->rf[0]  != 0) { ad->rfline = ad->mem + pos; pos += z2-z1+2; } else { ad->rfline = NULL; }
-  if (om->mm[0]  != 0) { ad->mmline = ad->mem + pos; pos += z2-z1+2; } else { ad->mmline = NULL; }
+  //if (om->mm[0]  != 0) { ad->mmline = ad->mem + pos; pos += z2-z1+2; } else { ad->mmline = NULL; }
+  ad->mmline = NULL;
   if (om->cs[0]  != 0) { ad->csline = ad->mem + pos; pos += z2-z1+2; } else { ad->csline = NULL; }
   ad->model   = ad->mem + pos;  pos += z2-z1+2;
   ad->mline   = ad->mem + pos;  pos += z2-z1+2;
   ad->aseq    = ad->mem + pos;  pos += z2-z1+2;
+
   if (tr->pp != NULL)  { ad->ppline = ad->mem + pos;  pos += z2-z1+2;} else { ad->ppline = NULL; }
-  if (ddef_app != NULL)    { ad->appline = ad->mem + pos;  pos += z2-z1+2;} else { ad->appline = NULL; }
   ad->hmmname = ad->mem + pos;  pos += hmm_namelen +1;
   ad->hmmacc  = ad->mem + pos;  pos += hmm_acclen +1;
   ad->hmmdesc = ad->mem + pos;  pos += hmm_desclen +1;
@@ -224,6 +139,7 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
   strcpy(ad->hmmname, om->name);
   if (om->acc  != NULL) strcpy(ad->hmmacc,  om->acc);  else ad->hmmacc[0]  = 0;
   if (om->desc != NULL) strcpy(ad->hmmdesc, om->desc); else ad->hmmdesc[0] = 0;
+
   strcpy(ad->sqname,  sq->name);
   strcpy(ad->sqacc,   sq->acc);
   strcpy(ad->sqdesc,  sq->desc);
@@ -232,11 +148,11 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
   ad->hmmfrom = tr->k[z1];
   ad->hmmto   = tr->k[z2];
   ad->M       = om->M;
+	
   ad->sqfrom  = tr->i[z1];
-  ad->sqto    = tr->i[z2];
+  ad->sqto    = tr->i[z2];		 
   ad->L       = sq->n;
 
-
   /* optional rf line */
   if (ad->rfline != NULL) {
     for (z = z1; z <= z2; z++) ad->rfline[z-z1] = ((tr->st[z] == p7T_I) ? '.' : om->rf[tr->k[z]]);
@@ -261,16 +177,7 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
     ad->ppline[z-z1] = '\0';
   }
 
-
-  /* optional app line */
-  if (ddef_app != NULL && ddef_app->show_app) {
-    for (z = z1, s = z-2  ; z <= z2; z++)  ad->appline[z-z1] = ( (tr->st[z] == p7T_D) ? '.' : p7_alidisplay_EncodeAliPostProb(ddef_app->mocc[s++], ddef_app->app_hi, ddef_app->app_med, ddef_app->app_lo ));
-    ad->appline[z-z1] = '\0';
-  } else {
-    ad->appline = NULL;
-  }
-
-
+  
   /* mandatory three alignment display lines: model, mline, aseq */
   for (z = z1; z <= z2; z++) 
     {
@@ -298,6 +205,7 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
         ad->model [z-z1] = om->consensus[k];
         ad->mline [z-z1] = ' ';
         ad->aseq  [z-z1] = '-';
+
         break;
 
       default: ESL_XEXCEPTION(eslEINVAL, "invalid state in trace: not M,D,I");
@@ -308,14 +216,9 @@ p7_alidisplay_Create(const P7_TRACE *tr, int which, const P7_OPROFILE *om, const
   ad->aseq  [z2-z1+1] = '\0';
   ad->N = z2-z1+1;
 
+  esl_sq_Destroy(ntorfseqtxt);  
 
-  if (ddef_app != NULL) {
-    /* pick APP-based hq bounds.  This is done after mline is created,
-     * as mline is used in the trimming process
-     */
-    p7_alidisplay_pickHqBounds(ddef_app, ad, tr, z1, z2, ddef_app->app_med);
-  }
-
+	
   return ad;
 
  ERROR:
@@ -342,7 +245,7 @@ p7_alidisplay_Clone(const P7_ALIDISPLAY *ad)
   int status;
 
   ESL_ALLOC(ad2, sizeof(P7_ALIDISPLAY));
-  ad2->rfline  = ad2->mmline = ad2->csline = ad2->model   = ad2->mline  = ad2->aseq = ad2->ppline = NULL;
+  ad2->rfline  = ad2->mmline = ad2->csline = ad2->model   = ad2->mline  = ad2->aseq = ad2->ntseq = ad2->ppline = NULL;
   ad2->hmmname = ad2->hmmacc = ad2->hmmdesc = NULL;
   ad2->sqname  = ad2->sqacc  = ad2->sqdesc  = NULL;
   ad2->mem     = NULL;
@@ -360,6 +263,7 @@ p7_alidisplay_Clone(const P7_ALIDISPLAY *ad)
       ad2->model  = ad2->mem + (ad->model  - ad->mem);
       ad2->mline  = ad2->mem + (ad->mline  - ad->mem);
       ad2->aseq   = ad2->mem + (ad->aseq   - ad->mem);
+      ad2->ntseq  = (ad->ntseq  ? ad2->mem + (ad->ntseq  - ad->mem) : NULL );
       ad2->ppline = (ad->ppline ? ad2->mem + (ad->ppline - ad->mem) : NULL );
       ad2->N      = ad->N;
 
@@ -385,6 +289,7 @@ p7_alidisplay_Clone(const P7_ALIDISPLAY *ad)
       if ( esl_strdup(ad->model,  -1, &(ad2->model))  != eslOK) goto ERROR;
       if ( esl_strdup(ad->mline,  -1, &(ad2->mline))  != eslOK) goto ERROR;
       if ( esl_strdup(ad->aseq,   -1, &(ad2->aseq))   != eslOK) goto ERROR;
+      if ( esl_strdup(ad->ntseq,  -1, &(ad2->ntseq))  != eslOK) goto ERROR;
       if ( esl_strdup(ad->ppline, -1, &(ad2->ppline)) != eslOK) goto ERROR;
       ad2->N = ad->N;
 
@@ -434,6 +339,7 @@ p7_alidisplay_Sizeof(const P7_ALIDISPLAY *ad)
   if (ad->csline) n += ad->N+1; 
   if (ad->ppline) n += ad->N+1; 
   n += 3 * (ad->N+1);	          /* model, mline, aseq */
+  if (ad->ntseq)  n += (3 * ad->N) + 1;	          /* ntseq */
   n += 1 + strlen(ad->hmmname);	  
   n += 1 + strlen(ad->hmmacc);	  /* optional acc, desc fields: when not present, just "" ("\0") */
   n += 1 + strlen(ad->hmmdesc);
@@ -481,6 +387,7 @@ p7_alidisplay_Serialize(P7_ALIDISPLAY *ad)
   memcpy(ad->mem+pos, ad->model,  ad->N+1); free(ad->model); ad->model = ad->mem+pos; pos += ad->N+1; 
   memcpy(ad->mem+pos, ad->mline,  ad->N+1); free(ad->mline); ad->mline = ad->mem+pos; pos += ad->N+1; 
   memcpy(ad->mem+pos, ad->aseq,   ad->N+1); free(ad->aseq);  ad->aseq  = ad->mem+pos; pos += ad->N+1; 
+  if (ad->ntseq)  { memcpy(ad->mem+pos, ad->ntseq, (3*ad->N)+1); free(ad->ntseq);  ad->ntseq  = ad->mem+pos; pos += (3*ad->N)+1; } 
   if (ad->ppline) { memcpy(ad->mem+pos, ad->ppline, ad->N+1); free(ad->ppline); ad->ppline = ad->mem+pos;  pos += ad->N+1; }
   n = 1 + strlen(ad->hmmname);  memcpy(ad->mem + pos, ad->hmmname, n); free(ad->hmmname); ad->hmmname = ad->mem+pos; pos += n;
   n = 1 + strlen(ad->hmmacc);   memcpy(ad->mem + pos, ad->hmmacc,  n); free(ad->hmmacc);  ad->hmmacc  = ad->mem+pos; pos += n;
@@ -529,6 +436,7 @@ p7_alidisplay_Deserialize(P7_ALIDISPLAY *ad)
   ESL_ALLOC(ad->model, sizeof(char) * ad->N+1); memcpy(ad->model, ad->mem+pos, ad->N+1); pos += ad->N+1; 
   ESL_ALLOC(ad->mline, sizeof(char) * ad->N+1); memcpy(ad->mline, ad->mem+pos, ad->N+1); pos += ad->N+1; 
   ESL_ALLOC(ad->aseq,  sizeof(char) * ad->N+1); memcpy(ad->aseq,  ad->mem+pos, ad->N+1); pos += ad->N+1; 
+  if (ad->ntseq)  { ESL_ALLOC(ad->ntseq,  sizeof(char) * (3*ad->N)+1); memcpy(ad->ntseq,  ad->mem+pos, (3*ad->N)+1); pos += (3*ad->N)+1; }
   if (ad->ppline) { ESL_ALLOC(ad->ppline, sizeof(char) * ad->N+1); memcpy(ad->ppline, ad->mem+pos, ad->N+1); pos += ad->N+1; }
   n = 1 + strlen(ad->mem+pos);  ESL_ALLOC(ad->hmmname,  sizeof(char) * n); memcpy(ad->hmmname,  ad->mem+pos, n); pos += n;
   n = 1 + strlen(ad->mem+pos);  ESL_ALLOC(ad->hmmacc,   sizeof(char) * n); memcpy(ad->hmmacc,   ad->mem+pos, n); pos += n;
@@ -552,6 +460,7 @@ p7_alidisplay_Deserialize(P7_ALIDISPLAY *ad)
   if (ad->model != ad->mem+pos) { free(ad->model); ad->model = ad->mem+pos; }  pos += ad->N+1; 
   if (ad->mline != ad->mem+pos) { free(ad->mline); ad->mline = ad->mem+pos; }  pos += ad->N+1; 
   if (ad->aseq  != ad->mem+pos) { free(ad->aseq);  ad->aseq  = ad->mem+pos; }  pos += ad->N+1; 
+  if (ad->ntseq)  { if (ad->ntseq  != ad->mem+pos) { free(ad->ntseq);  ad->ntseq  = ad->mem+pos; }  pos += (3*ad->N)+1; }
   if (ad->ppline) { if (ad->ppline != ad->mem+pos) { free(ad->ppline); ad->ppline = ad->mem+pos; }  pos += ad->N+1; }
 
   n = 1 + strlen(ad->hmmname);  if (ad->hmmname != ad->mem+pos) { free(ad->hmmname); ad->hmmname = ad->mem+pos; }  pos += n;
@@ -583,6 +492,7 @@ p7_alidisplay_Destroy(P7_ALIDISPLAY *ad)
       if (ad->model)   free(ad->model);
       if (ad->mline)   free(ad->mline);
       if (ad->aseq)    free(ad->aseq);
+      if (ad->ntseq)   free(ad->ntseq);
       if (ad->ppline)  free(ad->ppline);
       if (ad->hmmname) free(ad->hmmname);
       if (ad->hmmacc)  free(ad->hmmacc);
@@ -609,29 +519,6 @@ integer_textwidth(long n)
   return w;
 }
 
-/* Function:  p7_alidisplay_EncodeAliPostProb()
- * Synopsis:  Convert the posterior probability of a position aligning
- *            toa core model state into a char code.
- *
- * Purpose:   Convert the posterior probability <p> to
- *            a character code used to visually assess the 
- *            confidence that a position should be included 
- *            at the boundary of an alignment.
- *
- *            Returns a character from the class [HML-],
- *            with 'H' if <p>   >= <hi>
- *                 'M' if <hi>  >  <p>   >= <med>
- *                 'L' if <med> >  <p>   >= <lo>
- *                 '-' if <p> < <lo>
- *
- * Returns:   the encoded character.
- */
-char
-p7_alidisplay_EncodeAliPostProb(float p, float hi, float med, float lo)
-{
-  //return (p >= 0.97) ? '*' : (p<0.7? '0' : (char) ((p-.67) * (100.0/3)) + '0');
-  return (p >= hi ) ? 'H' : (p>med ? 'M' : (p>lo ? 'L' : '-'));
-}
 
 /* Function:  p7_alidisplay_EncodePostProb()
  * Synopsis:  Convert a posterior probability to a char code.
@@ -680,7 +567,6 @@ p7_alidisplay_DecodePostProb(char pc)
 }
 
 
-
 /* Function:  p7_alidisplay_Print()
  * Synopsis:  Human readable output of <P7_ALIDISPLAY>
  *
@@ -702,7 +588,36 @@ p7_alidisplay_DecodePostProb(char pc)
  * Throws:    <eslEWRITE> on write error, such as filling the disk.
  */
 int
-p7_alidisplay_Print(FILE *fp, P7_ALIDISPLAY *ad, int min_aliwidth, int linewidth, int show_accessions)
+p7_alidisplay_Print(FILE *fp, P7_ALIDISPLAY *ad, int min_aliwidth, int linewidth, P7_PIPELINE *pli)
+{
+   int status;
+   if ((status = p7_nontranslated_alidisplay_Print(fp, ad, min_aliwidth, linewidth, pli->show_accessions)) != eslOK) return status;
+
+	return status;
+}
+
+/* Function:  p7_nontranslated_alidisplay_Print()
+ * Synopsis:  Human readable output of <P7_ALIDISPLAY>
+ *
+ * Purpose:   Prints alignment <ad> to stream <fp>.
+ *            
+ *            Put at least <min_aliwidth> alignment characters per
+ *            line; try to make lines no longer than <linewidth>
+ *            characters, including name, coords, and spacing.  The
+ *            width of lines may exceed <linewidth>, if that's what it
+ *            takes to put a name, coords, and <min_aliwidth>
+ *            characters of alignment on a line.
+ *            
+ *            As a special case, if <linewidth> is negative or 0, then
+ *            alignments are formatted in a single block of unlimited
+ *            line length.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on write error, such as filling the disk.
+ */
+int
+p7_nontranslated_alidisplay_Print(FILE *fp, P7_ALIDISPLAY *ad, int min_aliwidth, int linewidth, int show_accessions)
 {
   char *buf          = NULL;
   char *show_hmmname = NULL;
@@ -759,7 +674,6 @@ p7_alidisplay_Print(FILE *fp, P7_ALIDISPLAY *ad, int min_aliwidth, int linewidth
       else        { strncpy(buf, ad->aseq+pos, aliwidth); if (fprintf(fp, "  %*s %*s %s %*s\n",    namewidth, show_seqname, coordwidth, "-", buf, coordwidth, "-") < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "alignment display write failed");  }
 
       if (ad->ppline != NULL)  { strncpy(buf, ad->ppline+pos, aliwidth);  if (fprintf(fp, "  %*s %s PP\n", namewidth+coordwidth+1, "", buf)  < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "alignment display write failed");  }
-      if (ad->appline != NULL) { strncpy(buf, ad->appline+pos, aliwidth); if (fprintf(fp, "  %*s %s APP\n", namewidth+coordwidth+1, "", buf) < 0) ESL_XEXCEPTION_SYS(eslEWRITE, "alignment display write failed");  }
 
       k1 += nk;
       if   (ad->sqfrom < ad->sqto)  i1 += ni;
@@ -774,7 +688,6 @@ p7_alidisplay_Print(FILE *fp, P7_ALIDISPLAY *ad, int min_aliwidth, int linewidth
   return status;
 }  
 
-
 /* Function:  p7_alidisplay_Backconvert()
  * Synopsis:  Convert an alidisplay to a faux trace and subsequence.
  *
@@ -968,6 +881,7 @@ p7_alidisplay_Compare(const P7_ALIDISPLAY *ad1, const P7_ALIDISPLAY *ad2)
   if (esl_strcmp(ad1->model,   ad2->model)   != eslOK) return eslFAIL;
   if (esl_strcmp(ad1->mline,   ad2->mline)   != eslOK) return eslFAIL;
   if (esl_strcmp(ad1->aseq,    ad2->aseq)    != eslOK) return eslFAIL;
+  if (esl_strcmp(ad1->ntseq,   ad2->ntseq)   != eslOK) return eslFAIL;
   if (esl_strcmp(ad1->ppline,  ad2->ppline)  != eslOK) return eslFAIL;
   if (ad1->N != ad2->N)                                return eslFAIL;
 
@@ -1171,7 +1085,7 @@ create_faux_alidisplay(ESL_RANDOMNESS *rng, int N, P7_ALIDISPLAY **ret_ad)
   guidestring[N] = '\0';
 
   ESL_ALLOC(ad, sizeof(P7_ALIDISPLAY));
-  ad->rfline  = ad->mmline = ad->csline = ad->model   = ad->mline  = ad->aseq = ad->ppline = NULL;
+  ad->rfline  = ad->mmline = ad->csline = ad->model   = ad->mline  = ad->aseq = ad->ntseq = ad->ppline = NULL;
   ad->hmmname = ad->hmmacc = ad->hmmdesc = NULL;
   ad->sqname  = ad->sqacc  = ad->sqdesc  = NULL;
   ad->mem     = NULL;
@@ -1455,7 +1369,7 @@ main(int argc, char **argv)
   p7_pli_NewSeq(pli, sq);
   p7_bg_SetLength(bg, sq->n);
   p7_oprofile_ReconfigLength(om, sq->n);
-  p7_Pipeline(pli, om, bg, sq, hitlist);
+  p7_Pipeline(pli, om, bg, sq, NULL, hitlist);
 
   if (hitlist->N == 0) { p7_Fail("target sequence doesn't hit"); }
 
@@ -1480,13 +1394,15 @@ main(int argc, char **argv)
 #endif /*p7ALIDISPLAY_EXAMPLE*/
 
 
-/*****************************************************************
+/****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
- *****************************************************************/
-
+ *
+ * SVN $Id$
+ * SVN $URL$
+ ****************************************************************/
