@@ -70,7 +70,7 @@ p7_builder_Create(const ESL_GETOPTS *go, const ESL_ALPHABET *abc)
       bld->arch_strategy = p7_ARCH_FAST;
       bld->wgt_strategy  = p7_WGT_PB;
       bld->effn_strategy = p7_EFFN_ENTROPY;
-      seed               = 0;
+      seed               = 42;
     }
   else 
     {
@@ -84,6 +84,7 @@ p7_builder_Create(const ESL_GETOPTS *go, const ESL_ALPHABET *abc)
       else if (esl_opt_GetBoolean(go, "--wgiven"))  bld->wgt_strategy = p7_WGT_GIVEN;
 
       if      (esl_opt_GetBoolean(go, "--eent"))    bld->effn_strategy = p7_EFFN_ENTROPY;
+      else if (esl_opt_GetBoolean(go, "--eentexp")) bld->effn_strategy = p7_EFFN_ENTROPY_EXP;
       else if (esl_opt_GetBoolean(go, "--eclust"))  bld->effn_strategy = p7_EFFN_CLUST;
       else if (esl_opt_GetBoolean(go, "--enone"))   bld->effn_strategy = p7_EFFN_NONE;
       else if (esl_opt_IsOn      (go, "--eset"))  { bld->effn_strategy = p7_EFFN_SET;      bld->eset = esl_opt_GetReal(go, "--eset"); }
@@ -91,7 +92,7 @@ p7_builder_Create(const ESL_GETOPTS *go, const ESL_ALPHABET *abc)
       seed = esl_opt_GetInteger(go, "--seed");
     }
 
-  bld->do_uniform_insert = FALSE; // default
+  bld->max_insert_len = 0;
 
   /* The default RE target is alphabet dependent. */
   if (go != NULL &&  esl_opt_IsOn (go, "--ere")) 
@@ -123,6 +124,7 @@ p7_builder_Create(const ESL_GETOPTS *go, const ESL_ALPHABET *abc)
    * As a special case, seed==0 means choose an arbitrary seed and shut off the
    * reinitialization; this allows run-to-run variation.
    */
+
   bld->r            = esl_randomness_CreateFast(seed);
   bld->do_reseeding = (seed == 0) ? FALSE : TRUE;
 
@@ -137,10 +139,28 @@ p7_builder_Create(const ESL_GETOPTS *go, const ESL_ALPHABET *abc)
       default:       bld->prior = p7_prior_CreateLaplace(abc); break;
       }
       if (bld->prior == NULL) goto ERROR;
+      /*
+      if      (go != NULL) {
+        if (esl_opt_IsOn(go, "--tmm"))  bld->prior->tm->alpha[0][0] = esl_opt_GetReal(go, "--tmm"); // TMM
+        if (esl_opt_IsOn(go, "--tmi"))  bld->prior->tm->alpha[0][1] = esl_opt_GetReal(go, "--tmi"); // TMM
+        if (esl_opt_IsOn(go, "--tmd"))  bld->prior->tm->alpha[0][2] = esl_opt_GetReal(go, "--tmd"); // TMM
+
+        if (esl_opt_IsOn(go, "--tim"))  bld->prior->ti->alpha[0][0] = esl_opt_GetReal(go, "--tim"); // TMM
+        if (esl_opt_IsOn(go, "--tii"))  bld->prior->ti->alpha[0][1] = esl_opt_GetReal(go, "--tii"); // TMM
+
+        if (esl_opt_IsOn(go, "--tdm"))  bld->prior->td->alpha[0][0] = esl_opt_GetReal(go, "--tdm"); // TMM
+        if (esl_opt_IsOn(go, "--tdd"))  bld->prior->td->alpha[0][1] = esl_opt_GetReal(go, "--tdd"); // TMM
+      }
+      */
     }
 
+
   bld->abc       = abc;
   bld->errbuf[0] = '\0';
+
+  bld->popen   = -1;
+  bld->pextend = -1;
+
   return bld;
   
  ERROR:
@@ -291,7 +311,11 @@ p7_builder_SetScoreSystem(P7_BUILDER *bld, const char *mxfile, const char *env,
   if ((bld->S  = esl_scorematrix_Create(bld->abc)) == NULL) { status = eslEMEM; goto ERROR; }
   if (mxfile == NULL) 
     {
-      if ((status = esl_scorematrix_Set("BLOSUM62", bld->S)) != eslOK) goto ERROR;
+      if (bld->abc->type == eslAMINO) {
+        if ((status = esl_scorematrix_Set("BLOSUM62", bld->S)) != eslOK) goto ERROR;
+      } else {
+        if ((status = esl_scorematrix_Set("DNA1", bld->S)) != eslOK) goto ERROR;
+      }
     } 
   else 
     {
@@ -358,7 +382,6 @@ p7_builder_Destroy(P7_BUILDER *bld)
 static int    validate_msa         (P7_BUILDER *bld, ESL_MSA *msa);
 static int    relative_weights     (P7_BUILDER *bld, ESL_MSA *msa);
 static int    build_model          (P7_BUILDER *bld, ESL_MSA *msa, P7_HMM **ret_hmm, P7_TRACE ***opt_tr);
-static int    homogonize_inserts   (P7_HMM *hmm);
 static int    effective_seqnumber  (P7_BUILDER *bld, const ESL_MSA *msa, P7_HMM *hmm, const P7_BG *bg);
 static int    parameterize         (P7_BUILDER *bld, P7_HMM *hmm);
 static int    annotate             (P7_BUILDER *bld, const ESL_MSA *msa, P7_HMM *hmm);
@@ -404,8 +427,9 @@ static int    make_post_msa        (P7_BUILDER *bld, const ESL_MSA *premsa, cons
 int
 p7_Builder(P7_BUILDER *bld, ESL_MSA *msa, P7_BG *bg,
 	   P7_HMM **opt_hmm, P7_TRACE ***opt_trarr, P7_PROFILE **opt_gm, P7_OPROFILE **opt_om,
-	   ESL_MSA **opt_postmsa)
+	   ESL_MSA **opt_postmsa, FILE *seqweights_w_fp, FILE *seqweights_e_fp)
 {
+  int i,j;
   uint32_t    checksum = 0;	/* checksum calculated for the input MSA. hmmalign --mapali verifies against this. */
   P7_HMM     *hmm      = NULL;
   P7_TRACE  **tr       = NULL;
@@ -414,18 +438,38 @@ p7_Builder(P7_BUILDER *bld, ESL_MSA *msa, P7_BG *bg,
   if ((status =  validate_msa         (bld, msa))                       != eslOK) goto ERROR;
   if ((status =  esl_msa_Checksum     (msa, &checksum))                 != eslOK) ESL_XFAIL(status, bld->errbuf, "Failed to calculate checksum"); 
   if ((status =  relative_weights     (bld, msa))                       != eslOK) goto ERROR;
+  if (seqweights_w_fp != NULL) {
+    for (i = 0; i < msa->nseq; i++)
+      fprintf( seqweights_w_fp, "%.2f  %s\n", msa->wgt[i], msa->sqname[i]) ;
+  }
+
   if ((status =  esl_msa_MarkFragments(msa, bld->fragthresh))           != eslOK) goto ERROR;
   if ((status =  build_model          (bld, msa, &hmm, tr_ptr))         != eslOK) goto ERROR;
 
-  if (bld->do_uniform_insert)
-    if ((status =  homogonize_inserts (hmm))                            != eslOK) goto ERROR;
+
+  //Ensures that the weighted-average I->I count <=  bld->max_insert_len
+  //(MI currently contains the number of observed insert-starts)
+  if (bld->max_insert_len>0)
+    for (i=1; i<hmm->M; i++ )
+      hmm->t[i][p7H_II] = ESL_MIN(hmm->t[i][p7H_II], bld->max_insert_len*hmm->t[i][p7H_MI]);
 
   if ((status =  effective_seqnumber  (bld, msa, hmm, bg))              != eslOK) goto ERROR;
+  if (seqweights_e_fp != NULL) {
+    for (i = 0; i < msa->nseq; i++)
+      fprintf( seqweights_e_fp, "%.4f  %s\n", msa->wgt[i], msa->sqname[i]) ;
+  }
   if ((status =  parameterize         (bld, hmm))                       != eslOK) goto ERROR;
   if ((status =  annotate             (bld, msa, hmm))                  != eslOK) goto ERROR;
   if ((status =  calibrate            (bld, hmm, bg, opt_gm, opt_om))   != eslOK) goto ERROR;
   if ((status =  make_post_msa        (bld, msa, hmm, tr, opt_postmsa)) != eslOK) goto ERROR;
 
+  //force masked positions to background  (it'll be close already, so no relevant impact on weighting)
+  if (hmm->mm != NULL)
+    for (i=1; i<hmm->M; i++ )
+      if (hmm->mm[i] == 'm')
+        for (j=0; j<hmm->abc->K; j++)
+          hmm->mat[i][j] = bg->f[j];
+
   if ( bld->abc->type == eslDNA ||  bld->abc->type == eslRNA ) {
 	  if (bld->w_len > 0)           hmm->max_length = bld->w_len;
 	  else if (bld->w_beta == 0.0)  hmm->max_length = hmm->M *4;
@@ -487,6 +531,13 @@ p7_SingleBuilder(P7_BUILDER *bld, ESL_SQ *sq, P7_BG *bg, P7_HMM **opt_hmm,
   if ((status = p7_hmm_SetConsensus(hmm, sq))                                                                   != eslOK) goto ERROR; 
   if ((status = calibrate(bld, hmm, bg, opt_gm, opt_om))                                                        != eslOK) goto ERROR;
 
+  if ( bld->abc->type == eslDNA ||  bld->abc->type == eslRNA ) {
+    if (bld->w_len > 0)           hmm->max_length = bld->w_len;
+    else if (bld->w_beta == 0.0)  hmm->max_length = hmm->M *4;
+    else if ( (status =  p7_Builder_MaxLength(hmm, bld->w_beta)) != eslOK) goto ERROR;
+  }
+
+
   /* build a faux trace: relative to core model (B->M_1..M_L->E) */
   if (opt_tr != NULL) 
     {
@@ -523,8 +574,8 @@ p7_SingleBuilder(P7_BUILDER *bld, ESL_SQ *sq, P7_BG *bg, P7_HMM **opt_hmm,
  *
  * The idea is to find the length such that all but e.g. 1e-7 sequences emitted
  * by the model are at most that long. The method conceptually fills in a table of
- * length at most max_len (set to 100,000), though in practice, only two columns are
- * used to store values;
+ * length at most length_bound (usually 20 * model_length, up to at most 100,000),
+ * though in practice, only two columns are used to store values;
  *
  * Letting i correspond to the ith state of the model,
  *         j to a length j of emitted sequence, and
@@ -609,6 +660,9 @@ p7_SingleBuilder(P7_BUILDER *bld, ESL_SQ *sq, P7_BG *bg, P7_HMM **opt_hmm,
  * (1) each M[i] should contribute (1-t_md)M[i] to Y.
  * (2) each D[i] should contribute (1-t_dd)D[i] to Y.
  *
+ * If the probability threshold isn't met before reaching length_bound, then MAXL is
+ * simply set to length_bound (usually 20 * model_length).
+ *
  *
  * Args:      hmm         - p7_HMM (required for the transition probabilities)
  *
@@ -624,11 +678,11 @@ p7_Builder_MaxLength (P7_HMM *hmm, double emit_thresh)
   double   surv;                  // surviving probability mass at length L; Y from above
   int      k;                     // active state in model
   int      i;
-  int      length_bound = 200000; // default cap on # iterations (aka max model length)
   double **I            = NULL;
   double **M            = NULL;
   double **D            = NULL;
-  int      model_len    = hmm->M; // model length                
+  int      model_len    = hmm->M; // model length
+  int      length_bound = ESL_MAX(model_len, ESL_MIN(20*model_len, 100000)); // cap on # iterations (aka max model length)
   int      status;
   
   if (model_len==1) {
@@ -636,6 +690,8 @@ p7_Builder_MaxLength (P7_HMM *hmm, double emit_thresh)
     return eslOK;
   }
 
+  hmm->max_length = length_bound;  //default, if it never reaches the target surviving density
+
 
   //    double I[model_len+1][2], M[model_len+1][2], D[model_len+1][2]; //2 columns for each way of ending a subpath
   ESL_ALLOC(I, (model_len+1) * sizeof(double*)); 
@@ -716,7 +772,7 @@ p7_Builder_MaxLength (P7_HMM *hmm, double emit_thresh)
   free(M);
   free(D);
 
-  if (hmm->max_length >= length_bound) return eslERANGE;
+  if (hmm->max_length > length_bound) return eslERANGE;
   return eslOK;
   
  ERROR:
@@ -821,54 +877,6 @@ build_model(P7_BUILDER *bld, ESL_MSA *msa, P7_HMM **ret_hmm, P7_TRACE ***opt_tr)
   return status;
 }
 
-/* homogonize_inserts()
- *
- * <hmm> comes in with weighted observed counts. It goes out with
- * the observed counts for transitions p7H_II and p7H_MI distributed
- * equally among all positions.
- */
-static int
-homogonize_inserts(P7_HMM *hmm)
-{
-  int i;
-  float tii = 0.0;
-  float tmi = 0.0;
-  float dmi;
-  float tm_md;
-
-  /* Get total counts for M->I and I->I transitions
-   * Ignore position M, as its insert state isn't reached.
-   */
-  for (i=1; i<hmm->M; i++ ) {
-    tii += hmm->t[i][p7H_II];
-    tmi += hmm->t[i][p7H_MI];
-  }
-
-
-  //these is the avg counts per position
-  tii /= (hmm->M-1);
-  tmi /= (hmm->M-1);
-
-  //assign those avg counts per position
-  for (i=1; i<hmm->M; i++ ) {
-
-    hmm->t[i][p7H_II]  = tii;
-    hmm->t[i][p7H_IM]  = tmi; //(I->M == M->I by design).
-
-    dmi = tmi - hmm->t[i][p7H_MI];  //change
-    hmm->t[i][p7H_MI]  = tmi;
-
-    /* the change to M->I counts should be offset by
-     * changes to the M->M or M->D counts
-     */
-    tm_md = hmm->t[i][p7H_MM] + hmm->t[i][p7H_MD];
-    hmm->t[i][p7H_MM] -= (tm_md == 0 ? 0 : (dmi * hmm->t[i][p7H_MM]/tm_md)) ;
-    hmm->t[i][p7H_MD] -= (tm_md == 0 ? 0 : (dmi * hmm->t[i][p7H_MD]/tm_md)) ;
-
-  }
-
-  return eslOK;
-}
 
 
 /* set_effective_seqnumber()
@@ -887,35 +895,63 @@ static int
 effective_seqnumber(P7_BUILDER *bld, const ESL_MSA *msa, P7_HMM *hmm, const P7_BG *bg)
 {
   int    status;
+  int    i;
 
-  if      (bld->effn_strategy == p7_EFFN_NONE)    hmm->eff_nseq = msa->nseq;
-  else if (bld->effn_strategy == p7_EFFN_SET)     hmm->eff_nseq = bld->eset;
-  else if (bld->effn_strategy == p7_EFFN_CLUST)
-    {
-      int nclust;
-
-      status = esl_msacluster_SingleLinkage(msa, bld->eid, NULL, NULL, &nclust);
-      if      (status == eslEMEM) ESL_XFAIL(status, bld->errbuf, "memory allocation failed");
-      else if (status != eslOK)   ESL_XFAIL(status, bld->errbuf, "single linkage clustering algorithm (at %d%% id) failed", (int)(100 * bld->eid));
-
-      hmm->eff_nseq = (double) nclust;
-    }
-
-  else if (bld->effn_strategy == p7_EFFN_ENTROPY)
-    {
+  if (bld->effn_strategy == p7_EFFN_ENTROPY_EXP) {
       double etarget; 
       double eff_nseq;
-
+      double exp;
       etarget = (bld->esigma - eslCONST_LOG2R * log( 2.0 / ((double) hmm->M * (double) (hmm->M+1)))) / (double) hmm->M; /* xref J5/36. */
       etarget = ESL_MAX(bld->re_target, etarget);
 
-      status = p7_EntropyWeight(hmm, bg, bld->prior, etarget, &eff_nseq);
+      status = p7_EntropyWeight_exp(hmm, bg, bld->prior, etarget, &exp);
       if      (status == eslEMEM) ESL_XFAIL(status, bld->errbuf, "memory allocation failed");
       else if (status != eslOK)   ESL_XFAIL(status, bld->errbuf, "internal failure in entropy weighting algorithm");
+
+      p7_hmm_ScaleExponential(hmm, exp);
+
+      for (i = 1; i <= hmm->M; i++)
+        eff_nseq +=  esl_vec_FSum(hmm->mat[i], hmm->abc->K);
+
+
+      eff_nseq /= hmm->M;
       hmm->eff_nseq = eff_nseq;
+
+  } else {
+
+    if      (bld->effn_strategy == p7_EFFN_NONE)    hmm->eff_nseq = msa->nseq;
+    else if (bld->effn_strategy == p7_EFFN_SET)     hmm->eff_nseq = bld->eset;
+    else if (bld->effn_strategy == p7_EFFN_CLUST)
+    {
+        int nclust;
+
+        status = esl_msacluster_SingleLinkage(msa, bld->eid, NULL, NULL, &nclust);
+        if      (status == eslEMEM) ESL_XFAIL(status, bld->errbuf, "memory allocation failed");
+        else if (status != eslOK)   ESL_XFAIL(status, bld->errbuf, "single linkage clustering algorithm (at %d%% id) failed", (int)(100 * bld->eid));
+
+        hmm->eff_nseq = (double) nclust;
     }
-    
-  p7_hmm_Scale(hmm, hmm->eff_nseq / (double) hmm->nseq);
+    else if (bld->effn_strategy == p7_EFFN_ENTROPY)
+    {
+        double etarget;
+        double eff_nseq;
+        etarget = (bld->esigma - eslCONST_LOG2R * log( 2.0 / ((double) hmm->M * (double) (hmm->M+1)))) / (double) hmm->M; /* xref J5/36. */
+        etarget = ESL_MAX(bld->re_target, etarget);
+
+        status = p7_EntropyWeight(hmm, bg, bld->prior, etarget, &eff_nseq);
+        if      (status == eslEMEM) ESL_XFAIL(status, bld->errbuf, "memory allocation failed");
+        else if (status != eslOK)   ESL_XFAIL(status, bld->errbuf, "internal failure in entropy weighting algorithm");
+        hmm->eff_nseq = eff_nseq;
+    }
+
+    p7_hmm_Scale(hmm, hmm->eff_nseq / (double) hmm->nseq);
+
+  }
+
+  //this re-assignment of the wgt values is done in support of hmmbuild's --seq_weights_e flag
+  for (i = 0; i < msa->nseq; i++)
+    msa->wgt[i] *= (hmm->eff_nseq / (double) hmm->nseq);
+
   return eslOK;
 
  ERROR:
@@ -1003,7 +1039,7 @@ static int
 make_post_msa(P7_BUILDER *bld, const ESL_MSA *premsa, const P7_HMM *hmm, P7_TRACE **tr, ESL_MSA **opt_postmsa)
 {
   ESL_MSA  *postmsa  = NULL;
-  int       optflags = p7_ALL_CONSENSUS_COLS;
+  int       optflags = p7_DEFAULT;
   int       status;
 
   if (opt_postmsa == NULL) return eslOK;
@@ -1030,13 +1066,13 @@ make_post_msa(P7_BUILDER *bld, const ESL_MSA *premsa, const P7_HMM *hmm, P7_TRAC
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  *
- * SVN $Id: p7_builder.c 4285 2012-11-05 10:59:57Z nawrockie $
- * SVN $URL: https://svn.janelia.org/eddylab/eddys/src/hmmer/branches/infernal/1.1/src/p7_builder.c $
+ * SVN $Id$
+ * SVN $URL$
  *****************************************************************/
