@@ -17,9 +17,9 @@
  *********************************************************************/
 
 
-/* Function:  p7_hmm_initWindows()
+/* Function:  p7_hmmwindow_init()
  *
- * Synopsis:  initialize the object used to store a list of windows around seeds
+ * Synopsis:  initialize the object used to store a list of sequence windows
  *
  * Returns:   eslEMEM in event of allocation failure, otherwise eslOK
  */
@@ -37,20 +37,21 @@ ERROR:
 
 }
 
-/* Function:  p7_hmm_newWindow()
+/* Function:  p7_hmmwindow_new()
  *
- * Synopsis:  return a pointer to the next window element on the list,
- *            increasing the size of the list, if necessary.
+ * Synopsis:  Return a pointer to the next window element on the list
  *
- * Purpose:   accepts <id>, <pos>, <fm_pos>, <k>, <length>, <score>,
+ * Purpose:   Accepts <id>, <pos>, <fm_pos>, <k>, <length>, <score>,
  *            and <complementarity>, assigns those to the next window
- *            element, then returns it.
+ *            element, then returns it, increasing the size of the
+ *            list, if necessary.
  *
  * Returns:   NULL in event of allocation failure, otherwise pointer to
  *            the next seed diagonal
  */
+
 P7_HMM_WINDOW *
-p7_hmmwindow_new (P7_HMM_WINDOWLIST *list, uint32_t id, uint32_t pos, uint32_t fm_pos, uint16_t k, uint32_t length, float score, uint8_t complementarity) {
+p7_hmmwindow_new (P7_HMM_WINDOWLIST *list, uint32_t id, uint32_t pos, uint32_t fm_pos, uint16_t k, uint32_t length, float score, uint8_t complementarity, uint32_t target_len) {
   int status;
   P7_HMM_WINDOW *window;
 
@@ -60,13 +61,14 @@ p7_hmmwindow_new (P7_HMM_WINDOWLIST *list, uint32_t id, uint32_t pos, uint32_t f
   }
   window = list->windows + list->count;
 
-  window->id      = id;
-  window->n       = pos;
-  window->fm_n    = fm_pos;
-  window->k       = k;
-  window->length  = length;
-  window->score   = score;
+  window->id               = id;
+  window->n                = pos;
+  window->fm_n             = fm_pos;
+  window->k                = k;
+  window->length           = length;
+  window->score            = score;
   window->complementarity  = complementarity;
+  window->target_len       = target_len;
 
   list->count++;
 
@@ -93,8 +95,8 @@ ERROR:
 
 /************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
