@@ -7,7 +7,7 @@
  *   4. Copyright and license.
  * 
  * SRE, Tue Oct 21 19:38:19 2008 [Casa de Gatos]
- * SVN $Id: tracealign.c 3960 2012-03-22 21:42:50Z wheelert $
+ * SVN $Id$
  */
 #include "p7_config.h"
 
@@ -267,22 +267,25 @@ p7_tracealign_computeTraces(P7_HMM *hmm, ESL_SQ  **sq, int offset, int N, P7_TRA
    */
   for (idx = offset; idx < offset+ N; idx++)
   {
-      p7_omx_GrowTo(oxf, hmm->M, sq[idx]->n, sq[idx]->n);
-      p7_omx_GrowTo(oxb, hmm->M, sq[idx]->n, sq[idx]->n);
+    /* special case: a sequence of length 0. HMMER model can't generate 0 length seq. Set tr->N == 0 as a flag. (bug #h100 fix) */
+    if (sq[idx]->n == 0) { tr[idx]->N = 0; continue; }
 
-      p7_oprofile_ReconfigLength(om, sq[idx]->n);
+    p7_omx_GrowTo(oxf, hmm->M, sq[idx]->n, sq[idx]->n);
+    p7_omx_GrowTo(oxb, hmm->M, sq[idx]->n, sq[idx]->n);
 
-      p7_Forward (sq[idx]->dsq, sq[idx]->n, om,      oxf, &fwdsc);
-      p7_Backward(sq[idx]->dsq, sq[idx]->n, om, oxf, oxb, NULL);
+    p7_oprofile_ReconfigLength(om, sq[idx]->n);
 
-      status = p7_Decoding(om, oxf, oxb, oxb);      /* <oxb> is now overwritten with post probabilities     */
+    p7_Forward (sq[idx]->dsq, sq[idx]->n, om,      oxf, &fwdsc);
+    p7_Backward(sq[idx]->dsq, sq[idx]->n, om, oxf, oxb, NULL);
 
-      if (status == eslOK)
+    status = p7_Decoding(om, oxf, oxb, oxb);      /* <oxb> is now overwritten with post probabilities     */
+
+    if (status == eslOK)
       {
         p7_OptimalAccuracy(om, oxb, oxf, &oasc);      /* <oxf> is now overwritten with OA scores              */
         p7_OATrace        (om, oxb, oxf, tr[idx]);    /* tr[idx] is now an OA traceback for seq #idx          */
       }
-      else if (status == eslERANGE)
+    else if (status == eslERANGE)
       {
         /* Work around the numeric overflow problem in Decoding()
          * xref J3/119-121 for commentary;
@@ -325,25 +328,25 @@ p7_tracealign_computeTraces(P7_HMM *hmm, ESL_SQ  **sq, int offset, int N, P7_TRA
       }
 
 
-      /* the above steps aren't storing the tfrom/tto values in the trace,
-       * which are required for downstream processing in this case, so
-       * hack them here. Note - this treats the whole thing as one domain,
-       * even if there are really multiple domains.
-       */
-      // skip the parts of the trace that precede the first match state
-      tfrom = 2;
-      while (tr[idx]->st[tfrom] != p7T_M)   tfrom++;
+    /* the above steps aren't storing the tfrom/tto values in the trace,
+     * which are required for downstream processing in this case, so
+     * hack them here. Note - this treats the whole thing as one domain,
+     * even if there are really multiple domains.
+     */
+    // skip the parts of the trace that precede the first match state
+    tfrom = 2;
+    while (tr[idx]->st[tfrom] != p7T_M)   tfrom++;
 
-      tto = tfrom + 1;
-      //run until the model is exited
-      while (tr[idx]->st[tto] != p7T_E)     tto++;
+    tto = tfrom + 1;
+    //run until the model is exited
+    while (tr[idx]->st[tto] != p7T_E)     tto++;
 
-      tr[idx]->tfrom[0]  = tfrom;
-      tr[idx]->tto[0]    = tto - 1;
+    tr[idx]->tfrom[0]  = tfrom;
+    tr[idx]->tto[0]    = tto - 1;
 
 
-      p7_omx_Reuse(oxf);
-      p7_omx_Reuse(oxb);
+    p7_omx_Reuse(oxf);
+    p7_omx_Reuse(oxb);
   }
 
 #if 0
@@ -468,19 +471,19 @@ ERROR:
  *****************************************************************/
 
 /* map_new_msa()
- * 
+ *
  * Construct <inscount[0..M]>, <matuse[1..M]>, and <matmap[1..M]>
  * arrays for mapping model consensus nodes <1..M> onto columns
  * <1..alen> of a new MSA.
- * 
+ *
  * Here's the problem. We want to align the match states in columns,
  * but some sequences have inserted symbols in them; we need some
  * sort of overall knowledge of where the inserts are and how long
  * they are in order to create the alignment.
- * 
+ *
  * Here's our trick. inscount[] is a 0..M array; inserts[k] stores
  * the maximum number of times insert substate k was used. This
- * is the maximum number of gaps to insert between canonical 
+ * is the maximum number of gaps to insert between canonical
  * column k and k+1.  inserts[0] is the N-term tail; inserts[M] is
  * the C-term tail.
  * 
@@ -490,22 +493,22 @@ ERROR:
  * the <p7_ALL_CONSENSUS_COLS> option flag is set, though, all
  * matuse[1..M] are set <TRUE>. (matuse[0] is unused, always <FALSE>.)
  * 
- * Then, using these arrays, we construct matmap[] and determine alen. 
+ * Then, using these arrays, we construct matmap[] and determine alen.
  * If match state k is represented as an alignment column,
  * matmap[1..M] = that position, <1..alen>.
  * If match state k is not in the alignment (<matuse[k] == FALSE>),
  * matmap[k] = matmap[k-1] = the last alignment column that a match
- * state did map to; this is a trick to make some apos coordinate setting 
- * work cleanly. 
- * Because of this trick, you can't just assume because matmap[k] is 
- * nonzero that match state k maps somewhere in the alignment; 
+ * state did map to; this is a trick to make some apos coordinate setting
+ * work cleanly.
+ * Because of this trick, you can't just assume because matmap[k] is
+ * nonzero that match state k maps somewhere in the alignment;
  * you have to check matuse[k] == TRUE, then look at what matmap[k] says.
  * Remember that N and C emit on transition, hence the check for an
- * N->N or C->C transition before bumping nins. 
+ * N->N or C->C transition before bumping nins.
  * <matmap[0]> is unused; by convention, <matmap[0] = 0>.
  */
 static int
-map_new_msa(P7_TRACE **tr, int nseq, int M, int optflags, int **ret_inscount, 
+map_new_msa(P7_TRACE **tr, int nseq, int M, int optflags, int **ret_inscount,
 	    int **ret_matuse, int **ret_matmap, int *ret_alen)
 {
   int *inscount = NULL;	  /* inscount[k=0..M] == max # of inserts in node k */
@@ -1193,7 +1196,7 @@ main(int argc, char **argv)
   }
 
   /* Determine output alignment file format */
-  outfmt = eslx_msafile_EncodeFormat(esl_opt_GetString(go, "--outformat"));
+  outfmt = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--outformat"));
   if (outfmt == eslMSAFILE_UNKNOWN)    esl_fatal(argv[0], "%s is not a recognized output MSA file format\n", esl_opt_GetString(go, "--outformat"));
 
   /* Open output stream */
@@ -1265,7 +1268,7 @@ main(int argc, char **argv)
 
   p7_tracealign_getMSAandStats(hmm, sq, totseq, &msa, pp, relent, scores);
 
-  eslx_msafile_Write(ofp, msa, outfmt);
+  esl_msafile_Write(ofp, msa, outfmt);
 
   for (idx = 0; idx < totseq; idx++) {
     printf("%s\n------------------\n", sq[idx]->name);
@@ -1303,8 +1306,8 @@ main(int argc, char **argv)
 
 /*****************************************************************
  * HMMER - Biological sequence analysis with profile HMMs
- * Version i1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 3.1b3; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * HMMER is distributed under the terms of the GNU General Public License
