@@ -1,6 +1,6 @@
 /* alphabet.c
  * SRE, Tue Aug  1 10:31:11 2000 [St. Louis]
- * SVN $Id: alphabet.c 2550 2008-08-23 19:06:45Z nawrockie $
+ * SVN $Id$
  * 
  * Stuff having to do with manipulating symbols in the (RNA) alphabet.
  * 
@@ -12,6 +12,7 @@
  */
 
 #include "esl_config.h"
+#include "p7_config.h"
 #include "config.h"
 
 #include <string.h>
@@ -24,8 +25,7 @@
 #include "esl_randomseq.h"
 #include "esl_vectorops.h"
 
-#include "funcs.h"
-#include "structs.h"
+#include "infernal.h"
 
 /* Function: PairCount()
  * Date:     SRE, Tue Aug  1 10:34:20 2000 [St. Louis]
@@ -48,38 +48,88 @@ PairCount(const ESL_ALPHABET *abc, float *counters, ESL_DSQ syml, ESL_DSQ symr,
   if (syml < abc->K && symr < abc->K) 
     counters[(int) (syml * abc->K + symr)] += wt;
   else {
-    /*    float left = NULL;
-	  float right = NULL;
-	  ESL_ALLOC(left,  sizeof(float) * esl->abc->K);
-	  ESL_ALLOC(right, sizeof(float) * esl->abc->K);*/
-    float left[MAXABET];
-    float right[MAXABET];
-
+    int status;
+    float *left = NULL;
+    float *right = NULL;
     int   l,r;
-    
-    esl_vec_FSet(left, MAXABET, 0.);
-    esl_vec_FSet(right, MAXABET, 0.);
-    esl_abc_FCount(abc, left,  syml, wt);
-    esl_abc_FCount(abc, right, symr, wt);
+    ESL_ALLOC(left,  (sizeof(float) * abc->K));
+    ESL_ALLOC(right, (sizeof(float) * abc->K));
+
+    /* Be careful to set weights as 1.0 for the FCount calls (not <wt>), 
+     * because what we want to know is the fraction of all possible
+     * residues that agree with the degenerate for each residue, i.e.:
+     *
+     * if  syml == 'N': left[A]  == left[C] == left[G] == left[U] == 0.25
+     * and symr == 'R': right[A] == right[G] == 0.5
+     *
+     * Then when we increment counters by left[l] * left[r] * <wt>.
+     * That way if <wt> = 0.8, counters is set as follows:
+     * 
+     * if syml == 'N' && symr == 'R': 
+     * counters['AA'] == counters['CA'] == counters ['GA'] == counters['UA'] == 
+     * counters['AG'] == counters['CG'] == counters ['GG'] == counters['UG'] == 0.1
+     * and all others are = 0.0.
+     *
+     * Previously (Infernal versions 0.55 (maybe even earlier) until
+     * 1.0.2) this was done incorrectly by passing <wt> to the
+     * FCount() calls and setting counters[] = left[l] *
+     * right[r]. Somehow, Stefan Janssen tracked this down. Thanks
+     * Stefan!  The bug is i23, logged in Bugs/BUGTRAX. 
+     * EPN, Mon Dec 6 13:09:52 2010
+     */
+    esl_vec_FSet(left, abc->K, 0.);
+    esl_vec_FSet(right, abc->K, 0.);
+    esl_abc_FCount(abc, left,  syml, 1.0);
+    esl_abc_FCount(abc, right, symr, 1.0);
 
     for (l = 0; l < abc->K; l++)
       for (r = 0; r < abc->K; r++)
-	counters[l*abc->K +r] += left[l] * right[r];
+	counters[l*abc->K +r] += left[l] * right[r] * wt;
+
+    free(left);
+    free(right);
   }
   return;
+
+ ERROR:
+  cm_Fail("Memory allocation error.");
+  return; /* never reached */
 }
+
 float
 DegeneratePairScore(const ESL_ALPHABET *abc, float *esc, ESL_DSQ syml, ESL_DSQ symr)
 {
-  float left[MAXABET], right[MAXABET];
+  float *left = NULL;
+  float *right = NULL;
+  int status;
+  ESL_ALLOC(left,  (sizeof(float) * abc->K));
+  ESL_ALLOC(right, (sizeof(float) * abc->K));
   int l,r;
   float sc;
 
-  if (syml < abc->K && symr < abc->K) 
+  if (syml < abc->K && symr < abc->K) /* canonical pair */
+  {
+    free(left);
+    free(right);
     return esc[(int) (syml*abc->K+symr)];
+  }
 
-  esl_vec_FSet(left, MAXABET, 0.);
-  esl_vec_FSet(right, MAXABET, 0.);
+  if (syml == abc->K || symr == abc->K)  /* gap, this gets an IMPOSSIBLE sc */
+  {
+    free(left);
+    free(right);
+    return IMPOSSIBLE;
+  }
+
+  if (syml == (abc->Kp-1) || symr == (abc->Kp-1))  /* missing data, this gets an IMPOSSIBLE sc */
+  {
+    free(left);
+    free(right);
+    return IMPOSSIBLE;
+  }
+
+  esl_vec_FSet(left, abc->K, 0.);
+  esl_vec_FSet(right, abc->K, 0.);
   esl_abc_FCount(abc, left,  syml, 1.);
   esl_abc_FCount(abc, right, symr, 1.);
   
@@ -87,30 +137,55 @@ DegeneratePairScore(const ESL_ALPHABET *abc, float *esc, ESL_DSQ syml, ESL_DSQ s
   for (l = 0; l < abc->K; l++)
     for (r = 0; r < abc->K; r++)
       sc += esc[l*abc->K+r] * left[l] * right[r];
+
+  free(left);
+  free(right);
+
   return sc;
+
+ ERROR:
+  cm_Fail("Memory allocation error.");
+  return 0.0; /* never reached */
 }
 int
 iDegeneratePairScore(const ESL_ALPHABET *abc, int *iesc, ESL_DSQ syml, ESL_DSQ symr)
 {
-  float left[MAXABET], right[MAXABET];
+  float *left = NULL;
+  float *right = NULL;
+  int status;
+  ESL_ALLOC(left,  (sizeof(float) * abc->K));
+  ESL_ALLOC(right, (sizeof(float) * abc->K));
   int l,r;
   float sc;
 
   if (syml < abc->K && symr < abc->K) 
     return iesc[(int) (syml*abc->K+symr)];
 
-  esl_vec_FSet(left, MAXABET, 0.);
-  esl_vec_FSet(right, MAXABET, 0.);
+  if (syml == abc->K || symr == abc->K)  /* gap, this gets an -INFTY sc */
+    return -INFTY;
+
+  if (syml == (abc->Kp-1) || symr == (abc->Kp-1))  /* missing data, this gets an -INFTY sc */
+    return -INFTY;
+
+  esl_vec_FSet(left, abc->K, 0.);
+  esl_vec_FSet(right, abc->K, 0.);
   esl_abc_FCount(abc, left,  syml, 1.);
   esl_abc_FCount(abc, right, symr, 1.);
 
   sc = 0.;
-  for (l = 0; l < MAXABET; l++)
-    for (r = 0; r < MAXABET; r++)
+  for (l = 0; l < abc->K; l++)
+    for (r = 0; r < abc->K; r++)
       sc += iesc[l*abc->K+r] * left[l] * right[r];
-  return (int) sc;
-}
 
+  free(left);
+  free(right);
+
+  return (int) sc;
+
+ ERROR:
+  cm_Fail("Memory allocation error.");
+  return status; /* never reached */
+}
 /* EPN, Wed Aug 20 13:44:16 2008
  * FastPairScore*() functions: 
  * Written to calculate base pairs scores involving 1 or 2 canonical 
@@ -253,81 +328,6 @@ iFastPairScoreRightOnlyDegenerate(int K, int *iesc, float *right, ESL_DSQ syml)
   return (int) sc;
 }
 
-/* Function: LeftMarginalScore()
- * Author:   DLK
- *
- * Purpose:  Calculate marginal probability for left half
- *           of an emission pair.  Implicitly assumes
- *           a uniform background distribution
- */
-float
-LeftMarginalScore(const ESL_ALPHABET *abc, float *esc, ESL_DSQ dres)
-{
-   float left[MAXABET];
-   int i;
-   float sc;
-
-   if (dres < abc->K) 
-   {
-      sc = esl_vec_FLogSum(&(esc[dres*abc->K]),abc->K);
-      sc -= sreLOG2(abc->K);
-   }
-   else /* degenerate */
-   {
-      esl_vec_FSet(left, MAXABET, 0.);
-      esl_abc_FCount(abc, left, dres, 1.);
-
-      sc = 0.;
-      for (i = 0; i < MAXABET; i++)
-      {
-         sc += esl_vec_FLogSum(&(esc[i*abc->K]),abc->K)*left[i];
-         sc -= sreLOG2(abc->K)*left[i];
-      }
-   }
-
-   return sc;
-}
-
-/* Function: RightMarginalScore()
- * Author:   DLK
- *
- * Purpose:  Calculate marginal probability for right half
- *           of an emission pair.  Implicitly assumes
- *           a uniform background distribution
- */
-float
-RightMarginalScore(const ESL_ALPHABET *abc, float *esc, ESL_DSQ dres)
-{
-   float right[MAXABET];
-   int i,j;
-   float sc;
-   float row[abc->K];
-
-   if (dres < abc->K)
-   {
-      for (i=0; i<abc->K; i++)
-         row[i] = esc[i*abc->K+dres];
-      sc = esl_vec_FLogSum(row,abc->K);
-      sc -= sreLOG2(abc->K);
-   }
-   else /* degenerate */
-   {
-      esl_vec_FSet(right, MAXABET, 0.);
-      esl_abc_FCount(abc, right, dres, 1.);
-
-      sc = 0.;
-      for (i=0; i < MAXABET; i++)
-      {
-         for (j=0; j<abc->K; j++)
-            row[j] = esc[j*abc->K+dres];
-         sc += esl_vec_FLogSum(row,abc->K)*right[i];
-         sc -= sreLOG2(abc->K)*right[i];
-      }
-   }
-
-   return sc;
-}
-
 /* Following funcs from RSEARCH, only used by RSEARCH code */
 
 static char random_from_string (ESL_RANDOMNESS *r, char *s);
