@@ -26,14 +26,14 @@
  * xref STL7 p.130.
  ******************************************************************
  * Infernal - inference of RNA secondary structure alignments
- * Version 1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 1.1.2; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * Infernal is distributed under the terms of the GNU General Public License
  * (GPLv3). See the LICENSE file for details.
  *****************************************************************  
- * SVN $Id: bandcyk-truncation-test.c 3955 2012-03-20 10:17:48Z nawrockie $
+ * SVN $Id$
  */
 
 #include "esl_config.h"
@@ -59,7 +59,6 @@ static ESL_OPTIONS options[] = {
   { "--betaW",   eslARG_REAL,  "1E-5", NULL, "x>0.",NULL,  NULL, NULL, "set tail probability thresh for W calculation to <x>", 0 },
   { "--beta1",   eslARG_REAL,  "1E-5", NULL, "x>0.",NULL,  NULL, NULL, "set tail probability thresh for dmin1/dmax1 to <x>",   0 },
   { "--beta2",   eslARG_REAL,  "1E-6", NULL, "x>0.",NULL,  NULL, NULL, "set tail probability thresh for dmin2/dmax2 to <x>",  0 },
-  { "--verbose", eslARG_NONE,   FALSE, NULL, NULL,  NULL,  NULL, NULL, "show output (default: silently return 0 on success)", 0 },
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 static char usage[]  = "[-options] <cmfile> <Z1> <Z2> where width Z1 < Z2.";
@@ -86,7 +85,6 @@ main(int argc, char **argv)
   double   gbeta1;		/* geometric decay constant, truncation error calc */
   double   gbeta2;		/* geometric decay constant, truncation error calc */
   double   g;			/* log of an estimated gamma[n] */
-  int      be_verbose;
   ESL_ALPHABET   *abc     = NULL;
   char            errbuf[eslERRBUFSIZE]; /* for error messages */
   CM_QDBINFO     *qdbinfo1 = NULL; /* holds dmin1/dmax1 and dmin2/dmax2 for first  choice of Z */
@@ -101,7 +99,6 @@ main(int argc, char **argv)
   betaW      = esl_opt_GetReal   (go, "--betaW");
   beta1      = esl_opt_GetReal   (go, "--beta1");
   beta2      = esl_opt_GetReal   (go, "--beta2");
-  be_verbose = esl_opt_GetBoolean(go, "--verbose");
 
   if (Z1 >= Z2) cm_Fail("Please set a width Z1 < width Z2, else the check won't work right.");
 
