@@ -1,6 +1,6 @@
 /* cm_modelmaker.c
  * SRE, 28 Feb 2000 
- * SVN $Id: cm_modelmaker.c 4066 2012-06-08 14:04:21Z nawrockie $
+ * SVN $Id$
  *
  * Construct a model from an alignment. 
  *
@@ -27,8 +27,8 @@
  * 
  *****************************************************************
  * Infernal - inference of RNA secondary structure alignments
- * Version 1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 1.1.2; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * Infernal is distributed under the terms of the GNU General Public License
@@ -1837,7 +1837,7 @@ cm_check_before_detaching(CM_t *cm, int insert1, int insert2)
   int ret_val;
   int i, yoffset;
   int to_detach;
-  int to_keep;
+  /*int to_keep;*/
   float diff;
   
   ret_val = FALSE;
@@ -1849,7 +1849,7 @@ cm_check_before_detaching(CM_t *cm, int insert1, int insert2)
     {
       ret_val = TRUE;
       to_detach = insert1;
-      to_keep   = insert2;
+      /*to_keep   = insert2;*/
     }
   if(cm->sttype[insert2+1] == E_st)
     {
@@ -1857,7 +1857,7 @@ cm_check_before_detaching(CM_t *cm, int insert1, int insert2)
 	cm_Fail("ERROR: in cm_check_before_detaching() insert1: %d and insert2: %d both map to END_E-1 states.\n", insert1, insert2);
       ret_val = TRUE;
       to_detach = insert2;
-      to_keep   = insert1;
+      /*to_keep   = insert1;*/
     }
 
   /* check to make sure we have 0.0 counts in to_detach */
