@@ -6,8 +6,8 @@
  *
  *****************************************************************
  * Infernal - inference of RNA secondary structure alignments
- * Version 1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 1.1.2; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * Infernal is distributed under the terms of the GNU General Public License
@@ -357,7 +357,6 @@ cm_p7_Tau(ESL_RANDOMNESS *r, char *errbuf, P7_OPROFILE *om, P7_PROFILE *gm, P7_B
   double   gmu, glam;
   int      status;
   int      i;
-  int      M;
   int do_generic;
 
   if(om == NULL && gm == NULL) { status = eslEINVAL; goto ERROR; }
@@ -367,12 +366,10 @@ cm_p7_Tau(ESL_RANDOMNESS *r, char *errbuf, P7_OPROFILE *om, P7_PROFILE *gm, P7_B
   if(do_generic) { 
     gx = p7_gmx_Create(gm->M, L); /* DP matrix: for ForwardParser,  L rows */
     if (gx == NULL) { status = eslEMEM; goto ERROR; }
-    M  = gm->M;
   }
   else { 
     ox = p7_omx_Create(om->M, 0, L);     /* DP matrix: for ForwardParser,  L rows */
     if (ox == NULL) { status = eslEMEM; goto ERROR; }
-    M  = om->M;
   }
 
   ESL_ALLOC(xv,  sizeof(double)  * N);
