@@ -1,7 +1,7 @@
 /************************************************************
  * Infernal - inference of RNA secondary structure alignments
- * Version 1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 1.1.2; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * Infernal is distributed under the terms of the GNU General Public License
@@ -106,16 +106,16 @@ CP9_fake_tracebacks(ESL_MSA *msa, int *matassign, CP9trace_t ***ret_tr)
   int  apos;                    /* position in alignment columns   */
   int  tpos;			/* position in traceback           */
   int  first_match;             /* first match column */
-  int  last_match;              /* last match column */
+  /*int  last_match;*/              /* last match column, not used */
 
   ESL_ALLOC(tr, sizeof(CP9trace_t *) * msa->nseq);
   
   first_match = -1;
-  last_match  = -1;
+  /*last_match  = -1;*/
   for (apos = 0; apos < msa->alen; apos++)
     {
       if(matassign[apos+1] && first_match == -1) first_match = apos;
-      if(matassign[apos+1]) last_match = apos;
+      /*if(matassign[apos+1]) last_match = apos;*/
     }
 
   for (idx = 0; idx < msa->nseq; idx++)
