@@ -7,8 +7,8 @@
  * 
  *****************************************************************
  * Infernal - inference of RNA secondary structure alignments
- * Version 1.1.1; July 2014
- * Copyright (C) 2014 Howard Hughes Medical Institute.
+ * Version 1.1.2; July 2016
+ * Copyright (C) 2016 Howard Hughes Medical Institute.
  * Other copyrights also apply. See the COPYRIGHT file for a full list.
  * 
  * Infernal is distributed under the terms of the GNU General Public License
@@ -86,8 +86,7 @@ SSE_CYKScan(CM_t *cm, char *errbuf, CM_SCAN_MX *smx, ESL_DSQ *dsq, int i0, int j
   int       W;                  /* max d; max size of a hit, this is min(L, smx->W) */
   int       sd;                 /* StateDelta(cm->sttype[v]), # emissions from v */
   int      *dnA, *dxA;          /* tmp ptr to 1 row of dnAA, dxAA */
-  int       dn,   dx;           /* minimum/maximum valid d for current state */
-  int       cnum;               /* number of children for current state */
+  /*int       dn,   dx; */      /* minimum/maximum valid d for current state */
   int      *jp_wA;              /* rolling pointer index for B states, gets precalc'ed */
   float   **init_scAA;          /* [0..v..cm->M-1][0..d..W] initial score for each v, d for all j */
   double  **act;                /* [0..j..W-1][0..a..abc->K-1], alphabet count, count of residue a in dsq from 1..jp where j = jp%(W+1) */
@@ -393,9 +392,8 @@ SSE_CYKScan(CM_t *cm, char *errbuf, CM_SCAN_MX *smx, ESL_DSQ *dsq, int i0, int j
 	  jp_v = (cm->stid[v] == BEGL_S) ? (j % (W+1)) : cur;
 	  jp_y = (StateRightDelta(cm->sttype[v]) > 0) ? prv : cur;
 	  sd   = StateDelta(cm->sttype[v]);
-	  cnum = cm->cnum[v];
-	  dn   = dnA[v];
-	  dx   = dxA[v];
+	  /*dn   = dnA[v];*/
+	  /*dx   = dxA[v];*/
 
 	  if(cm->sttype[v] == B_st) {
 	    float *vec_access;
@@ -717,7 +715,7 @@ SSE_CYKScan(CM_t *cm, char *errbuf, CM_SCAN_MX *smx, ESL_DSQ *dsq, int i0, int j
     /* first, set srcL for all hits to length of sequence, this is required for overlap removal */
     for(h = 0; h < tmp_hitlist->N; h++) tmp_hitlist->unsrt[h].srcL = L;
     cm_tophits_SortForOverlapRemoval(tmp_hitlist);
-    if((status = cm_tophits_RemoveOverlaps(tmp_hitlist, errbuf)) != eslOK) return status;
+    if((status = cm_tophits_RemoveOrMarkOverlaps(tmp_hitlist, FALSE, errbuf)) != eslOK) return status;
     for(h = 0; h < tmp_hitlist->N; h++) { 
       if(! (tmp_hitlist->hit[h]->flags & CM_HIT_IS_REMOVED_DUPLICATE)) { 
 	if((status = cm_tophits_CloneHitMostly(tmp_hitlist, h, hitlist)) != eslOK) ESL_FAIL(status, errbuf, "problem copying hit to hitlist, out of memory?");
