@@ -10,6 +10,7 @@
  * Separate file started May 17, 2002
  */
 #include "esl_config.h"
+#include "p7_config.h"
 #include "config.h"
 
 #include <stdio.h>
@@ -18,89 +19,23 @@
 #include <ctype.h>
 #include <string.h>
 
-
 #include "easel.h"
 #include "esl_exponential.h"
 #include "esl_histogram.h"
 #include "esl_random.h"
 #include "esl_sq.h"
 #include "esl_sqio.h"
+#include "esl_ssi.h"
 #include "esl_vectorops.h"
 
-#include "cm_dispatch.h"
-#include "funcs.h"		/* external functions                   */
-#include "structs.h"		/* data structures, macros, #define's   */
+#include "hmmer.h"
 
-/* Function: AllocCMStats()
- * Date:     EPN, Wed May  2 14:10:25 2007      
- *
- * Purpose:  Allocate a CMStats_t data structure given
- *           the number of partitions and relevant info needed
- *           for creating the HMMFilter_t objects.
- * 
- * Args:     np - number of partitions 
- */
-CMStats_t *
-AllocCMStats(int np)
+#include "infernal.h"
+
+
+int debug_print_expinfo_array(CM_t *cm, char *errbuf, ExpInfo_t **expA)
 {
   int status;
-  CMStats_t  *cmstats;
-  int i, p;
-
-  ESL_ALLOC(cmstats, sizeof(struct cmstats_s));
-
-  cmstats->np = np;
-  ESL_ALLOC(cmstats->ps, sizeof(int) * cmstats->np);
-  ESL_ALLOC(cmstats->pe, sizeof(int) * cmstats->np);
-  ESL_ALLOC(cmstats->expAA, sizeof(ExpInfo_t **) * EXP_NMODES);
-  ESL_ALLOC(cmstats->hfiA,  sizeof(HMMFilterInfo_t *) * FTHR_NMODES);
-  for(i = 0; i < EXP_NMODES; i++) {
-    ESL_ALLOC(cmstats->expAA[i], sizeof(ExpInfo_t *) * cmstats->np);
-    for(p = 0; p < cmstats->np; p++) {
-      cmstats->expAA[i][p] = CreateExpInfo();
-      if(cmstats->expAA[i][p] == NULL) goto ERROR; /* memory error */
-    }
-  }
-  for(i = 0; i < FTHR_NMODES; i++) {
-    cmstats->hfiA[i] = CreateHMMFilterInfo();
-    if(cmstats->hfiA[i] == NULL) goto ERROR; /* memory error */
-  }
-  return cmstats;
-
- ERROR:
-  cm_Fail("AllocCMStats() memory allocation error.");
-  return NULL; /* never reached */
-}
-
-/* Function: FreeCMStats()
- * Returns: (void) 
- */
-void 
-FreeCMStats(CMStats_t *cmstats)
-{
-  int i, p;
-  for(i = 0; i < EXP_NMODES; i++)
-    {
-      for(p = 0; p < cmstats->np; p++)
-	if(cmstats->expAA[i][p] != NULL) free(cmstats->expAA[i][p]);
-      free(cmstats->expAA[i]);
-    }
-  free(cmstats->expAA);
-  for(i = 0; i < FTHR_NMODES; i++)
-    FreeHMMFilterInfo(cmstats->hfiA[i]);
-  free(cmstats->hfiA);
-  free(cmstats->ps);
-  free(cmstats->pe);
-  free(cmstats);
-}  
-
-
-/* Function: debug_print_cmstats
- */
-int debug_print_cmstats(CM_t *cm, char *errbuf, CMStats_t *cmstats, int has_fthr)
-{
-  int status;
-  int p;
   char *namedashes;
   int ni;
   int namewidth = strlen(cm->name); 
@@ -108,46 +43,22 @@ int debug_print_cmstats(CM_t *cm, char *errbuf, CMStats_t *cmstats, int has_fthr
   namedashes[namewidth] = '\0';
   for(ni = 0; ni < namewidth; ni++) namedashes[ni] = '-';
 
-  printf("Num partitions: %d\n", cmstats->np);
-  for (p = 0; p < cmstats->np; p++)
-    {
-      printf("Partition %d: start: %d end: %d\n", p, cmstats->ps[p], cmstats->pe[p]);
-      printf("cm_lc  exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CM_LC][p]);
-      printf("cm_gc  exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CM_GC][p]);
-      printf("cm_li  exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CM_LI][p]);
-      printf("cm_gi  exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CM_GI][p]);
-      printf("cp9_lv exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CP9_LV][p]);
-      printf("cp9_gv exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CP9_GV][p]);
-      printf("cp9_lf exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CP9_LF][p]);
-      printf("cp9_gf exp tail:\t");
-      debug_print_expinfo(cmstats->expAA[EXP_CP9_GF][p]);
-      printf("\n\n");
-    }
-
-  if(has_fthr)
-    {
-      printf("Filter CM_LC info:\n");
-      if((status = DumpHMMFilterInfo(stdout, cmstats->hfiA[FTHR_CM_LC], errbuf, cm, EXP_CM_LC, EXP_CP9_LF, cmstats->hfiA[FTHR_CM_LC]->dbsize, 1, namewidth, namedashes)) != eslOK) return status;
-      printf("Filter CM_LI info:\n");
-      if((status = DumpHMMFilterInfo(stdout, cmstats->hfiA[FTHR_CM_LI], errbuf, cm, EXP_CM_LI, EXP_CP9_LF, cmstats->hfiA[FTHR_CM_LI]->dbsize, 1, namewidth, namedashes)) != eslOK) return status;
-      printf("Filter CM_GC info:\n");
-      if((status = DumpHMMFilterInfo(stdout, cmstats->hfiA[FTHR_CM_GC], errbuf, cm, EXP_CM_GC, EXP_CP9_GF, cmstats->hfiA[FTHR_CM_GC]->dbsize, 1, namewidth, namedashes)) != eslOK) return status;
-      printf("Filter CM_GI info:\n");
-      if((status = DumpHMMFilterInfo(stdout, cmstats->hfiA[FTHR_CM_GI], errbuf, cm, EXP_CM_GI, EXP_CP9_GF, cmstats->hfiA[FTHR_CM_GI]->dbsize, 1, namewidth, namedashes)) != eslOK) return status;
-      printf("\n\n");
-    }
+  if(expA != NULL) { 
+    printf("cm_lc  exp tail:\t");
+    debug_print_expinfo(expA[EXP_CM_LC]);
+    printf("cm_gc  exp tail:\t");
+    debug_print_expinfo(expA[EXP_CM_GC]);
+    printf("cm_li  exp tail:\t");
+    debug_print_expinfo(expA[EXP_CM_LI]);
+    printf("cm_gi  exp tail:\t");
+    debug_print_expinfo(expA[EXP_CM_GI]);
+    printf("\n\n");
+  }
   free(namedashes);
   return eslOK;
 
  ERROR:
-  ESL_FAIL(status, errbuf, "Memory allocation error in debug_print_cmstats().");
+  ESL_FAIL(status, errbuf, "Memory allocation error in debug_print_expinfo_array().");
   return status; /* NEVERREACHED */
 }
 
@@ -155,7 +66,7 @@ int debug_print_cmstats(CM_t *cm, char *errbuf, CMStats_t *cmstats, int has_fthr
  */
 int debug_print_expinfo(ExpInfo_t *exp)
 {
-  if(exp->is_valid) printf("cur_eff_dbsize: %ld lambda: %f mu_extrap: %f mu_orig: %f dbsize: %ld nrandhits: %d tailp: %f (valid)\n", exp->cur_eff_dbsize, exp->lambda, exp->mu_extrap, exp->mu_orig, exp->dbsize, exp->nrandhits, exp->tailp);
+  if(exp->is_valid) printf("cur_eff_dbsize: %f lambda: %f mu_extrap: %f mu_orig: %f dbsize: %.0f nrandhits: %d tailp: %f (valid)\n", exp->cur_eff_dbsize, exp->lambda, exp->mu_extrap, exp->mu_orig, exp->dbsize, exp->nrandhits, exp->tailp);
   else              printf("invalid (not yet set)\n");
   fflush(stdout);
   return eslOK;
@@ -246,11 +157,14 @@ int get_alphabet_comp(const ESL_ALPHABET *abc, ESL_DSQ *dsq, int start, int stop
  *
  * Date:     EPN, Wed Apr  2 16:33:19 2008
  *
- * Purpose:  Given a sequence file name, determine the total size of the
- *           seqs in the file.
+ * Purpose:  Given a sequence file ptr, determine the number, and summed 
+ *           size of the seqs in the file. 
  *
- * Args:     sqfp     - open sequence file
- *           ret_N    - RETURN: total length (residues) or all seqs in seqfile
+ * Args:     sqfp          - open sequence file
+ *           errbuf        - for writing error messages
+ *           ret_N         - RETURN: total length (residues) or all seqs in sqfp
+ *           ret_nseq      - RETURN: number of seqs in sqfp
+ *           ret_namewidth - RETURN: max length of name in sqfp
  *
  * Returns:  eslOK on success, other status on failure, errbuf filled with error message.
  */
@@ -263,17 +177,19 @@ int GetDBSize (ESL_SQFILE *sqfp, char *errbuf, long *ret_N, int *ret_nseq, int *
   int     nseq = 0;
 
   sq = esl_sq_Create(); 
+
   while ((status = esl_sqio_ReadInfo(sqfp, sq)) == eslOK) {
-    nseq++;
     N += sq->L;
     namewidth = ESL_MAX(namewidth, strlen(sq->name));
     esl_sq_Reuse(sq); 
+    nseq++;
   } 
-  if (status != eslEOF) 
-    ESL_FAIL(status, errbuf, "Parse failed, line %" PRId64 ", file %s:\n%s", 
-	     sqfp->linenumber, sqfp->filename, sqfp->errbuf); 
+  if (status != eslEOF) { 
+    ESL_FAIL(status, errbuf, "Parse failed, file %s:\n%s", 
+	     sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
+  }
   esl_sq_Destroy(sq); 
-  esl_sqfile_Position(sqfp, (off_t) 0);
+  esl_sqfile_Position(sqfp, (off_t) 0); /* rewind sequence file to beginning */
 
   if(ret_N != NULL)          *ret_N         = N;
   if(ret_nseq != NULL)       *ret_nseq      = nseq;
@@ -347,8 +263,8 @@ int GetDBInfo (const ESL_ALPHABET *abc, ESL_SQFILE *sqfp, char *errbuf, long *re
     esl_sq_Reuse(sq); 
   } 
   if (status != eslEOF) 
-    ESL_FAIL(status, errbuf, "Parse failed, line %" PRId64 ", file %s:\n%s",
-	     sqfp->linenumber, sqfp->filename, sqfp->errbuf); 
+    ESL_FAIL(status, errbuf, "Parse failed, file %s:\n%s",
+	     sqfp->filename, esl_sqfile_GetErrorBuf(sqfp));
   esl_sq_Destroy(sq); 
   esl_sqfile_Position(sqfp, (off_t) 0);
 
@@ -362,39 +278,6 @@ int GetDBInfo (const ESL_ALPHABET *abc, ESL_SQFILE *sqfp, char *errbuf, long *re
   ESL_FAIL(status, errbuf, "GetDBInfo(): memory allocation error.");
 }
 
-/* Function: E2MinScore()
- * Date:     EPN, Tue Dec 11 15:40:25 2007 
- *           (morphed from RSEARCH: RJK, Mon April 15 2002 [St. Louis])
- *
- * Purpose:  Given an E-value <E> and a CM with valid exp tail stats 
- *           determine the minimal bit score that will give an E-value 
- *           of <E> across all partitions. This will be a safe bit score
- *           cutoff to use when returning hits in DispatchSearch().
- *           Because no score less than this will have an E-value 
- *           less than E.
- *
- * Returns:  eslOK on success, <ret_sc> filled with bit score
- *           error code on failure, errbuf filled with message
- */
-int E2MinScore (CM_t *cm, char *errbuf, int exp_mode, float E, float *ret_sc)
-{
-  int status;
-  float low_sc, sc;
-  int p;
-
-  /* contract checks */
-  if(!(cm->flags & CMH_EXPTAIL_STATS)) ESL_FAIL(eslEINCOMPAT, errbuf, "E2MinScore, CM's CMH_EXPTAIL_STATS flag is down.");
-  if(ret_sc == NULL)                   ESL_FAIL(eslEINCOMPAT, errbuf, "E2MinScore, ret_sc is NULL");
-
-  if((status = E2ScoreGivenExpInfo(cm->stats->expAA[exp_mode][0], errbuf, E, &low_sc)) != eslOK) return status;
-  for(p = 1; p < cm->stats->np; p++) {
-    if((status = E2ScoreGivenExpInfo(cm->stats->expAA[exp_mode][p], errbuf, E, &sc)) != eslOK) return status;
-    low_sc = ESL_MIN(low_sc, sc);
-  }
-  *ret_sc = low_sc;
-  return eslOK;
-}
-
 /* Function: E2ScoreGivenExpInfo()
  * Date:     EPN, Thu Apr  3 15:57:34 2008
  *
@@ -405,53 +288,36 @@ int E2MinScore (CM_t *cm, char *errbuf, int exp_mode, float E, float *ret_sc)
  * Returns:  eslOK on success, <ret_sc> filled with bit score
  *           error code on failure, errbuf filled with message
  */
-int E2ScoreGivenExpInfo(ExpInfo_t *exp, char *errbuf, float E, float *ret_sc)
+int E2ScoreGivenExpInfo(ExpInfo_t *exp, char *errbuf, double E, float *ret_sc)
 {
   float sc;
   /* contract checks */
-  if(ret_sc == NULL)                   ESL_FAIL(eslEINCOMPAT, errbuf, "E2ScoreGivenExpInfo, ret_sc is NULL");
+  if(ret_sc == NULL) ESL_FAIL(eslEINCOMPAT, errbuf, "E2ScoreGivenExpInfo, ret_sc is NULL");
   sc  = exp->mu_extrap + ((log(E/exp->cur_eff_dbsize)) / (-1 * exp->lambda));
   *ret_sc = sc;
   return eslOK;
 }
 
-
-/* Function: Score2MaxE()
- * Date:     EPN, Wed Jan 16 14:25:44 2008
+/* Function: P2ScoreGivenExpInfo()
+ * Date:     EPN, Sun Mar 18 10:58:17 2012
  *
- * Purpose:  Given a bit score <sc> and a CM with valid exp tail stats
- *           determine the maximal E-value that will be assigned to 
- *           bit score of <sc> across all partitions. 
- *           This will be a 'safe' E-value cutoff to use 
- *           to always return hits with bit score of <sc> or greater.
- *           because no E-value higher than this will be assigned to
- *           a bit score greater than <sc>.
+ * Purpose:  Given a P-value <P> <E> and a exp tail stat structure
+ *           determine the bit score that will give a P-value 
+ *           of <P>.
  *
- * Returns:  eslOK on success, <ret_E> filled with E value
+ * Returns:  eslOK on success, <ret_sc> filled with bit score
  *           error code on failure, errbuf filled with message
  */
-int Score2MaxE (CM_t *cm, char *errbuf, int exp_mode, float sc, float *ret_E)
+int P2ScoreGivenExpInfo(ExpInfo_t *exp, char *errbuf, double P, float *ret_sc)
 {
-  float high_E, E;
-  int p;
-
+  float sc;
   /* contract checks */
-  if(!(cm->flags & CMH_EXPTAIL_STATS)) ESL_FAIL(eslEINCOMPAT, errbuf, "Score2E(), CM's CMH_EXPTAIL_STATS flag is down.");
-  if(ret_E == NULL)                   ESL_FAIL(eslEINCOMPAT, errbuf, "Score2E(), ret_E is NULL");
-
-  high_E = Score2E(sc, cm->stats->expAA[exp_mode][0]->mu_extrap, cm->stats->expAA[exp_mode][0]->lambda, cm->stats->expAA[exp_mode][0]->cur_eff_dbsize);
-
-  if(! cm->stats->expAA[exp_mode][0]->is_valid) ESL_FAIL(eslEINCOMPAT, errbuf, "Score2E(), CM's exp tail stats for mode: %d partition: %d are invalid.", exp_mode, p);
-  for(p = 1; p < cm->stats->np; p++) {
-    if(! cm->stats->expAA[exp_mode][p]->is_valid) ESL_FAIL(eslEINCOMPAT, errbuf, "Score2E(), CM's exp tail stats for mode: %d partition: %d are invalid.", exp_mode, p);
-    E = Score2E(sc, cm->stats->expAA[exp_mode][p]->mu_extrap, cm->stats->expAA[exp_mode][p]->lambda, cm->stats->expAA[exp_mode][p]->cur_eff_dbsize);
-    high_E = ESL_MAX(high_E, E);
-  }
-  *ret_E = high_E;
+  if(ret_sc == NULL) ESL_FAIL(eslEINCOMPAT, errbuf, "P2ScoreGivenExpInfo, ret_sc is NULL");
+  sc  = exp->mu_extrap + (log(P) / (-1 * exp->lambda));
+  *ret_sc = sc;
   return eslOK;
 }
 
-
 /* Function: Score2E()
  * Date:     EPN, Fri Feb 15 12:40:21 2008
  *
@@ -461,77 +327,39 @@ int Score2MaxE (CM_t *cm, char *errbuf, int exp_mode, float sc, float *ret_E)
  *
  * Returns:  E value of <x>
  */
-double Score2E (float x, double mu, double lambda, long eff_dbsize)
+double Score2E (float x, double mu, double lambda, double eff_dbsize)
 {
-  return esl_exp_surv(x, mu, lambda) * (double) eff_dbsize;
+  return esl_exp_surv(x, mu, lambda) * eff_dbsize;
 }
 
-/* Function: CM2ExpMode
- * Date:     EPN, Mon May  7 17:43:28 2007
- * Purpose:  Return the exp_mode for the CM and HMM
- *           given the flags and search options in the
- *           CM data structure.
+/* Function: cm_p7_E2Score()
+ * Date:     EPN, Tue Apr 17 14:46:51 2012
+ *
+ * Purpose:  Given an E-value <E>, a database size in residues <Z>, a
+ *           fixed hit length <hitlen>, and a mu and lambda for a
+ *           P7_HMM, determine the bit score that will give an E-value
+ *           of <E>. This function computes the E-value the same way
+ *           nhmmer does, as of svn r3990.
+ *
+ * Returns:  bit score corresponding to E-value <E>.
  */
-int CM2ExpMode(CM_t *cm, int search_opts, int *ret_cm_exp_mode, int *ret_cp9_exp_mode)
+float cm_p7_E2Score(double E, double Z, int hitlen, float mu, float lambda)
 {
-  int cm_exp_mode;
-  int cp9_exp_mode;
-
-  /* check contract */
-  if(!(cm->flags & CMH_CP9) || cm->cp9 == NULL) cm_Fail("ERROR no CP9 in CM2ExpMode()\n");
-  if(cm->flags & CMH_LOCAL_BEGIN) {
-    if(search_opts & CM_SEARCH_INSIDE) cm_exp_mode = EXP_CM_LI;
-    else               	               cm_exp_mode = EXP_CM_LC;
-  }
-  else {
-    if(search_opts & CM_SEARCH_INSIDE) cm_exp_mode = EXP_CM_GI;
-    else        	               cm_exp_mode = EXP_CM_GC;
-  }
-
-  if(cm->cp9->flags & CPLAN9_LOCAL_BEGIN) {
-    if(search_opts & CM_SEARCH_HMMFORWARD) cp9_exp_mode = EXP_CP9_LF;
-    else                                   cp9_exp_mode = EXP_CP9_LV;
-  }
-  else {
-    if(search_opts & CM_SEARCH_HMMFORWARD) cp9_exp_mode = EXP_CP9_GF;
-    else                                   cp9_exp_mode = EXP_CP9_GV;
-  }
-
-  if(ret_cm_exp_mode  != NULL) *ret_cm_exp_mode  = cm_exp_mode;
-  if(ret_cp9_exp_mode != NULL) *ret_cp9_exp_mode = cp9_exp_mode;
-  return eslOK;
+  return mu + ((log(E/(Z / (float) hitlen))) / (-1 * lambda));
 }
 
-
-/*
- * Function: CM2FthrMode
- * Date:     EPN, Tue Dec 11 13:16:35 2007
- * Purpose:  Return the filter threshold mode for the CM 
- *           given CM's flags and a passed in search options
- *           int.
- * 
- * Returns: eslOK on success, eslEINCOMPAT if search_opts indicate
- *          we're doing HMM search, errbuf is filled with error message.
+/* Function: cm_p7_P2Score()
+ * Date:     EPN, Tue Apr 17 14:46:51 2012
+ *
+ * Purpose:  Given a P value and a mu and lambda for a
+ *           P7_HMM, determine the bit score that corresponds with 
+ *           that P value.
+ *
+ * Returns:  bit score corresponding to P-value <P>.
  */
-int CM2FthrMode(CM_t *cm, char *errbuf, int search_opts, int *ret_fthr_mode)
+float cm_p7_P2Score(double P, float mu, float lambda)
 {
-  int fthr_mode;
-
-  /* check contract */
-  if(search_opts & CM_SEARCH_HMMVITERBI) ESL_FAIL(eslEINCOMPAT, errbuf, "CM2FThrMode(), search_opts CM_SEARCH_HMMVITERBI flag raised.\n");
-  if(search_opts & CM_SEARCH_HMMFORWARD) ESL_FAIL(eslEINCOMPAT, errbuf, "CM2FThrMode(), search_opts CM_SEARCH_HMMFORWARD flag raised.\n");
-  if(ret_fthr_mode == NULL) ESL_FAIL(eslEINCOMPAT, errbuf, "CM2FThrMode(), ret_fthr_mode is NULL.");
-
-  if(cm->flags & CMH_LOCAL_BEGIN) {
-    if(search_opts & CM_SEARCH_INSIDE) fthr_mode = FTHR_CM_LI;
-    else               	               fthr_mode = FTHR_CM_LC;
-  }
-  else {
-    if(search_opts & CM_SEARCH_INSIDE) fthr_mode = FTHR_CM_GI;
-    else        	               fthr_mode = FTHR_CM_GC;
-  }
-  if(ret_fthr_mode  != NULL) *ret_fthr_mode  = fthr_mode;
-  return eslOK;
+  return mu + (log(P) / (-1 * lambda));
 }
 
 /* Function: ExpModeIsLocal()
@@ -549,14 +377,10 @@ ExpModeIsLocal(int exp_mode)
   switch (exp_mode) {
   case EXP_CM_LC: 
   case EXP_CM_LI: 
-  case EXP_CP9_LV: 
-  case EXP_CP9_LF: 
     return TRUE;
     break;
   case EXP_CM_GC:
   case EXP_CM_GI: 
-  case EXP_CP9_GV: 
-  case EXP_CP9_GF: 
     return FALSE;
     break;
   default: 
@@ -565,71 +389,31 @@ ExpModeIsLocal(int exp_mode)
   return FALSE; /* never reached */
 }
 
-
-/* Function: ExpModeIsForCM()
+/* Function: ExpModeIsInside()
  * Date:     EPN, Mon Dec 10 09:11:55 2007
  * Purpose:  Given a exp tail mode, return TRUE if it corresponds to 
- *           a CM (not a CP9 HMM).
+ *           Inside, false if it corresponds to Inside.
  *
- * Args:     exp_mode     - the mode 0..EXP_NMODES-1
+ * Args:     exp_mode  - the mode 0..EXP_NMODES-1
  */
 int
-ExpModeIsForCM(int exp_mode)
+ExpModeIsInside(int exp_mode)
 {
   ESL_DASSERT1((exp_mode >= 0 && exp_mode < EXP_NMODES));
 
   switch (exp_mode) {
-  case EXP_CM_LC: 
   case EXP_CM_LI: 
-  case EXP_CM_GC:
-  case EXP_CM_GI: 
+  case EXP_CM_GI:
     return TRUE;
     break;
-  case EXP_CP9_LV: 
-  case EXP_CP9_LF: 
-  case EXP_CP9_GV: 
-  case EXP_CP9_GF: 
+
+  case EXP_CM_LC: 
+  case EXP_CM_GC: 
     return FALSE;
     break;
-  default: 
-    cm_Fail("ExpModeIsForCM(): bogus exp_mode: %d\n", exp_mode);
-  }
-  return FALSE; /* never reached */
-}
 
-/* Function: ExpModeToFthrMode()
- * Date:     EPN, Mon Dec 10 09:31:48 2007
- * Purpose:  Given a exp tail mode, return it's corresponding
- *           filter threshold mode, or -1 if there is none
- *
- * Args:     exp_mode     - the mode 0..EXP_NMODES-1
- */
-int
-ExpModeToFthrMode(int exp_mode)
-{
-  ESL_DASSERT1((exp_mode >= 0 && exp_mode < EXP_NMODES));
-
-  switch (exp_mode) {
-  case EXP_CM_GC: 
-    return FTHR_CM_GC;
-    break;
-  case EXP_CM_GI: 
-    return FTHR_CM_GI;
-    break;
-  case EXP_CM_LC: 
-    return FTHR_CM_LC;
-    break;
-  case EXP_CM_LI: 
-    return FTHR_CM_LI;
-    break;
-  case EXP_CP9_LV: 
-  case EXP_CP9_LF: 
-  case EXP_CP9_GV: 
-  case EXP_CP9_GF: 
-    return -1;
-    break;
   default: 
-    cm_Fail("ExpModeToFthrMode(): bogus exp_mode: %d\n", exp_mode);
+    cm_Fail("ExpModeIsInside(): bogus exp_mode: %d\n", exp_mode);
   }
   return FALSE; /* never reached */
 }
@@ -649,7 +433,7 @@ CreateExpInfo()
   ExpInfo_t *exp = NULL;
   ESL_ALLOC(exp, sizeof(ExpInfo_t));
 
-  exp->cur_eff_dbsize = 0;
+  exp->cur_eff_dbsize = 0.;
   exp->lambda         = 0.;
   exp->mu_extrap      = 0.;
   exp->mu_orig        = 0.;
@@ -673,7 +457,7 @@ CreateExpInfo()
  * Returns:  void
  */
 void 
-SetExpInfo(ExpInfo_t *exp, double lambda, double mu_orig, long dbsize, int nrandhits, double tailp)
+SetExpInfo(ExpInfo_t *exp, double lambda, double mu_orig, double dbsize, int nrandhits, double tailp)
 {
   exp->lambda    = lambda;
   exp->mu_orig   = mu_orig;
@@ -686,7 +470,7 @@ SetExpInfo(ExpInfo_t *exp, double lambda, double mu_orig, long dbsize, int nrand
   /* initialize exp->cur_eff_dbsize as effective database size as exp->nrandhits, this is
    * effective database size if we searched a database of size <exp->dbsize>, which we 
    * just calibrated for. we'll update this in cmsearch for the target database size. */
-  exp->cur_eff_dbsize = (long) exp->nrandhits;
+  exp->cur_eff_dbsize = (double) exp->nrandhits;
 
   exp->is_valid  = TRUE; /* we can now write Exp Info to a cm file */
   return;
@@ -733,60 +517,228 @@ char *
 DescribeExpMode(int exp_mode)
 {
   switch (exp_mode) {
-  case EXP_CP9_GV: return "hmm  glc  vit";
-  case EXP_CP9_GF: return "hmm  glc  fwd";
   case EXP_CM_GC:  return " cm  glc  cyk";
   case EXP_CM_GI:  return " cm  glc  ins";
-  case EXP_CP9_LV: return "hmm  loc  vit";
-  case EXP_CP9_LF: return "hmm  loc  fwd";
   case EXP_CM_LC:  return " cm  loc  cyk";
   case EXP_CM_LI:  return " cm  loc  ins";
   default:     return "?";
   }
 }
 
-
-/* Function:  DescribeFthrMode()
- * Incept:    EPN, Mon Jan  7 18:41:41 2008
- *
- * Purpose:   Returns the Filter thresold mode in text.
- *            For example, <DescribeFThrMode(EXP_CM_GC)>
- *            returns "glocal CM CYK".
- */
-char *
-DescribeFthrMode(int fthr_mode)
-{
-  switch (fthr_mode) {
-  case FTHR_CM_GC:  return "glc  cyk";
-  case FTHR_CM_GI:  return "glc  ins";
-  case FTHR_CM_LC:  return "loc  cyk";
-  case FTHR_CM_LI:  return "loc  ins";
-  default:     return "?";
-  }
-}
-
 /* Function: UpdateExpsForDBSize()
  * Date:     EPN, Thu Jan 17 09:38:06 2008
  *
  * Purpose:  Update the <cur_eff_dbsize> parameter of the
- *           ExpInfo_t objects in a CM's cm->stats object 
+ *           ExpInfo_t objects in a CM's cm->expA object 
  *           to reflect a database of size <dbsize>.
  *            
  * Returns:  eslOK on success, other Easel status code on contract 
  *           violation with informative error message in errbuf.
  */
 int 
-UpdateExpsForDBSize(CM_t *cm, char *errbuf, long dbsize)
+UpdateExpsForDBSize(CM_t *cm, char *errbuf, double dbsize)
 {
-  int i, p;
+  int i;
   /* contract checks */
   if(! (cm->flags & CMH_EXPTAIL_STATS)) ESL_FAIL(eslEINCOMPAT, errbuf, "UpdateExpsForDBSize(), cm does not have Exp stats.");
 
   for(i = 0; i < EXP_NMODES; i++) { 
-    for(p = 0; p < cm->stats->np; p++) {
-      cm->stats->expAA[i][p]->cur_eff_dbsize = (long) ((((double) dbsize / (double) cm->stats->expAA[i][p]->dbsize) * 
-							((double) cm->stats->expAA[i][p]->nrandhits)) + 0.5);
-    }
+    cm->expA[i]->cur_eff_dbsize = (dbsize / cm->expA[i]->dbsize) * ((double) cm->expA[i]->nrandhits);
   }
+
   return eslOK;
 }  
+
+
+
+/* Function: CreateGenomicHMM()
+ * Date:     EPN, Tue May 20 17:40:54 2008
+ * 
+ * Purpose: Create the three arrays that make up the parameters of the
+ *          fully connected 5 state HMM that emits 'realistic' genomic
+ *          sequence for calculating E-value statistics. 
+ *           
+ *          The HMM was trained by EM from 30 Mb of 100 Kb chunks of
+ *          real genomes of hand selected GC contents (10 Mb each from
+ *          Archaea, Bacteria, Eukarya genomes). See
+ *          ~nawrockie/notebook/8_0326_inf_default_gc/ for more info.
+ *
+ *          There were larger HMMs that 'performed' better, but this 5
+ *          state guy was a good balance b/t performance and number of
+ *          parameters. Performance was judged by how similar the
+ *          generated sequence was to the training 30 Mb genomic
+ *          sequence.
+ * 
+ *          abc - alphabet, must be eslRNA | eslDNA
+ *          errbuf - for error messages
+ *          ret_sA  - RETURN: start probabilities [0..nstates-1]
+ *          ret_tAA - RETURN: transition probabilities [0..nstates-1][0..nstates-1]
+ *          ret_eAA - RETURN: emission probabilities   [0..nstates-1][0..abc->K-1]
+ *          ret_nstates - RETURN: number of states (5)
+ */
+int
+CreateGenomicHMM(const ESL_ALPHABET *abc, char *errbuf, double **ret_sA, double ***ret_tAA, double ***ret_eAA, int *ret_nstates)
+{
+  int      status;
+  int      nstates = 5;
+  int      i;
+
+  /* contract check, make sure we're in a valid mode */
+  if(abc->type != eslRNA && abc->type != eslDNA) ESL_FAIL(eslEINCOMPAT, errbuf, "get_genomic_sequence_from_hmm(), abc is not eslRNA nor eslDNA");
+
+  /* start probabilities */
+  double *sA;
+  ESL_ALLOC(sA, sizeof(double) * nstates);
+
+  sA[0] = 0.157377049180328;
+  sA[1] = 0.39344262295082;
+  sA[2] = 0.265573770491803; 
+  sA[3] = 0.00327868852459016; 
+  sA[4] = 0.180327868852459;
+  esl_vec_DNorm(sA, nstates);
+
+  /* transition probabilities */
+  double **tAA;
+  ESL_ALLOC(tAA, sizeof(double *) * nstates);
+  for(i = 0; i < nstates; i ++) ESL_ALLOC(tAA[i], sizeof(double) * nstates);
+
+  tAA[0][0] = 0.999483637183643;
+  tAA[0][1] = 0.000317942006440604; 
+  tAA[0][2] = 0.000185401071732768; 
+  tAA[0][3] = 2.60394763669618e-07; 
+  tAA[0][4] = 1.27593434198113e-05;
+  esl_vec_DNorm(tAA[0], nstates);
+
+  tAA[1][0] = 9.76333640771184e-05; 
+  tAA[1][1] = 0.99980020511745; 
+  tAA[1][2] = 9.191359010352e-05; 
+  tAA[1][3] = 7.94413051888677e-08; 
+  tAA[1][4] = 1.01684870641751e-05;
+  esl_vec_DNorm(tAA[1], nstates);
+
+  tAA[2][0] = 1.3223694798182e-07; 
+  tAA[2][1] = 0.000155642887774602; 
+  tAA[2][2] = 0.999700615549769; 
+  tAA[2][3] = 9.15079680034191e-05; 
+  tAA[2][4] = 5.21013575048369e-05;
+  esl_vec_DNorm(tAA[2], nstates);
+
+  tAA[3][0] = 0.994252873563218; 
+  tAA[3][1] = 0.0014367816091954; 
+  tAA[3][2] = 0.0014367816091954; 
+  tAA[3][3] = 0.0014367816091954; 
+  tAA[3][4] = 0.0014367816091954;
+  esl_vec_DNorm(tAA[3], nstates);
+
+  tAA[4][0] = 8.32138798088677e-06; 
+  tAA[4][1] = 2.16356087503056e-05; 
+  tAA[4][2] = 6.42411152124459e-05; 
+  tAA[4][3] = 1.66427759617735e-07; 
+  tAA[4][4] = 0.999905635460297;
+  esl_vec_DNorm(tAA[4], nstates);
+
+  /* emission probabilities */
+  double **eAA;
+  ESL_ALLOC(eAA, sizeof(double *) * nstates);
+  for(i = 0; i < nstates; i ++) ESL_ALLOC(eAA[i], sizeof(double) * abc->K);
+
+  eAA[0][0] = 0.370906566523225;
+  eAA[0][1] = 0.129213995153577;
+  eAA[0][2] = 0.130511270043053;
+  eAA[0][3] = 0.369368168280145;
+  esl_vec_DNorm(eAA[0], abc->K);
+
+  eAA[1][0] = 0.305194882571888;
+  eAA[1][1] = 0.194580936415687;
+  eAA[1][2] = 0.192343972160245;
+  eAA[1][3] = 0.307880208852179;
+  esl_vec_DNorm(eAA[1], abc->K);
+
+  eAA[2][0] = 0.238484980800698;
+  eAA[2][1] = 0.261262845707113;
+  eAA[2][2] = 0.261810301531792;
+  eAA[2][3] = 0.238441871960397;
+  esl_vec_DNorm(eAA[2], abc->K);
+
+  eAA[3][0] = 0.699280575539568;
+  eAA[3][1] = 0.00143884892086331;
+  eAA[3][2] = 0.00143884892086331;
+  eAA[3][3] = 0.297841726618705;
+  esl_vec_DNorm(eAA[3], abc->K);
+
+  eAA[4][0] = 0.169064007664923;
+  eAA[4][1] = 0.331718611320207;
+  eAA[4][2] = 0.33045427183482;
+  eAA[4][3] = 0.16876310918005;
+  esl_vec_DNorm(eAA[4], abc->K);
+
+  *ret_sA = sA;
+  *ret_tAA = tAA;
+  *ret_eAA = eAA;
+  *ret_nstates = nstates;
+
+  return eslOK;
+
+ ERROR:
+  return status;
+}  
+
+/* Function: SampleGenomicSequenceFromHMM()
+ * Date:     EPN, Tue May 20 17:40:54 2008
+ * 
+ * Purpose: Sample a sequence of length L from an HMM. The HMM defined
+ *          by three arrays:
+ * 
+ *          sA  - start probabilities [0..nstates-1]
+ *          tAA - transition probabilities [0..nstates-1][0..nstates-1]
+ *          eAA - emission probabilities   [0..nstates-1][0..abc->K-1]
+ *
+ *          ret_dsq - the sampled sequence
+ */
+int
+SampleGenomicSequenceFromHMM(ESL_RANDOMNESS *r, const ESL_ALPHABET *abc, char *errbuf, double *sA, double **tAA, double **eAA, int nstates, int L, ESL_DSQ **ret_dsq)
+{
+  int status;
+  ESL_DSQ *dsq = NULL;
+  int      si, x;
+
+  ESL_ALLOC(dsq, sizeof(ESL_DSQ) * (L+2));
+  dsq[0] = dsq[L+1] = eslDSQ_SENTINEL;
+
+  /* pick initial state to emit from */
+  si = esl_rnd_DChoose(r, sA, nstates);
+  for (x = 1; x <= L; x++) {
+    dsq[x] = esl_rnd_DChoose(r, eAA[si], abc->K); /* emit residue */
+    si = esl_rnd_DChoose(r, tAA[si], nstates);    /* make transition */
+  }
+
+  *ret_dsq = dsq;
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
+/* Function: CopyExpInfo()
+ * Date:     EPN, Tue May 24 09:59:14 2011
+ *
+ * Purpose:  Copy an ExpInfo_t object.
+ *            
+ * Returns:  eslOK on success.
+ */
+int
+CopyExpInfo(ExpInfo_t *src, ExpInfo_t *dest)
+{
+  dest->cur_eff_dbsize = src->cur_eff_dbsize;
+  dest->lambda         = src->lambda;
+  dest->mu_extrap      = src->mu_extrap;
+  dest->mu_orig        = src->mu_orig;
+  dest->dbsize         = src->dbsize;
+  dest->nrandhits      = src->nrandhits;
+  dest->tailp          = src->tailp;
+  dest->is_valid       = src->is_valid;
+
+  return eslOK;
+}  
+
+
